function Connect-NtapStorageSystem {
    [CmdletBinding(DefaultParameterSetName = 'ONTAP')]
    param (
        [Parameter(Mandatory, Position = 0)]
        [ValidateNotNullOrEmpty()]
        [Alias('ClusterName', 'FilerName', 'SystemName', 'SystemId', 'ProxyUrl', 'Target', 'StorageGridUri')]
        [string]$Name,

        [Parameter()]
        [ValidateNotNullOrEmpty()]
        [Alias('Cred')]
        [pscredential]
        [System.Management.Automation.Credential()]
        $Credential = [pscredential]::Empty,

        [Parameter()]
        [ValidateRange([uint16]::MinValue, [uint16]::MaxValue)]
        [uint16]$Port,

        [Parameter()]
        [ValidateRange(0, [int]::MaxValue)]
        [int]$Timeout,

        [Parameter(ParameterSetName = 'ONTAP')]
        [Parameter(ParameterSetName = '7-Mode')]
        [Alias('ZapiRetryCount')]
        [ValidateRange(0, 5)]
        [uint16]$RetryCount = 3,

        [Parameter(ParameterSetName = '7-Mode')]
        [switch]$ForceRPC,

        [Parameter(ParameterSetName = 'ONTAP')]
        [Parameter(ParameterSetName = '7-Mode')]
        [Alias('ForceHTTPS')]
        [switch]$ForceSecure,

        [Parameter(ParameterSetName = 'OCI')]
        [switch]$Insecure,

        [Parameter(ParameterSetName = '7-Mode')]
        [switch]$NaController,

        [Parameter(ParameterSetName = 'ONTAP')]
        [switch]$NcController,

        [Parameter(ParameterSetName = 'SANtricity')]
        [switch]$NeCredential,

        [Parameter(ParameterSetName = 'SolidFire')]
        [switch]$SFCluster,

        [Parameter(ParameterSetName = 'SolidFire')]
        [switch]$Node,

        [Parameter(ParameterSetName = 'StorageGRID')]
        [switch]$StorageGrid,

        [Parameter(ParameterSetName = 'OCI')]
        [switch]$OciServer,

        [Parameter(ParameterSetName = 'VMware')]
        [switch]$VMware
    )

    $params = @{ }

    $sbVerboseMsg = New-Object -TypeName System.Text.StringBuilder

    try {
        switch -Exact ($PSCmdlet.ParameterSetName) {
            'SANtricity' {
                $params.Add('ProxyUrl', $Name)

                if ($Credential -ne [pscredential]::Empty) {
                    [void]$sbVerboseMsg.Append('Using supplied credentials')

                    $params.Add('SystemCredential', $Credential)
                }
                elseif (($cachedCredential = Get-NeCredential -CredentialName $Name)) {
                    [void]$sbVerboseMsg.Append('Using cached credentials')

                    $params.Add('Credential', $cachedCredential.Credential)
                }
                else {
                    throw 'Connection failed (No valid credentials specified for connection).'
                }
            }
            'SolidFire' {
                $params.Add('Target', $Name)

                if ($Credential -ne [pscredential]::Empty) {
                    [void]$sbVerboseMsg.Append('Using supplied credentials')

                    $params.Add('Credential', $Credential)
                }
                elseif (($cachedCredential = Get-NcCredential -Name $Name)) {
                    [void]$sbVerboseMsg.Append('Using cached credentials')

                    $params.Add('Credential', $cachedCredential.Credential)
                }
                else {
                    throw 'Connection failed (No valid credentials specified for connection).'
                }

                if ($PSCmdlet.MyInvocation.BoundParameters.ContainsKey('Port')) {
                    [void]$sbVerboseMsg.Append(" (specified port: $Port)")

                    $params.Add('Port', $Port)
                }

                if ($PSCmdlet.MyInvocation.BoundParameters.ContainsKey('Timeout')) {
                    $params.Add('Timeout', $Timeout)
                }

                if ($Node.IsPresent) {
                    $params.Add('Node', [switch]::IsPresent)
                }
            }
            'StorageGRID' {
                $params.Add(
                    'Headers', @{
                        'Accept'          = 'application/json'
                        'Accept-Encoding' = 'gzip'
                        'Content-Type'    = 'application/json'
                    }
                )

                if ($Credential -ne [pscredential]::Empty) {
                    [void]$sbVerboseMsg.Append('Using supplied credentials')

                    $params.Add(
                        'Body', $(
                            @{
                                username  = $Credential.UserName
                                password  = $Credential.GetNetworkCredential().Password
                                cookie    = $true
                                csrfToken = $false
                            } | ConvertTo-Json
                        )
                    )
                }
                elseif (($cachedCredential = Get-NcCredential -Name $Name)) {
                    [void]$sbVerboseMsg.Append('Using cached credentials')

                    $params.Add(
                        'Body', $(
                            @{
                                username  = $cachedCredential.Credential.UserName
                                password  = $cachedCredential.Credential.GetNetworkCredential().Password
                                cookie    = $true
                                csrfToken = $false
                            } | ConvertTo-Json
                        )
                    )
                }
                else {
                    throw 'Connection failed (No valid credentials specified for connection).'
                }

                if ($PSVersionTable.PSVersion.Major -gt 6) {
                    $params.Add('SkipCertificateCheck', $true)
                }
            }
            'VMware' {
                $params.Add('Server', $Name)

                if ($Credential -ne [pscredential]::Empty) {
                    [void]$sbVerboseMsg.Append('Using supplied credentials')

                    $params.Add('Credential', $Credential)
                }
                elseif (Get-NcCredential -Name $Name) {
                    [void]$sbVerboseMsg.Append('Using cached credentials')

                    $params.Add('Credential', (Get-NcCredential -Name $Name).Credential)
                }
                else {
                    throw 'Connection failed (No valid credentials specified for connection).'
                }

                if ($PSCmdlet.MyInvocation.BoundParameters.ContainsKey('Port')) {
                    [void]$sbVerboseMsg.Append(" (specified port: $Port)")

                    $params.Add('Port', $Port)
                }
            }
            default {
                $params.Add('Name', $Name)

                if ($Credential -ne [pscredential]::Empty) {
                    [void]$sbVerboseMsg.Append('Using supplied credentials')

                    $params.Add('Credential', $Credential)
                }
                elseif ($ForceRPC.IsPresent) {
                    [void]$sbVerboseMsg.Append('Forcing RPC')

                    $params.Add('RPC', [switch]::Present)
                }
                elseif (($cachedCredential = Get-NcCredential -Name $Name)) {
                    [void]$sbVerboseMsg.Append('Using cached credentials')

                    $params.Add('Credential', $cachedCredential.Credential)
                }
                else {
                    throw 'Connection failed (No valid credentials specified for connection).'
                }

                if ($PSCmdlet.MyInvocation.BoundParameters.ContainsKey('Port')) {
                    [void]$sbVerboseMsg.Append(" (specified port: $Port)")

                    $params.Add('Port', $Port)
                }

                if ($PSCmdlet.MyInvocation.BoundParameters.ContainsKey('Timeout')) {
                    $params.Add('Timeout', $Timeout)
                }

                if ($ForceSecure.IsPresent) {
                    if ($sbVerboseMsg[-1] -eq ')') {
                        [void]$sbVerboseMsg.Remove($sbVerboseMsg.Length - 1, 1)
                        [void]$sbVerboseMsg.Append(' / forcing secure connection)')
                    }
                    else {
                        [void]$sbVerboseMsg.Append(' (forcing secure connection)')
                    }

                    $params.Add('HTTPS', [switch]::Present)
                }

                if ($PSCmdlet.ParameterSetName -eq 'OCI' -and $Insecure.IsPresent) {
                    $params.Add('Insecure', [switch]::Present)
                }
            }
        }

        Write-Verbose -Message $sbVerboseMsg.ToString()

        switch -Exact ($PSCmdlet.ParameterSetName) {
            '7-Mode' {
                $controller = Connect-NaController @params -Transient -ErrorAction Stop

                $controller.RetryCount = $RetryCount
                #$controller.ApplicationName = 'NetAppDocs'
            }
            'ONTAP' {
                $controller = Connect-NcController @params -Transient -ErrorAction Stop

                $controller.RetryCount = $RetryCount
                #$controller.ApplicationName = 'NetAppDocs'
            }
            'SANtricity' {
                $controller = & (Get-Command -Name New-NeCredential -CommandType Cmdlet) @params -ErrorAction Stop
            }
            'SolidFire' {
                [void](Connect-SFCluster @params -ErrorAction Stop)
                $controller = $SFConnection

                Remove-Variable -Name SFConnection -ErrorAction Ignore
            }
            'StorageGRID' {
                $response = Invoke-WebRequest -Uri "$( $Name )api/v4/authorize" -Method Post @params -SessionVariable webSession -UserAgent "NetAppDocs/$( (Get-NtapDocsVersion -AsPatchVersion).ToString(3) )" -Verbose:$false | ConvertFrom-Json
                $webSession.Headers.Add('Authorization', "Bearer $( $response.data )")

                $controller = $response | Select-Object -Property @{
                    Name       = 'Uri'
                    Expression = { [uri]::new($Name) }
                }, @{
                    Name       = 'Type'
                    Expression = { 'StorageGRID' }
                }, @{
                    Name       = 'ApiVersion'
                    Expression = { $_.ApiVersion }
                }, @{
                    Name       = 'ReferenceDT'
                    Expression = { [datetimeoffset]::Parse($_.responseTime) }
                }, @{
                    Name       = 'Websession'
                    Expression = { $webSession }
                }
            }
            'OCI' {
                $controller = Connect-OciServer @params -ErrorAction Stop
            }
            'VMware' {
                [void](Set-PowerCLIConfiguration -Scope Session -DefaultVIServerMode Single -InvalidCertificateAction Ignore -ParticipateInCeip $false -DisplayDeprecationWarnings $false -Confirm:$false)
                $controller = Connect-VIServer @params -NotDefault -ErrorAction Stop
            }
        }

        return $controller
    }
    catch {
        $Global:Error.RemoveAt(0)

        $PSCmdlet.ThrowTerminatingError((New-Object -TypeName System.Management.Automation.ErrorRecord -ArgumentList ($_.Exception.Message, 'ConnectionFailed', [System.Management.Automation.ErrorCategory]::ConnectionError, $Name)))
    }
}

function Test-NtapStorageSystemConnection {
    [CmdletBinding()]
    [OutputType([System.Boolean])]
    param (
        [Parameter(Mandatory, Position = 0)]
        [ValidateNotNullOrEmpty()]
        [Alias('ProxyCredential')]
        [System.Object]$Controller,

        [Parameter()]
        [switch]$DisableLifCheck
    )

    try {
        switch -Exact ($Controller.GetType().FullName) {
            'NetApp.Ontapi.Filer.NaController' {
                try {
                    Write-Verbose -Message 'Validating successful connection'
                    [void](Get-NaSystemVersion -Controller $Controller -ErrorAction Stop)
                }
                catch {
                    throw "Connection failed ($( $_.Exception.GetBaseException().Message ))"
                }

                Write-Verbose -Message 'Validating user capabilities'
                if (-not (Test-7ModeUserCapability -AbstractController $Controller -Capabilities 'login-http-admin', 'api-*', 'cli-*', 'security-api-vfiler', 'security-priv-advanced')) {
                    throw "User account does not have one or more of the required capabilities (Required: 'login-http-admin', 'api-*', 'cli-*', 'security-api-vfiler', 'security-priv-advanced')."
                }
            }
            'NetApp.Ontapi.Filer.C.NcController' {
                try {
                    Write-Verbose -Message 'Validating successful connection'
                    [void](Get-NcSystemVersion -Controller $Controller -ErrorAction Stop)
                }
                catch {
                    $Global:Error.RemoveAt(0)

                    throw "Connection failed ($( $_.Exception.GetBaseException().Message ))"
                }

                Write-Verbose -Message 'Validating user roles'

                if ($Controller.Credentials.UserName -ne 'fsxadmin') {
                    if ((-not ($adminVserverName = (Get-NcVserver -Query @{ VserverType = 'admin' } -Controller $Controller -ErrorAction Ignore).Vserver))) {
                        throw 'Unable to determine the admin SVM name.'
                    }

                    if ((-not ($users = Get-NcUser -Vserver $adminVserverName -Name $Controller.Credentials.UserName -AuthMethod 'password', 'domain' -Application 'console', 'ontapi', 'ssh' -Controller $Controller -ErrorAction Ignore))) {
                        Write-Warning -Message 'Unable to determine user account rights. Continuing, but errors may be encountered if the specified account does not have the necessary rights.'
                    }
                    else {
                        $roles = Get-NcRole -Role $users.RoleName -Vserver $adminVserverName -Controller $Controller -ErrorAction Ignore

                        if (-not (Test-OntapUserApplication -Users $users) -or -not ($roles) -or -not (Test-OntapUserRole -Users $users -Roles $roles)) {
                            throw 'User account does not have the minimum access level required. See the FAQ entry in the help files for more information.'
                        }
                    }
                }

                if (-not $DisableLifCheck.IsPresent) {
                    Write-Verbose -Message 'Validating connection LIF type'

                    if ((Get-NcNetInterface -Query @{ Address = $Controller.Address.IpAddressToString } -Controller $Controller -ErrorAction SilentlyContinue).Role -notmatch 'cluster_mgmt|node_mgmt') {
                        throw 'Only the cluster_mgmt or a node_mgmt interface may be used to gather the data from the cluster.'
                    }
                }
            }
            'Netapp.PowerShell.Credential.NeCredential' {
                try {
                    Write-Verbose -Message 'Validating successful connection'
                    [void](Get-NeAbout -Credential $Controller -Verbose:$false -ErrorAction Stop)
                }
                catch {
                    $Global:Error.RemoveAt(0)

                    throw "Connection failed ($( $_.Exception.GetBaseException().Message ))"
                }
            }
            'SolidFire.Core.Objects.SFConnection' {
                try {
                    Write-Verbose -Message 'Validating successful connection'
                    [void](Get-SFClusterVersionInfo -SFConnection $Controller -ErrorAction Stop)
                }
                catch {
                    $Global:Error.RemoveAt(0)

                    throw "Connection Failed ($( $_.Exception.GetBaseException().Message ))"
                }
            }
            'OCI.Server' {
                try {
                    Write-Verbose -Message 'Validating successful connection'
                    #Currently no way to test this connection
                }
                catch {
                    $Global:Error.RemoveAt(0)

                    throw "Connection Failed ($( $_.Exception.GetBaseException().Message ))"
                }
            }
            'VMware.VimAutomation.ViCore.Impl.V1.VIServerImpl' {
                try {
                    Write-Verbose -Message 'Validating successful connection'
                    #Currently no way to test this connection
                }
                catch {
                    $Global:Error.RemoveAt(0)

                    throw "Connection Failed ($( $_.Exception.GetBaseException().Message ))"
                }
            }
            'System.Management.Automation.PSCustomObject' {
                switch -Exact ($Controller.Type) {
                    'StorageGRID' {
                        $params = @{
                            UseBasicParsing = $true
                        }

                        if ($PSVersionTable.PSVersion.Major -gt 6) {
                            $params.Add('SkipCertificateCheck', $true)
                        }

                        try {
                            [void](Invoke-WebRequest -Uri "$( [string]::Format('{0}://{1}', $Controller.Uri.Scheme, $Controller.Uri.Authority) )/api/v4/grid/config" -WebSession $Controller.WebSession @params -Verbose:$false)
                        }
                        catch {
                            #$Global:Error.RemoveAt(0)

                            $msg = New-Object -TypeName System.Text.StringBuilder
                            [void]$msg.Append('Connection failed (Reason: ')

                            if (-not [string]::IsNullOrWhiteSpace($_.ErrorDetails.Message)) {
                                switch -Wildcard ($_.ErrorDetails.Message) {
                                    '*ErrorUnauthorized*' {
                                        [void]$msg.Append('Invalid credentials specified')
                                    }
                                    default {
                                        [void]$msg.Append('Unknown error')
                                    }
                                }
                            }
                            else {
                                switch -Wildcard ($_.Exception.Message) {
                                    '*401 (Unauthorized)*' {
                                        [void]$msg.Append('Invalid credentials specified')
                                    }
                                    default {
                                        [void]$msg.Append('Unknown error')
                                    }
                                }
                            }

                            [void]$msg.Append(').')

                            throw $msg
                        }
                    }
                }
            }
        }

        return $true
    }
    catch {
        $Global:Error.RemoveAt(0)

        $PSCmdlet.ThrowTerminatingError((New-Object -TypeName System.Management.Automation.ErrorRecord -ArgumentList ($_.Exception.Message, 'ConnectionFailed', [System.Management.Automation.ErrorCategory]::ConnectionError, $Controller)))
    }
}

function Test-7ModeUserCapability {
    param (
        [Parameter(Mandatory)]
        [NetApp.Ontapi.AbstractController]$AbstractController,

        [Parameter(Mandatory)]
        [string[]]$Capabilities
    )

    if ([version]::Parse($AbstractController.Ontapi) -le [version]::Parse('1.7')) {
        Write-Warning -Message 'Unable to determine user account rights due to missing API for this version of Data ONTAP. Continuing, but errors may be encountered if the specified account does not have the necessary rights.'

        return $true
    }

    $failedCapabilities = @( )
    $userName = $(
        if ($AbstractController.Credentials.Domain) {
            [string]::Format('{0}\{1}', $AbstractController.Credentials.Domain, $AbstractController.Credentials.UserName)
        }
        else {
            $AbstractController.Credentials.UserName
        }
    )

    foreach ($capability in $Capabilities) {
        try {
            if ((Invoke-NaSystemApi -Request "<useradmin-user-has-capability><user-name>$userName</user-name><capability>$capability</capability></useradmin-user-has-capability>" -Controller $AbstractController -ErrorAction Stop).results.'is-user-capable' -ne $true) {
                $failedCapabilities += $capability
            }
        }
        catch {
            $Global:Error.RemoveAt(0)

            return $false
        }
    }

    if ($failedCapabilities.Count -gt 0) {
        Write-Warning -Message "User account does not have these required capabilities: $( $failedCapabilities -join ', ' )"

        return $false
    }

    return $true
}

function Test-OntapUserApplication {
    param (
        [Parameter(Mandatory)]
        [AllowEmptyCollection()]
        [pscustomobject[]]$Users
    )

    if (
        ($Users | Where-Object { $_.Application -eq 'ontapi' } | Measure-Object).Count -eq 0 -or
        ($Users | Where-Object { $_.Application -in @( 'console', 'ssh' ) } | Measure-Object).Count -eq 0
    ) {
        Write-Warning -Message "User account does not have the required applications created. Application(s) required: 'ontapi' and 'console' or 'ssh'"

        return $false
    }

    return $true
}

function Test-OntapUserRole {
    param (
        [Parameter(Mandatory)]
        [AllowEmptyCollection()]
        [pscustomobject[]]$Users,

        [Parameter(Mandatory)]
        [pscustomobject[]]$Roles
    )

    foreach ($user in ($Users | Where-Object { $_.RoleName -ne 'admin' })) {
        $role = $Roles | Where-Object { $_.RoleName -eq ($user | Where-Object { $_.Application -eq $user.Application }).RoleName }
        $roleName = ($user | Where-Object { $_.Application -eq $user.Application }).RoleName

        $warningMsg = New-Object -TypeName System.Text.StringBuilder

        switch -Exact (($role | Where-Object { $_.CommandDirectoryName -eq 'DEFAULT' }).AccessLevel) {
            'none' {
                [void]$warningMsg.AppendLine("DEFAULT command directory access level is set to 'none' for role: $( $roleName )")
            }
            'all' {
                if (($role | Where-Object { $_.CommandDirectoryname -eq 'set' }) -and ($role | Where-Object { $_.CommandDirectoryname -eq 'set' }).AccessLevel -match 'readonly|none') {
                    [void]$warningMsg.AppendLine("'set' command directory is missing or the access level is set to 'readonly' or 'none' for role: $( $roleName )")
                }

                if (($role | Where-Object { $_.CommandDirectoryname -eq 'system node run' }) -and ($role | Where-Object { $_.CommandDirectoryname -eq 'system node run' }).AccessLevel -match 'readonly|none') {
                    [void]$warningMsg.AppendLine("'system node run' command directory is missing or the access level is set to 'readonly' or 'none' for role: $( $roleName )")
                }
            }
            'readonly' {
                if (-not ($role | Where-Object { $_.CommandDirectoryname -eq 'set' }) -or ($role | Where-Object { $_.CommandDirectoryname -eq 'set' }).AccessLevel -ne 'all') {
                    [void]$warningMsg.AppendLine("'set' command directory is missing or the access level is not set to 'all' for role: $( $roleName )")
                }

                if (-not ($role | Where-Object { $_.CommandDirectoryname -eq 'system node run' }) -or ($role | Where-Object { $_.CommandDirectoryname -eq 'system node run' }).AccessLevel -ne 'all') {
                    [void]$warningMsg.AppendLine("'system node run' command directory is missing or the access level is not set to 'all' for role: $( $roleName )")
                }
            }
        }

        if ($warningMsg.Length -gt 0) {
            Write-Warning -Message $warningMsg.ToString()

            return $false
        }
    }

    return $true
}

# SIG # Begin signature block
# MIIy0QYJKoZIhvcNAQcCoIIywjCCMr4CAQExDzANBglghkgBZQMEAgEFADB5Bgor
# BgEEAYI3AgEEoGswaTA0BgorBgEEAYI3AgEeMCYCAwEAAAQQH8w7YFlLCE63JNLG
# KX7zUQIBAAIBAAIBAAIBAAIBADAxMA0GCWCGSAFlAwQCAQUABCDYqJR5mWScapUI
# H2d1yXJUGGC83eg4Un2D2rnbKCbvtaCCFxcwggQ+MIIDJqADAgECAgRKU4woMA0G
# CSqGSIb3DQEBCwUAMIG+MQswCQYDVQQGEwJVUzEWMBQGA1UEChMNRW50cnVzdCwg
# SW5jLjEoMCYGA1UECxMfU2VlIHd3dy5lbnRydXN0Lm5ldC9sZWdhbC10ZXJtczE5
# MDcGA1UECxMwKGMpIDIwMDkgRW50cnVzdCwgSW5jLiAtIGZvciBhdXRob3JpemVk
# IHVzZSBvbmx5MTIwMAYDVQQDEylFbnRydXN0IFJvb3QgQ2VydGlmaWNhdGlvbiBB
# dXRob3JpdHkgLSBHMjAeFw0wOTA3MDcxNzI1NTRaFw0zMDEyMDcxNzU1NTRaMIG+
# MQswCQYDVQQGEwJVUzEWMBQGA1UEChMNRW50cnVzdCwgSW5jLjEoMCYGA1UECxMf
# U2VlIHd3dy5lbnRydXN0Lm5ldC9sZWdhbC10ZXJtczE5MDcGA1UECxMwKGMpIDIw
# MDkgRW50cnVzdCwgSW5jLiAtIGZvciBhdXRob3JpemVkIHVzZSBvbmx5MTIwMAYD
# VQQDEylFbnRydXN0IFJvb3QgQ2VydGlmaWNhdGlvbiBBdXRob3JpdHkgLSBHMjCC
# ASIwDQYJKoZIhvcNAQEBBQADggEPADCCAQoCggEBALqEtnLbngxr4pnpMAGnduoy
# uJVBGsnaYU5Ycs/+9oJ5v3NhBgqlJ9izX9NFThxy1k4y8nKKD/eDGdBqgIAARR6w
# x+eavxJXJxyjaC8Kh71qaw5eZfMcd9XUhY1wIbSzMueLotWGOQKxuNJHzuTJScQ7
# p977VH1XvvDobsJ5sjoLVeJQmBYyE1wveFbBwpSz8lrkJ5qfJNfG7NCbJYLjzMLE
# RcWMl3oGayoRn6kKbkg7b9vUERlC948Hv/VTX5w+9Bcs5mmsTjJMYnfqt+jluzS8
# GYuunFHnt361U7EzIuVtz3A8Gvrim2e2g/SNpa9iTE3gWKxkNBID+LaNlGMkpHEC
# AwEAAaNCMEAwDgYDVR0PAQH/BAQDAgEGMA8GA1UdEwEB/wQFMAMBAf8wHQYDVR0O
# BBYEFGpyJnrQHu995ztpUdRsjZ+QEmarMA0GCSqGSIb3DQEBCwUAA4IBAQB5nx2W
# xrZ5PyKNh9OHAwRgamuaLlmJcxGsQ9H1E/+NOSvA8r1PcIypL+oXxAtUntQblpgz
# PKitYqIAdqtZaW4GHX7EuUSNmK8S1GHbChlGR/Pr92PBQAVApdK39LWaNr+piHaI
# BFUEK5yHfxo3PH4tpRrY1Ileyr2sPWzYba/V83YPzTuIOCKdbJOaxD2/ghtlP6YP
# Xar85bIVyrWtxrw90ITo6gZysE05Mni/PhGcC6SdmiHz8JsLMHjbwdyHQ/68Y5rK
# xcIcyceN/zsSWAjmtj3seixO+4OWzgw8aYdUc6RzwpP/URCsFVQB2PwFsYmhf3SD
# mknX3E57ikhvi0X2MIIF3zCCBMegAwIBAgIQTkDkN1Tt5owAAAAAUdOUfzANBgkq
# hkiG9w0BAQsFADCBvjELMAkGA1UEBhMCVVMxFjAUBgNVBAoTDUVudHJ1c3QsIElu
# Yy4xKDAmBgNVBAsTH1NlZSB3d3cuZW50cnVzdC5uZXQvbGVnYWwtdGVybXMxOTA3
# BgNVBAsTMChjKSAyMDA5IEVudHJ1c3QsIEluYy4gLSBmb3IgYXV0aG9yaXplZCB1
# c2Ugb25seTEyMDAGA1UEAxMpRW50cnVzdCBSb290IENlcnRpZmljYXRpb24gQXV0
# aG9yaXR5IC0gRzIwHhcNMjEwNTA3MTU0MzQ1WhcNMzAxMTA3MTYxMzQ1WjBpMQsw
# CQYDVQQGEwJVUzEWMBQGA1UECgwNRW50cnVzdCwgSW5jLjFCMEAGA1UEAww5RW50
# cnVzdCBDb2RlIFNpZ25pbmcgUm9vdCBDZXJ0aWZpY2F0aW9uIEF1dGhvcml0eSAt
# IENTQlIxMIICIjANBgkqhkiG9w0BAQEFAAOCAg8AMIICCgKCAgEAp4GP9xRFtmJD
# 8tiu0yVeSE9Rv8V9n1AcNdHWfmEqlBltJ0akphpd91RRaoAixqhmdU1Ug8leaBur
# 9ltksK2tIL1U70ZrbQLnRa519o6KuTIui7h3HFJNeYhfpToYyVAslyctv9oAfWN/
# 7zLsRodj25qfw1ohNnv5m9XKoG5yLPzh8Z5wTQhWFW+Qq/tIurnXwYJ4hWUuf7XJ
# wOIUtzcRQQbiiuCo9uV+tngFAcNg7U8HQS4KE0njkJt/3b36rL9kUdFcm7T1XOdc
# /zubpaAa130JssK3/24cvMh95ukr/HKzFOlKVRKEnEQldR32KvBPpSA9aCXrYZd8
# D+W2PfOuw8ERvBuOzOBHMF5CAIZx41isBsplH3uUpktXZwx+Xq14Z1tV417rx9js
# TG6Gy/Pc+J+HqnJYEg99pvj4Qjk7PCzkMk1JjODhAMI4oJz6hD5B3G5WrsYaW/Rn
# aAUBzRu/roe8nVP2Lui2a+SZ3sVPh1io0mUeyB/Vcm7uWRxXOwlyndfKt5DGzXtF
# kpFCA0x9P8ryqrjCDobzEJ9GLqRmhmhaaBhwKTgRgGBrikOjc2zjs2s3/+adZwGS
# ht8vSNH7UGDVXP4h0wFCY/7vcLQXwI+o7tPBS18S6v39Lg6HRGDjqfTCGKPj/c4M
# hCIN86d42pPz2zjPuS8zxv8HPF6+RdMCAwEAAaOCASswggEnMA4GA1UdDwEB/wQE
# AwIBhjASBgNVHRMBAf8ECDAGAQH/AgEBMB0GA1UdJQQWMBQGCCsGAQUFBwMDBggr
# BgEFBQcDCDA7BgNVHSAENDAyMDAGBFUdIAAwKDAmBggrBgEFBQcCARYaaHR0cDov
# L3d3dy5lbnRydXN0Lm5ldC9ycGEwMwYIKwYBBQUHAQEEJzAlMCMGCCsGAQUFBzAB
# hhdodHRwOi8vb2NzcC5lbnRydXN0Lm5ldDAwBgNVHR8EKTAnMCWgI6Ahhh9odHRw
# Oi8vY3JsLmVudHJ1c3QubmV0L2cyY2EuY3JsMB0GA1UdDgQWBBSCutY9l86fz3Ho
# kjev/bO1aTVXzzAfBgNVHSMEGDAWgBRqciZ60B7vfec7aVHUbI2fkBJmqzANBgkq
# hkiG9w0BAQsFAAOCAQEAH15BBLaDcCRTLFVzHWU6wOy0ewSYXlk4EwmkWZRCXlC/
# T2xuJSCQk1hADfUZtGLuJF7CAVgVAh0QCW+o1PuSfjc4Pi8UfY8dQzZks2YTXxTM
# pXH3WyFLxpe+3JX8cH0RHNMh3dAkOSnF/goapc97ee46b97cv+kR3RaDCNMsjX9N
# qBR5LwVhUjjrYPMUaH3LsoqtwJRc5CYOLIrdRsPO5FZRxVbjhbhNm0VyiwfxivtJ
# uF/R8paBXWlSJPEII9LWIw/ri9d+i8GTa/rxYntY6VCbl24XiA3hxkOY14FhtoWd
# R+yxnq4/IDtDndiiHODUfAjCr3YG+GJmerb3+sivNTCCBmcwggRPoAMCAQICEEbM
# 41QNEgkq80rzt39+/oswDQYJKoZIhvcNAQELBQAwYzELMAkGA1UEBhMCVVMxFjAU
# BgNVBAoTDUVudHJ1c3QsIEluYy4xPDA6BgNVBAMTM0VudHJ1c3QgRXh0ZW5kZWQg
# VmFsaWRhdGlvbiBDb2RlIFNpZ25pbmcgQ0EgLSBFVkNTMjAeFw0yNDEwMjMxNTIy
# MTRaFw0yNTExMDkxNTIyMTNaMIHCMQswCQYDVQQGEwJVUzETMBEGA1UECBMKQ2Fs
# aWZvcm5pYTERMA8GA1UEBxMIU2FuIEpvc2UxEzARBgsrBgEEAYI3PAIBAxMCVVMx
# GTAXBgsrBgEEAYI3PAIBAhMIRGVsYXdhcmUxFDASBgNVBAoTC05ldEFwcCwgSW5j
# MR0wGwYDVQQPExRQcml2YXRlIE9yZ2FuaXphdGlvbjEQMA4GA1UEBRMHMzQ1Mjc0
# MzEUMBIGA1UEAxMLTmV0QXBwLCBJbmMwggGiMA0GCSqGSIb3DQEBAQUAA4IBjwAw
# ggGKAoIBgQCIHyusYBqN6d8MjDVGb9yKy/jxOJi5Ry9vtMsEpSe0eX1NwlzjdPpo
# R+kSTTj0HXwjn48v4UtO3lOWVVdqDuAuECTvG+n31c1RmaB4qS0qg9VEDsUP0/gr
# R4GlmEnGRIeDNWnRIV4oHGTqA1NjNU3esZYLQSV9j5DSzh0W+vyq10stDZiLBNai
# KoBzi5fs+C7QWW+xd+fSFV3Z+zIfE/jHFa4ZKP0n8Bva/5XKZSg6XVBN09oY/EF2
# 67wlz2018jiSwKZi++kCqTIpp9j7dKP5x+0/bK4GlGeivT/NObhjDN8e0Iu5rC2L
# tCalpr5CBMzjKmwfV8mVd9L/qPKk8QZxBdBrUpA1qHDCHbd9BRcd2zT8doIbM0vT
# dHnT9pRPZYws7BfT33KMoV7nShbrNViWhkP3i3UkQv7W9RptiZwfEDpt4Lxn0Uw8
# Zf94fbwMQaV3A28ItUbueLN23+k41N0jqdFTbpCaqQpUT/NxlRHECtn0Lp6kjZTy
# wKF7FqH+jpcCAwEAAaOCATUwggExMAwGA1UdEwEB/wQCMAAwHQYDVR0OBBYEFGzw
# 5EcKP/wp/cuZ0BIs8j6slZQcMB8GA1UdIwQYMBaAFM6JT4JRqhWihGLKMSNh0mH7
# +P54MA4GA1UdDwEB/wQEAwIHgDATBgNVHSUEDDAKBggrBgEFBQcDAzBnBggrBgEF
# BQcBAQRbMFkwIwYIKwYBBQUHMAGGF2h0dHA6Ly9vY3NwLmVudHJ1c3QubmV0MDIG
# CCsGAQUFBzAChiZodHRwOi8vYWlhLmVudHJ1c3QubmV0L2V2Y3MyLWNoYWluLnA3
# YzAxBgNVHR8EKjAoMCagJKAihiBodHRwOi8vY3JsLmVudHJ1c3QubmV0L2V2Y3My
# LmNybDAgBgNVHSAEGTAXMAcGBWeBDAEDMAwGCmCGSAGG+mwKAQIwDQYJKoZIhvcN
# AQELBQADggIBADAwfASY7H1rvIVgDB4LCw7V6xYShLM1XBV0YV8mgn50EVUgkXRr
# qwdn9JgVJvPU6jQ30ab9bb2505N/e1hWQN+Skw21Kbixw3mJ2lZNjY9CvwPUAc4B
# 9xXFEl/w7w1GuATgHO77YCpe0QTbYpbuSTXA0YBs1LlzmuoMxCD+E82srnED8Crg
# m4IHe71kpqqbsIYDnxujU+w/+/CAijPCuxXD2Ff1D7Ms97n9OFvLZHU2e+7xTP66
# RFXF81uCNcLYZSAMiXpjFCiXz74yNHtqZYecnNKvA3hUnvE0BH2IbJFpFiwQO9H2
# rSxdf6qTJANWvCTm7AkDMPhCneufSIFeGHyINH5sKNc+MgsNawZwy2Bfld2YE+ii
# +RyoUgQvl+OZu58dcPYvbgSkc+nE7Xlq/nzE4qNB83ZUuo4MQuGgZ1ZdROSgO6BD
# lCDa1Mr+viR84DtT6dwMYDf0PYWe0LoDYBVLRknZHpSNCMULURCTENBJdJo1mrOn
# hmmm5ZJ3Hg/CmyrujK67YvNFUsQnpLKYEut7ntmJM5ptEPm7Ql5zzaOp3o68a/Q6
# 0+s6p4NwZQYo4ekHk+DM4oaPiv8K+dEPxvppQCSoSPG0S59A8oFBXzoaidOTC06j
# xfGFkPFtOEIO269lK0NCIgr2lJvlUdlUPSnM2eWkEkslz42ShJgON0SaMIIGgzCC
# BGugAwIBAgIQNa+3e500H2r8j4RGqzE1KzANBgkqhkiG9w0BAQ0FADBpMQswCQYD
# VQQGEwJVUzEWMBQGA1UECgwNRW50cnVzdCwgSW5jLjFCMEAGA1UEAww5RW50cnVz
# dCBDb2RlIFNpZ25pbmcgUm9vdCBDZXJ0aWZpY2F0aW9uIEF1dGhvcml0eSAtIENT
# QlIxMB4XDTIxMDUwNzE5MTk1MloXDTQwMTIyOTIzNTkwMFowYzELMAkGA1UEBhMC
# VVMxFjAUBgNVBAoTDUVudHJ1c3QsIEluYy4xPDA6BgNVBAMTM0VudHJ1c3QgRXh0
# ZW5kZWQgVmFsaWRhdGlvbiBDb2RlIFNpZ25pbmcgQ0EgLSBFVkNTMjCCAiIwDQYJ
# KoZIhvcNAQEBBQADggIPADCCAgoCggIBAL69pznJpX3sXWXx9Cuph9DnrRrFGjsY
# zuGhUY1y+s5YH1y4JEIPRtUxl9BKTeObMMm6l6ic/kU2zyeA53u4bsEkt9+ndNyF
# 8qMkWEXMlJQ7AuvEjXxG9VxmguOkwdMfrG4MUyMO1Dr62kLxg1RfNTJW8rV4m1cA
# SB6pYWEnDnMDQ7bWcJL71IWaMMaz5ppeS+8dKthmqxZG/wvYD6aJSgJRV0E8QThO
# l8dRMm1njmahXk2fNSKv1Wq3f0BfaDXMafrxBfDqhabqMoXLwcHKg2lFSQbcCWy6
# SWUZjPm3NyeMZJ414+Xs5wegnahyvG+FOiymFk49nM8I5oL1RH0owL2JrWwv3C94
# eRHXHHBL3Z0ITF4u+o29p91j9n/wUjGEbjrY2VyFRJ5jBmnQhlh4iZuHu1gcpChs
# xv5pCpwerBFgal7JaWUu7UMtafF4tzstNfKqT+If4wFvkEaq1agNBFegtKzjbb2d
# GyiAJ0bH2qpnlfHRh3vHyCXphAyPiTbSvjPhhcAz1aA8GYuvOPLlk4C/xsOre5PE
# PZ257kV2wNRobzBePLQ2+ddFQuASBoDbpSH85wV6KI20jmB798i1SkesFGaXoFpp
# cjFXa1OEzWG6cwcVcDt7AfynP4wtPYeM+wjX5S8Xg36Cq08J8inhflV3ZZQFHVnU
# Ct2TfuMUXeK7AgMBAAGjggErMIIBJzASBgNVHRMBAf8ECDAGAQH/AgEAMB0GA1Ud
# DgQWBBTOiU+CUaoVooRiyjEjYdJh+/j+eDAfBgNVHSMEGDAWgBSCutY9l86fz3Ho
# kjev/bO1aTVXzzAzBggrBgEFBQcBAQQnMCUwIwYIKwYBBQUHMAGGF2h0dHA6Ly9v
# Y3NwLmVudHJ1c3QubmV0MDEGA1UdHwQqMCgwJqAkoCKGIGh0dHA6Ly9jcmwuZW50
# cnVzdC5uZXQvY3NicjEuY3JsMA4GA1UdDwEB/wQEAwIBhjATBgNVHSUEDDAKBggr
# BgEFBQcDAzBEBgNVHSAEPTA7MDAGBFUdIAAwKDAmBggrBgEFBQcCARYaaHR0cDov
# L3d3dy5lbnRydXN0Lm5ldC9ycGEwBwYFZ4EMAQMwDQYJKoZIhvcNAQENBQADggIB
# AD4AVLgq849mr2EWxFiTZPRBi2RVjRs1M6GbkdirRsqrX7y+fnDk0tcHqJYH14bR
# VwoI0NB4Tfgq37IE85rh13zwwQB6wUCh34qMt8u0HQFh8piapt24gwXKqSwW3Jwt
# Dv6nl+RQqZeVwUsqjFHjxALga3w1TVO8S5QTi1MYFl6mCqe4NMFssess5DF9DCzG
# fOGkVugtdtWyE3XqgwCuAHfGb6k97mMUgVAW/FtPEhkOWw+N6kvOBkyJS64gzI5H
# pnXWZe4vMOhdNI8fgk1cQqbyFExQIJwJonQkXDnYiTKFPK+M5Wqe5gQ6pRP/qh3N
# R0suAgW0ao/rhU+B7wrbfZ8pj6XCP1I4UkGVO7w+W1QwQiMJY95QjYk1RfqruA+P
# oq17ehGT8Y8ohHtoeUdq6GQpTR/0HS9tHsiUhjzTWpl6a3yrNfcrOUtPuT8Wku8p
# jI2rrAEazHFEOctAPiASzghw40f+3IDXCADRC2rqIbV5ZhfpaqpW3c0VeLEDwBSt
# PkcYde0KU0syk83/gLGQ1hPl5EF4Iu1BguUO37DOlSFF5osB0xn39CtVrNlWc2MQ
# 4LigbctUlpigmSFRBqqmDDorY8t52kO50hLM3o9VeukJ8+Ka0yXBezaS2uDlUmfN
# 4+ZUCqWd1HOj0y9dBmSFA3d/YNjCvHTJlZFot7d+YRl1MYIbEDCCGwwCAQEwdzBj
# MQswCQYDVQQGEwJVUzEWMBQGA1UEChMNRW50cnVzdCwgSW5jLjE8MDoGA1UEAxMz
# RW50cnVzdCBFeHRlbmRlZCBWYWxpZGF0aW9uIENvZGUgU2lnbmluZyBDQSAtIEVW
# Q1MyAhBGzONUDRIJKvNK87d/fv6LMA0GCWCGSAFlAwQCAQUAoIGEMBgGCisGAQQB
# gjcCAQwxCjAIoAKAAKECgAAwGQYJKoZIhvcNAQkDMQwGCisGAQQBgjcCAQQwHAYK
# KwYBBAGCNwIBCzEOMAwGCisGAQQBgjcCARUwLwYJKoZIhvcNAQkEMSIEIFe57qDN
# l6JDnnCaRIflmW0ddf624DXTvkVogN718YwyMA0GCSqGSIb3DQEBAQUABIIBgCpE
# Ebru5CnprgU3z2fqrCdESqTS9SP6Kem5UjALyX+Yx62CLcgA17VaHnTjfBt55i/m
# oeOj2G7VhZSrFVl9oYm9bcpSA8LjRzIXALYwgo8Rzug1P6lu21WRybqNl+C/5pN8
# 6t3srqGDx8jiOZoTHhNHcbtSquGJlFS6tFJJ0tdFdUvNIjnOnTaT7fe9qadftrnt
# cWXGmT993KhYWpThrqS2H0rrkDuvElRnS8Ntge4Y+y1q4bUaG1Z4iIBK+h8InJo0
# zCUoRv2H5acsvJDRHbb/Rtmq6fJDU85QU/EJm78mee5Q1WR/GO1UskgL5rkrqx55
# JuD+aLvXvbnRHtLEGgUHDaNptsrjtmeE4X/9izvlqWGAd8TKH9KaMLu6034JRD0U
# PvVsMPCPz1ntq3IFevgCTdrdo8C8VLYq84LnGRGocX28GvGjtUgtahRDXj1E+3j5
# OE8+h26xzWZhsmYSEHvbc+PAmTVWxCsGZMHTousOtGoryE7XhiENcfumfAZK6KGC
# GGMwghhfBgorBgEEAYI3AwMBMYIYTzCCGEsGCSqGSIb3DQEHAqCCGDwwghg4AgED
# MQ0wCwYJYIZIAWUDBAIDMIHzBgsqhkiG9w0BCRABBKCB4wSB4DCB3QIBAQYKYIZI
# AYb6bAoDBTAxMA0GCWCGSAFlAwQCAQUABCARgHQqh+6b07VOr2KdHChvf0XWLPh4
# vQ/fRnqEHVqLxwIIeD7hvcfJhyMYDzIwMjUwMTA5MDMwNjI5WjADAgEBoHmkdzB1
# MQswCQYDVQQGEwJDQTEQMA4GA1UECBMHT250YXJpbzEPMA0GA1UEBxMGT3R0YXdh
# MRYwFAYDVQQKEw1FbnRydXN0LCBJbmMuMSswKQYDVQQDEyJFbnRydXN0IFRpbWVz
# dGFtcCBBdXRob3JpdHkgLSBUU0EyoIITDjCCBd8wggTHoAMCAQICEE5A5DdU7eaM
# AAAAAFHTlH8wDQYJKoZIhvcNAQELBQAwgb4xCzAJBgNVBAYTAlVTMRYwFAYDVQQK
# Ew1FbnRydXN0LCBJbmMuMSgwJgYDVQQLEx9TZWUgd3d3LmVudHJ1c3QubmV0L2xl
# Z2FsLXRlcm1zMTkwNwYDVQQLEzAoYykgMjAwOSBFbnRydXN0LCBJbmMuIC0gZm9y
# IGF1dGhvcml6ZWQgdXNlIG9ubHkxMjAwBgNVBAMTKUVudHJ1c3QgUm9vdCBDZXJ0
# aWZpY2F0aW9uIEF1dGhvcml0eSAtIEcyMB4XDTIxMDUwNzE1NDM0NVoXDTMwMTEw
# NzE2MTM0NVowaTELMAkGA1UEBhMCVVMxFjAUBgNVBAoMDUVudHJ1c3QsIEluYy4x
# QjBABgNVBAMMOUVudHJ1c3QgQ29kZSBTaWduaW5nIFJvb3QgQ2VydGlmaWNhdGlv
# biBBdXRob3JpdHkgLSBDU0JSMTCCAiIwDQYJKoZIhvcNAQEBBQADggIPADCCAgoC
# ggIBAKeBj/cURbZiQ/LYrtMlXkhPUb/FfZ9QHDXR1n5hKpQZbSdGpKYaXfdUUWqA
# IsaoZnVNVIPJXmgbq/ZbZLCtrSC9VO9Ga20C50WudfaOirkyLou4dxxSTXmIX6U6
# GMlQLJcnLb/aAH1jf+8y7EaHY9uan8NaITZ7+ZvVyqBuciz84fGecE0IVhVvkKv7
# SLq518GCeIVlLn+1ycDiFLc3EUEG4orgqPblfrZ4BQHDYO1PB0EuChNJ45Cbf929
# +qy/ZFHRXJu09VznXP87m6WgGtd9CbLCt/9uHLzIfebpK/xysxTpSlUShJxEJXUd
# 9irwT6UgPWgl62GXfA/ltj3zrsPBEbwbjszgRzBeQgCGceNYrAbKZR97lKZLV2cM
# fl6teGdbVeNe68fY7Exuhsvz3Pifh6pyWBIPfab4+EI5Ozws5DJNSYzg4QDCOKCc
# +oQ+QdxuVq7GGlv0Z2gFAc0bv66HvJ1T9i7otmvkmd7FT4dYqNJlHsgf1XJu7lkc
# VzsJcp3XyreQxs17RZKRQgNMfT/K8qq4wg6G8xCfRi6kZoZoWmgYcCk4EYBga4pD
# o3Ns47NrN//mnWcBkobfL0jR+1Bg1Vz+IdMBQmP+73C0F8CPqO7TwUtfEur9/S4O
# h0Rg46n0whij4/3ODIQiDfOneNqT89s4z7kvM8b/BzxevkXTAgMBAAGjggErMIIB
# JzAOBgNVHQ8BAf8EBAMCAYYwEgYDVR0TAQH/BAgwBgEB/wIBATAdBgNVHSUEFjAU
# BggrBgEFBQcDAwYIKwYBBQUHAwgwOwYDVR0gBDQwMjAwBgRVHSAAMCgwJgYIKwYB
# BQUHAgEWGmh0dHA6Ly93d3cuZW50cnVzdC5uZXQvcnBhMDMGCCsGAQUFBwEBBCcw
# JTAjBggrBgEFBQcwAYYXaHR0cDovL29jc3AuZW50cnVzdC5uZXQwMAYDVR0fBCkw
# JzAloCOgIYYfaHR0cDovL2NybC5lbnRydXN0Lm5ldC9nMmNhLmNybDAdBgNVHQ4E
# FgQUgrrWPZfOn89x6JI3r/2ztWk1V88wHwYDVR0jBBgwFoAUanImetAe733nO2lR
# 1GyNn5ASZqswDQYJKoZIhvcNAQELBQADggEBAB9eQQS2g3AkUyxVcx1lOsDstHsE
# mF5ZOBMJpFmUQl5Qv09sbiUgkJNYQA31GbRi7iRewgFYFQIdEAlvqNT7kn43OD4v
# FH2PHUM2ZLNmE18UzKVx91shS8aXvtyV/HB9ERzTId3QJDkpxf4KGqXPe3nuOm/e
# 3L/pEd0WgwjTLI1/TagUeS8FYVI462DzFGh9y7KKrcCUXOQmDiyK3UbDzuRWUcVW
# 44W4TZtFcosH8Yr7Sbhf0fKWgV1pUiTxCCPS1iMP64vXfovBk2v68WJ7WOlQm5du
# F4gN4cZDmNeBYbaFnUfssZ6uPyA7Q53Yohzg1HwIwq92BvhiZnq29/rIrzUwggZv
# MIIEV6ADAgECAhAlvCvzKcoQfx6puoiF1J07MA0GCSqGSIb3DQEBDQUAMGkxCzAJ
# BgNVBAYTAlVTMRYwFAYDVQQKDA1FbnRydXN0LCBJbmMuMUIwQAYDVQQDDDlFbnRy
# dXN0IENvZGUgU2lnbmluZyBSb290IENlcnRpZmljYXRpb24gQXV0aG9yaXR5IC0g
# Q1NCUjEwHhcNMjEwNTA3MTkyMjE0WhcNNDAxMjI5MjM1OTAwWjBOMQswCQYDVQQG
# EwJVUzEWMBQGA1UEChMNRW50cnVzdCwgSW5jLjEnMCUGA1UEAxMeRW50cnVzdCBU
# aW1lIFN0YW1waW5nIENBIC0gVFMyMIICIjANBgkqhkiG9w0BAQEFAAOCAg8AMIIC
# CgKCAgEAtQMqBraETzTMnjG+6o7Ge24gJA8D172BNjPmb9I+u8SeqqDMZl1CYDRR
# 4i1hg7o8nmi4S8NmjqXeK7qbLUGl3tDLVwUtiNnARohtPDUNGZCsuAwZ8EzF20Vb
# 3DvCvI1EAOA4//9dyM8Imj3Usut7lcPEpQDgYgFkWV+M43uLqflFt8awogo3Q7hG
# gauGfn8v3vBf680gzKUtMeOoLpmb4Hjd9NtdgmN9MDmIfiQG5nxghHjIXCnllcv1
# e0/PWNyQz6eiexRUxOA3mDDBm7aunCn2ayVD5xJqPhmLvsDngddKMAh+fI3fPPNI
# kyHegYNqnQA+12C60bNiCJbptkkAOPRcdg3ad2o9oJXeHqHMWXpen8/pdl1A8rUV
# tWBoLPNaySOD2PUaa2kopLbUrmIG3iZFvd7pVneq3GSwQc3ME/FdcXSylAhLTzJu
# fnC4d+nVMjfCet5CEuhxHOLYYr+qa0GGBlvmab4mCfSXwv5M9IByhG7b3sUHqY4V
# om3hG+AJD/WgB6C7xHdzCTwHk40BtRJnLU0ZtbjFyRCrcM9C/6UnHdYS1uQYRGD7
# OV2Eu2a1UxKdAxSCZSm07Ely/bND5cMCKSS6H/i4vl2jty6p/M9i2fqXxBN441W5
# WnqE6/+7z2mIwLb5i2CFQRBQxwUQ0l52nT417PZPivkSez+vCzcCAwEAAaOCASww
# ggEoMBIGA1UdEwEB/wQIMAYBAf8CAQAwHQYDVR0OBBYEFCYP8MRICBvN3ZH1VFS2
# s7P8mfEIMB8GA1UdIwQYMBaAFIK61j2Xzp/PceiSN6/9s7VpNVfPMDMGCCsGAQUF
# BwEBBCcwJTAjBggrBgEFBQcwAYYXaHR0cDovL29jc3AuZW50cnVzdC5uZXQwMQYD
# VR0fBCowKDAmoCSgIoYgaHR0cDovL2NybC5lbnRydXN0Lm5ldC9jc2JyMS5jcmww
# DgYDVR0PAQH/BAQDAgGGMBMGA1UdJQQMMAoGCCsGAQUFBwMIMEUGA1UdIAQ+MDww
# MAYEVR0gADAoMCYGCCsGAQUFBwIBFhpodHRwOi8vd3d3LmVudHJ1c3QubmV0L3Jw
# YTAIBgZngQwBBAIwDQYJKoZIhvcNAQENBQADggIBAHY9RmiFXwnAzsn4x7jXfqdc
# OwtGc84iU4s49hG6d8TChyGPJ9kMlevoS+Vc15AVb6gqLIpq4CIRNHn6vZZ5yHYW
# zbFE6nFRD5tVyRh81YJ5nPOmt5OpBThyAgaXoaiyeX5eCOJubHmaqIeUzn+PVTZA
# D58/UdVmvnurhuZuh7pOY12x2I4fy2fDDXDaeuuTshSE0j59kz8WDDagXHkAnwtu
# 5YQq2IjBbQS3N/OSmnDgOwP9zbsngs4v/I3kssq++CWFxu10yD01XzgJPXlP6xvg
# QV6ldIE/URXA+EmIj7q7BdStRYIzecEljV9UPZHkuJ8XmABdrSjlT18b+/hUtaXE
# JPzwdjv4DV7MR7DKqrtuUd9TtUka8WCHt/n9aQZ1X3PWEZDygFiO5cMPA5iMBFOz
# KYffP/ew6UxXgSREeWj+GzYH1mU97G/WwrWqYXxmBuzS5WmA+HGSLlziEUYYRUvW
# +gVk77veBspDJb3fjckBdGGI2/DHpj3jDHG8f0vnyOBu4NqA+QCewGsl1aO6XCv+
# xGAgFEoHIzHlGsuRRLKae0PxW5i1IUu4MTsiVO55YfealBtmJyrG3+FlXe9xOeZQ
# Jf2HNLmuXXwnO+dlk/gohLFvhpMsk5O2cK+h7sikYamEO/sKt/i4bSbuR071XgX6
# nA8RfdfLka5DIPcVWWO8MIIGtDCCBJygAwIBAgIQW3AmzJb2eBq7hfYR9W27bjAN
# BgkqhkiG9w0BAQ0FADBOMQswCQYDVQQGEwJVUzEWMBQGA1UEChMNRW50cnVzdCwg
# SW5jLjEnMCUGA1UEAxMeRW50cnVzdCBUaW1lIFN0YW1waW5nIENBIC0gVFMyMB4X
# DTI0MDExOTE2NDc0N1oXDTM1MDQxODAwMDAwMFowdTELMAkGA1UEBhMCQ0ExEDAO
# BgNVBAgTB09udGFyaW8xDzANBgNVBAcTBk90dGF3YTEWMBQGA1UEChMNRW50cnVz
# dCwgSW5jLjErMCkGA1UEAxMiRW50cnVzdCBUaW1lc3RhbXAgQXV0aG9yaXR5IC0g
# VFNBMjCCAiIwDQYJKoZIhvcNAQEBBQADggIPADCCAgoCggIBAKqGBDhej2i/g/Va
# /VSbxX06dZ3Epy1PbenQBR9R4zO7L186sZZUiukIfDStiYNVpJ8CzCML2Bt6QKTM
# n0LqE398uOC3SgIF4oNJBG8Ghs1zQamD1Jhn+vRSGXrjSlkoh1YXXUjxhpELuCJk
# 20H5XVS1NaRuONwIDRpWGEDDfpg8QjOEDcWlYVHOIG1PAMtC8Kh1G9tG/incG57X
# +BgFUXYwm3S9lR8ZU2UVRyxWY2vpukisCji2wXL9IAqjdBprB3/R3awXzrUcx3qJ
# DOxtHDCNBfCvPZfMNbS1BQNmup52/DnayJrWEhLkQcKFIkDwy7qI6dKHwqvGNDvl
# qw0ABg4GtYt70Qsi2jyoyVZ78DUElupZRoyP1KC1qYOwzoT7S0TlQagW86/a0aAn
# cN8H7zfzrpHO1CF/hpjcXu6PwNpH22nKELmRlYAVY/ErGsKWoR/C9zzU9n0kByWo
# VJRvClXb2yRDbi47ontJu7XWGI/ZBLO+6se1ZJpLzlJhHisFHmM3Rp2xydVeh6Tq
# 1zSyIUIXuDooOl6l7T79Nxd/YyDye9ZfyvMUP+HO/4p3lGZticHg7H8EcDhkJ/Wn
# F31NygyBG8Xl8TbFeq59KaRHNTj6DfRTAykF/G+hi8nN8biIQNPS254lMa3X5lrY
# BLSEuwRGYWb/XTLe/NM+fE4Y737fAgMBAAGjggFlMIIBYTAMBgNVHRMBAf8EAjAA
# MB0GA1UdDgQWBBT1dhr4EJiSoRt3qQAs8ZViAgt5JzAfBgNVHSMEGDAWgBQmD/DE
# SAgbzd2R9VRUtrOz/JnxCDBoBggrBgEFBQcBAQRcMFowIwYIKwYBBQUHMAGGF2h0
# dHA6Ly9vY3NwLmVudHJ1c3QubmV0MDMGCCsGAQUFBzAChidodHRwOi8vYWlhLmVu
# dHJ1c3QubmV0L3RzMi1jaGFpbjI1Ni5wN2MwMQYDVR0fBCowKDAmoCSgIoYgaHR0
# cDovL2NybC5lbnRydXN0Lm5ldC90czJjYS5jcmwwDgYDVR0PAQH/BAQDAgeAMBYG
# A1UdJQEB/wQMMAoGCCsGAQUFBwMIMEwGA1UdIARFMEMwNwYKYIZIAYb6bAoBBzAp
# MCcGCCsGAQUFBwIBFhtodHRwczovL3d3dy5lbnRydXN0Lm5ldC9ycGEwCAYGZ4EM
# AQQCMA0GCSqGSIb3DQEBDQUAA4ICAQCpq32/GgCFW9ztcSZelIKvXIJL+hw5xVO1
# wwbdA2B++WGaQSDCrkEXLqVCCJEJXpcwmIFF1rK1hD2L6YcH2gE/4AhYZxEglB7A
# Fo5jd72bbAWNPbXLmjGpSlxiMqITzpCcDceQT2fD4Kh/h4RL1/LBIQ8Q5xe5+70J
# zL6KpgH+TYJPaSCadWvkDfdQJlP8M/f+G66nasCo6x6vItM9YBWfuuTVKaS1V5+x
# ZfaxTGHV0sjat6yqGB8gGIqm+PUH+XIr9mPYJZwW/vuU34Z0oiC08gjfFu85gHY6
# krHQloNq2nxTiVkbDVzxuynyIU9hLCSJ+HYJf9lokI5XDGTVdgBxWfPUhMg8oYh/
# qG/zvW5/sKiO5pNs6RTFdRwMK87yB/u1rtRBMb/BzGmYGK5bCI+rXGS5hGq9XDih
# 309Ct0tbUtPe9zvLn2/CWUIiTgsYtyLIL/xqCYXtUClResSUS7ZxZYpKhppLeEtW
# iqHeL4uTWzLMobnvqbYM2AYLFzC/DW+Pk6zLEfXxxwCe0BnXnF+OVHxrqUM7Y9zB
# RKu9jDpD8HI/JJXNhJFdIQANeGsNXCMkn0yvNMsUWt6x5EVQ/a9vpwWCp3PPz+mS
# 6nMqseHdogPmILwdZxpWwi72rF/KToGw5lodc8/SFPNQeCgFThVuhST4M6iIr3Rt
# 9FLOjKiEnjGCBBowggQWAgEBMGIwTjELMAkGA1UEBhMCVVMxFjAUBgNVBAoTDUVu
# dHJ1c3QsIEluYy4xJzAlBgNVBAMTHkVudHJ1c3QgVGltZSBTdGFtcGluZyBDQSAt
# IFRTMgIQW3AmzJb2eBq7hfYR9W27bjALBglghkgBZQMEAgOgggGLMBoGCSqGSIb3
# DQEJAzENBgsqhkiG9w0BCRABBDAcBgkqhkiG9w0BCQUxDxcNMjUwMTA5MDMwNjI5
# WjArBgkqhkiG9w0BCTQxHjAcMAsGCWCGSAFlAwQCA6ENBgkqhkiG9w0BAQ0FADBP
# BgkqhkiG9w0BCQQxQgRAbZOH5gjPmFYiGPBuVo8FCkaM71luH0Qa9G8ekGjXW69l
# NzdHQbEIQT2AqI5wKHb7KKgalsKUikXVzxQdiQfvvjCB0AYLKoZIhvcNAQkQAi8x
# gcAwgb0wgbowgbcwCwYJYIZIAWUDBAIDBEA5EUIuFwI+qpkkmXQODsjo0nLTVfxc
# 9mz5EVavl1U05ICv07x8TFtX79H/vNt1FGXg1AVahU6bETnZ9+xV1f4kMGYwUqRQ
# ME4xCzAJBgNVBAYTAlVTMRYwFAYDVQQKEw1FbnRydXN0LCBJbmMuMScwJQYDVQQD
# Ex5FbnRydXN0IFRpbWUgU3RhbXBpbmcgQ0EgLSBUUzICEFtwJsyW9ngau4X2EfVt
# u24wDQYJKoZIhvcNAQENBQAEggIAjHuk29A3c2dN2Ou2jIf+n7bv2R5vcPP1AhIv
# FqiSGbetTIFfpTR1oHXa86X84NUotVorlTfA4xAQCMiifVWEIsVoNiqdI7YF9Nqn
# lVoG1BLs2MG6uH6uq8bv2nq22UP7xlFtsH8gsMq65DmigQq6A00XzLzEjcZK6vYS
# 48VnXeZGTqvQyranxprWjxLABPZGdrR7OJv3ylXpy6Wge5r91PqdtUlyW3O26T4C
# sUfvIcMOJSzVkuEYdV0KEB1TihDbRJmLA1eQmjY1DZnx3+AqyV2P1iXhHCL2AUDU
# ii5dlMsjUyxFr3/VaL3QLgzIPHX7E7ByK4mRKPtU4zXaBNRVnXH874bAAzcUZ+vj
# X5n++UIBkcOE1dSnOYVH/uhZOvgZck91HROup8xkjKwS0R7IulRaYqZwhBsWctto
# /GJtdWnRa9Ylrv1JZktGMfXXSdRR8nj1yXDx/PK2Fg4Bh4jdqlHSjp3Q+StdjD9K
# urMZiZDv5IxdtBAd2p9aMmIF7HltXXwEUUlgmr/BKXAJl0hRvUzBz//oYLUnOYZJ
# a1H8KDsg/Enb1WPas+XXy+K0W39sv7G2jzLQcBqMBwgY3odq6nrlkylyN2L2Onyv
# Nmu7adhkkjsep4xzewGFeQSQeCiPTEptlXuREFSGl4cEhoLVAog5zagq5O6C4Kfp
# 975OTik=
# SIG # End signature block
