function Get-Match {
    [CmdletBinding()]
    param (
        [Parameter(ValueFromPipeline)]
        [System.Object]$InputObject,

        [Parameter(Mandatory)]
        [string]$Pattern
    )

    begin {
        try {
            $regex = New-Object -TypeName System.Text.RegularExpressions.Regex -ArgumentList $Pattern
        }
        catch {
            $Global:Error.RemoveAt(0)

            $PSCmdlet.ThrowTerminatingError((New-Object -TypeName System.Management.Automation.ErrorRecord -ArgumentList ("Get-Match: Pattern not correct. '$Pattern' is not a valid regular expression.", 'InvalidRegEx', [System.Management.Automation.ErrorCategory]::InvalidArgument, $InputObject)))
        }

        $groups = @(
            $regex.GetGroupNames() | Where-Object { -not [int]::TryParse($_, [ref]$null) } | ForEach-Object {
                $_.ToString()
            }
        )
    }

    process {
        foreach ($line in $InputObject) {
            foreach ($match in $regex.Matches($line)) {
                if ($groups.Count -eq 0) {
                    ([System.Object[]]$match.Groups)[-1].Value
                }
                else {
                    $rv = 1 | Select-Object -Property $groups

                    $groups | ForEach-Object {
                        $rv.$_ = $match.Groups[$_].Value
                    }

                    $rv
                }
            }
        }
    }
}

function Convert-DateTimeString {
    param (
        [Parameter(Mandatory)]
        [AllowEmptyString()]
        [string]$String,

        [Parameter()]
        [string[]]$Format
    )

    if ([string]::IsNullOrWhiteSpace($String) -or $String -in '-', 'none', '0') {
        return [string]::Empty
    }

    $dateTime = [datetime]::Now

    if ([string]::IsNullOrWhiteSpace($Format)) {
        $convertible = [System.DateTimeOffset]::TryParse(
            $String,
            [cultureinfo]::InvariantCulture,
            [System.Globalization.DateTimeStyles]::None,
            [ref]$dateTime
        )
    }
    else {
        $convertible = [System.DateTimeOffset]::TryParseExact(
            $String,
            $Format,
            [cultureinfo]::InvariantCulture,
            [System.Globalization.DateTimeStyles]::AllowWhiteSpaces,
            [ref]$dateTime
        )
    }

    if (-not $convertible) {
        return [string]::Empty
    }

    $dateTime.DateTime
}

function Convert-TimeSpanString {
    param (
        [Parameter(Mandatory)]
        [string]$String,

        [Parameter()]
        [string]$Format
    )

    $timeSpan = [timespan]::Zero

    if (-not [string]::IsNullOrWhiteSpace($Format)) {
        $style = [System.Globalization.TimeSpanStyles]::None
        $timeSpanFormat = $Format

        if ($String.StartsWith('-')) {
            $timeSpanFormat = $Format.Insert(0, "'-'")
            $style = [System.Globalization.TimeSpanStyles]::AssumeNegative
        }

        $convertible = [timespan]::TryParseExact(
            $String,
            $timeSpanFormat,
            [cultureinfo]::InvariantCulture,
            $style,
            [ref]$timeSpan
        )
    }
    elseif (($convertible = $string -match '^(?<Negative>[-])?(?<Hours>\d+)[:](?<Minutes>\d+)[:](?<Seconds>\d+)$')) {
        if ($Matches.ContainsKey('Negative')) {
            $timeSpan = New-TimeSpan -Hours "$( $Matches.Negative )$( $Matches.Hours)" -Minutes "$( $Matches.Negative )$( $Matches.Minutes )" -Seconds "$( $Matches.Negative )$( $Matches.Seconds )"
        }
        else {
            $timeSpan = New-TimeSpan -Hours "$( $Matches.Hours)" -Minutes "$( $Matches.Minutes )" -Seconds "$( $Matches.Seconds )"
        }

        $Matches.Clear()
    }
    else {
        $convertible = [timespan]::TryParse(
            $String,
            [cultureinfo]::InvariantCulture,
            [ref]$timeSpan
        )
    }

    if ($convertible) {
        $timeSpan
    }
}

function ConvertFrom-UnixTimestamp {
    [CmdletBinding(DefaultParameterSetName = 'Seconds')]
    [OutputType([System.DateTime])]
    param (
        [Parameter(Mandatory, ParameterSetName = 'Seconds')]
        [AllowEmptyString()]
        [string]$Seconds,

        [Parameter(Mandatory, ParameterSetName = 'Milliseconds')]
        [AllowEmptyString()]
        [string]$Milliseconds
    )

    switch -Exact ($PSCmdlet.ParameterSetName) {
        'Milliseconds' {
            if (-not [long]::TryParse($Milliseconds, [ref]$null)) {
                return
            }

            [System.DateTimeOffset]::FromUnixTimeMilliseconds($Milliseconds).DateTime
        }
        'Seconds' {
            if (-not [long]::TryParse($Seconds, [ref]$null)) {
                return
            }

            [System.DateTimeOffset]::FromUnixTimeSeconds($Seconds).DateTime
        }
    }
}

function ConvertTo-UnixTimestamp {
    param (
        [Parameter(Mandatory)]
        [datetime]$DateTime,

        [Parameter()]
        [switch]$AsMilliseconds
    )

    if ($AsMilliseconds.IsPresent) {
        [System.DateTimeOffset]::Parse($DateTime, [cultureinfo]::InvariantCulture, [System.Globalization.DateTimeStyles]::AssumeUniversal).ToUnixTimeMilliseconds()
    }
    else {
        [System.DateTimeOffset]::Parse($DateTime, [cultureinfo]::InvariantCulture, [System.Globalization.DateTimeStyles]::AssumeUniversal).ToUnixTimeSeconds()
    }
}

function Import-WithRegex {
    [CmdletBinding()]
    [OutputType([System.Management.Automation.PSCustomObject])]
    param (
        [Parameter(Mandatory, ValueFromPipeline)]
        [string]$CliOutput,

        [Parameter(Mandatory)]
        [string]$RegexString,

        [int]$Skip = 0,

        [switch]$ByLine
    )

    process {
        if ($ByLine.IsPresent) {
            $CliOutput -split '\n' | Where-Object { -not [string]::IsNullOrWhiteSpace($_) } | Select-Object -Skip $Skip | ForEach-Object {
                if ($_ -match $RegexString) {
                    [pscustomobject](Get-RegexNamedGroup -Hashtable $Matches)
                }
            }
        }
        else {
            if ($CliOutput -match $RegexString) {
                [pscustomobject](Get-RegexNamedGroup -Hashtable $Matches)
            }
        }
    }
}

function Get-RegexNamedGroup {
    [Diagnostics.CodeAnalysis.SuppressMessageAttribute('PSUseLiteralInitializerForHashtable', '')]
    param (
        [Parameter(Mandatory)]
        [hashtable]$Hashtable
    )

    $newHashtable = New-Object -TypeName System.Collections.Hashtable

    $Hashtable.Keys | Where-Object { $_ -and $_ -notmatch '^\d+$' } | ForEach-Object {
        $newHashtable[$_] = $Hashtable[$_].Trim()
    }

    return $newHashtable
}

function Get-ObjectFromKeyValuePair {
    [Diagnostics.CodeAnalysis.SuppressMessageAttribute('PSUseLiteralInitializerForHashtable', '')]
    [CmdletBinding()]
    [OutputType([System.Management.Automation.PSCustomObject])]
    param (
        [Parameter(Mandatory, ValueFromPipeline)]
        [System.Object]$KeyValuePair
    )

    begin {
        $hashtable = New-Object -TypeName System.Collections.Hashtable
    }

    process {
        $KeyValuePair | ForEach-Object {
            $hashtable.Add($_.Name, $_.Value)
        }
    }

    end {
        [pscustomobject]$hashtable
    }
}

function Convert-ListToBuckets {
    [Diagnostics.CodeAnalysis.SuppressMessageAttribute('PSUseSingularNouns', '')]
    [CmdletBinding()]
    [OutputType([System.Object[]])]
    param (
        [Parameter(Mandatory)]
        [ValidateNotNullOrEmpty()]
        [object[]]$List,

        [Parameter(Mandatory, ParameterSetName = 'NumberOfBuckets')]
        [ValidateRange(0, [int]::MaxValue)]
        [int]$NumberOfBuckets,

        [Parameter(Mandatory, ParameterSetName = 'BucketSize')]
        [ValidateRange(0, [int]::MaxValue)]
        [int]$BucketSize
    )

    $aggregateList = @( )
    $start = 0
    $leftOver = 0

    if ($NumberOfBuckets -gt 0) {
        $blockCount = [System.Math]::Ceiling($List.Count / $NumberOfBuckets)
        $loopCount = $NumberOfBuckets
        $endBlockCount = $blockCount
    }
    else {
        $blockCount = [System.Math]::Floor($List.Count / $BucketSize)
        $leftOver = $List.Count % $BucketSize
        $loopCount = $blockCount
        $endBlockCount = $BucketSize
    }

    for ($i = 0; $i -lt $loopCount; $i++) {
        $end = $endBlockCount * ($i + 1) - 1

        $aggregateList += , $List[$start..$end]
        $start = $end + 1
    }

    if ($leftOver -gt 0) {
        $aggregateList += , $List[$start..($end + $leftOver)]
    }

    $aggregateList
}

function Show-ErrorMessage {
    [CmdletBinding()]
    param (
        [Parameter(Mandatory)]
        [string]$Message
    )

    if ($Host.Name -in 'Default Host', 'ConsoleHost') {
        [System.Console]::ForegroundColor = [System.ConsoleColor]::Red
        [System.Console]::WriteLine("ERROR: $Message")
        [System.Console]::ResetColor()
    }
    else {
        $Host.UI.WriteLine([System.ConsoleColor]::Red, $Host.UI.RawUI.BackgroundColor, "ERROR: $Message")
    }
}

function Show-WarningMessage {
    [CmdletBinding()]
    param (
        [Parameter(Mandatory)]
        [string]$Message
    )

    if ($Host.Name -in 'Default Host', 'ConsoleHost') {
        [System.Console]::ForegroundColor = [System.ConsoleColor]::Yellow
        [System.Console]::WriteLine("WARNING: $Message")
        [System.Console]::ResetColor()
    }
    else {
        $Host.UI.WriteLine([System.ConsoleColor]::Yellow, $Host.UI.RawUI.BackgroundColor, "WARNING: $Message")
    }
}

function Show-InformationMessage {
    [CmdletBinding()]
    param (
        [Parameter(Mandatory)]
        [string]$Message
    )

    if ($Host.Name -in 'Default Host', 'ConsoleHost') {
        [System.Console]::ForegroundColor = [System.ConsoleColor]::White
        [System.Console]::WriteLine("INFORMATION: $Message")
        [System.Console]::ResetColor()
    }
    else {
        $Host.UI.WriteLine([System.ConsoleColor]::White, $Host.UI.RawUI.BackgroundColor, "INFORMATION: $Message")
    }
}

function Get-NtapDocsDataSetSchemaVersion {
    param (
        [Parameter(Mandatory)]
        [System.Data.DataSet]$DataSet
    )

    if ($DataSet.ExtendedProperties.ContainsKey('SchemaVersion')) {
        [version]::Parse($DataSet.ExtendedProperties['SchemaVersion'])
    }
    else {
        $ntapDocsVersionColumnName = ($DataSet.Tables['DocumentVersionInformation'].Columns | Where-Object { $_.ColumnName -like '*DocsVersion' }).ColumnName
        [version]::Parse(
            (
                $DataSet.Tables['DocumentVersionInformation'].Rows | Select-Object -Property @{
                    Name       = 'Version'
                    Expression = { ($_.$ntapDocsVersionColumnName -split '\s*[|]\s*')[1] -Replace '^[^:]*[:]\s*' }
                } -Unique | Sort-Object -Property Version | Select-Object -First 1
            ).Version
        )
    }
}

function Get-VersionString {
    [CmdletBinding(DefaultParameterSetName = 'none')]
    [OutputType([System.String])]
    param (
        [Parameter(Mandatory, Position = 0)]
        [object]$InputVersion,

        [Parameter(Mandatory, ParameterSetName = 'PSVersion')]
        [switch]$PSVersion,

        [Parameter(Mandatory, ParameterSetName = 'AsFeatureVersion')]
        [switch]$AsFeatureVersion,

        [Parameter(Mandatory, ParameterSetName = 'AsPatchVersion')]
        [switch]$AsPatchVersion
    )

    switch ($PSCmdlet.ParameterSetName) {
        'PSVersion' {
            if ($InputVersion.psobject.TypeNames[0] -like '*SemanticVersion') {
                $InputVersion.ToString()
            }
            else {
                [string]::Format('{0}.{1}', $InputVersion.Major, $InputVersion.Minor)
            }
        }
        'AsFeatureVersion' {
            [string]::Format('{0}.{1}', $InputVersion.Major, $InputVersion.Minor)
        }
        'AsPatchVersion' {
            if ($InputVersion.psobject.TypeNames[0] -like '*SemanticVersion') {
                [string]::Format('{0}.{1}.{2}', $InputVersion.Major, $InputVersion.Minor, ($InputVersion.Patch, 0)[[byte](! ($InputVersion.Patch -gt 0))])
            }
            else {
                [string]::Format('{0}.{1}.{2}', $InputVersion.Major, $InputVersion.Minor, ($InputVersion.Build, 0)[[byte](! ($InputVersion.Build -gt 0))])
            }
        }
        default {
            $InputVersion.ToString()
        }
    }
}

function ConvertFrom-ShelfLogicalUid {
    <#
        .SYNOPSIS
            Converts a LogicalUid string into the Int64 referenced by ONTAP (disk show output 'shelf-uid' field).

        .DESCRIPTION
            ONTAP lists the ShelfUid that the disk resides on as an Int64. This function calculates that Int64 from a given
            shelf UID.

        .EXAMPLE
            PS> ConvertFrom-ShelfLogicalUid -LogicalUid '50:05:0c:c1:02:03:2f:ca'
            14568866629930452304

        .LINK
            https://community.netapp.com/t5/Microsoft-Virtualization-Discussions/bd-p/microsoft-cloud-and-virtualization-discussions
    #>

    param (
        [Parameter(Mandatory)]
        [AllowEmptyString()]
        [string]$LogicalUid
    )

    if ([string]::IsNullOrWhiteSpace($LogicalUid)) {
        return
    }

    # ONTAP sometimes adds an extra :00 element to the end of the UID during the calculation
    $uidHexElements = ($LogicalUid -split ':') + '00'

    # Reverse sort for Big Endian conversion
    [array]::Reverse($uidHexElements)

    # If the ':00' is added by ONTAP then the number becomes larger than the Int64.MaxValue, so we must use a BigInteger instead
    $bigint = [bigint]::Zero

    if (
        [bigint]::TryParse(
            ($uidHexElements -join ''),
            [System.Globalization.NumberStyles]::HexNumber,
            [cultureinfo]::InvariantCulture,
            [ref]$bigint
        )
    ) {
        $bigint
    }
}

function ConvertTo-ShelfLogicalUid {
    <#
        .SYNOPSIS
          Converts the Int64 value referenced by ONTAP back to the proper LogicalUID (disk show output 'shelf-uid' field).

        .DESCRIPTION
            ONTAP lists the ShelfUID that the disk resides on as an Int64. This function converts that Int64 value back to
            the proper shelf UID.

        .EXAMPLE
            PS> ConvertTo-ShelfLogicalUid -LogicalUid 14568866629930452304
            50:05:0c:c1:02:03:2f:ca

        .LINK
            https://community.netapp.com/t5/Microsoft-Virtualization-Discussions/bd-p/microsoft-cloud-and-virtualization-discussions
    #>

    param (
        [Parameter(Mandatory)]
        [AllowEmptyString()]
        [string]$Number
    )

    if ([string]::IsNullOrWhiteSpace($Number)) {
        return
    }

    # We must use a BigInteger as the number may be larger than the Int64.MaxValue
    $bigint = [bigint]::Zero

    if ([bigint]::TryParse($Number, [ref]$bigint)) {
        (
            $bigint.ToByteArray() | Select-Object -First 8 | ForEach-Object {
                $_.ToString('x2')
            }
        ) -join ':'
    }
}

function Get-SessionInfo ([int]$HistoryId) {
    $getMachineGuid = {
        $machineGuid = [guid]::Empty

        if ($PSVersionTable.PSVersion.Major -lt 6 -or $IsWindows) {
            $machineGuidRegValue = $(
                if ([System.Environment]::Is64BitOperatingSystem -and -not [System.Environment]::Is64BitProcess) {
                    try {
                        $baseKey = [Microsoft.Win32.RegistryKey]::OpenBaseKey([Microsoft.Win32.RegistryHive]::LocalMachine, [Microsoft.Win32.RegistryView]::Registry64)
                        $subKey = $baseKey.OpenSubKey('Software\Microsoft\Cryptography', [Microsoft.Win32.RegistryKeyPermissionCheck]::ReadSubTree)
                        $subKey.GetValue('MachineGuid')
                    }
                    catch {
                        $Global:Error.RemoveAt(0)
                    }
                    finally {
                        $subKey.Close()
                        $baseKey.Close()
                    }
                }
                else {
                    (Get-ItemProperty -Path 'HKLM:SOFTWARE\Microsoft\Cryptography' -Name MachineGuid -ErrorAction Ignore).MachineGuid
                }
            )

            if (-not [guid]::TryParse($machineGuidRegValue, [ref]$machineGuid)) {
                [void][guid]::TryParse((Get-CimInstance -ClassName Win32_ComputerSystemProduct -Property UUID -ErrorAction Ignore).Uuid, [ref]$machineGuid)
            }
        }
        elseif ($IsLinux) {
            [void][guid]::TryParse((Get-Content -Path '/etc/machine-id'), [ref]$machineGuid)

            #if ($machineGuid -eq [guid]::Empty) {
            #    #suggest running: systemd-machine-id-setup
            #}
        }
        elseif ($IsMacOS) {
            [void][guid]::TryParse((& ioreg -d2 -c IOPlatformExpertDevice | Select-String -Pattern '"IOPlatformUUID"\s*[=]\s*"(.+?)"$').Matches.Groups[1].Value, [ref]$machineGuid)
        }

        $machineGuid
    }

    [pscustomobject]@{
        ModuleSessionGuid   = [guid]::NewGuid()
        PipelineSessionGuid = [guid]::Empty
        HistoryId           = (Get-PSCallStack)[0].InvocationInfo.HistoryId
        OSInfo              = [System.Environment]::OSVersion | Select-Object -ExcludeProperty Platform -Property @{
            Name       = 'Platform'
            Expression = {
                if (Test-Path -Path './.dockerenv' -ErrorAction Ignore) {
                    "$( $_.Platform ) (Docker)"
                }
                else {
                    $_.Platform
                }
            }
        }, Version, @{
            Name       = 'Caption'
            Expression = {
                if ($PSVersionTable.PSVersion.Major -lt 6 -or $IsWindows) {
                    (Get-CimInstance -ClassName Win32_OperatingSystem -Property Caption).Caption
                }
                elseif ($IsLinux) {
                    [regex]::Match((Get-Content -Path '/etc/os-release' -Raw), '(?s)(?=.*\bPRETTY_NAME\s*[:|=]\s*\"(?<Caption>.+?)\")(?=.*\bVERSION_ID\s*[:|=]\s*\"(?<Version>.+?)\").*$').Groups['Caption'].Value
                }
                elseif ($IsMacOS) {
                    [regex]::Match(((& sw_vers) -join "`n"), '(?s)^(?=.*ProductName\s*[:]\s*(?<ProductName>.+?)\n)(?=.*ProductVersion\s*[:]\s*(?<ProductVersion>.+?)\n)(?=.*BuildVersion\s*[:]\s*(?<BuildVersion>.+?)\b).*$').Groups['ProductName'].Value
                }
            }
        }, @{
            Name       = 'Architecture'
            Expression = {
                $(
                    if ([System.Environment]::Is64BitOperatingSystem) {
                        '64-bit'
                    }
                    else {
                        '32-bit'
                    }
                )
            }
        }
        NetAppDocsVersion   = (Test-ModuleManifest -Path "$( (Get-PSCallStack)[0].InvocationInfo.ScriptName -replace '[.]psm1', '.psd1' )" -ErrorAction Ignore) | ForEach-Object { [version]::Parse([string]::Format('{0}.{1}', $_.Version.ToString(3), $_.PrivateData.BuildNumber)) }
        PowerShellVersion   = $PSVersionTable.PSVersion
        MachineGuid         = $getMachineGuid.InvokeReturnAsIs()
        LocationInfo        = [pscustomobject]@{
            CountryCode = [System.Globalization.RegionInfo]::CurrentRegion.ThreeLetterISORegionName
        }
    }
}

function FilterByVserver {
    param ([string]$VserverName)

    process {
        if ($_.VolumeIdAttributes.OwningVserverName -ceq $VserverName -or $_.VserverName -ceq $VserverName -or $_.Vserver -ceq $VserverName -or $_.Vs -ceq $VserverName -or $_.VserverId -ceq $VserverName) {
            $_
        }
    }
}

function FilterByVolume {
    param ([string]$VserverName, [string]$VolumeName)

    process {
        if (
            ([string]::IsNullOrWhiteSpace($VserverName) -or ($_.VolumeIdAttributes.OwningVserverName -ceq $VserverName -or $_.VserverName -ceq $VserverName -or $_.Vserver -ceq $VserverName -or $_.Vs -ceq $VserverName -or $_.VserverId -ceq $VserverName) ) -and
            ($_.VolumeIdAttributes.Name -ceq $VolumeName -or $_.VolumeName -ceq $VolumeName -or $_.Volume -ceq $VolumeName -or $_.Vol -ceq $VolumeName -or $_.VolId -ceq $VolumeName -or $_.V -ceq $VolumeName)
        ) {
            $_
        }
    }
}

function FilterByQtree {
    param ([string]$VserverName, [string]$VolumeName, [string]$QtreeName)

    process {
        if (
            ([string]::IsNullOrWhiteSpace($VserverName) -or ($_.VolumeIdAttributes.OwningVserverName -ceq $VserverName -or $_.VserverName -ceq $VserverName -or $_.Vserver -ceq $VserverName -or $_.Vs -ceq $VserverName -or $_.VserverId -ceq $VserverName) ) -and
            ($_.VolumeIdAttributes.Name -ceq $VolumeName -or $_.VolumeName -ceq $VolumeName -or $_.Volume -ceq $VolumeName -or $_.Vol -ceq $VolumeName -or $_.VolId -ceq $VolumeName -or $_.V -ceq $VolumeName) -and
            ($_.Qtree -ceq $QtreeName -or $_.Qt -ceq $QtreeName)
        ) {
            $_
        }
    }
}

function FilterBySnapshot {
    param ([string]$VserverName, [string]$VolumeName, [string]$SnapshotName)

    process {
        if (
            ($_.VolumeIdAttributes.OwningVserverName -ceq $VserverName -or $_.VserverName -ceq $VserverName -or $_.Vserver -ceq $VserverName -or $_.Vs -ceq $VserverName -or $_.VserverId -ceq $VserverName) -and
            ($_.VolumeIdAttributes.Name -ceq $VolumeName -or $_.VolumeName -ceq $VolumeName -or $_.Volume -ceq $VolumeName -or $_.Vol -ceq $VolumeName -or $_.VolId -ceq $VolumeName -or $_.V -ceq $VolumeName) -and
            ($_.Name -eq $SnapshotName -or $_.Snapshot -eq $SnapshotName)
        ) {
            $_
        }
    }
}

function FilterByCifsShare {
    param ([string]$VserverName, [string]$CifsShareName)

    process {
        if (
            ($_.VserverName -ceq $VserverName -or $_.Vserver -ceq $VserverName -or $_.Vs -ceq $VserverName -or $_.VserverId -ceq $VserverName) -and
            ($_.ShareName -ceq $CifsShareName -or $_.Share -ceq $CifsShareName -or $_.CifsShareId -ceq $CifsShareName)
        ) {
            $_
        }
    }
}

function FilterByExportPolicyName {
    param ([string]$VserverName, [string]$PolicyName)

    process {
        if (
            ([string]::IsNullOrWhiteSpace($VserverName) -or ($_.VolumeIdAttributes.OwningVserverName -ceq $VserverName -or $_.VserverName -ceq $VserverName -or $_.Vserver -ceq $VserverName -or $_.Vs -ceq $VserverName -or $_.VserverId -ceq $VserverName) ) -and
            ($_.VolumeExportAttributes.Policy -ceq $PolicyName -or $_.PolicyName -eq $PolicyName -or $_.Policy -ceq $PolicyName)
        ) {
            $_
        }
    }
}

function Test-IsLibGdiPlusPackageInstalled {
    [CmdletBinding()]
    [OutputType([System.Boolean])]
    param ( )

    if ($PSVersionTable.PSVersion -lt [version]::Parse('6.0') -or $IsWindows) {
        return $true
    }

    if ($IsLinux -or $IsMacOS) {
        if ($PSVersionTable.PSVersion -ge [version]::Parse('7.2')) {
            $msg = @'
NetAppDocs is currently not supported on non-Windows platforms
with PowerShell 7.2+. This is due to the current limitation with the
components used to generate the MS Office documentation. NetAppDocs will
be updated to use the new components as soon as they are available.
The only solution at this time is to install and use the older 7.1.x version
of PowerShell on your MAC and Linux platforms.

'@

            Show-ErrorMessage -Message $msg

            return $false
        }

        $excelFile = New-Object -TypeName Aspose.Cells.Workbook

        try {
            $excelFile.Worksheets[0].AutoFitColumns()

            return $true
        }
        catch {
            if ($IsLinux) {
                $msg = @'
The following dependency packages must be installed on your system using your
package manager:
libgdiplus
libc6-dev

ubuntu/debian:
apt-get -y update && apt-get install -y --no-install-recommends libgdiplus libc6-dev

fedora/centos:
yum install libgdiplus libc6-dev

openuse linux:
zypper install libgdiplus libc6-dev

'@
            }

            if ($IsMacOS) {
                $msg = @'
The following dependency package must be installed on your system using the Homebrew
package manager:
mono-libgdiplus

brew install mono-libgdiplus

'@
            }

            Show-ErrorMessage -Message $msg

            return $false
        }
        finally {
            $excelFile.Dispose()
        }
    }
}

function Join-Object {
    <#
        .SYNOPSIS
            Join data from two sets of objects based on a common value

        .DESCRIPTION
            Join data from two sets of objects based on a common value
            For more details, see the accompanying blog post:
                http://ramblingcookiemonster.github.io/Join-Object/
            For even more details, see the original code and discussions that this borrows from:
                Dave Wyatt's Join-Object - http://powershell.org/wp/forums/topic/merging-very-large-collections
                Lucio Silveira's Join-Object - http://blogs.msdn.com/b/powershell/archive/2012/07/13/join-object.aspx

        .PARAMETER Left
            'Left' collection of objects to join. You can use the pipeline for Left.
            The objects in this collection should be consistent.
            We look at the properties on the first object for a baseline.

        .PARAMETER Right
            'Right' collection of objects to join.
            The objects in this collection should be consistent.
            We look at the properties on the first object for a baseline.

        .PARAMETER LeftJoinProperty
            Property on Left collection objects that we match up with RightJoinProperty on the Right collection

        .PARAMETER RightJoinProperty
            Property on Right collection objects that we match up with LeftJoinProperty on the Left collection

        .PARAMETER LeftProperties
            One or more properties to keep from Left. Default is to keep all Left properties (*).
            Each property can:
                - Be a plain property name like "Name"
                - Contain wildcards like "*"
                - Be a Hashtable like @{Name="Product Name";Expression={$_.Name}}.
                    Name is the output property name
                    Expression is the property value ($_ as the current object)
                    Alternatively, use the Suffix or Prefix parameter to avoid collisions
                    Each property using this Hashtable syntax will be excluded from suffixes and prefixes

        .PARAMETER RightProperties
            One or more properties to keep from Right. Default is to keep all Right properties (*).
            Each property can:
                - Be a plain property name like "Name"
                - Contain wildcards like "*"
                - Be a Hashtable like @{Name="Product Name";Expression={$_.Name}}.
                    Name is the output property name
                    Expression is the property value ($_ as the current object)
                    Alternatively, use the Suffix or Prefix parameter to avoid collisions
                    Each property using this Hashtable syntax will be excluded from suffixes and prefixes

        .PARAMETER Prefix
            If specified, prepend Right object property names with this prefix to avoid collisions
            Example:
                Property Name                   = 'Name'
                Suffix                          = 'j_'
                Resulting Joined Property Name  = 'j_Name'

        .PARAMETER Suffix
            If specified, append Right object property names with this suffix to avoid collisions
            Example:
                Property Name                   = 'Name'
                Suffix                          = '_j'
                Resulting Joined Property Name  = 'Name_j'

        .PARAMETER Type
            Type of join. Default is AllInLeft.
            AllInLeft will have all elements from Left at least once in the output, and might appear more than once
            if the where clause is true for more than one element in right, Left elements with matches in Right are
            preceded by elements with no matches.
            SQL equivalent: outer left join (or simply left join)
            AllInRight is similar to AllInLeft.
            OnlyIfInBoth will cause all elements from Left to be placed in the output, only if there is at least one
            match in Right.
            SQL equivalent: inner join (or simply join)
            AllInBoth will have all entries in right and left in the output. Specifically, it will have all entries
            in right with at least one match in left, followed by all entries in Right with no matches in left,
            followed by all entries in Left with no matches in Right.
            SQL equivalent: full join

        .EXAMPLE
            #
            #Define some input data.
            $l = 1..5 | Foreach-Object {
                [PSCustomObject]@{
                    Name = "jsmith$_"
                    Birthday = (Get-Date).AddDays(-1)
                }
            }

            $r = 4..7 | Foreach-Object{
                [PSCustomObject]@{
                    Department = "Department $_"
                    Name = "Department $_"
                    Manager = "jsmith$_"
                }
            }
            #We have a name and Birthday for each manager, how do we find their department, using an inner join?
            Join-Object -Left $l -Right $r -LeftJoinProperty Name -RightJoinProperty Manager -Type OnlyIfInBoth -RightProperties Department
                # Name    Birthday             Department
                # ----    --------             ----------
                # jsmith4 4/14/2015 3:27:22 PM Department 4
                # jsmith5 4/14/2015 3:27:22 PM Department 5

        .EXAMPLE
            #
            #Define some input data.
            $l = 1..5 | Foreach-Object {
                [PSCustomObject]@{
                    Name = "jsmith$_"
                    Birthday = (Get-Date).AddDays(-1)
                }
            }

            $r = 4..7 | Foreach-Object{
                [PSCustomObject]@{
                    Department = "Department $_"
                    Name = "Department $_"
                    Manager = "jsmith$_"
                }
            }
            #We have a name and Birthday for each manager, how do we find all related department data, even if there are conflicting properties?
            $l | Join-Object -Right $r -LeftJoinProperty Name -RightJoinProperty Manager -Type AllInLeft -Prefix j_
                # Name    Birthday             j_Department j_Name       j_Manager
                # ----    --------             ------------ ------       ---------
                # jsmith1 4/14/2015 3:27:22 PM
                # jsmith2 4/14/2015 3:27:22 PM
                # jsmith3 4/14/2015 3:27:22 PM
                # jsmith4 4/14/2015 3:27:22 PM Department 4 Department 4 jsmith4
                # jsmith5 4/14/2015 3:27:22 PM Department 5 Department 5 jsmith5

        .EXAMPLE
            #
            #Hey! You know how to script right? Can you merge these two CSVs, where Path1's IP is equal to Path2's IP_ADDRESS?
            #Get CSV data
            $s1 = Import-CSV $Path1
            $s2 = Import-CSV $Path2
            #Merge the data, using a full outer join to avoid omitting anything, and export it
            Join-Object -Left $s1 -Right $s2 -LeftJoinProperty IP_ADDRESS -RightJoinProperty IP -Prefix 'j_' -Type AllInBoth |
                Export-CSV $MergePath -NoTypeInformation

        .EXAMPLE
            #
            # "Hey Warren, we need to match up SSNs to Active Directory users, and check if they are enabled or not.
            #  I'll e-mail you an unencrypted CSV with all the SSNs from gmail, what could go wrong?"
            # Import some SSNs.
            $SSNs = Import-CSV -Path D:\SSNs.csv
            #Get AD users, and match up by a common value, SamAccountName in this case:
            Get-ADUser -Filter "SamAccountName -like 'WFrame*'" |
                Join-Object -LeftJoinProperty SamAccountName -Right $SSNs `
                            -RightJoinProperty SamAccountName -RightProperties ssn `
                            -LeftProperties SamAccountName, enabled, ObjectClass

        .NOTES
            This borrows from:
                Dave Wyatt's Join-Object - http://powershell.org/wp/forums/topic/merging-very-large-collections/
                Lucio Silveira's Join-Object - http://blogs.msdn.com/b/powershell/archive/2012/07/13/join-object.aspx
            Changes:
                Always display full set of properties
                Display properties in order (left first, right second)
                If specified, add suffix or prefix to right object property names to avoid collisions
                Use a Hashtable rather than OrderedDictionary (avoid case sensitivity)

        .LINK
            http://ramblingcookiemonster.github.io/Join-Object/

        .FUNCTIONALITY
            PowerShell Language
    #>

    [Diagnostics.CodeAnalysis.SuppressMessageAttribute('PSUseDeclaredVarsMoreThanAssignments', '')]
    [Diagnostics.CodeAnalysis.SuppressMessageAttribute('PSUseProcessBlockForPipelineCommand', '')]
    [CmdletBinding(DefaultParameterSetName = 'Default')]
    [OutputType([System.Array])]
    param (
        [Parameter(Mandatory = $true, ValueFromPipeLine = $true)]
        $Left,

        [Parameter(Mandatory = $true)]
        [AllowNull()]
        $Right,

        [Parameter(Mandatory = $true)]
        [string[]]$LeftJoinProperty,

        [Parameter(Mandatory = $true)]
        [string[]]$RightJoinProperty,

        $LeftJoinScript,

        $RightJoinScript,

        [ValidateScript({ $_ -is [System.Collections.Hashtable] -or $_ -is [string] -or $_ -is [System.Collections.Specialized.OrderedDictionary] })]
        $LeftProperties = '*',

        [ValidateScript({ $_ -is [System.Collections.Hashtable] -or $_ -is [string] -or $_ -is [System.Collections.Specialized.OrderedDictionary] })]
        $RightProperties = '*',

        [string[]]$ExcludeLeftProperties,

        [string[]]$ExcludeRightProperties,

        [switch]$KeepRightJoinProperty,

        [ValidateSet('AllInLeft', 'OnlyIfInBoth')] #, 'AllInBoth')]
        [Parameter(Mandatory = $false)]
        [string]$Type = 'AllInLeft',

        [string]$Prefix,

        [string]$Suffix,

        [Parameter(Mandatory, ParameterSetName = 'PassThru')]
        [switch]$PassThru,

        [Parameter(Mandatory, ParameterSetName = 'DataTable')]
        [switch]$DataTable,

        [Parameter(ParameterSetName = 'PassThru')]
        [Parameter(ParameterSetName = 'DataTable')]
        [hashtable]$DataTableTypes,

        [ValidateSet('SingleOnly', 'DuplicateLines', 'SubGroups')]
        [string]$LeftMultiMode = 'SingleOnly',

        [ValidateSet('SingleOnly', 'DuplicateLines', 'SubGroups')]
        [string]$RightMultiMode = 'SingleOnly',

        [String]$AddKey,

        [switch]$AllowColumnsMerging,

        [System.Collections.Generic.IEqualityComparer[Object]]$Comparer
    )

    # if $Right is null then just return $Left as-is
    if (-not $Right) {
        return $Left
    }

    #region Validate Params

    #if ($PassThru -and ($Type -in @( 'AllInBoth', 'OnlyIfInBoth' ) -or ($Type -eq 'AllInLeft' -and $RightMultiMode -eq 'DuplicateLines'))) {
    if ($PassThru -and ($Type -eq 'OnlyIfInBoth' -or ($Type -eq 'AllInLeft' -and $RightMultiMode -eq 'DuplicateLines'))) {
        $PSCmdlet.ThrowTerminatingError(
            [Management.Automation.ErrorRecord]::new(
                [ArgumentException]::new('"-PassThru" compatible only with "-Type AllInLeft" with "-RightMultiMode" "SingleOnly" or "SubGroups"'),
                'Incompatible Arguments',
                [Management.Automation.ErrorCategory]::InvalidArgument,
                $Type
            )
        )
    }

    <#if ($AddKey -and $Type -ne 'AllInBoth') {
        $PSCmdlet.ThrowTerminatingError(
            [Management.Automation.ErrorRecord]::new(
                [ArgumentException]::new('"-AddKey" support only "-Type AllInBoth"'),
                'Incompatible Arguments',
                [Management.Automation.ErrorCategory]::InvalidArgument,
                $AddKey
            )
        )
    }#>

    if ($Type -in @( 'AllInLeft', 'OnlyIfInBoth' )) {
        if ($PSBoundParameters['LeftMultiMode'] -ne 'DuplicateLines' -and $null -ne $PSBoundParameters['LeftMultiMode']) {
            $PSCmdlet.ThrowTerminatingError(
                [Management.Automation.ErrorRecord]::new(
                    [ArgumentException]::new('"-Type AllInLeft" and "-Type OnlyIfInBoth" support only "-LeftMultiMode DuplicateLines"'),
                    'Incompatible Arguments',
                    [Management.Automation.ErrorCategory]::InvalidArgument,
                    $Type
                )
            )
        }

        $Attributes = (Get-Variable 'LeftMultiMode').Attributes
        [void]$Attributes.Remove($Attributes.Where({ $_.TypeId.Name -eq 'ValidateSetAttribute' })[0])
        $ValidateSetAttribute = [System.Management.Automation.ValidateSetAttribute]::new('SingleOnly', 'DuplicateLines', 'SubGroups', $null)
        $Attributes.Add($ValidateSetAttribute)
        $LeftMultiMode = $null
    }

    if ($Type -eq 'OnlyIfInBoth') {
        if ($PSBoundParameters['RightMultiMode'] -ne 'DuplicateLines' -and $null -ne $PSBoundParameters['RightMultiMode']) {
            $PSCmdlet.ThrowTerminatingError(
                [Management.Automation.ErrorRecord]::new(
                    [ArgumentException]::new('"-Type OnlyIfInBoth" support only "-RightMultiMode DuplicateLines"'),
                    'Incompatible Arguments',
                    [Management.Automation.ErrorCategory]::InvalidArgument,
                    $Type
                )
            )
        }

        $Attributes = (Get-Variable 'RightMultiMode').Attributes
        [void]$Attributes.Remove($Attributes.Where({ $_.TypeId.Name -eq 'ValidateSetAttribute' })[0])
        $ValidateSetAttribute = [System.Management.Automation.ValidateSetAttribute]::new('SingleOnly', 'DuplicateLines', 'SubGroups', $null)
        $Attributes.Add($ValidateSetAttribute)
        $RightMultiMode = $null
    }

    if ($AllowColumnsMerging -and !$DataTable -and !($PassThru -and $Left -is [DataTable])) {
        $PSCmdlet.ThrowTerminatingError(
            [Management.Automation.ErrorRecord]::new(
                [ArgumentException]::new('"-AllowColumnsMerging" support only on DataTable output'),
                'Incompatible Arguments',
                [Management.Automation.ErrorCategory]::InvalidArgument,
                $AllowColumnsMerging
            )
        )
    }

    #endregion Validate Params

    #region Set $SelectedLeftProperties and $SelectedRightProperties

    function Get-Properties {
        [Diagnostics.CodeAnalysis.SuppressMessageAttribute('PSUseSingularNouns', '')]
        [CmdletBinding()]
        [OutputType([System.Collections.Specialized.OrderedDictionary])]
        param (
            $Object,
            $SelectProperties,
            $ExcludeProperties,
            $Prefix,
            $Suffix
        )

        $Properties = [ordered]@{ }

        if ($Object -is [System.Data.DataTable]) {
            $ObjectProperties = $Object.Columns.ColumnName
        }
        else {
            #$ObjectProperties = $Object[0].PSObject.Properties.Name
            $ObjectProperties = ($Object | Group-Object -Property { @( $_.psobject.properties ).Count } | Select-Object -Last 1).Group[0].psobject.properties.name
        }

        if ($SelectProperties -is [hashtable] -or $SelectProperties -is [System.Collections.Specialized.OrderedDictionary]) {
            $SelectProperties.GetEnumerator() | Where-Object { $_.Key -notin $ExcludeProperties } | ForEach-Object { $Properties.Add($_.Key, $Prefix + $_.Value + $Suffix) }
        }
        elseif ($SelectProperties -eq '*') {
            $ObjectProperties | Where-Object { $_ -notin $ExcludeProperties } | ForEach-Object { $Properties.Add($_, $Prefix + $_ + $Suffix) }
        }
        else {
            $SelectProperties | Where-Object { $_ -notin $ExcludeProperties } | ForEach-Object { $Properties.Add($_, $Prefix + $_ + $Suffix) }
        }

        $Properties
    }

    $SelectedLeftProperties = Get-Properties -Object $Left -SelectProperties $LeftProperties -ExcludeProperties $ExcludeLeftProperties

    if (!$KeepRightJoinProperty) {
        $ExcludeRightProperties = $null -ne @( $ExcludeRightProperties ) + @( $RightJoinProperty )
    }

    $SelectedRightProperties = Get-Properties -Object $Right -SelectProperties $RightProperties -ExcludeProperties $ExcludeRightProperties -Prefix $Prefix -Suffix $Suffix

    #endregion Set $SelectedLeftProperties and $SelectedRightProperties

    #region Importing package MoreLinq

    <#if ($Type -eq 'AllInBoth') {
        try {
            if ($PSScriptRoot) {
                $ScriptRoot = $PSScriptRoot
            }
            elseif ($psISE.CurrentFile.IsUntitled -eq $false) {
                $ScriptRoot = Split-Path -Path $psISE.CurrentFile.FullPath
            }
            elseif ($null -ne $psEditor.GetEditorContext().CurrentFile.Path -and $psEditor.GetEditorContext().CurrentFile.Path -notlike 'untitled:*') {
                $ScriptRoot = Split-Path -Path $psEditor.GetEditorContext().CurrentFile.Path
            }
            else {
                $ScriptRoot = '.'
            }

            if (!('MoreLinq.MoreEnumerable' -as [type])) {
                Add-Type -Path (Resolve-Path -Path "$ScriptRoot\morelinq.*\lib\Core\MoreLinq.dll")
            }
        }
        catch {
            $PSCmdlet.ThrowTerminatingError(
                [Management.Automation.ErrorRecord]::new(
                    [TypeLoadException]::new('Importing package MoreLinq failed: {0}' -f $_.Exception.Message, $_.Exception),
                    'Importing package',
                    [Management.Automation.ErrorCategory]::NotInstalled,
                    $null
                )
            )
        }
    }#>

    #endregion Importing package MoreLinq

    #region Set $RightJoinScript and $LeftJoinScript

    function Get-JoinScript {
        [CmdletBinding()]
        [OutputType([System.Func[Object, String]])]
        param (
            $JoinScript,
            $JoinProperty,
            $Side,
            $Object
        )

        if ($JoinScript) {
            if ($JoinScript.GetType().Name -eq 'Func`2') {
                $JoinScript #.GetNewClosure()
            }
            else {
                [System.Func[Object, String]]$JoinScript
            }
        }
        else {
            $JoinScript = if ($JoinProperty.Count -gt 1) {
                {
                    param ($_Side_Line)
                    ($_Side_Line | Select-Object -Property $_Side_JoinProperty).PSObject.Properties.Value
                }
            }
            else {
                if ($Object -is [Data.DataTable]) {
                    {
                        param ($_Side_Line)
                        $_Side_Line[$_Side_JoinProperty]
                    }
                }
                else {
                    {
                        param ($_Side_Line)
                        $_Side_Line.$_Side_JoinProperty
                    }
                }
            }

            [System.Func[Object, String]][Scriptblock]::Create($JoinScript.ToString().Replace('_Side_', $Side))
        }
    }

    $LeftJoinScript = Get-JoinScript -JoinScript $LeftJoinScript -JoinProperty $LeftJoinProperty -Side 'Left' -Object $Left
    $RightJoinScript = Get-JoinScript -JoinScript $RightJoinScript -JoinProperty $RightJoinProperty -Side 'Right' -Object $Right

    #endregion Set $RightJoinScript and $LeftJoinScript

    #region Prepare Data

    function Set-OutDataTable {
        [Diagnostics.CodeAnalysis.SuppressMessageAttribute('PSUseShouldProcessForStateChangingFunctions', '')]
        param (
            $OutDataTable,
            $Object,
            $SelectedProperties,
            $AllowColumnsMerging
        )

        # Create Columns
        foreach ($item in $SelectedProperties.GetEnumerator()) {
            if (!$AllowColumnsMerging -or !$OutDataTable.Columns.Item($item.Value)) {
                if ($Object -is [Data.DataTable]) {
                    [void]$OutDataTable.Columns.Add($item.Value, $Object.Columns.Item($item.Name).DataType)
                }
                else {
                    if ($null -ne $DataTableTypes.($item.Value)) {
                        [void]$OutDataTable.Columns.Add($item.Value, $DataTableTypes.($item.Value))
                    }
                    else {
                        [void]$OutDataTable.Columns.Add($item.Value)
                    }
                }
            }
        }
    }

    if ($DataTable) {
        $OutDataTable = [Data.DataTable]::new('Joined')

        if ($AddKey) {
            [void]$OutDataTable.Columns.Add($AddKey)
        }

        if ($LeftMultiMode -eq 'SubGroups') {
            $OutDataTableSubGroupTemplateLeft = [Data.DataTable]::new('LeftGroup')
            Set-OutDataTable -OutDataTable $OutDataTableSubGroupTemplateLeft -Object $Left -SelectedProperties $SelectedLeftProperties

            [void]$OutDataTable.Columns.Add('LeftGroup', [Object])
        }
        else {
            Set-OutDataTable -OutDataTable $OutDataTable -Object $Left -SelectedProperties $SelectedLeftProperties
        }

        if ($RightMultiMode -eq 'SubGroups') {
            $OutDataTableSubGroupTemplateRight = [Data.DataTable]::new('RightGroup')
            Set-OutDataTable -OutDataTable $OutDataTableSubGroupTemplateRight -Object $Right -SelectedProperties $SelectedRightProperties

            [void]$OutDataTable.Columns.Add('RightGroup', [Object])
        }
        else {
            Set-OutDataTable -OutDataTable $OutDataTable -Object $Right -SelectedProperties $SelectedRightProperties -AllowColumnsMerging $AllowColumnsMerging
        }
    }
    elseif ($PassThru) {
        if ($Left -is [Data.DataTable]) {
            # Remove LeftLine
            foreach ($ColumnName in $Left.Columns.ColumnName) {
                if ($ColumnName -notin $SelectedLeftProperties.Keys) {
                    $Left.Columns.Remove($ColumnName)
                }
            }

            # Rename LeftLine
            foreach ($item in $SelectedLeftProperties.GetEnumerator()) {
                if ($item.Key -ne $item.Value -and ($Column = $Left.Columns.Item($item.Key))) {
                    $Column.ColumnName = $item.Value
                }
            }

            if ($RightMultiMode -eq 'SubGroups') {
                [void]$Left.Columns.Add('RightGroup', [Object])
            }
            else {
                # Add RightLine to LeftLine
                foreach ($item in $SelectedRightProperties.GetEnumerator()) {
                    if (!$AllowColumnsMerging -or !$Left.Columns.Item($item.Value)) {
                        if ($null -ne $DataTableTypes.($item.Value)) {
                            [void]$Left.Columns.Add($item.Value, $DataTableTypes.($item.Value))
                        }
                        else {
                            [void]$Left.Columns.Add($item.Value)
                        }
                    }
                }
            }
        }

        if ($Right -is [Data.DataTable] -and $RightMultiMode -eq 'SubGroups') {
            $OutDataTableSubGroupTemplateRight = [Data.DataTable]::new('RightGroup')

            Set-OutDataTable -OutDataTable $OutDataTableSubGroupTemplateRight -Object $Right -SelectedProperties $SelectedRightProperties
        }
    }

    #endregion Prepare Data

    #region Main

    #region Main: Set $QueryParts

    $QueryParts = @{
        'IfSideLine'                       = {
            if ($_Side_Line) {
                _SideScript_
            }
        }.ToString()
        'DataTableFromAny'                 = {
            foreach ($item in $Selected_Side_Properties.GetEnumerator()) {
                $Value = $_Side_Line.($item.Key)

                if ($null -ne $Value) {
                    $_Row_[$item.Value] = $Value
                }
            }
        }.ToString()
        'DataTableFromDataTable'           = {
            foreach ($item in $Selected_Side_Properties.GetEnumerator()) {
                $Value = $_Side_Line[$item.Key]

                if ($Value -isnot [System.DBNull]) {
                    $_Row_[$item.Value] = $Value
                }
            }
        }.ToString()
        'DataTableFromSubGroup'            = {
            if ($_Side_Lines) {
                _SubGroup_
                $_Row_['_Side_Group'] = $OutSubGroup_Side_
            }
        }.ToString()
        'SubGroupFromDataTable'            = {
            $OutSubGroup_Side_ = $OutDataTableSubGroupTemplate_Side_.Clone()
            foreach ($_Side_Line in $_Side_Lines) {
                $RowSubGroup = $OutSubGroup_Side_.Rows.Add()

                _DataTable_
            }
        }.ToString()
        'SubGroupFromPSCustomObject'       = {
            $OutSubGroup_Side_ = @( )

            foreach ($_Side_Line in $_Side_Lines) {
                $RowSubGroup = [ordered]@{ }

                _PSCustomObject_

                $OutSubGroup_Side_ += [PSCustomObject]$RowSubGroup
            }
        }.ToString()
        'PSCustomObjectFromPSCustomObject' = {
            foreach ($item in $Selected_Side_Properties.GetEnumerator()) {
                $_Row_.Add($item.Value, $_Side_Line.($item.Key))
            }
        }.ToString()
        'PSCustomObjectFromAny'            = {
            foreach ($item in $Selected_Side_Properties.GetEnumerator()) {
                $Value = $_Side_Line.($item.Key)

                if ($Value -is [System.DBNull]) {
                    $Value = $null
                }

                $_Row_.Add($item.Value, $Value)
            }
        }.ToString()
        'PSCustomObjectFromSubGroup'       = {
            if ($_Side_Lines) {
                _SubGroup_

                $_Row_.Add('_Side_Group', $OutSubGroup_Side_)
            }
            else {
                $_Row_.Add('_Side_Group', $null)
            }
        }.ToString()
    }

    #endregion Main: Set $QueryParts

    #region Main: Set $Query

    $Query = if ($PassThru) {
        if ($Left -is [Data.DataTable]) {
            $QueryTemp = @{
                Main        = { _SidesScript_ }.ToString()
                SideReplace = '_Row_', 'LeftLine'
            }

            if ($RightMultiMode -eq 'SubGroups') {
                $QueryTemp['SideSubGroupBase'] = $QueryParts['DataTableFromSubGroup']

                if ($Right -is [Data.DataTable]) {
                    $QueryTemp['SideSubGroup'] = $QueryParts['SubGroupFromDataTable']
                }
                else {
                    $QueryTemp['SideSubGroup'] = $QueryParts['SubGroupFromPSCustomObject']
                }
            }
            else {
                $QueryTemp['Right'] = { _DataTable_ }.ToString()
            }

            $QueryTemp
        }
        else {
            # Left is PSCustomObject
            $QueryTemp = @{
                # Edit PSCustomObject
                Main        = {
                    # Add to LeftLine (Rename)
                    foreach ($item in $SelectedLeftProperties.GetEnumerator()) {
                        if ($item.Value -notin $LeftLine.PSObject.Properties.Name) {
                            $LeftLine.PSObject.Properties.Add([Management.Automation.PSNoteProperty]::new($item.Value, $LeftLine.($item.Key)))
                        }
                    }
                    # Remove from LeftLine
                    foreach ($item in $LeftLine.PSObject.Properties.Name) {
                        if ($item -notin $SelectedLeftProperties.Values) {
                            $LeftLine.PSObject.Properties.Remove($item)
                        }
                    }

                    _SidesScript_
                }.ToString()

                SideReplace = '_Row_\.Add([^\r\n]*)', 'LeftLine.PSObject.Properties.Add([Management.Automation.PSNoteProperty]::new$1)'
            }

            if ($RightMultiMode -eq 'SubGroups') {
                $QueryTemp['SideSubGroupBase'] = $QueryParts['PSCustomObjectFromSubGroup']

                if ($Right -is [Data.DataTable]) {
                    $QueryTemp['SideSubGroup'] = $QueryParts['SubGroupFromDataTable']
                }
                else {
                    $QueryTemp['SideSubGroup'] = $QueryParts['SubGroupFromPSCustomObject']
                }
            }
            else {
                $QueryTemp['Right'] = { _PSCustomObject_ }.ToString()
            }

            $QueryTemp
        }
    }
    elseif ($DataTable) {
        $QueryTemp = @{
            Main        = {
                $RowMain = $OutDataTable.Rows.Add()
                _SidesScript_
            }.ToString()

            SideReplace = '_Row_', 'RowMain'
        }

        if ($LeftMultiMode -eq 'SubGroups' -or $RightMultiMode -eq 'SubGroups') {
            $QueryTemp['SideSubGroupBase'] = $QueryParts['DataTableFromSubGroup']
            $QueryTemp['SideSubGroup'] = $QueryParts['SubGroupFromDataTable']
        }

        if ($LeftMultiMode -ne 'SubGroups' -or $RightMultiMode -ne 'SubGroups') {
            $QueryTemp['Side'] = { _DataTable_ }.ToString()
        }

        $QueryTemp
    }
    else {
        # PSCustomObject
        $QueryTemp = @{
            Main        = {
                $RowMain = [ordered]@{ }
                _SidesScript_
                [PSCustomObject]$RowMain
            }.ToString()

            SideReplace = '_Row_', 'RowMain'
        }

        if ($LeftMultiMode -eq 'SubGroups' -or $RightMultiMode -eq 'SubGroups') {
            $QueryTemp['SideSubGroupBase'] = $QueryParts['PSCustomObjectFromSubGroup']
            $QueryTemp['SideSubGroup'] = $QueryParts['SubGroupFromPSCustomObject']
        }

        if ($LeftMultiMode -ne 'SubGroups' -or $RightMultiMode -ne 'SubGroups') {
            $QueryTemp['Side'] = { _PSCustomObject_ }.ToString()
        }

        $QueryTemp
    }

    $Query['Base'] = {
        param (
            #_$Key_,
            $LeftLine,
            $RightLine
        )
    }.ToString()

    #endregion Main: Set $Query

    #region Main: Assemble $Query

    function Invoke-AssembledQuery {
        param (
            $MultiMode,
            $Side,
            $Object
        )

        if ($MultiMode -eq 'SingleOnly') {
            $Query[$Side + 'Enumerable'] = { $_Side_Line = [System.Linq.Enumerable]::SingleOrDefault($_Side_Line) }
        }
        elseif ($MultiMode -eq 'DuplicateLines') {
            $Query[$Side + 'Enumerable'] = { $_Side_Lines = [System.Linq.Enumerable]::DefaultIfEmpty($_Side_Line) }

            $Query['Main'] = {
                foreach ($_Side_Line in $_Side_Lines) {
                    _MainScript_
                }
            }.ToString().Replace('_Side_', $Side).Replace('_MainScript_', $Query['Main'])
        }

        if ($MultiMode -eq 'SubGroups') {
            $Query[$Side + 'Enumerable'] = { $_Side_Lines = if ($_Side_Line.Count) { $_Side_Line } }

            $Query[$Side] = if ($Object -is [Data.DataTable]) {
                ($Query['SideSubGroupBase'] -Replace $Query['SideReplace']).Replace('_SubGroup_', $Query['SideSubGroup']).Replace('_DataTable_', $QueryParts['DataTableFromDataTable']).Replace('_PSCustomObject_', $QueryParts['PSCustomObjectFromAny'])
            }
            else {
                ($Query['SideSubGroupBase'] -Replace $Query['SideReplace']).Replace('_SubGroup_', $Query['SideSubGroup']).Replace('_DataTable_', $QueryParts['DataTableFromAny']).Replace('_PSCustomObject_', $QueryParts['PSCustomObjectFromPSCustomObject'])
            }

            $Query[$Side] = $Query[$Side].Replace('_Row_', 'RowSubGroup')
        }
        else {
            if ($Query[$Side] -or $Query['Side']) {
                if ($null -eq $Query[$Side]) {
                    $Query[$Side] = $Query['Side']
                }

                if ($null -ne $MultiMode -and $Query[$Side] -like '*_DataTable_*') {
                    $Query[$Side] = $QueryParts['IfSideLine'].Replace('_SideScript_', $Query[$Side])
                }

                $Query[$Side] = if ($Object -is [Data.DataTable]) {
                    $Query[$Side].Replace('_DataTable_', $QueryParts['DataTableFromDataTable']).Replace('_PSCustomObject_', $QueryParts['PSCustomObjectFromAny'])
                }
                else {
                    $Query[$Side].Replace('_DataTable_', $QueryParts['DataTableFromAny']).Replace('_PSCustomObject_', $QueryParts['PSCustomObjectFromPSCustomObject'])
                }

                $Query[$Side] = $Query[$Side] -Replace $Query['SideReplace']
            }
        }

        if ($Query[$Side]) {
            $Query[$Side] = $Query[$Side].Replace('_Side_', $Side)
        }

        if ($Query[$Side + 'Enumerable']) {
            $Query[$Side + 'Enumerable'] = $Query[$Side + 'Enumerable'].ToString().Replace('_Side_', $Side)
        }
    }

    Invoke-AssembledQuery -MultiMode $LeftMultiMode -Side 'Left' -Object $Left
    Invoke-AssembledQuery -MultiMode $RightMultiMode -Side 'Right' -Object $Right

    if ($AddKey) {
        $KeyScript = {
            $RowMain._Key_ = $Key

            _SidesScript_
        }.ToString()

        $KeyScript = if ($DataTable) {
            $KeyScript.Replace('._Key_', '[$AddKey]')
        }
        else {
            $KeyScript.Replace('_Key_', '$AddKey')
        }

        $Query['Main'] = $Query['Main'].Replace('_SidesScript_', $KeyScript)
    }

    $Query['Main'] = $Query['Main'].Replace('_SidesScript_', $Query['Left'] + $Query['Right'])

    if ($Type -eq 'OnlyIfInBoth') {
        [System.Func[System.Object, System.Object, System.Object]]$Query = [Scriptblock]::Create($Query['Base'] + $Query['Main'])
    }
    elseif ($Type -eq 'AllInLeft') {
        [System.Func[System.Object, [System.Collections.Generic.IEnumerable[System.Object]], System.Object]]$Query = [Scriptblock]::Create($Query['Base'] + $Query['RightEnumerable'] + $Query['Main'])
    }
    # elseif ($Type -eq 'AllInBoth') {
    #     [System.Func[System.Object, [System.Collections.Generic.IEnumerable[System.Object]], [System.Collections.Generic.IEnumerable[System.Object]], System.Object]]$Query = [Scriptblock]::Create($Query['Base'].Replace('#_$Key_', '$Key') + $Query['LeftEnumerable'] + "`n" + $Query['RightEnumerable'] + $Query['Main'])
    # }

    #endregion Main: Assemble $Query

    #endregion Main

    #region Execute

    if ($Left -is [Data.DataTable]) {
        $LeftNew = [DataTableExtensions]::AsEnumerable($Left)
    }
    elseif ($Left -is [PSCustomObject] -or $Left -is [System.Collections.ArrayList]) {
        $LeftNew = @( $Left )
    }
    else {
        $LeftNew = $Left
    }

    if ($Right -is [Data.DataTable]) {
        $RightNew = [DataTableExtensions]::AsEnumerable($Right)
    }
    elseif ($Right -is [PSCustomObject] -or $Right -is [System.Collections.ArrayList]) {
        $RightNew = @( $Right )
    }
    else {
        $RightNew = $Right
    }

    try {
        $Result = if ($Type -eq 'OnlyIfInBoth') {
            [System.Linq.Enumerable]::ToArray(
                [System.Linq.Enumerable]::Join($LeftNew, $RightNew, $LeftJoinScript, $RightJoinScript, $query, $Comparer)
            )
        }
        # elseif ($Type -eq 'AllInBoth') {
        #     [System.Linq.Enumerable]::ToArray(
        #         [MoreLinq.MoreEnumerable]::FullGroupJoin($LeftNew, $RightNew, $LeftJoinScript, $RightJoinScript, $query, $Comparer)
        #     )
        # }
        else {
            [System.Linq.Enumerable]::ToArray(
                [System.Linq.Enumerable]::GroupJoin($LeftNew, $RightNew, $LeftJoinScript, $RightJoinScript, $query, $Comparer)
            )
        }

        if ($PassThru) {
            , $Left
        }
        elseif ($DataTable) {
            , $OutDataTable
        }
        elseif ($LeftMultiMode -eq 'DuplicateLines' -or $RightMultiMode -eq 'DuplicateLines') {
            $Result.ForEach({ $_ })
        }
        else {
            $Result
        }
    }
    catch {
        $PSCmdlet.ThrowTerminatingError($_)
    }

    #endregion Execute
}

function ConvertTo-MaskLength {
    <#
        .SYNOPSIS
            Convert from a netmask to the masklength

        .EXAMPLE
            PS> ConvertTo-MaskLength -NetMask 255.255.255.0
    #>

    param (
        # The subnet mask to convert.
        [Parameter(Mandatory)]
        [AllowEmptyString()]
        [string]$Netmask
    )

    if ([uri]::CheckHostName($Netmask) -ne 'IPv4') {
        return $null
    }

    $maskLength = 0

    foreach ($octet in $Netmask.Split('.')) {
        0..7 | ForEach-Object {
            if (($octet - [System.Math]::Pow(2, (7 - $_))) -ge 0) {
                $octet = $octet - [System.Math]::Pow(2, (7 - $_))
                $maskLength++
            }
        }
    }

    return $maskLength
}

function Get-SubnetFromCidr {
    [CmdletBinding()]
    [OutputType([System.Net.IPAddress])]
    param (
        # The Classless inter-domain routing notation for the subnet mask
        [Parameter()]
        [Int32]
        $CIDR
    )

    [double] $_zeroSubnetMaskBits = [System.Math]::pow(2, 31)
    [double] $_subnetMaskBits = 0

    for ( $i = 0; $i -lt $CIDR; $i++ ) {
        $_subnetMaskBits = $_subnetMaskBits + $_zeroSubnetMaskBits
        $_zeroSubnetMaskBits = $_zeroSubnetMaskBits / 2
    }

    ( [IPAddress]$_subnetMaskBits ).IPAddressToString -as [IPAddress]
}

function ConvertTo-SortableVersionString {
    param (
        [Parameter(Mandatory)]
        [AllowEmptyString()]
        [string]$VersionString
    )

    if (-not [string]::IsNullOrWhiteSpace($VersionString)) {
        (
            ($VersionString -replace '\s\(7-Mode\)' -replace '[.]*P', '.0.9' -replace '[.]*RC', '.0.0.7' -replace '[.]*D', '.0.0.0.5' -replace '[.]*X', '.0.0.0.0.3' -replace '[.]*L', '.0.0.0.0.0.2' -replace '[.]*[a-z]+', '.0.0.0.0.0.0.1') -split '[.]' | ForEach-Object {
                '{0:D5}' -f [int]$_
            }
        ) -join '.'
    }
}

function ConvertFrom-JobScheduleDescription {
    param (
        [Parameter(Mandatory)]
        [AllowEmptyString()]
        [string]$JobScheduleDescription
    )

    if ([string]::IsNullOrWhiteSpace($JobScheduleDescription)) {
        return $null
    }

    $cronFieldAliases = @{
        'Month'     = @{
            'Jan' = 0
            'Feb' = 1
            'Mar' = 2
            'Apr' = 3
            'May' = 4
            'Jun' = 5
            'Jul' = 6
            'Aug' = 7
            'Sep' = 8
            'Oct' = 9
            'Nov' = 10
            'Dec' = 11
        };
        'DayOfWeek' = @{
            'Sun' = 0
            'Mon' = 1
            'Tue' = 2
            'Wed' = 3
            'Thu' = 4
            'Fri' = 5
            'Sat' = 6
        };
    }

    $outputObj = [pscustomobject]@{
        JobScheduleCronMonth       = New-Object -TypeName System.Collections.Generic.List[System.Int32]
        JobScheduleCronDayOfWeek   = New-Object -TypeName System.Collections.Generic.List[System.Int32]
        JobScheduleCronDay         = New-Object -TypeName System.Collections.Generic.List[System.Int32]
        JobScheduleCronHour        = New-Object -TypeName System.Collections.Generic.List[System.Int32]
        JobScheduleCronMinute      = New-Object -TypeName System.Collections.Generic.List[System.Int32]
        JobScheduleIntervalDays    = $null
        JobScheduleIntervalHours   = $null
        JobScheduleIntervalMinutes = $null
        JobScheduleIntervalSeconds = $null
        JobScheduleDescription     = $JobScheduleDescription
    }

    switch -Wildcard ($JobScheduleDescription) {
        'Every*' {
            $intervalRegex = '^Every\s(?:(?<Days>\d+)d)?(?:(?<Hours>\d+)h)?(?:(?<Minutes>\d+)m)?(?:(?<Seconds>\d+)s)?'

            if ($JobScheduleDescription -match $intervalRegex) {
                $outputObj.JobScheduleIntervalDays = $Matches['Days']
                $outputObj.JobScheduleIntervalHours = $Matches['Hours']
                $outputObj.JobScheduleIntervalMinutes = $Matches['Minutes']
                $outputObj.JobScheduleIntervalSeconds = $Matches['Seconds']

                $Matches.Clear()
            }
        }
        default {
            $dayAndMonthEntry, $hourAndMinuteEntry = $JobScheduleDescription -split '[@]'

            foreach ($entry in ($dayAndMonthEntry -split '[,|\s]')) {
                if ($entry -like '*-*') {
                    $first, $last = $entry -split '-'

                    if ([int]::TryParse($first, [ref]$null)) {
                        $first .. $last | ForEach-Object {
                            $outputObj.JobScheduleCronDay.Add($_)
                        }
                    }

                    if ($cronFieldAliases['Month'].ContainsKey($first)) {
                        $cronFieldAliases['Month'][$first] .. $cronFieldAliases['Month'][$last] | ForEach-Object {
                            $outputObj.JobScheduleCronMonth.Add($_)
                        }
                    }
                    elseif ($cronFieldAliases['DayOfWeek'].ContainsKey($first)) {
                        $cronFieldAliases['DayOfWeek'][$first] .. $cronFieldAliases['DayOfWeek'][$last] | ForEach-Object {
                            $outputObj.JobScheduleCronDayOfWeek.Add($_)
                        }
                    }
                }
                else {
                    if ([int]::TryParse($entry, [ref]$null)) {
                        $outputObj.JobScheduleCronDay.Add($entry)
                    }
                    else {
                        if ($cronFieldAliases['Month'].ContainsKey($entry)) {
                            $outputObj.JobScheduleCronMonth.Add($cronFieldAliases['Month'][$entry])
                        }
                        elseif ($cronFieldAliases['DayOfWeek'].ContainsKey($entry)) {
                            $outputObj.JobScheduleCronDayOfWeek.Add($cronFieldAliases['DayOfWeek'][$entry])
                        }
                    }
                }
            }

            foreach ($entry in ($hourAndMinuteEntry -split ',')) {
                $hour, $minute = $entry -split ':'

                if ([int]::TryParse($hour, [ref]$null) -and -not $outputObj.JobScheduleCronHour.Contains($hour)) {
                    $outputObj.JobScheduleCronHour.Add($hour)
                }

                if ([int]::TryParse($minute, [ref]$null) -and -not $outputObj.JobScheduleCronMinute.Contains($minute)) {
                    $outputObj.JobScheduleCronMinute.Add($minute)
                }
            }
        }
    }

    $outputObj
}

# SIG # Begin signature block
# MIIy0QYJKoZIhvcNAQcCoIIywjCCMr4CAQExDzANBglghkgBZQMEAgEFADB5Bgor
# BgEEAYI3AgEEoGswaTA0BgorBgEEAYI3AgEeMCYCAwEAAAQQH8w7YFlLCE63JNLG
# KX7zUQIBAAIBAAIBAAIBAAIBADAxMA0GCWCGSAFlAwQCAQUABCBMadKLCtSD2t8K
# +wXNeAqalyupae+bI/0njACy5sR2x6CCFxcwggQ+MIIDJqADAgECAgRKU4woMA0G
# CSqGSIb3DQEBCwUAMIG+MQswCQYDVQQGEwJVUzEWMBQGA1UEChMNRW50cnVzdCwg
# SW5jLjEoMCYGA1UECxMfU2VlIHd3dy5lbnRydXN0Lm5ldC9sZWdhbC10ZXJtczE5
# MDcGA1UECxMwKGMpIDIwMDkgRW50cnVzdCwgSW5jLiAtIGZvciBhdXRob3JpemVk
# IHVzZSBvbmx5MTIwMAYDVQQDEylFbnRydXN0IFJvb3QgQ2VydGlmaWNhdGlvbiBB
# dXRob3JpdHkgLSBHMjAeFw0wOTA3MDcxNzI1NTRaFw0zMDEyMDcxNzU1NTRaMIG+
# MQswCQYDVQQGEwJVUzEWMBQGA1UEChMNRW50cnVzdCwgSW5jLjEoMCYGA1UECxMf
# U2VlIHd3dy5lbnRydXN0Lm5ldC9sZWdhbC10ZXJtczE5MDcGA1UECxMwKGMpIDIw
# MDkgRW50cnVzdCwgSW5jLiAtIGZvciBhdXRob3JpemVkIHVzZSBvbmx5MTIwMAYD
# VQQDEylFbnRydXN0IFJvb3QgQ2VydGlmaWNhdGlvbiBBdXRob3JpdHkgLSBHMjCC
# ASIwDQYJKoZIhvcNAQEBBQADggEPADCCAQoCggEBALqEtnLbngxr4pnpMAGnduoy
# uJVBGsnaYU5Ycs/+9oJ5v3NhBgqlJ9izX9NFThxy1k4y8nKKD/eDGdBqgIAARR6w
# x+eavxJXJxyjaC8Kh71qaw5eZfMcd9XUhY1wIbSzMueLotWGOQKxuNJHzuTJScQ7
# p977VH1XvvDobsJ5sjoLVeJQmBYyE1wveFbBwpSz8lrkJ5qfJNfG7NCbJYLjzMLE
# RcWMl3oGayoRn6kKbkg7b9vUERlC948Hv/VTX5w+9Bcs5mmsTjJMYnfqt+jluzS8
# GYuunFHnt361U7EzIuVtz3A8Gvrim2e2g/SNpa9iTE3gWKxkNBID+LaNlGMkpHEC
# AwEAAaNCMEAwDgYDVR0PAQH/BAQDAgEGMA8GA1UdEwEB/wQFMAMBAf8wHQYDVR0O
# BBYEFGpyJnrQHu995ztpUdRsjZ+QEmarMA0GCSqGSIb3DQEBCwUAA4IBAQB5nx2W
# xrZ5PyKNh9OHAwRgamuaLlmJcxGsQ9H1E/+NOSvA8r1PcIypL+oXxAtUntQblpgz
# PKitYqIAdqtZaW4GHX7EuUSNmK8S1GHbChlGR/Pr92PBQAVApdK39LWaNr+piHaI
# BFUEK5yHfxo3PH4tpRrY1Ileyr2sPWzYba/V83YPzTuIOCKdbJOaxD2/ghtlP6YP
# Xar85bIVyrWtxrw90ITo6gZysE05Mni/PhGcC6SdmiHz8JsLMHjbwdyHQ/68Y5rK
# xcIcyceN/zsSWAjmtj3seixO+4OWzgw8aYdUc6RzwpP/URCsFVQB2PwFsYmhf3SD
# mknX3E57ikhvi0X2MIIF3zCCBMegAwIBAgIQTkDkN1Tt5owAAAAAUdOUfzANBgkq
# hkiG9w0BAQsFADCBvjELMAkGA1UEBhMCVVMxFjAUBgNVBAoTDUVudHJ1c3QsIElu
# Yy4xKDAmBgNVBAsTH1NlZSB3d3cuZW50cnVzdC5uZXQvbGVnYWwtdGVybXMxOTA3
# BgNVBAsTMChjKSAyMDA5IEVudHJ1c3QsIEluYy4gLSBmb3IgYXV0aG9yaXplZCB1
# c2Ugb25seTEyMDAGA1UEAxMpRW50cnVzdCBSb290IENlcnRpZmljYXRpb24gQXV0
# aG9yaXR5IC0gRzIwHhcNMjEwNTA3MTU0MzQ1WhcNMzAxMTA3MTYxMzQ1WjBpMQsw
# CQYDVQQGEwJVUzEWMBQGA1UECgwNRW50cnVzdCwgSW5jLjFCMEAGA1UEAww5RW50
# cnVzdCBDb2RlIFNpZ25pbmcgUm9vdCBDZXJ0aWZpY2F0aW9uIEF1dGhvcml0eSAt
# IENTQlIxMIICIjANBgkqhkiG9w0BAQEFAAOCAg8AMIICCgKCAgEAp4GP9xRFtmJD
# 8tiu0yVeSE9Rv8V9n1AcNdHWfmEqlBltJ0akphpd91RRaoAixqhmdU1Ug8leaBur
# 9ltksK2tIL1U70ZrbQLnRa519o6KuTIui7h3HFJNeYhfpToYyVAslyctv9oAfWN/
# 7zLsRodj25qfw1ohNnv5m9XKoG5yLPzh8Z5wTQhWFW+Qq/tIurnXwYJ4hWUuf7XJ
# wOIUtzcRQQbiiuCo9uV+tngFAcNg7U8HQS4KE0njkJt/3b36rL9kUdFcm7T1XOdc
# /zubpaAa130JssK3/24cvMh95ukr/HKzFOlKVRKEnEQldR32KvBPpSA9aCXrYZd8
# D+W2PfOuw8ERvBuOzOBHMF5CAIZx41isBsplH3uUpktXZwx+Xq14Z1tV417rx9js
# TG6Gy/Pc+J+HqnJYEg99pvj4Qjk7PCzkMk1JjODhAMI4oJz6hD5B3G5WrsYaW/Rn
# aAUBzRu/roe8nVP2Lui2a+SZ3sVPh1io0mUeyB/Vcm7uWRxXOwlyndfKt5DGzXtF
# kpFCA0x9P8ryqrjCDobzEJ9GLqRmhmhaaBhwKTgRgGBrikOjc2zjs2s3/+adZwGS
# ht8vSNH7UGDVXP4h0wFCY/7vcLQXwI+o7tPBS18S6v39Lg6HRGDjqfTCGKPj/c4M
# hCIN86d42pPz2zjPuS8zxv8HPF6+RdMCAwEAAaOCASswggEnMA4GA1UdDwEB/wQE
# AwIBhjASBgNVHRMBAf8ECDAGAQH/AgEBMB0GA1UdJQQWMBQGCCsGAQUFBwMDBggr
# BgEFBQcDCDA7BgNVHSAENDAyMDAGBFUdIAAwKDAmBggrBgEFBQcCARYaaHR0cDov
# L3d3dy5lbnRydXN0Lm5ldC9ycGEwMwYIKwYBBQUHAQEEJzAlMCMGCCsGAQUFBzAB
# hhdodHRwOi8vb2NzcC5lbnRydXN0Lm5ldDAwBgNVHR8EKTAnMCWgI6Ahhh9odHRw
# Oi8vY3JsLmVudHJ1c3QubmV0L2cyY2EuY3JsMB0GA1UdDgQWBBSCutY9l86fz3Ho
# kjev/bO1aTVXzzAfBgNVHSMEGDAWgBRqciZ60B7vfec7aVHUbI2fkBJmqzANBgkq
# hkiG9w0BAQsFAAOCAQEAH15BBLaDcCRTLFVzHWU6wOy0ewSYXlk4EwmkWZRCXlC/
# T2xuJSCQk1hADfUZtGLuJF7CAVgVAh0QCW+o1PuSfjc4Pi8UfY8dQzZks2YTXxTM
# pXH3WyFLxpe+3JX8cH0RHNMh3dAkOSnF/goapc97ee46b97cv+kR3RaDCNMsjX9N
# qBR5LwVhUjjrYPMUaH3LsoqtwJRc5CYOLIrdRsPO5FZRxVbjhbhNm0VyiwfxivtJ
# uF/R8paBXWlSJPEII9LWIw/ri9d+i8GTa/rxYntY6VCbl24XiA3hxkOY14FhtoWd
# R+yxnq4/IDtDndiiHODUfAjCr3YG+GJmerb3+sivNTCCBmcwggRPoAMCAQICEEbM
# 41QNEgkq80rzt39+/oswDQYJKoZIhvcNAQELBQAwYzELMAkGA1UEBhMCVVMxFjAU
# BgNVBAoTDUVudHJ1c3QsIEluYy4xPDA6BgNVBAMTM0VudHJ1c3QgRXh0ZW5kZWQg
# VmFsaWRhdGlvbiBDb2RlIFNpZ25pbmcgQ0EgLSBFVkNTMjAeFw0yNDEwMjMxNTIy
# MTRaFw0yNTExMDkxNTIyMTNaMIHCMQswCQYDVQQGEwJVUzETMBEGA1UECBMKQ2Fs
# aWZvcm5pYTERMA8GA1UEBxMIU2FuIEpvc2UxEzARBgsrBgEEAYI3PAIBAxMCVVMx
# GTAXBgsrBgEEAYI3PAIBAhMIRGVsYXdhcmUxFDASBgNVBAoTC05ldEFwcCwgSW5j
# MR0wGwYDVQQPExRQcml2YXRlIE9yZ2FuaXphdGlvbjEQMA4GA1UEBRMHMzQ1Mjc0
# MzEUMBIGA1UEAxMLTmV0QXBwLCBJbmMwggGiMA0GCSqGSIb3DQEBAQUAA4IBjwAw
# ggGKAoIBgQCIHyusYBqN6d8MjDVGb9yKy/jxOJi5Ry9vtMsEpSe0eX1NwlzjdPpo
# R+kSTTj0HXwjn48v4UtO3lOWVVdqDuAuECTvG+n31c1RmaB4qS0qg9VEDsUP0/gr
# R4GlmEnGRIeDNWnRIV4oHGTqA1NjNU3esZYLQSV9j5DSzh0W+vyq10stDZiLBNai
# KoBzi5fs+C7QWW+xd+fSFV3Z+zIfE/jHFa4ZKP0n8Bva/5XKZSg6XVBN09oY/EF2
# 67wlz2018jiSwKZi++kCqTIpp9j7dKP5x+0/bK4GlGeivT/NObhjDN8e0Iu5rC2L
# tCalpr5CBMzjKmwfV8mVd9L/qPKk8QZxBdBrUpA1qHDCHbd9BRcd2zT8doIbM0vT
# dHnT9pRPZYws7BfT33KMoV7nShbrNViWhkP3i3UkQv7W9RptiZwfEDpt4Lxn0Uw8
# Zf94fbwMQaV3A28ItUbueLN23+k41N0jqdFTbpCaqQpUT/NxlRHECtn0Lp6kjZTy
# wKF7FqH+jpcCAwEAAaOCATUwggExMAwGA1UdEwEB/wQCMAAwHQYDVR0OBBYEFGzw
# 5EcKP/wp/cuZ0BIs8j6slZQcMB8GA1UdIwQYMBaAFM6JT4JRqhWihGLKMSNh0mH7
# +P54MA4GA1UdDwEB/wQEAwIHgDATBgNVHSUEDDAKBggrBgEFBQcDAzBnBggrBgEF
# BQcBAQRbMFkwIwYIKwYBBQUHMAGGF2h0dHA6Ly9vY3NwLmVudHJ1c3QubmV0MDIG
# CCsGAQUFBzAChiZodHRwOi8vYWlhLmVudHJ1c3QubmV0L2V2Y3MyLWNoYWluLnA3
# YzAxBgNVHR8EKjAoMCagJKAihiBodHRwOi8vY3JsLmVudHJ1c3QubmV0L2V2Y3My
# LmNybDAgBgNVHSAEGTAXMAcGBWeBDAEDMAwGCmCGSAGG+mwKAQIwDQYJKoZIhvcN
# AQELBQADggIBADAwfASY7H1rvIVgDB4LCw7V6xYShLM1XBV0YV8mgn50EVUgkXRr
# qwdn9JgVJvPU6jQ30ab9bb2505N/e1hWQN+Skw21Kbixw3mJ2lZNjY9CvwPUAc4B
# 9xXFEl/w7w1GuATgHO77YCpe0QTbYpbuSTXA0YBs1LlzmuoMxCD+E82srnED8Crg
# m4IHe71kpqqbsIYDnxujU+w/+/CAijPCuxXD2Ff1D7Ms97n9OFvLZHU2e+7xTP66
# RFXF81uCNcLYZSAMiXpjFCiXz74yNHtqZYecnNKvA3hUnvE0BH2IbJFpFiwQO9H2
# rSxdf6qTJANWvCTm7AkDMPhCneufSIFeGHyINH5sKNc+MgsNawZwy2Bfld2YE+ii
# +RyoUgQvl+OZu58dcPYvbgSkc+nE7Xlq/nzE4qNB83ZUuo4MQuGgZ1ZdROSgO6BD
# lCDa1Mr+viR84DtT6dwMYDf0PYWe0LoDYBVLRknZHpSNCMULURCTENBJdJo1mrOn
# hmmm5ZJ3Hg/CmyrujK67YvNFUsQnpLKYEut7ntmJM5ptEPm7Ql5zzaOp3o68a/Q6
# 0+s6p4NwZQYo4ekHk+DM4oaPiv8K+dEPxvppQCSoSPG0S59A8oFBXzoaidOTC06j
# xfGFkPFtOEIO269lK0NCIgr2lJvlUdlUPSnM2eWkEkslz42ShJgON0SaMIIGgzCC
# BGugAwIBAgIQNa+3e500H2r8j4RGqzE1KzANBgkqhkiG9w0BAQ0FADBpMQswCQYD
# VQQGEwJVUzEWMBQGA1UECgwNRW50cnVzdCwgSW5jLjFCMEAGA1UEAww5RW50cnVz
# dCBDb2RlIFNpZ25pbmcgUm9vdCBDZXJ0aWZpY2F0aW9uIEF1dGhvcml0eSAtIENT
# QlIxMB4XDTIxMDUwNzE5MTk1MloXDTQwMTIyOTIzNTkwMFowYzELMAkGA1UEBhMC
# VVMxFjAUBgNVBAoTDUVudHJ1c3QsIEluYy4xPDA6BgNVBAMTM0VudHJ1c3QgRXh0
# ZW5kZWQgVmFsaWRhdGlvbiBDb2RlIFNpZ25pbmcgQ0EgLSBFVkNTMjCCAiIwDQYJ
# KoZIhvcNAQEBBQADggIPADCCAgoCggIBAL69pznJpX3sXWXx9Cuph9DnrRrFGjsY
# zuGhUY1y+s5YH1y4JEIPRtUxl9BKTeObMMm6l6ic/kU2zyeA53u4bsEkt9+ndNyF
# 8qMkWEXMlJQ7AuvEjXxG9VxmguOkwdMfrG4MUyMO1Dr62kLxg1RfNTJW8rV4m1cA
# SB6pYWEnDnMDQ7bWcJL71IWaMMaz5ppeS+8dKthmqxZG/wvYD6aJSgJRV0E8QThO
# l8dRMm1njmahXk2fNSKv1Wq3f0BfaDXMafrxBfDqhabqMoXLwcHKg2lFSQbcCWy6
# SWUZjPm3NyeMZJ414+Xs5wegnahyvG+FOiymFk49nM8I5oL1RH0owL2JrWwv3C94
# eRHXHHBL3Z0ITF4u+o29p91j9n/wUjGEbjrY2VyFRJ5jBmnQhlh4iZuHu1gcpChs
# xv5pCpwerBFgal7JaWUu7UMtafF4tzstNfKqT+If4wFvkEaq1agNBFegtKzjbb2d
# GyiAJ0bH2qpnlfHRh3vHyCXphAyPiTbSvjPhhcAz1aA8GYuvOPLlk4C/xsOre5PE
# PZ257kV2wNRobzBePLQ2+ddFQuASBoDbpSH85wV6KI20jmB798i1SkesFGaXoFpp
# cjFXa1OEzWG6cwcVcDt7AfynP4wtPYeM+wjX5S8Xg36Cq08J8inhflV3ZZQFHVnU
# Ct2TfuMUXeK7AgMBAAGjggErMIIBJzASBgNVHRMBAf8ECDAGAQH/AgEAMB0GA1Ud
# DgQWBBTOiU+CUaoVooRiyjEjYdJh+/j+eDAfBgNVHSMEGDAWgBSCutY9l86fz3Ho
# kjev/bO1aTVXzzAzBggrBgEFBQcBAQQnMCUwIwYIKwYBBQUHMAGGF2h0dHA6Ly9v
# Y3NwLmVudHJ1c3QubmV0MDEGA1UdHwQqMCgwJqAkoCKGIGh0dHA6Ly9jcmwuZW50
# cnVzdC5uZXQvY3NicjEuY3JsMA4GA1UdDwEB/wQEAwIBhjATBgNVHSUEDDAKBggr
# BgEFBQcDAzBEBgNVHSAEPTA7MDAGBFUdIAAwKDAmBggrBgEFBQcCARYaaHR0cDov
# L3d3dy5lbnRydXN0Lm5ldC9ycGEwBwYFZ4EMAQMwDQYJKoZIhvcNAQENBQADggIB
# AD4AVLgq849mr2EWxFiTZPRBi2RVjRs1M6GbkdirRsqrX7y+fnDk0tcHqJYH14bR
# VwoI0NB4Tfgq37IE85rh13zwwQB6wUCh34qMt8u0HQFh8piapt24gwXKqSwW3Jwt
# Dv6nl+RQqZeVwUsqjFHjxALga3w1TVO8S5QTi1MYFl6mCqe4NMFssess5DF9DCzG
# fOGkVugtdtWyE3XqgwCuAHfGb6k97mMUgVAW/FtPEhkOWw+N6kvOBkyJS64gzI5H
# pnXWZe4vMOhdNI8fgk1cQqbyFExQIJwJonQkXDnYiTKFPK+M5Wqe5gQ6pRP/qh3N
# R0suAgW0ao/rhU+B7wrbfZ8pj6XCP1I4UkGVO7w+W1QwQiMJY95QjYk1RfqruA+P
# oq17ehGT8Y8ohHtoeUdq6GQpTR/0HS9tHsiUhjzTWpl6a3yrNfcrOUtPuT8Wku8p
# jI2rrAEazHFEOctAPiASzghw40f+3IDXCADRC2rqIbV5ZhfpaqpW3c0VeLEDwBSt
# PkcYde0KU0syk83/gLGQ1hPl5EF4Iu1BguUO37DOlSFF5osB0xn39CtVrNlWc2MQ
# 4LigbctUlpigmSFRBqqmDDorY8t52kO50hLM3o9VeukJ8+Ka0yXBezaS2uDlUmfN
# 4+ZUCqWd1HOj0y9dBmSFA3d/YNjCvHTJlZFot7d+YRl1MYIbEDCCGwwCAQEwdzBj
# MQswCQYDVQQGEwJVUzEWMBQGA1UEChMNRW50cnVzdCwgSW5jLjE8MDoGA1UEAxMz
# RW50cnVzdCBFeHRlbmRlZCBWYWxpZGF0aW9uIENvZGUgU2lnbmluZyBDQSAtIEVW
# Q1MyAhBGzONUDRIJKvNK87d/fv6LMA0GCWCGSAFlAwQCAQUAoIGEMBgGCisGAQQB
# gjcCAQwxCjAIoAKAAKECgAAwGQYJKoZIhvcNAQkDMQwGCisGAQQBgjcCAQQwHAYK
# KwYBBAGCNwIBCzEOMAwGCisGAQQBgjcCARUwLwYJKoZIhvcNAQkEMSIEILVLbU9y
# oHWdBUPKnCAylDdPyweBtfVaSri8Y7vF10hkMA0GCSqGSIb3DQEBAQUABIIBgH14
# zAuVL5fGLsv72GkwbPtxkrUMnHjofxOpG8jFzq7x1xSgDqfpvGgFsbglgEH9cZVY
# iPExxN3xQPgjODCcFTGjfwRITKGSDangwHxxJOPrYB1mRjTIs3ojdHJyZ3MW7yEP
# sq8q7o2Pbd0Wn44kx/Yg/4W+iuVI5nlROMQqZoNQ4zivahb7eSplDPrlrcYde5YH
# 4DmZpzY/YcGwCIQeuZkuyqtDAFcuhdyAVPJgxJ+ZvEylzqsQZPoF5vOLXv65Gzf8
# zN862r6D6SliDKRIbwSmcagCn1iy2uTj25/qvExdi7y5IYcU+uc1u0XQErujC+cC
# x96PoNo9xGmcBEqrwsNtRmlo7Tmy5mlpUaBadvGAMtgvWUmDquLPSEIScLdiGC4i
# 385hg7C+dUAWRJXqTfgpwODhKZ7skNIP5nLmdjXMEUos4mrWVNX8H7FO7/p5TpP4
# 8uU3kYAkEui5lL1Krz1RaMTGWtt4Fgmw5YmDTxK5YFNC4eIZbLDXDQOAoVKwbKGC
# GGMwghhfBgorBgEEAYI3AwMBMYIYTzCCGEsGCSqGSIb3DQEHAqCCGDwwghg4AgED
# MQ0wCwYJYIZIAWUDBAIDMIHzBgsqhkiG9w0BCRABBKCB4wSB4DCB3QIBAQYKYIZI
# AYb6bAoDBTAxMA0GCWCGSAFlAwQCAQUABCAG2O6clp5poISUgtjprbhjswVdbh5K
# y2RsVyIfxaiNVgIIWwATRQc8W6AYDzIwMjUwMTA5MDMwNjI5WjADAgEBoHmkdzB1
# MQswCQYDVQQGEwJDQTEQMA4GA1UECBMHT250YXJpbzEPMA0GA1UEBxMGT3R0YXdh
# MRYwFAYDVQQKEw1FbnRydXN0LCBJbmMuMSswKQYDVQQDEyJFbnRydXN0IFRpbWVz
# dGFtcCBBdXRob3JpdHkgLSBUU0EyoIITDjCCBd8wggTHoAMCAQICEE5A5DdU7eaM
# AAAAAFHTlH8wDQYJKoZIhvcNAQELBQAwgb4xCzAJBgNVBAYTAlVTMRYwFAYDVQQK
# Ew1FbnRydXN0LCBJbmMuMSgwJgYDVQQLEx9TZWUgd3d3LmVudHJ1c3QubmV0L2xl
# Z2FsLXRlcm1zMTkwNwYDVQQLEzAoYykgMjAwOSBFbnRydXN0LCBJbmMuIC0gZm9y
# IGF1dGhvcml6ZWQgdXNlIG9ubHkxMjAwBgNVBAMTKUVudHJ1c3QgUm9vdCBDZXJ0
# aWZpY2F0aW9uIEF1dGhvcml0eSAtIEcyMB4XDTIxMDUwNzE1NDM0NVoXDTMwMTEw
# NzE2MTM0NVowaTELMAkGA1UEBhMCVVMxFjAUBgNVBAoMDUVudHJ1c3QsIEluYy4x
# QjBABgNVBAMMOUVudHJ1c3QgQ29kZSBTaWduaW5nIFJvb3QgQ2VydGlmaWNhdGlv
# biBBdXRob3JpdHkgLSBDU0JSMTCCAiIwDQYJKoZIhvcNAQEBBQADggIPADCCAgoC
# ggIBAKeBj/cURbZiQ/LYrtMlXkhPUb/FfZ9QHDXR1n5hKpQZbSdGpKYaXfdUUWqA
# IsaoZnVNVIPJXmgbq/ZbZLCtrSC9VO9Ga20C50WudfaOirkyLou4dxxSTXmIX6U6
# GMlQLJcnLb/aAH1jf+8y7EaHY9uan8NaITZ7+ZvVyqBuciz84fGecE0IVhVvkKv7
# SLq518GCeIVlLn+1ycDiFLc3EUEG4orgqPblfrZ4BQHDYO1PB0EuChNJ45Cbf929
# +qy/ZFHRXJu09VznXP87m6WgGtd9CbLCt/9uHLzIfebpK/xysxTpSlUShJxEJXUd
# 9irwT6UgPWgl62GXfA/ltj3zrsPBEbwbjszgRzBeQgCGceNYrAbKZR97lKZLV2cM
# fl6teGdbVeNe68fY7Exuhsvz3Pifh6pyWBIPfab4+EI5Ozws5DJNSYzg4QDCOKCc
# +oQ+QdxuVq7GGlv0Z2gFAc0bv66HvJ1T9i7otmvkmd7FT4dYqNJlHsgf1XJu7lkc
# VzsJcp3XyreQxs17RZKRQgNMfT/K8qq4wg6G8xCfRi6kZoZoWmgYcCk4EYBga4pD
# o3Ns47NrN//mnWcBkobfL0jR+1Bg1Vz+IdMBQmP+73C0F8CPqO7TwUtfEur9/S4O
# h0Rg46n0whij4/3ODIQiDfOneNqT89s4z7kvM8b/BzxevkXTAgMBAAGjggErMIIB
# JzAOBgNVHQ8BAf8EBAMCAYYwEgYDVR0TAQH/BAgwBgEB/wIBATAdBgNVHSUEFjAU
# BggrBgEFBQcDAwYIKwYBBQUHAwgwOwYDVR0gBDQwMjAwBgRVHSAAMCgwJgYIKwYB
# BQUHAgEWGmh0dHA6Ly93d3cuZW50cnVzdC5uZXQvcnBhMDMGCCsGAQUFBwEBBCcw
# JTAjBggrBgEFBQcwAYYXaHR0cDovL29jc3AuZW50cnVzdC5uZXQwMAYDVR0fBCkw
# JzAloCOgIYYfaHR0cDovL2NybC5lbnRydXN0Lm5ldC9nMmNhLmNybDAdBgNVHQ4E
# FgQUgrrWPZfOn89x6JI3r/2ztWk1V88wHwYDVR0jBBgwFoAUanImetAe733nO2lR
# 1GyNn5ASZqswDQYJKoZIhvcNAQELBQADggEBAB9eQQS2g3AkUyxVcx1lOsDstHsE
# mF5ZOBMJpFmUQl5Qv09sbiUgkJNYQA31GbRi7iRewgFYFQIdEAlvqNT7kn43OD4v
# FH2PHUM2ZLNmE18UzKVx91shS8aXvtyV/HB9ERzTId3QJDkpxf4KGqXPe3nuOm/e
# 3L/pEd0WgwjTLI1/TagUeS8FYVI462DzFGh9y7KKrcCUXOQmDiyK3UbDzuRWUcVW
# 44W4TZtFcosH8Yr7Sbhf0fKWgV1pUiTxCCPS1iMP64vXfovBk2v68WJ7WOlQm5du
# F4gN4cZDmNeBYbaFnUfssZ6uPyA7Q53Yohzg1HwIwq92BvhiZnq29/rIrzUwggZv
# MIIEV6ADAgECAhAlvCvzKcoQfx6puoiF1J07MA0GCSqGSIb3DQEBDQUAMGkxCzAJ
# BgNVBAYTAlVTMRYwFAYDVQQKDA1FbnRydXN0LCBJbmMuMUIwQAYDVQQDDDlFbnRy
# dXN0IENvZGUgU2lnbmluZyBSb290IENlcnRpZmljYXRpb24gQXV0aG9yaXR5IC0g
# Q1NCUjEwHhcNMjEwNTA3MTkyMjE0WhcNNDAxMjI5MjM1OTAwWjBOMQswCQYDVQQG
# EwJVUzEWMBQGA1UEChMNRW50cnVzdCwgSW5jLjEnMCUGA1UEAxMeRW50cnVzdCBU
# aW1lIFN0YW1waW5nIENBIC0gVFMyMIICIjANBgkqhkiG9w0BAQEFAAOCAg8AMIIC
# CgKCAgEAtQMqBraETzTMnjG+6o7Ge24gJA8D172BNjPmb9I+u8SeqqDMZl1CYDRR
# 4i1hg7o8nmi4S8NmjqXeK7qbLUGl3tDLVwUtiNnARohtPDUNGZCsuAwZ8EzF20Vb
# 3DvCvI1EAOA4//9dyM8Imj3Usut7lcPEpQDgYgFkWV+M43uLqflFt8awogo3Q7hG
# gauGfn8v3vBf680gzKUtMeOoLpmb4Hjd9NtdgmN9MDmIfiQG5nxghHjIXCnllcv1
# e0/PWNyQz6eiexRUxOA3mDDBm7aunCn2ayVD5xJqPhmLvsDngddKMAh+fI3fPPNI
# kyHegYNqnQA+12C60bNiCJbptkkAOPRcdg3ad2o9oJXeHqHMWXpen8/pdl1A8rUV
# tWBoLPNaySOD2PUaa2kopLbUrmIG3iZFvd7pVneq3GSwQc3ME/FdcXSylAhLTzJu
# fnC4d+nVMjfCet5CEuhxHOLYYr+qa0GGBlvmab4mCfSXwv5M9IByhG7b3sUHqY4V
# om3hG+AJD/WgB6C7xHdzCTwHk40BtRJnLU0ZtbjFyRCrcM9C/6UnHdYS1uQYRGD7
# OV2Eu2a1UxKdAxSCZSm07Ely/bND5cMCKSS6H/i4vl2jty6p/M9i2fqXxBN441W5
# WnqE6/+7z2mIwLb5i2CFQRBQxwUQ0l52nT417PZPivkSez+vCzcCAwEAAaOCASww
# ggEoMBIGA1UdEwEB/wQIMAYBAf8CAQAwHQYDVR0OBBYEFCYP8MRICBvN3ZH1VFS2
# s7P8mfEIMB8GA1UdIwQYMBaAFIK61j2Xzp/PceiSN6/9s7VpNVfPMDMGCCsGAQUF
# BwEBBCcwJTAjBggrBgEFBQcwAYYXaHR0cDovL29jc3AuZW50cnVzdC5uZXQwMQYD
# VR0fBCowKDAmoCSgIoYgaHR0cDovL2NybC5lbnRydXN0Lm5ldC9jc2JyMS5jcmww
# DgYDVR0PAQH/BAQDAgGGMBMGA1UdJQQMMAoGCCsGAQUFBwMIMEUGA1UdIAQ+MDww
# MAYEVR0gADAoMCYGCCsGAQUFBwIBFhpodHRwOi8vd3d3LmVudHJ1c3QubmV0L3Jw
# YTAIBgZngQwBBAIwDQYJKoZIhvcNAQENBQADggIBAHY9RmiFXwnAzsn4x7jXfqdc
# OwtGc84iU4s49hG6d8TChyGPJ9kMlevoS+Vc15AVb6gqLIpq4CIRNHn6vZZ5yHYW
# zbFE6nFRD5tVyRh81YJ5nPOmt5OpBThyAgaXoaiyeX5eCOJubHmaqIeUzn+PVTZA
# D58/UdVmvnurhuZuh7pOY12x2I4fy2fDDXDaeuuTshSE0j59kz8WDDagXHkAnwtu
# 5YQq2IjBbQS3N/OSmnDgOwP9zbsngs4v/I3kssq++CWFxu10yD01XzgJPXlP6xvg
# QV6ldIE/URXA+EmIj7q7BdStRYIzecEljV9UPZHkuJ8XmABdrSjlT18b+/hUtaXE
# JPzwdjv4DV7MR7DKqrtuUd9TtUka8WCHt/n9aQZ1X3PWEZDygFiO5cMPA5iMBFOz
# KYffP/ew6UxXgSREeWj+GzYH1mU97G/WwrWqYXxmBuzS5WmA+HGSLlziEUYYRUvW
# +gVk77veBspDJb3fjckBdGGI2/DHpj3jDHG8f0vnyOBu4NqA+QCewGsl1aO6XCv+
# xGAgFEoHIzHlGsuRRLKae0PxW5i1IUu4MTsiVO55YfealBtmJyrG3+FlXe9xOeZQ
# Jf2HNLmuXXwnO+dlk/gohLFvhpMsk5O2cK+h7sikYamEO/sKt/i4bSbuR071XgX6
# nA8RfdfLka5DIPcVWWO8MIIGtDCCBJygAwIBAgIQW3AmzJb2eBq7hfYR9W27bjAN
# BgkqhkiG9w0BAQ0FADBOMQswCQYDVQQGEwJVUzEWMBQGA1UEChMNRW50cnVzdCwg
# SW5jLjEnMCUGA1UEAxMeRW50cnVzdCBUaW1lIFN0YW1waW5nIENBIC0gVFMyMB4X
# DTI0MDExOTE2NDc0N1oXDTM1MDQxODAwMDAwMFowdTELMAkGA1UEBhMCQ0ExEDAO
# BgNVBAgTB09udGFyaW8xDzANBgNVBAcTBk90dGF3YTEWMBQGA1UEChMNRW50cnVz
# dCwgSW5jLjErMCkGA1UEAxMiRW50cnVzdCBUaW1lc3RhbXAgQXV0aG9yaXR5IC0g
# VFNBMjCCAiIwDQYJKoZIhvcNAQEBBQADggIPADCCAgoCggIBAKqGBDhej2i/g/Va
# /VSbxX06dZ3Epy1PbenQBR9R4zO7L186sZZUiukIfDStiYNVpJ8CzCML2Bt6QKTM
# n0LqE398uOC3SgIF4oNJBG8Ghs1zQamD1Jhn+vRSGXrjSlkoh1YXXUjxhpELuCJk
# 20H5XVS1NaRuONwIDRpWGEDDfpg8QjOEDcWlYVHOIG1PAMtC8Kh1G9tG/incG57X
# +BgFUXYwm3S9lR8ZU2UVRyxWY2vpukisCji2wXL9IAqjdBprB3/R3awXzrUcx3qJ
# DOxtHDCNBfCvPZfMNbS1BQNmup52/DnayJrWEhLkQcKFIkDwy7qI6dKHwqvGNDvl
# qw0ABg4GtYt70Qsi2jyoyVZ78DUElupZRoyP1KC1qYOwzoT7S0TlQagW86/a0aAn
# cN8H7zfzrpHO1CF/hpjcXu6PwNpH22nKELmRlYAVY/ErGsKWoR/C9zzU9n0kByWo
# VJRvClXb2yRDbi47ontJu7XWGI/ZBLO+6se1ZJpLzlJhHisFHmM3Rp2xydVeh6Tq
# 1zSyIUIXuDooOl6l7T79Nxd/YyDye9ZfyvMUP+HO/4p3lGZticHg7H8EcDhkJ/Wn
# F31NygyBG8Xl8TbFeq59KaRHNTj6DfRTAykF/G+hi8nN8biIQNPS254lMa3X5lrY
# BLSEuwRGYWb/XTLe/NM+fE4Y737fAgMBAAGjggFlMIIBYTAMBgNVHRMBAf8EAjAA
# MB0GA1UdDgQWBBT1dhr4EJiSoRt3qQAs8ZViAgt5JzAfBgNVHSMEGDAWgBQmD/DE
# SAgbzd2R9VRUtrOz/JnxCDBoBggrBgEFBQcBAQRcMFowIwYIKwYBBQUHMAGGF2h0
# dHA6Ly9vY3NwLmVudHJ1c3QubmV0MDMGCCsGAQUFBzAChidodHRwOi8vYWlhLmVu
# dHJ1c3QubmV0L3RzMi1jaGFpbjI1Ni5wN2MwMQYDVR0fBCowKDAmoCSgIoYgaHR0
# cDovL2NybC5lbnRydXN0Lm5ldC90czJjYS5jcmwwDgYDVR0PAQH/BAQDAgeAMBYG
# A1UdJQEB/wQMMAoGCCsGAQUFBwMIMEwGA1UdIARFMEMwNwYKYIZIAYb6bAoBBzAp
# MCcGCCsGAQUFBwIBFhtodHRwczovL3d3dy5lbnRydXN0Lm5ldC9ycGEwCAYGZ4EM
# AQQCMA0GCSqGSIb3DQEBDQUAA4ICAQCpq32/GgCFW9ztcSZelIKvXIJL+hw5xVO1
# wwbdA2B++WGaQSDCrkEXLqVCCJEJXpcwmIFF1rK1hD2L6YcH2gE/4AhYZxEglB7A
# Fo5jd72bbAWNPbXLmjGpSlxiMqITzpCcDceQT2fD4Kh/h4RL1/LBIQ8Q5xe5+70J
# zL6KpgH+TYJPaSCadWvkDfdQJlP8M/f+G66nasCo6x6vItM9YBWfuuTVKaS1V5+x
# ZfaxTGHV0sjat6yqGB8gGIqm+PUH+XIr9mPYJZwW/vuU34Z0oiC08gjfFu85gHY6
# krHQloNq2nxTiVkbDVzxuynyIU9hLCSJ+HYJf9lokI5XDGTVdgBxWfPUhMg8oYh/
# qG/zvW5/sKiO5pNs6RTFdRwMK87yB/u1rtRBMb/BzGmYGK5bCI+rXGS5hGq9XDih
# 309Ct0tbUtPe9zvLn2/CWUIiTgsYtyLIL/xqCYXtUClResSUS7ZxZYpKhppLeEtW
# iqHeL4uTWzLMobnvqbYM2AYLFzC/DW+Pk6zLEfXxxwCe0BnXnF+OVHxrqUM7Y9zB
# RKu9jDpD8HI/JJXNhJFdIQANeGsNXCMkn0yvNMsUWt6x5EVQ/a9vpwWCp3PPz+mS
# 6nMqseHdogPmILwdZxpWwi72rF/KToGw5lodc8/SFPNQeCgFThVuhST4M6iIr3Rt
# 9FLOjKiEnjGCBBowggQWAgEBMGIwTjELMAkGA1UEBhMCVVMxFjAUBgNVBAoTDUVu
# dHJ1c3QsIEluYy4xJzAlBgNVBAMTHkVudHJ1c3QgVGltZSBTdGFtcGluZyBDQSAt
# IFRTMgIQW3AmzJb2eBq7hfYR9W27bjALBglghkgBZQMEAgOgggGLMBoGCSqGSIb3
# DQEJAzENBgsqhkiG9w0BCRABBDAcBgkqhkiG9w0BCQUxDxcNMjUwMTA5MDMwNjI5
# WjArBgkqhkiG9w0BCTQxHjAcMAsGCWCGSAFlAwQCA6ENBgkqhkiG9w0BAQ0FADBP
# BgkqhkiG9w0BCQQxQgRAasO4Mn93V2Q6ebBmoe2rUMfuIyYVIAujJfrnsq5TzEAO
# jYScIH5qef+pHrpx08LuF+YFF/1cGl3etsBF1VneUDCB0AYLKoZIhvcNAQkQAi8x
# gcAwgb0wgbowgbcwCwYJYIZIAWUDBAIDBEA5EUIuFwI+qpkkmXQODsjo0nLTVfxc
# 9mz5EVavl1U05ICv07x8TFtX79H/vNt1FGXg1AVahU6bETnZ9+xV1f4kMGYwUqRQ
# ME4xCzAJBgNVBAYTAlVTMRYwFAYDVQQKEw1FbnRydXN0LCBJbmMuMScwJQYDVQQD
# Ex5FbnRydXN0IFRpbWUgU3RhbXBpbmcgQ0EgLSBUUzICEFtwJsyW9ngau4X2EfVt
# u24wDQYJKoZIhvcNAQENBQAEggIAhqzvtsQLE1f6O2fWb0tJyKIokyi4pCB1BnkA
# XXfqXpKPwWyJriB+vJXW/cGSGts4CsT0jsMUcZx2I8rA+pWIll9h3wWf0DrVoWG1
# 0kthzJ4VFAkwOJN8ONQJESqHJXpfWNDWiEB8jjThUrpgK2mOpfBWc9VL/looyPhp
# 62jLwOWcCIgEfBZ1vOYwC9C6WxpCxwol7GUY2GsTxQ5+5Zpkh9PORfC6uJKc+PAo
# TPAnLdtf0zWPsKZwVJWVHplRrH8OZKWlgLdJLkeUm+Q2DHTZ8Cs0PHT9pW6aU/Jg
# usxv1ZP4h8XPT5AZqb5tQVYw8sFJY6QF5R6Fu3/hIhYzFS9+7rsNbV7HvIJu6/Za
# WS/pvkuD8/dg38RL4CR0KgK60RMTKyNwsd2IGseAS0SxlerwRSkTnR4yM9Qh3J/J
# TA3Avn12OBDiANl/JZP85t75Y1VR/U49foekj+8hTa7hkEuHbE5Ry3u558PoLaDZ
# b7qD7Kd6tD5lqiPcxSVI+EVrQ5uW87z6JvHbXceGs3oHzMEIeOODTJs9+29PJfVQ
# Wv0tOlhokAZEoW7nKVl0C+MYlSw31+x0DkGdEZyjENa3whyxw2wdgX4kmFJ15hk1
# 7E6rSedvPOHTE0xlqSpQ9OR79zFxLwyVIsVAFv1RXDP93oKLX98Evc9wGJqxEkjx
# uVGkdBI=
# SIG # End signature block
