function Expand-GzipFile {
    [CmdletBinding(SupportsShouldProcess)]
    param (
        [Parameter(Mandatory, Position = 0, ValueFromPipeline)]
        [ValidateNotNullOrEmpty()]
        [ValidateScript({
                if ([System.IO.Path]::GetExtension($_) -ne '.gz') {
                    throw (New-Object -TypeName System.Management.Automation.ErrorRecord -ArgumentList ([System.ArgumentException]'Invalid file extension specified for the InFile parameter.', 'ArgumentException', [System.Management.Automation.ErrorCategory]::InvalidArgument, $_))
                }

                $true
            })]
        [string]$InFile,

        [Parameter()]
        [string]$OutFile,

        [Parameter()]
        [switch]$DeleteSrcFile,

        [Parameter()]
        [switch]$PassThru
    )

    process {
        try {
            $inFileInfo = $(
                if ($InFile -is [System.IO.FileInfo]) {
                    [System.IO.FileInfo]$ExecutionContext.SessionState.Path.GetResolvedPSPathFromPSPath($InFile.FullName).ProviderPath
                }
                else {
                    [System.IO.FileInfo]$ExecutionContext.SessionState.Path.GetResolvedPSPathFromPSPath($InFile).ProviderPath
                }
            )
        }
        catch {
            $Global:Error.RemoveAt(0)

            $PSCmdlet.ThrowTerminatingError((New-Object -TypeName System.Management.Automation.ErrorRecord -ArgumentList ([System.ArgumentException]'Input file specified does not exist.', $_.FullyQualifiedErrorId, [System.Management.Automation.ErrorCategory]::InvalidArgument, $InFile)))
        }

        if (-not [string]::IsNullOrWhiteSpace($OutFile)) {
            try {
                $outFileInfo = [System.IO.FileInfo]$ExecutionContext.SessionState.Path.GetUnresolvedProviderPathFromPSPath($OutFile)
            }
            catch {
                $Global:Error.RemoveAt(0)

                $PSCmdlet.ThrowTerminatingError((New-Object -TypeName System.Management.Automation.ErrorRecord -ArgumentList ([System.ArgumentException]'Invalid file path specified for the OutFile parameter.', $_.FullyQualifiedErrorId, [System.Management.Automation.ErrorCategory]::InvalidArgument, $OutFile)))
            }
        }
        else {
            $outFileInfo = [System.IO.FileInfo]($inFileInfo.FullName -replace '.gz$')
        }

        $inputStream = New-Object -TypeName System.IO.FileStream -ArgumentList $inFileInfo.FullName, ([System.IO.FileMode]::Open), ([System.IO.FileAccess]::Read), ([System.IO.FileShare]::Read)
        $outputStream = New-Object -TypeName System.IO.FileStream -ArgumentList $outFileInfo.FullName, ([System.IO.FileMode]::Create), ([System.IO.FileAccess]::Write), ([System.IO.FileShare]::None)
        $gzipStream = New-Object -TypeName System.IO.Compression.GZipStream -ArgumentList $inputStream, ([System.IO.Compression.CompressionMode]::DeCompress)

        try {
            if ($PSCmdlet.ShouldProcess($inFileInfo.FullName, "Expanding file to $( $outFileInfo.FullName ).")) {
                $buffer = New-Object -TypeName byte[] -ArgumentList 1024

                while ($true) {
                    $read = $gzipStream.Read($buffer, 0, 1024)

                    if ($read -le 0) {
                        break
                    }

                    $outputStream.Write($buffer, 0, $read)
                }
            }
        }
        finally {
            $gzipStream.Close()
            $outputStream.Close()
            $inputStream.Close()

            if ($DeleteSrcFile.IsPresent) {
                if (Test-Path -LiteralPath $outFileInfo.FullName) {
                    if ($PSCmdlet.ShouldProcess($inFileInfo.FullName, 'Removing file.')) {
                        Remove-Item -LiteralPath $inFileInfo.FullName
                    }
                }
            }

            if ($PassThru.IsPresent) {
                $outFileInfo
            }
        }
    }
}

function Out-GzipFile {
    [CmdletBinding(SupportsShouldProcess)]
    param (
        [Parameter(Mandatory, Position = 0, ValueFromPipeline)]
        [ValidateNotNullOrEmpty()]
        [string]$InFile,

        [Parameter()]
        [string]$OutFile,

        [Parameter()]
        [switch]$DeleteSrcFile,

        [Parameter()]
        [switch]$PassThru
    )

    process {
        try {
            $inFileInfo = $(
                if ($InFile -is [System.IO.FileInfo]) {
                    [System.IO.FileInfo]$ExecutionContext.SessionState.Path.GetResolvedPSPathFromPSPath($InFile.FullName).ProviderPath
                }
                else {
                    [System.IO.FileInfo]$ExecutionContext.SessionState.Path.GetResolvedPSPathFromPSPath($InFile).ProviderPath
                }
            )
        }
        catch {
            $Global:Error.RemoveAt(0)

            $PSCmdlet.ThrowTerminatingError((New-Object -TypeName System.Management.Automation.ErrorRecord -ArgumentList ([System.ArgumentException]'Input file specified does not exist.', $_.FullyQualifiedErrorId, [System.Management.Automation.ErrorCategory]::InvalidArgument, $InFile)))
        }

        if (-not [string]::IsNullOrWhiteSpace($OutFile)) {
            try {
                $outFileInfo = [System.IO.FileInfo]$ExecutionContext.SessionState.Path.GetUnresolvedProviderPathFromPSPath($OutFile)
            }
            catch {
                $Global:Error.RemoveAt(0)

                $PSCmdlet.ThrowTerminatingError((New-Object -TypeName System.Management.Automation.ErrorRecord -ArgumentList ([System.ArgumentException]'Invalid file path specified for the OutFile parameter.', $_.FullyQualifiedErrorId, [System.Management.Automation.ErrorCategory]::InvalidArgument, $OutFile)))
            }
        }
        else {
            $outFileInfo = [System.IO.FileInfo]"$( $inFileInfo.FullName ).gz"
        }

        $inputStream = New-Object -TypeName System.IO.FileStream -ArgumentList $inFileInfo.FullName, ([System.IO.FileMode]::Open), ([System.IO.FileAccess]::Read), ([System.IO.FileShare]::Read)
        $outputStream = New-Object -TypeName System.IO.FileStream -ArgumentList $outFileInfo.FullName, ([System.IO.FileMode]::Create), ([System.IO.FileAccess]::Write), ([System.IO.FileShare]::None)
        $gzipStream = New-Object -TypeName System.IO.Compression.GzipStream -ArgumentList $outputStream, ([System.IO.Compression.CompressionMode]::Compress)

        try {
            if ($PSCmdlet.ShouldProcess($inFileInfo.FullName, 'Compressing file.')) {
                $buffer = New-Object -TypeName byte[] -ArgumentList 1024

                while ($true) {
                    $read = $inputStream.Read($buffer, 0, 1024)

                    if ($read -le 0) {
                        break
                    }

                    $gzipStream.Write($buffer, 0, $read)
                }
            }
        }
        finally {
            $gzipStream.Close()
            $outputStream.Close()
            $inputStream.Close()

            if ($DeleteSrcFile.IsPresent) {
                if (Test-Path -LiteralPath $outFileInfo.FullName) {
                    if ($PSCmdlet.ShouldProcess($inFileInfo.FullName, 'Removing file.')) {
                        Remove-Item -LiteralPath $inFileInfo.FullName
                    }
                }
            }

            if ($PassThru.IsPresent) {
                $outFileInfo
            }
        }
    }
}

# SIG # Begin signature block
# MIIy0QYJKoZIhvcNAQcCoIIywjCCMr4CAQExDzANBglghkgBZQMEAgEFADB5Bgor
# BgEEAYI3AgEEoGswaTA0BgorBgEEAYI3AgEeMCYCAwEAAAQQH8w7YFlLCE63JNLG
# KX7zUQIBAAIBAAIBAAIBAAIBADAxMA0GCWCGSAFlAwQCAQUABCCMVXPe0+FiChlE
# i0Ib90j7EyJnzoKJmgxp3spoUXDqtaCCFxcwggQ+MIIDJqADAgECAgRKU4woMA0G
# CSqGSIb3DQEBCwUAMIG+MQswCQYDVQQGEwJVUzEWMBQGA1UEChMNRW50cnVzdCwg
# SW5jLjEoMCYGA1UECxMfU2VlIHd3dy5lbnRydXN0Lm5ldC9sZWdhbC10ZXJtczE5
# MDcGA1UECxMwKGMpIDIwMDkgRW50cnVzdCwgSW5jLiAtIGZvciBhdXRob3JpemVk
# IHVzZSBvbmx5MTIwMAYDVQQDEylFbnRydXN0IFJvb3QgQ2VydGlmaWNhdGlvbiBB
# dXRob3JpdHkgLSBHMjAeFw0wOTA3MDcxNzI1NTRaFw0zMDEyMDcxNzU1NTRaMIG+
# MQswCQYDVQQGEwJVUzEWMBQGA1UEChMNRW50cnVzdCwgSW5jLjEoMCYGA1UECxMf
# U2VlIHd3dy5lbnRydXN0Lm5ldC9sZWdhbC10ZXJtczE5MDcGA1UECxMwKGMpIDIw
# MDkgRW50cnVzdCwgSW5jLiAtIGZvciBhdXRob3JpemVkIHVzZSBvbmx5MTIwMAYD
# VQQDEylFbnRydXN0IFJvb3QgQ2VydGlmaWNhdGlvbiBBdXRob3JpdHkgLSBHMjCC
# ASIwDQYJKoZIhvcNAQEBBQADggEPADCCAQoCggEBALqEtnLbngxr4pnpMAGnduoy
# uJVBGsnaYU5Ycs/+9oJ5v3NhBgqlJ9izX9NFThxy1k4y8nKKD/eDGdBqgIAARR6w
# x+eavxJXJxyjaC8Kh71qaw5eZfMcd9XUhY1wIbSzMueLotWGOQKxuNJHzuTJScQ7
# p977VH1XvvDobsJ5sjoLVeJQmBYyE1wveFbBwpSz8lrkJ5qfJNfG7NCbJYLjzMLE
# RcWMl3oGayoRn6kKbkg7b9vUERlC948Hv/VTX5w+9Bcs5mmsTjJMYnfqt+jluzS8
# GYuunFHnt361U7EzIuVtz3A8Gvrim2e2g/SNpa9iTE3gWKxkNBID+LaNlGMkpHEC
# AwEAAaNCMEAwDgYDVR0PAQH/BAQDAgEGMA8GA1UdEwEB/wQFMAMBAf8wHQYDVR0O
# BBYEFGpyJnrQHu995ztpUdRsjZ+QEmarMA0GCSqGSIb3DQEBCwUAA4IBAQB5nx2W
# xrZ5PyKNh9OHAwRgamuaLlmJcxGsQ9H1E/+NOSvA8r1PcIypL+oXxAtUntQblpgz
# PKitYqIAdqtZaW4GHX7EuUSNmK8S1GHbChlGR/Pr92PBQAVApdK39LWaNr+piHaI
# BFUEK5yHfxo3PH4tpRrY1Ileyr2sPWzYba/V83YPzTuIOCKdbJOaxD2/ghtlP6YP
# Xar85bIVyrWtxrw90ITo6gZysE05Mni/PhGcC6SdmiHz8JsLMHjbwdyHQ/68Y5rK
# xcIcyceN/zsSWAjmtj3seixO+4OWzgw8aYdUc6RzwpP/URCsFVQB2PwFsYmhf3SD
# mknX3E57ikhvi0X2MIIF3zCCBMegAwIBAgIQTkDkN1Tt5owAAAAAUdOUfzANBgkq
# hkiG9w0BAQsFADCBvjELMAkGA1UEBhMCVVMxFjAUBgNVBAoTDUVudHJ1c3QsIElu
# Yy4xKDAmBgNVBAsTH1NlZSB3d3cuZW50cnVzdC5uZXQvbGVnYWwtdGVybXMxOTA3
# BgNVBAsTMChjKSAyMDA5IEVudHJ1c3QsIEluYy4gLSBmb3IgYXV0aG9yaXplZCB1
# c2Ugb25seTEyMDAGA1UEAxMpRW50cnVzdCBSb290IENlcnRpZmljYXRpb24gQXV0
# aG9yaXR5IC0gRzIwHhcNMjEwNTA3MTU0MzQ1WhcNMzAxMTA3MTYxMzQ1WjBpMQsw
# CQYDVQQGEwJVUzEWMBQGA1UECgwNRW50cnVzdCwgSW5jLjFCMEAGA1UEAww5RW50
# cnVzdCBDb2RlIFNpZ25pbmcgUm9vdCBDZXJ0aWZpY2F0aW9uIEF1dGhvcml0eSAt
# IENTQlIxMIICIjANBgkqhkiG9w0BAQEFAAOCAg8AMIICCgKCAgEAp4GP9xRFtmJD
# 8tiu0yVeSE9Rv8V9n1AcNdHWfmEqlBltJ0akphpd91RRaoAixqhmdU1Ug8leaBur
# 9ltksK2tIL1U70ZrbQLnRa519o6KuTIui7h3HFJNeYhfpToYyVAslyctv9oAfWN/
# 7zLsRodj25qfw1ohNnv5m9XKoG5yLPzh8Z5wTQhWFW+Qq/tIurnXwYJ4hWUuf7XJ
# wOIUtzcRQQbiiuCo9uV+tngFAcNg7U8HQS4KE0njkJt/3b36rL9kUdFcm7T1XOdc
# /zubpaAa130JssK3/24cvMh95ukr/HKzFOlKVRKEnEQldR32KvBPpSA9aCXrYZd8
# D+W2PfOuw8ERvBuOzOBHMF5CAIZx41isBsplH3uUpktXZwx+Xq14Z1tV417rx9js
# TG6Gy/Pc+J+HqnJYEg99pvj4Qjk7PCzkMk1JjODhAMI4oJz6hD5B3G5WrsYaW/Rn
# aAUBzRu/roe8nVP2Lui2a+SZ3sVPh1io0mUeyB/Vcm7uWRxXOwlyndfKt5DGzXtF
# kpFCA0x9P8ryqrjCDobzEJ9GLqRmhmhaaBhwKTgRgGBrikOjc2zjs2s3/+adZwGS
# ht8vSNH7UGDVXP4h0wFCY/7vcLQXwI+o7tPBS18S6v39Lg6HRGDjqfTCGKPj/c4M
# hCIN86d42pPz2zjPuS8zxv8HPF6+RdMCAwEAAaOCASswggEnMA4GA1UdDwEB/wQE
# AwIBhjASBgNVHRMBAf8ECDAGAQH/AgEBMB0GA1UdJQQWMBQGCCsGAQUFBwMDBggr
# BgEFBQcDCDA7BgNVHSAENDAyMDAGBFUdIAAwKDAmBggrBgEFBQcCARYaaHR0cDov
# L3d3dy5lbnRydXN0Lm5ldC9ycGEwMwYIKwYBBQUHAQEEJzAlMCMGCCsGAQUFBzAB
# hhdodHRwOi8vb2NzcC5lbnRydXN0Lm5ldDAwBgNVHR8EKTAnMCWgI6Ahhh9odHRw
# Oi8vY3JsLmVudHJ1c3QubmV0L2cyY2EuY3JsMB0GA1UdDgQWBBSCutY9l86fz3Ho
# kjev/bO1aTVXzzAfBgNVHSMEGDAWgBRqciZ60B7vfec7aVHUbI2fkBJmqzANBgkq
# hkiG9w0BAQsFAAOCAQEAH15BBLaDcCRTLFVzHWU6wOy0ewSYXlk4EwmkWZRCXlC/
# T2xuJSCQk1hADfUZtGLuJF7CAVgVAh0QCW+o1PuSfjc4Pi8UfY8dQzZks2YTXxTM
# pXH3WyFLxpe+3JX8cH0RHNMh3dAkOSnF/goapc97ee46b97cv+kR3RaDCNMsjX9N
# qBR5LwVhUjjrYPMUaH3LsoqtwJRc5CYOLIrdRsPO5FZRxVbjhbhNm0VyiwfxivtJ
# uF/R8paBXWlSJPEII9LWIw/ri9d+i8GTa/rxYntY6VCbl24XiA3hxkOY14FhtoWd
# R+yxnq4/IDtDndiiHODUfAjCr3YG+GJmerb3+sivNTCCBmcwggRPoAMCAQICEEbM
# 41QNEgkq80rzt39+/oswDQYJKoZIhvcNAQELBQAwYzELMAkGA1UEBhMCVVMxFjAU
# BgNVBAoTDUVudHJ1c3QsIEluYy4xPDA6BgNVBAMTM0VudHJ1c3QgRXh0ZW5kZWQg
# VmFsaWRhdGlvbiBDb2RlIFNpZ25pbmcgQ0EgLSBFVkNTMjAeFw0yNDEwMjMxNTIy
# MTRaFw0yNTExMDkxNTIyMTNaMIHCMQswCQYDVQQGEwJVUzETMBEGA1UECBMKQ2Fs
# aWZvcm5pYTERMA8GA1UEBxMIU2FuIEpvc2UxEzARBgsrBgEEAYI3PAIBAxMCVVMx
# GTAXBgsrBgEEAYI3PAIBAhMIRGVsYXdhcmUxFDASBgNVBAoTC05ldEFwcCwgSW5j
# MR0wGwYDVQQPExRQcml2YXRlIE9yZ2FuaXphdGlvbjEQMA4GA1UEBRMHMzQ1Mjc0
# MzEUMBIGA1UEAxMLTmV0QXBwLCBJbmMwggGiMA0GCSqGSIb3DQEBAQUAA4IBjwAw
# ggGKAoIBgQCIHyusYBqN6d8MjDVGb9yKy/jxOJi5Ry9vtMsEpSe0eX1NwlzjdPpo
# R+kSTTj0HXwjn48v4UtO3lOWVVdqDuAuECTvG+n31c1RmaB4qS0qg9VEDsUP0/gr
# R4GlmEnGRIeDNWnRIV4oHGTqA1NjNU3esZYLQSV9j5DSzh0W+vyq10stDZiLBNai
# KoBzi5fs+C7QWW+xd+fSFV3Z+zIfE/jHFa4ZKP0n8Bva/5XKZSg6XVBN09oY/EF2
# 67wlz2018jiSwKZi++kCqTIpp9j7dKP5x+0/bK4GlGeivT/NObhjDN8e0Iu5rC2L
# tCalpr5CBMzjKmwfV8mVd9L/qPKk8QZxBdBrUpA1qHDCHbd9BRcd2zT8doIbM0vT
# dHnT9pRPZYws7BfT33KMoV7nShbrNViWhkP3i3UkQv7W9RptiZwfEDpt4Lxn0Uw8
# Zf94fbwMQaV3A28ItUbueLN23+k41N0jqdFTbpCaqQpUT/NxlRHECtn0Lp6kjZTy
# wKF7FqH+jpcCAwEAAaOCATUwggExMAwGA1UdEwEB/wQCMAAwHQYDVR0OBBYEFGzw
# 5EcKP/wp/cuZ0BIs8j6slZQcMB8GA1UdIwQYMBaAFM6JT4JRqhWihGLKMSNh0mH7
# +P54MA4GA1UdDwEB/wQEAwIHgDATBgNVHSUEDDAKBggrBgEFBQcDAzBnBggrBgEF
# BQcBAQRbMFkwIwYIKwYBBQUHMAGGF2h0dHA6Ly9vY3NwLmVudHJ1c3QubmV0MDIG
# CCsGAQUFBzAChiZodHRwOi8vYWlhLmVudHJ1c3QubmV0L2V2Y3MyLWNoYWluLnA3
# YzAxBgNVHR8EKjAoMCagJKAihiBodHRwOi8vY3JsLmVudHJ1c3QubmV0L2V2Y3My
# LmNybDAgBgNVHSAEGTAXMAcGBWeBDAEDMAwGCmCGSAGG+mwKAQIwDQYJKoZIhvcN
# AQELBQADggIBADAwfASY7H1rvIVgDB4LCw7V6xYShLM1XBV0YV8mgn50EVUgkXRr
# qwdn9JgVJvPU6jQ30ab9bb2505N/e1hWQN+Skw21Kbixw3mJ2lZNjY9CvwPUAc4B
# 9xXFEl/w7w1GuATgHO77YCpe0QTbYpbuSTXA0YBs1LlzmuoMxCD+E82srnED8Crg
# m4IHe71kpqqbsIYDnxujU+w/+/CAijPCuxXD2Ff1D7Ms97n9OFvLZHU2e+7xTP66
# RFXF81uCNcLYZSAMiXpjFCiXz74yNHtqZYecnNKvA3hUnvE0BH2IbJFpFiwQO9H2
# rSxdf6qTJANWvCTm7AkDMPhCneufSIFeGHyINH5sKNc+MgsNawZwy2Bfld2YE+ii
# +RyoUgQvl+OZu58dcPYvbgSkc+nE7Xlq/nzE4qNB83ZUuo4MQuGgZ1ZdROSgO6BD
# lCDa1Mr+viR84DtT6dwMYDf0PYWe0LoDYBVLRknZHpSNCMULURCTENBJdJo1mrOn
# hmmm5ZJ3Hg/CmyrujK67YvNFUsQnpLKYEut7ntmJM5ptEPm7Ql5zzaOp3o68a/Q6
# 0+s6p4NwZQYo4ekHk+DM4oaPiv8K+dEPxvppQCSoSPG0S59A8oFBXzoaidOTC06j
# xfGFkPFtOEIO269lK0NCIgr2lJvlUdlUPSnM2eWkEkslz42ShJgON0SaMIIGgzCC
# BGugAwIBAgIQNa+3e500H2r8j4RGqzE1KzANBgkqhkiG9w0BAQ0FADBpMQswCQYD
# VQQGEwJVUzEWMBQGA1UECgwNRW50cnVzdCwgSW5jLjFCMEAGA1UEAww5RW50cnVz
# dCBDb2RlIFNpZ25pbmcgUm9vdCBDZXJ0aWZpY2F0aW9uIEF1dGhvcml0eSAtIENT
# QlIxMB4XDTIxMDUwNzE5MTk1MloXDTQwMTIyOTIzNTkwMFowYzELMAkGA1UEBhMC
# VVMxFjAUBgNVBAoTDUVudHJ1c3QsIEluYy4xPDA6BgNVBAMTM0VudHJ1c3QgRXh0
# ZW5kZWQgVmFsaWRhdGlvbiBDb2RlIFNpZ25pbmcgQ0EgLSBFVkNTMjCCAiIwDQYJ
# KoZIhvcNAQEBBQADggIPADCCAgoCggIBAL69pznJpX3sXWXx9Cuph9DnrRrFGjsY
# zuGhUY1y+s5YH1y4JEIPRtUxl9BKTeObMMm6l6ic/kU2zyeA53u4bsEkt9+ndNyF
# 8qMkWEXMlJQ7AuvEjXxG9VxmguOkwdMfrG4MUyMO1Dr62kLxg1RfNTJW8rV4m1cA
# SB6pYWEnDnMDQ7bWcJL71IWaMMaz5ppeS+8dKthmqxZG/wvYD6aJSgJRV0E8QThO
# l8dRMm1njmahXk2fNSKv1Wq3f0BfaDXMafrxBfDqhabqMoXLwcHKg2lFSQbcCWy6
# SWUZjPm3NyeMZJ414+Xs5wegnahyvG+FOiymFk49nM8I5oL1RH0owL2JrWwv3C94
# eRHXHHBL3Z0ITF4u+o29p91j9n/wUjGEbjrY2VyFRJ5jBmnQhlh4iZuHu1gcpChs
# xv5pCpwerBFgal7JaWUu7UMtafF4tzstNfKqT+If4wFvkEaq1agNBFegtKzjbb2d
# GyiAJ0bH2qpnlfHRh3vHyCXphAyPiTbSvjPhhcAz1aA8GYuvOPLlk4C/xsOre5PE
# PZ257kV2wNRobzBePLQ2+ddFQuASBoDbpSH85wV6KI20jmB798i1SkesFGaXoFpp
# cjFXa1OEzWG6cwcVcDt7AfynP4wtPYeM+wjX5S8Xg36Cq08J8inhflV3ZZQFHVnU
# Ct2TfuMUXeK7AgMBAAGjggErMIIBJzASBgNVHRMBAf8ECDAGAQH/AgEAMB0GA1Ud
# DgQWBBTOiU+CUaoVooRiyjEjYdJh+/j+eDAfBgNVHSMEGDAWgBSCutY9l86fz3Ho
# kjev/bO1aTVXzzAzBggrBgEFBQcBAQQnMCUwIwYIKwYBBQUHMAGGF2h0dHA6Ly9v
# Y3NwLmVudHJ1c3QubmV0MDEGA1UdHwQqMCgwJqAkoCKGIGh0dHA6Ly9jcmwuZW50
# cnVzdC5uZXQvY3NicjEuY3JsMA4GA1UdDwEB/wQEAwIBhjATBgNVHSUEDDAKBggr
# BgEFBQcDAzBEBgNVHSAEPTA7MDAGBFUdIAAwKDAmBggrBgEFBQcCARYaaHR0cDov
# L3d3dy5lbnRydXN0Lm5ldC9ycGEwBwYFZ4EMAQMwDQYJKoZIhvcNAQENBQADggIB
# AD4AVLgq849mr2EWxFiTZPRBi2RVjRs1M6GbkdirRsqrX7y+fnDk0tcHqJYH14bR
# VwoI0NB4Tfgq37IE85rh13zwwQB6wUCh34qMt8u0HQFh8piapt24gwXKqSwW3Jwt
# Dv6nl+RQqZeVwUsqjFHjxALga3w1TVO8S5QTi1MYFl6mCqe4NMFssess5DF9DCzG
# fOGkVugtdtWyE3XqgwCuAHfGb6k97mMUgVAW/FtPEhkOWw+N6kvOBkyJS64gzI5H
# pnXWZe4vMOhdNI8fgk1cQqbyFExQIJwJonQkXDnYiTKFPK+M5Wqe5gQ6pRP/qh3N
# R0suAgW0ao/rhU+B7wrbfZ8pj6XCP1I4UkGVO7w+W1QwQiMJY95QjYk1RfqruA+P
# oq17ehGT8Y8ohHtoeUdq6GQpTR/0HS9tHsiUhjzTWpl6a3yrNfcrOUtPuT8Wku8p
# jI2rrAEazHFEOctAPiASzghw40f+3IDXCADRC2rqIbV5ZhfpaqpW3c0VeLEDwBSt
# PkcYde0KU0syk83/gLGQ1hPl5EF4Iu1BguUO37DOlSFF5osB0xn39CtVrNlWc2MQ
# 4LigbctUlpigmSFRBqqmDDorY8t52kO50hLM3o9VeukJ8+Ka0yXBezaS2uDlUmfN
# 4+ZUCqWd1HOj0y9dBmSFA3d/YNjCvHTJlZFot7d+YRl1MYIbEDCCGwwCAQEwdzBj
# MQswCQYDVQQGEwJVUzEWMBQGA1UEChMNRW50cnVzdCwgSW5jLjE8MDoGA1UEAxMz
# RW50cnVzdCBFeHRlbmRlZCBWYWxpZGF0aW9uIENvZGUgU2lnbmluZyBDQSAtIEVW
# Q1MyAhBGzONUDRIJKvNK87d/fv6LMA0GCWCGSAFlAwQCAQUAoIGEMBgGCisGAQQB
# gjcCAQwxCjAIoAKAAKECgAAwGQYJKoZIhvcNAQkDMQwGCisGAQQBgjcCAQQwHAYK
# KwYBBAGCNwIBCzEOMAwGCisGAQQBgjcCARUwLwYJKoZIhvcNAQkEMSIEIEf4HeF4
# C+PscwDG+kl//UqgHhlZR/lxwPc/XisYE2opMA0GCSqGSIb3DQEBAQUABIIBgC3u
# qq2Q34xVHnMU6iYFKh3fyTx16238GAUNeUAh865eo+x1G1f6PsQaIArmRLomXiwU
# NN41vPioBJYyFGZs6hJKTYaa9RPQXRqhEaJ7GBu5H3K4RvP68KIyG2juhcz/GKUB
# ftsW7Mohnk6ecIIQiLtfg7o1UP141861UCIGmWhvRixM+xOMk2ZsyXjYPiiS1lIT
# gQDJgqDjSvFRGjTNEBzblYMGlWoiU7KtJ2bFFJDYBcFZVaEnKxtPe5ODT2WdsE7d
# aM5f3rmK5lYIhZ0HXG8NSbfHCeA/5MFpEY9gZrQbe8+J8wBoZvTlbumK0Ny2kIFh
# B1i1Pbh+uYHXfYLxarxjY3uinBbSvYH69gYBApWHPQTUGFjZZr/G//To4ez/zrkw
# dvWPN0wCcjeZh8MJfTSkxaFjvTkNuOfUsLqd9qAitrDBby1Tk3Q1qTrB8Dp3Lxlj
# 8DCSs14O4IkSjLQStb1JRZI5M7yVbaCNA/fbT7CqXa7xiO9GO9TP+Fv6ySX2paGC
# GGMwghhfBgorBgEEAYI3AwMBMYIYTzCCGEsGCSqGSIb3DQEHAqCCGDwwghg4AgED
# MQ0wCwYJYIZIAWUDBAIDMIHzBgsqhkiG9w0BCRABBKCB4wSB4DCB3QIBAQYKYIZI
# AYb6bAoDBTAxMA0GCWCGSAFlAwQCAQUABCDiGMqSrz1FBu55beS6L/mEv1A4bqYA
# bXqiFXYUE2yV0AIIC0musvXnGFkYDzIwMjUwMTA5MDMwNjI5WjADAgEBoHmkdzB1
# MQswCQYDVQQGEwJDQTEQMA4GA1UECBMHT250YXJpbzEPMA0GA1UEBxMGT3R0YXdh
# MRYwFAYDVQQKEw1FbnRydXN0LCBJbmMuMSswKQYDVQQDEyJFbnRydXN0IFRpbWVz
# dGFtcCBBdXRob3JpdHkgLSBUU0EyoIITDjCCBd8wggTHoAMCAQICEE5A5DdU7eaM
# AAAAAFHTlH8wDQYJKoZIhvcNAQELBQAwgb4xCzAJBgNVBAYTAlVTMRYwFAYDVQQK
# Ew1FbnRydXN0LCBJbmMuMSgwJgYDVQQLEx9TZWUgd3d3LmVudHJ1c3QubmV0L2xl
# Z2FsLXRlcm1zMTkwNwYDVQQLEzAoYykgMjAwOSBFbnRydXN0LCBJbmMuIC0gZm9y
# IGF1dGhvcml6ZWQgdXNlIG9ubHkxMjAwBgNVBAMTKUVudHJ1c3QgUm9vdCBDZXJ0
# aWZpY2F0aW9uIEF1dGhvcml0eSAtIEcyMB4XDTIxMDUwNzE1NDM0NVoXDTMwMTEw
# NzE2MTM0NVowaTELMAkGA1UEBhMCVVMxFjAUBgNVBAoMDUVudHJ1c3QsIEluYy4x
# QjBABgNVBAMMOUVudHJ1c3QgQ29kZSBTaWduaW5nIFJvb3QgQ2VydGlmaWNhdGlv
# biBBdXRob3JpdHkgLSBDU0JSMTCCAiIwDQYJKoZIhvcNAQEBBQADggIPADCCAgoC
# ggIBAKeBj/cURbZiQ/LYrtMlXkhPUb/FfZ9QHDXR1n5hKpQZbSdGpKYaXfdUUWqA
# IsaoZnVNVIPJXmgbq/ZbZLCtrSC9VO9Ga20C50WudfaOirkyLou4dxxSTXmIX6U6
# GMlQLJcnLb/aAH1jf+8y7EaHY9uan8NaITZ7+ZvVyqBuciz84fGecE0IVhVvkKv7
# SLq518GCeIVlLn+1ycDiFLc3EUEG4orgqPblfrZ4BQHDYO1PB0EuChNJ45Cbf929
# +qy/ZFHRXJu09VznXP87m6WgGtd9CbLCt/9uHLzIfebpK/xysxTpSlUShJxEJXUd
# 9irwT6UgPWgl62GXfA/ltj3zrsPBEbwbjszgRzBeQgCGceNYrAbKZR97lKZLV2cM
# fl6teGdbVeNe68fY7Exuhsvz3Pifh6pyWBIPfab4+EI5Ozws5DJNSYzg4QDCOKCc
# +oQ+QdxuVq7GGlv0Z2gFAc0bv66HvJ1T9i7otmvkmd7FT4dYqNJlHsgf1XJu7lkc
# VzsJcp3XyreQxs17RZKRQgNMfT/K8qq4wg6G8xCfRi6kZoZoWmgYcCk4EYBga4pD
# o3Ns47NrN//mnWcBkobfL0jR+1Bg1Vz+IdMBQmP+73C0F8CPqO7TwUtfEur9/S4O
# h0Rg46n0whij4/3ODIQiDfOneNqT89s4z7kvM8b/BzxevkXTAgMBAAGjggErMIIB
# JzAOBgNVHQ8BAf8EBAMCAYYwEgYDVR0TAQH/BAgwBgEB/wIBATAdBgNVHSUEFjAU
# BggrBgEFBQcDAwYIKwYBBQUHAwgwOwYDVR0gBDQwMjAwBgRVHSAAMCgwJgYIKwYB
# BQUHAgEWGmh0dHA6Ly93d3cuZW50cnVzdC5uZXQvcnBhMDMGCCsGAQUFBwEBBCcw
# JTAjBggrBgEFBQcwAYYXaHR0cDovL29jc3AuZW50cnVzdC5uZXQwMAYDVR0fBCkw
# JzAloCOgIYYfaHR0cDovL2NybC5lbnRydXN0Lm5ldC9nMmNhLmNybDAdBgNVHQ4E
# FgQUgrrWPZfOn89x6JI3r/2ztWk1V88wHwYDVR0jBBgwFoAUanImetAe733nO2lR
# 1GyNn5ASZqswDQYJKoZIhvcNAQELBQADggEBAB9eQQS2g3AkUyxVcx1lOsDstHsE
# mF5ZOBMJpFmUQl5Qv09sbiUgkJNYQA31GbRi7iRewgFYFQIdEAlvqNT7kn43OD4v
# FH2PHUM2ZLNmE18UzKVx91shS8aXvtyV/HB9ERzTId3QJDkpxf4KGqXPe3nuOm/e
# 3L/pEd0WgwjTLI1/TagUeS8FYVI462DzFGh9y7KKrcCUXOQmDiyK3UbDzuRWUcVW
# 44W4TZtFcosH8Yr7Sbhf0fKWgV1pUiTxCCPS1iMP64vXfovBk2v68WJ7WOlQm5du
# F4gN4cZDmNeBYbaFnUfssZ6uPyA7Q53Yohzg1HwIwq92BvhiZnq29/rIrzUwggZv
# MIIEV6ADAgECAhAlvCvzKcoQfx6puoiF1J07MA0GCSqGSIb3DQEBDQUAMGkxCzAJ
# BgNVBAYTAlVTMRYwFAYDVQQKDA1FbnRydXN0LCBJbmMuMUIwQAYDVQQDDDlFbnRy
# dXN0IENvZGUgU2lnbmluZyBSb290IENlcnRpZmljYXRpb24gQXV0aG9yaXR5IC0g
# Q1NCUjEwHhcNMjEwNTA3MTkyMjE0WhcNNDAxMjI5MjM1OTAwWjBOMQswCQYDVQQG
# EwJVUzEWMBQGA1UEChMNRW50cnVzdCwgSW5jLjEnMCUGA1UEAxMeRW50cnVzdCBU
# aW1lIFN0YW1waW5nIENBIC0gVFMyMIICIjANBgkqhkiG9w0BAQEFAAOCAg8AMIIC
# CgKCAgEAtQMqBraETzTMnjG+6o7Ge24gJA8D172BNjPmb9I+u8SeqqDMZl1CYDRR
# 4i1hg7o8nmi4S8NmjqXeK7qbLUGl3tDLVwUtiNnARohtPDUNGZCsuAwZ8EzF20Vb
# 3DvCvI1EAOA4//9dyM8Imj3Usut7lcPEpQDgYgFkWV+M43uLqflFt8awogo3Q7hG
# gauGfn8v3vBf680gzKUtMeOoLpmb4Hjd9NtdgmN9MDmIfiQG5nxghHjIXCnllcv1
# e0/PWNyQz6eiexRUxOA3mDDBm7aunCn2ayVD5xJqPhmLvsDngddKMAh+fI3fPPNI
# kyHegYNqnQA+12C60bNiCJbptkkAOPRcdg3ad2o9oJXeHqHMWXpen8/pdl1A8rUV
# tWBoLPNaySOD2PUaa2kopLbUrmIG3iZFvd7pVneq3GSwQc3ME/FdcXSylAhLTzJu
# fnC4d+nVMjfCet5CEuhxHOLYYr+qa0GGBlvmab4mCfSXwv5M9IByhG7b3sUHqY4V
# om3hG+AJD/WgB6C7xHdzCTwHk40BtRJnLU0ZtbjFyRCrcM9C/6UnHdYS1uQYRGD7
# OV2Eu2a1UxKdAxSCZSm07Ely/bND5cMCKSS6H/i4vl2jty6p/M9i2fqXxBN441W5
# WnqE6/+7z2mIwLb5i2CFQRBQxwUQ0l52nT417PZPivkSez+vCzcCAwEAAaOCASww
# ggEoMBIGA1UdEwEB/wQIMAYBAf8CAQAwHQYDVR0OBBYEFCYP8MRICBvN3ZH1VFS2
# s7P8mfEIMB8GA1UdIwQYMBaAFIK61j2Xzp/PceiSN6/9s7VpNVfPMDMGCCsGAQUF
# BwEBBCcwJTAjBggrBgEFBQcwAYYXaHR0cDovL29jc3AuZW50cnVzdC5uZXQwMQYD
# VR0fBCowKDAmoCSgIoYgaHR0cDovL2NybC5lbnRydXN0Lm5ldC9jc2JyMS5jcmww
# DgYDVR0PAQH/BAQDAgGGMBMGA1UdJQQMMAoGCCsGAQUFBwMIMEUGA1UdIAQ+MDww
# MAYEVR0gADAoMCYGCCsGAQUFBwIBFhpodHRwOi8vd3d3LmVudHJ1c3QubmV0L3Jw
# YTAIBgZngQwBBAIwDQYJKoZIhvcNAQENBQADggIBAHY9RmiFXwnAzsn4x7jXfqdc
# OwtGc84iU4s49hG6d8TChyGPJ9kMlevoS+Vc15AVb6gqLIpq4CIRNHn6vZZ5yHYW
# zbFE6nFRD5tVyRh81YJ5nPOmt5OpBThyAgaXoaiyeX5eCOJubHmaqIeUzn+PVTZA
# D58/UdVmvnurhuZuh7pOY12x2I4fy2fDDXDaeuuTshSE0j59kz8WDDagXHkAnwtu
# 5YQq2IjBbQS3N/OSmnDgOwP9zbsngs4v/I3kssq++CWFxu10yD01XzgJPXlP6xvg
# QV6ldIE/URXA+EmIj7q7BdStRYIzecEljV9UPZHkuJ8XmABdrSjlT18b+/hUtaXE
# JPzwdjv4DV7MR7DKqrtuUd9TtUka8WCHt/n9aQZ1X3PWEZDygFiO5cMPA5iMBFOz
# KYffP/ew6UxXgSREeWj+GzYH1mU97G/WwrWqYXxmBuzS5WmA+HGSLlziEUYYRUvW
# +gVk77veBspDJb3fjckBdGGI2/DHpj3jDHG8f0vnyOBu4NqA+QCewGsl1aO6XCv+
# xGAgFEoHIzHlGsuRRLKae0PxW5i1IUu4MTsiVO55YfealBtmJyrG3+FlXe9xOeZQ
# Jf2HNLmuXXwnO+dlk/gohLFvhpMsk5O2cK+h7sikYamEO/sKt/i4bSbuR071XgX6
# nA8RfdfLka5DIPcVWWO8MIIGtDCCBJygAwIBAgIQW3AmzJb2eBq7hfYR9W27bjAN
# BgkqhkiG9w0BAQ0FADBOMQswCQYDVQQGEwJVUzEWMBQGA1UEChMNRW50cnVzdCwg
# SW5jLjEnMCUGA1UEAxMeRW50cnVzdCBUaW1lIFN0YW1waW5nIENBIC0gVFMyMB4X
# DTI0MDExOTE2NDc0N1oXDTM1MDQxODAwMDAwMFowdTELMAkGA1UEBhMCQ0ExEDAO
# BgNVBAgTB09udGFyaW8xDzANBgNVBAcTBk90dGF3YTEWMBQGA1UEChMNRW50cnVz
# dCwgSW5jLjErMCkGA1UEAxMiRW50cnVzdCBUaW1lc3RhbXAgQXV0aG9yaXR5IC0g
# VFNBMjCCAiIwDQYJKoZIhvcNAQEBBQADggIPADCCAgoCggIBAKqGBDhej2i/g/Va
# /VSbxX06dZ3Epy1PbenQBR9R4zO7L186sZZUiukIfDStiYNVpJ8CzCML2Bt6QKTM
# n0LqE398uOC3SgIF4oNJBG8Ghs1zQamD1Jhn+vRSGXrjSlkoh1YXXUjxhpELuCJk
# 20H5XVS1NaRuONwIDRpWGEDDfpg8QjOEDcWlYVHOIG1PAMtC8Kh1G9tG/incG57X
# +BgFUXYwm3S9lR8ZU2UVRyxWY2vpukisCji2wXL9IAqjdBprB3/R3awXzrUcx3qJ
# DOxtHDCNBfCvPZfMNbS1BQNmup52/DnayJrWEhLkQcKFIkDwy7qI6dKHwqvGNDvl
# qw0ABg4GtYt70Qsi2jyoyVZ78DUElupZRoyP1KC1qYOwzoT7S0TlQagW86/a0aAn
# cN8H7zfzrpHO1CF/hpjcXu6PwNpH22nKELmRlYAVY/ErGsKWoR/C9zzU9n0kByWo
# VJRvClXb2yRDbi47ontJu7XWGI/ZBLO+6se1ZJpLzlJhHisFHmM3Rp2xydVeh6Tq
# 1zSyIUIXuDooOl6l7T79Nxd/YyDye9ZfyvMUP+HO/4p3lGZticHg7H8EcDhkJ/Wn
# F31NygyBG8Xl8TbFeq59KaRHNTj6DfRTAykF/G+hi8nN8biIQNPS254lMa3X5lrY
# BLSEuwRGYWb/XTLe/NM+fE4Y737fAgMBAAGjggFlMIIBYTAMBgNVHRMBAf8EAjAA
# MB0GA1UdDgQWBBT1dhr4EJiSoRt3qQAs8ZViAgt5JzAfBgNVHSMEGDAWgBQmD/DE
# SAgbzd2R9VRUtrOz/JnxCDBoBggrBgEFBQcBAQRcMFowIwYIKwYBBQUHMAGGF2h0
# dHA6Ly9vY3NwLmVudHJ1c3QubmV0MDMGCCsGAQUFBzAChidodHRwOi8vYWlhLmVu
# dHJ1c3QubmV0L3RzMi1jaGFpbjI1Ni5wN2MwMQYDVR0fBCowKDAmoCSgIoYgaHR0
# cDovL2NybC5lbnRydXN0Lm5ldC90czJjYS5jcmwwDgYDVR0PAQH/BAQDAgeAMBYG
# A1UdJQEB/wQMMAoGCCsGAQUFBwMIMEwGA1UdIARFMEMwNwYKYIZIAYb6bAoBBzAp
# MCcGCCsGAQUFBwIBFhtodHRwczovL3d3dy5lbnRydXN0Lm5ldC9ycGEwCAYGZ4EM
# AQQCMA0GCSqGSIb3DQEBDQUAA4ICAQCpq32/GgCFW9ztcSZelIKvXIJL+hw5xVO1
# wwbdA2B++WGaQSDCrkEXLqVCCJEJXpcwmIFF1rK1hD2L6YcH2gE/4AhYZxEglB7A
# Fo5jd72bbAWNPbXLmjGpSlxiMqITzpCcDceQT2fD4Kh/h4RL1/LBIQ8Q5xe5+70J
# zL6KpgH+TYJPaSCadWvkDfdQJlP8M/f+G66nasCo6x6vItM9YBWfuuTVKaS1V5+x
# ZfaxTGHV0sjat6yqGB8gGIqm+PUH+XIr9mPYJZwW/vuU34Z0oiC08gjfFu85gHY6
# krHQloNq2nxTiVkbDVzxuynyIU9hLCSJ+HYJf9lokI5XDGTVdgBxWfPUhMg8oYh/
# qG/zvW5/sKiO5pNs6RTFdRwMK87yB/u1rtRBMb/BzGmYGK5bCI+rXGS5hGq9XDih
# 309Ct0tbUtPe9zvLn2/CWUIiTgsYtyLIL/xqCYXtUClResSUS7ZxZYpKhppLeEtW
# iqHeL4uTWzLMobnvqbYM2AYLFzC/DW+Pk6zLEfXxxwCe0BnXnF+OVHxrqUM7Y9zB
# RKu9jDpD8HI/JJXNhJFdIQANeGsNXCMkn0yvNMsUWt6x5EVQ/a9vpwWCp3PPz+mS
# 6nMqseHdogPmILwdZxpWwi72rF/KToGw5lodc8/SFPNQeCgFThVuhST4M6iIr3Rt
# 9FLOjKiEnjGCBBowggQWAgEBMGIwTjELMAkGA1UEBhMCVVMxFjAUBgNVBAoTDUVu
# dHJ1c3QsIEluYy4xJzAlBgNVBAMTHkVudHJ1c3QgVGltZSBTdGFtcGluZyBDQSAt
# IFRTMgIQW3AmzJb2eBq7hfYR9W27bjALBglghkgBZQMEAgOgggGLMBoGCSqGSIb3
# DQEJAzENBgsqhkiG9w0BCRABBDAcBgkqhkiG9w0BCQUxDxcNMjUwMTA5MDMwNjI5
# WjArBgkqhkiG9w0BCTQxHjAcMAsGCWCGSAFlAwQCA6ENBgkqhkiG9w0BAQ0FADBP
# BgkqhkiG9w0BCQQxQgRAAKlLqqzn62DUVHmqj/vTr9H5eE2ZXd2UoLPcjPohGyG3
# W6Dm2ivatpZhrmK//AhLyFKgP153j5Jpaz9et+A54TCB0AYLKoZIhvcNAQkQAi8x
# gcAwgb0wgbowgbcwCwYJYIZIAWUDBAIDBEA5EUIuFwI+qpkkmXQODsjo0nLTVfxc
# 9mz5EVavl1U05ICv07x8TFtX79H/vNt1FGXg1AVahU6bETnZ9+xV1f4kMGYwUqRQ
# ME4xCzAJBgNVBAYTAlVTMRYwFAYDVQQKEw1FbnRydXN0LCBJbmMuMScwJQYDVQQD
# Ex5FbnRydXN0IFRpbWUgU3RhbXBpbmcgQ0EgLSBUUzICEFtwJsyW9ngau4X2EfVt
# u24wDQYJKoZIhvcNAQENBQAEggIAZN1Dx9cga0B1YeuUk4QU/TrxB0niRoVEFTGw
# ZWjo+ojxxfqZPFMTP+1miP1qiA+o/rRj+JWtlGWwY7uxpDqMJb49ni5iCJVpGOND
# PLejrdomn5xDJHeeX0nXIo7nXpYcXIA0bp0rI9tHGo0Tas8ulXZalNuT/sEA7RLN
# 0MpmpRO6gC9iK+lJu1KjpNwD3LKGwVBKn2x1dCVLLPXrZ3wov/HVIE//LYuZmvtQ
# +AElKq4xQ4+wzgcbmCAnyQCZ9OlZTuDIKWa0dklr8E4LR8NKR46h09XL948R2Inz
# A6cT7W01EF68WMjkUO/8PbjN7PdRdPzoatl8yKggrnmgq0NNRkQCLLBeKpUwEKmQ
# MC9ZYs8J95uX933X7QmD7IS2C+rYSRRScBsgrTRSaK8K0cesd5iGkuW8gZBpU6Vb
# 9zIjqyaI5v5K6KedNB3mTifXEPMxQlWVtOIXAcn5shsFsGdtsw1PFSel7z8DKBcL
# UzPpd4pPYVChhP600U1a7CBZnaen9EBqmgvOkCmFkrqA1eV7GVp8HEyhcRhg1KrA
# CzIshbbNAvoXaTLWLI14JpuXs1qMrC5yjGXUznDdPpy7oTafZ6cMuu6c315SJldV
# /iSz4JTj1tfqlCGxikqcXPJHkK+lE672AVhonQs+s9cL9iCGvNobhV43CUkDaJTq
# TAT0Q4w=
# SIG # End signature block
