function Get-NtapClusterDpsaData {
    <#
        .SYNOPSIS
            Collects data from ONTAP storage systems for generating a DPSA report.

        .DESCRIPTION
            Uses the NetApp ONTAP PowerShell Toolkit to gather information required for the DPSA deliverable.

        .EXAMPLE
            PS> Get-NtapClusterDpsaData -Name cluster1 | Export-Clixml -Path 'D:\Output\cluster1.xml'

            Collects the configuration information off the cluster 'cluster1' and saves the results to a .xml file for further
            processing.

        .EXAMPLE
            PS> Get-NtapClusterDpsaData -Name cluster1 -Credential (Get-Credential) | Export-Clixml -Path 'D:\Output\cluster1.xml'

            Collects the configuration information off the cluster 'cluster1' using alternate credentials. The Add-NcCredential
            cmdlet may be used to cache credentials for the cluster to be accessed.

        .EXAMPLE
            PS> Get-NtapClusterDpsaData -Name cluster1 | Format-NtapClusterDpsaData | Out-NtapDocument -ExcelFile 'D:\Output\cluster1.xlsx'

            Collects the configuration information from the cluster 'cluster1', formats the data and creates the specified
            Excel document.

        .LINK
            https://community.netapp.com/t5/NetAppDocs/bd-p/netappdocs

        .LINK
            Format-NtapClusterDpsaData

        .INPUTS
            NetApp.Ontapi.AbstractController

        .INPUTS
            System.String

        .OUTPUTS
            System.Management.Automation.PSCustomObject
    #>

    [CmdletBinding(DefaultParameterSetName = 'Name')]
    [OutputType([System.Management.Automation.PSCustomObject])]
    param (
        # The system name or IP address of the cluster admin SVM to gather the data from.
        [Parameter(Mandatory, Position = 0, ParameterSetName = 'Name', ValueFromPipeline, ValueFromPipelineByPropertyName)]
        [ValidateNotNullOrEmpty()]
        [Alias('ClusterName', 'SystemName')]
        [string[]]$Name,

        # A NetApp ONTAP PowerShell Toolkit controller object (NcController).
        [Parameter(Mandatory, Position = 0, ParameterSetName = 'ControllerObject', ValueFromPipeline)]
        [ValidateNotNullOrEmpty()]
        [NetApp.Ontapi.AbstractController[]]$AbstractController,

        # Specifies explicit credentials to connect to the storage system. (Alias: Cred)
        [Parameter(ParameterSetName = 'Name')]
        [ValidateNotNullOrEmpty()]
        [Alias('Cred')]
        [pscredential]
        [System.Management.Automation.Credential()]
        $Credential = [pscredential]::Empty,

        # The port on which to connect to the storage system. If not set, it defaults to 80 for HTTP and 443 for HTTPS.
        [Parameter(ParameterSetName = 'Name')]
        [ValidateRange([uint16]::MinValue, [uint16]::MaxValue)]
        [uint16]$Port,

        # Connection timeout in milliseconds. Defaults to 60000 milliseconds.
        [Parameter()]
        [ValidateRange(0, [int]::MaxValue)]
        [int]$Timeout,

        # Forces the use of HTTPS instead of HTTP for the connection to the storage system.
        [Parameter(ParameterSetName = 'Name')]
        [Alias('ForceHTTPS')]
        [switch]$ForceSecure,

        # During the initial connection to the storage system, this switch disables a check that is performed to ensure that the LIF
        # being used for the connection is a cluster mgmt or a node mgmt port. This should only be used for a very specific scenario
        # where port forwarding is configured on the host connecting to the storage system.
        [Parameter()]
        [switch]$DisableLifCheck,

        # Disables the ping test prior to connecting to the storage system (useful in the event ICMP is being blocked). (Deprecated)
        [Parameter()]
        [ValidateScript({
                Write-Warning -Message '-DisablePingCheck has been deprecated (no longer performing an ICMP ping to test connectivity).'

                $true
            })]
        [switch]$DisablePingCheck,

        # Disables the manual generation of an ASUP for the storage system. (Deprecated)
        [Parameter()]
        [ValidateScript({
                Write-Warning -Message 'The -DisableAsupInvoke switch has been deprecated. The function no longer triggers ASUPs by default (use the -InvokeAsup switch to explicitly trigger ASUPs)'

                $true
            })]
        [switch]$DisableAsupInvoke,

        # Invokes a user generated ASUP for the storage system.
        [Parameter()]
        [switch]$InvokeAsup
    )

    process {
        Write-Verbose -Message "Initializing function: $( $PSCmdlet.MyInvocation.MyCommand.Name ) ($( $PSCmdlet.MyInvocation.MyCommand.ModuleName ) v$( (Get-NtapDocsVersion -AsVersion).ToString() ))"

        switch -Exact ($PSCmdlet.ParameterSetName) {
            'Name' {
                foreach ($cluster in ($Name | Where-Object { -not [string]::IsNullOrWhiteSpace($_) })) {
                    try {
                        Write-Verbose -Message "Validating connectivity to system: $cluster"

                        $ip = [System.Net.Dns]::GetHostAddresses($cluster)
                        $tcpClient = New-Object -TypeName System.Net.Sockets.TCPClient

                        if ($PSCmdlet.MyInvocation.BoundParameters.ContainsKey('Port')) {
                            $tcpConnect = $tcpClient.BeginConnect($ip, $Port, $null, $null)

                            if (-not $tcpConnect.AsyncWaitHandle.WaitOne(7500, $false)) {
                                $tcpClient.Close()

                                throw 'Connection failed (ResourceUnavailable).'
                            }
                        }
                        elseif ($ForceSecure.IsPresent) {
                            $tcpConnect = $tcpClient.BeginConnect($ip, 443, $null, $null)

                            if (-not $tcpConnect.AsyncWaitHandle.WaitOne(7500, $false)) {
                                $tcpClient.Close()

                                throw 'Connection failed (ResourceUnavailable).'
                            }
                        }
                        else {
                            $tcpConnect = $tcpClient.BeginConnect($ip, 443, $null, $null)

                            if (-not $tcpConnect.AsyncWaitHandle.WaitOne(7500, $false)) {
                                $tcpClient.Close()

                                $tcpClient = New-Object -TypeName System.Net.Sockets.TCPClient
                                $tcpConnect = $tcpClient.BeginConnect($ip, 80, $null, $null)

                                if (-not $tcpConnect.AsyncWaitHandle.WaitOne(7500, $false)) {
                                    $tcpClient.Close()

                                    throw 'Connection failed (ResourceUnavailable).'
                                }
                            }
                        }

                        [void]$tcpClient.EndConnect($tcpConnect)
                        $tcpClient.Close()

                        $params = @{
                            Name        = $cluster
                            ForceSecure = $ForceSecure.IsPresent
                        }

                        if ($Credential -ne [pscredential]::Empty) {
                            $params.Add('Credential', $Credential)
                        }

                        if ($PSCmdlet.MyInvocation.BoundParameters.ContainsKey('Port')) {
                            $params.Add('Port', $Port)
                        }

                        if ($PSCmdlet.MyInvocation.BoundParameters.ContainsKey('Timeout')) {
                            $params.Add('Timeout', $Timeout)
                        }

                        $controller = Connect-NtapStorageSystem @params -NcController -ErrorAction Stop

                        if ($controller.GetType().FullName -ne 'NetApp.Ontapi.Filer.C.NcController') {
                            throw 'Incorrect controller type specified for the Get-NtapClusterDpsaData function.'
                        }

                        if (-not (Test-NtapStorageSystemConnection -Controller $controller -DisableLifCheck:$DisableLifCheck.IsPresent -ErrorAction Stop)) {
                            throw 'Unspecified error.'
                        }

                        Write-Verbose -Message "Valid connection established to system: $cluster"
                    }
                    catch {
                        $Global:Error.RemoveAt(0)

                        Write-Error -ErrorRecord (New-Object -TypeName System.Management.Automation.ErrorRecord -ArgumentList ("Unable to gather data from system. Reason: $( $_.Exception.GetBaseException().Message )", $_.FullyQualifiedErrorId, $_.CategoryInfo.Category, $cluster))

                        continue
                    }

                    Write-Verbose -Message "Gathering data from system: $( $controller.Name )"

                    try {
                        $functionStartTime = [datetime]::Now

                        $ntapData = Get-ClusterDpsaData -Controller $controller

                        $ntapData

                        if ($ntapData -and $InvokeAsup.IsPresent) {
                            foreach ($node in ($ntapData.Node | Where-Object { $_ })) {
                                try {
                                    Write-Verbose -Message "Invoking ASUP on node: $( $node.Node )"
                                    [void](Invoke-NcAutoSupport -Node $node.Node -Message "NetAppDocs_$( $ntapData.ReferenceDT.ToString('u') )" -Type all -Controller $controller)
                                }
                                catch {
                                    $Global:Error.RemoveAt(0)

                                    Show-ErrorMessage -Message "Unable to invoke ASUP on node: $( $node.Node )"
                                }
                            }
                        }

                        $statsParams = @{
                            FunctionPSCallStack = Get-PSCallStack
                            TargetDetails       = [pscustomobject]@{
                                SystemUid     = $ntapData.Cluster.Info.ClusterUuid
                                SystemType    = $ntapData.pstypenames[0]
                                SystemVersion = ([regex]::Split(([regex]::Split($ntapData.System.VersionInfo.Version, 'Release\s(?:\w+[__])?')[1]), '\s+'))[0].Replace(':', '')
                            }
                            ElapsedTicks        = ([datetime]::Now - $functionStartTime).Ticks
                            ExcludeParameter    = 'Name'
                        }

                        Write-Statistics @statsParams
                    }
                    catch {
                        Write-Error -ErrorRecord $_
                    }
                }
            }
            'ControllerObject' {
                foreach ($controller in ($AbstractController | Where-Object { $_ })) {
                    Write-Verbose -Message "Validating connectivity to system: $( $controller.Name )"

                    try {
                        if ($controller.GetType().FullName -ne 'NetApp.Ontapi.Filer.C.NcController') {
                            throw 'Incorrect controller type specified for the Get-NtapClusterDpsaData function.'
                        }

                        if ($controller.Protocol -like 'HTTP*' -and $PSCmdlet.MyInvocation.BoundParameters.ContainsKey('Timeout')) {
                            $controller.TimeoutMsec = $Timeout
                        }

                        if (-not (Test-NtapStorageSystemConnection -Controller $controller -DisableLifCheck:$DisableLifCheck.IsPresent -ErrorAction Stop)) {
                            throw 'Unspecified error.'
                        }

                        Write-Verbose -Message "Valid connection established to system: $( $controller.Name )"
                    }
                    catch {
                        $Global:Error.RemoveAt(0)

                        Write-Error -ErrorRecord (New-Object -TypeName System.Management.Automation.ErrorRecord -ArgumentList ("Unable to gather data from system. Reason: $( $_.Exception.GetBaseException().Message )", $_.FullyQualifiedErrorId, $_.CategoryInfo.Category, $controller.Name))

                        continue
                    }

                    Write-Verbose -Message "Gathering data from system: $( $controller.Name )"

                    try {
                        $functionStartTime = [datetime]::Now

                        $ntapData = Get-ClusterDpsaData -Controller $controller

                        $ntapData

                        if ($ntapData -and $InvokeAsup.IsPresent) {
                            foreach ($node in ($ntapData.Node | Where-Object { $_ })) {
                                try {
                                    Write-Verbose -Message "Invoking ASUP on node: $( $node.Node )"
                                    [void](Invoke-NcAutoSupport -Node $node.Node -Message "NetAppDocs_$( $ntapData.ReferenceDT.ToString('u') )" -Type all -Controller $controller)
                                }
                                catch {
                                    $Global:Error.RemoveAt(0)

                                    Show-ErrorMessage -Message "Unable to invoke ASUP on node: $( $node.Node )"
                                }
                            }
                        }

                        $statsParams = @{
                            FunctionPSCallStack  = Get-PSCallStack
                            TargetDetails        = [pscustomobject]@{
                                SystemUid     = $ntapData.Cluster.Info.ClusterUuid
                                SystemType    = $ntapData.pstypenames[0]
                                SystemVersion = ([regex]::Split(([regex]::Split($ntapData.System.VersionInfo.Version, 'Release\s(?:\w+[__])?')[1]), '\s+'))[0].Replace(':', '')
                            }
                            ElapsedTicks         = ([datetime]::Now - $functionStartTime).Ticks
                            ParameterToSetAsBool = 'AbstractController'
                        }

                        Write-Statistics @statsParams
                    }
                    catch {
                        Write-Error -ErrorRecord $_
                    }
                }
            }
        }
    }
}

function Get-ClusterDpsaData {
    [CmdletBinding()]
    param (
        [Parameter(Mandatory)]
        [ValidateNotNullOrEmpty()]
        [NetApp.Ontapi.Filer.C.NcController]$Controller
    )

    $systemData = [pscustomobject]@{ CollectionProfile = 'DPSA' }
    $systemData.pstypenames.Insert(0, 'NetAppDocs.NcController.Data')

    Write-Verbose -Message '- Capturing module version -'
    $systemData | Add-Member -MemberType NoteProperty -Name NetAppDocsVersion (Get-NtapDocsVersion -AsVersion)

    Write-Verbose -Message '- Capturing PowerShell version -'
    $systemData | Add-Member -MemberType NoteProperty -Name PowerShellVersion -Value $PSVersionTable.PSVersion

    $NetAppDocsError = @( )

    try {
        Write-Verbose -Message 'Calling Get-NcSystemApi'
        $useSystemCli = $null -ne (Get-NcSystemApi -Name 'system-cli*' -Controller $Controller -ErrorVariable +NetAppDocsError)

        #region Raw Sysconfig Data

        Write-Verbose -Message '- Collecting raw sysconfig data -'

        $sysconfigAc = $(
            if ($useSystemCli) {
                Write-Verbose -Message 'Calling Invoke-NcSystemApi (system node run -node * -command sysconfig -ac)'
                (Invoke-NcSystemApi -Request '<system-cli><args><arg>system node run -node * -command sysconfig -ac</arg></args></system-cli>' -Controller $Controller -ErrorVariable +NetAppDocsError).results.'cli-output'
            }
            else {
                Write-Verbose -Message 'Calling Invoke-NcSsh (system node run -node * -command sysconfig -ac)'
                Invoke-NcSsh -Command 'system node run -node * -command sysconfig -ac' -Controller $Controller -ErrorVariable +NetAppDocsError -Verbose:$false
            }
        )

        $sysconfigV = $(
            if ($useSystemCli) {
                Write-Verbose -Message 'Calling Invoke-NcSystemApi (system node run -node * -command sysconfig -v)'
                (Invoke-NcSystemApi -Request '<system-cli><args><arg>system node run -node * -command sysconfig -v</arg></args></system-cli>' -Controller $Controller -ErrorAction SilentlyContinue -ErrorVariable +NetAppDocsError).results.'cli-output'
            }
            else {
                Write-Verbose -Message 'Calling Invoke-NcSsh (system node run -node * -command sysconfig -v)'
                Invoke-NcSsh -Command 'system node run -node * -command sysconfig -v' -Controller $Controller -ErrorVariable +NetAppDocsError -Verbose:$false
            }
        )

        Write-Verbose -Message 'Calling Get-AdditionalSysconfigProperties'
        $sysconfigProperties = Get-AdditionalSysconfigProperties -Sysconfig $sysconfigV

        #endregion Raw Sysconfig Data

        #region System Information

        Write-Verbose -Message '- Gathering basic system information -'
        $systemData | Add-Member -MemberType NoteProperty -Name System -Value ([pscustomobject]@{ })

        Write-Verbose -Message 'Calling Get-NcSystemVersionInfo'
        $systemData.System | Add-Member -MemberType NoteProperty -Name VersionInfo -Value (Get-NcSystemVersionInfo -Controller $Controller -ErrorVariable +NetAppDocsError)

        Write-Verbose -Message 'Calling Get-NcSystemOntapiVersion'
        $systemData.System | Add-Member -MemberType NoteProperty -Name OntapiVersion -Value ([version]::Parse((Get-NcSystemOntapiVersion -Controller $Controller -ErrorVariable +NetAppDocsError).ToString()))

        $systemData.System | Add-Member -MemberType NoteProperty -Name CliSessionTimeout -Value $(
            if ($useSystemCli) {
                Write-Verbose -Message 'Calling Invoke-NcSystemApi (system timeout show)'
                ((Invoke-NcSystemApi -Request '<system-cli><args><arg>system timeout show</arg></args></system-cli>' -Controller $Controller -ErrorVariable +NetAppDocsError).results.'cli-output' | Convert-SystemCliTextInstance).CliSessionTimeout
            }
            else {
                Write-Verbose -Message 'Calling Invoke-NcSsh (system timeout show)'
                (Invoke-NcSsh -Command 'system timeout show' -Controller $Controller -ErrorVariable +NetAppDocsError -Verbose:$false | Convert-SystemCliFieldDelimitedText).CliSessionTimeout
            }
        )

        #endregion System Information

        #region Node Information

        Write-Verbose -Message '- Gathering node information-'

        Write-Verbose -Message 'Calling Get-NcNode'
        $systemData | Add-Member -MemberType NoteProperty -Name Node -Value (Get-NcNode -Controller $Controller -ErrorVariable +NetAppDocsError | Where-Object { -not [string]::IsNullOrWhiteSpace($_.Node) })

        #BURT 520762
        if ($systemData.System.OntapiVersion -lt [version]::Parse('1.17')) {
            $clusterShowCliOutput = $(
                if ($useSystemCli) {
                    Write-Verbose -Message 'Calling Invoke-NcSystemApi (cluster show (priv advanced))'
                    (Invoke-NcSystemApi -Request '<system-cli><priv>advanced</priv><args><arg>set -showseparator ~;</arg><arg>set -showallfields true;</arg><arg>cluster show</arg></args></system-cli>' -Controller $Controller -ErrorVariable +NetAppDocsError).results.'cli-output' | Convert-SystemCliFieldDelimitedText
                }
                else {
                    Write-Verbose -Message 'Calling Invoke-NcSsh (cluster show (priv advanced))'
                    Invoke-NcSsh -Command 'set -privilege advanced -confirmations off; set -showseparator ~; set -showallfields true; cluster show' -Controller $Controller -ErrorVariable +NetAppDocsError -Verbose:$false | Convert-SystemCliFieldDelimitedText
                }
            )

            foreach ($node in ($systemData.Node | Where-Object { $_ })) {
                $node | Add-Member -NotePropertyMembers @{
                    IsEpsilonNode         = ($clusterShowCliOutput | Where-Object { $_.NodeUuid -ceq $node.NodeUuid }).Epsilon
                    IsNodeHealthy         = ($clusterShowCliOutput | Where-Object { $_.NodeUuid -ceq $node.NodeUuid }).Health
                    IsNodeClusterEligible = ($clusterShowCliOutput | Where-Object { $_.NodeUuid -ceq $node.NodeUuid }).Eligibility
                }
            }
        }

        if ($systemData.System.OntapiVersion -ge [version]::Parse('1.140')) {
            #This API will throw an error if cluster is set to non_ha mode
            try {
                Write-Verbose -Message 'Calling Get-NcSystemCacheMirror'
                $systemCacheMirror = Get-NcSystemCacheMirror -Controller $Controller -ErrorAction Stop

                foreach ($node in ($systemData.Node | Where-Object { $_ })) {
                    $node | Add-Member -MemberType NoteProperty -Name IsCacheMirrorEnabled -Value ($systemCacheMirror | Where-Object { $_.Node -ceq $node.Node }).Enabled
                }
            }
            catch {
                if ($_.Exception.Message -like '*The cache mirror feature requires a HA configuration*') {
                    $Global:Error.RemoveAt(0)
                }
                else {
                    $NetAppDocsError += $_
                }
            }
        }

        Write-Verbose -Message 'Calling Get-NcNodeInfo'
        $systemData | Add-Member -MemberType NoteProperty -Name NodeInfo -Value (Get-NcNodeInfo -Controller $Controller -ErrorVariable +NetAppDocsError)

        Write-Verbose -Message 'Identifying additional properties from parsed sysconfig'
        foreach ($node in ($systemData.NodeInfo | Where-Object { $_ })) {
            $node | Add-Member -MemberType NoteProperty -Name IoxmPresent -Value ([bool]($sysconfigProperties | Where-Object { $_.NodeName -ceq $node.SystemName }).IoxmPresent)

            $node | Add-Member -NotePropertyMembers @{
                PlatformConfig = Get-PlatformConfig -SystemModel $node.SystemModel -IsClustered (! [string]::IsNullOrWhiteSpace($node.PartnerSystemId)) -IoxmPresent $node.IoxmPresent
                LoaderVersion  = ($sysconfigProperties | Where-Object { $_.NodeName -ceq $node.SystemName }).LoaderVersion
            }
        }

        #endregion Node Information

        #region Cluster Information

        Write-Verbose -Message '- Gathering cluster information -'
        $systemData | Add-Member -MemberType NoteProperty -Name Cluster -Value ([pscustomobject]@{ })

        Write-Verbose -Message 'Calling Get-NcCluster'
        $systemData.Cluster | Add-Member -MemberType NoteProperty -Name Info -Value (Get-NcCluster -Controller $Controller -ErrorVariable +NetAppDocsError)

        $systemData.Cluster | Add-Member -MemberType NoteProperty -Name RingReplication -Value $(
            if ($useSystemCli) {
                Write-Verbose -Message 'Calling Invoke-NcSystemApi (cluster ring show (priv advanced))'
                (Invoke-NcSystemApi -Request '<system-cli><priv>advanced</priv><args><arg>set -showseparator ~;</arg><arg>set -showallfields true;</arg><arg>cluster ring show</arg></args></system-cli>' -Controller $Controller -ErrorVariable +NetAppDocsError).results.'cli-output' | Convert-SystemCliFieldDelimitedText
            }
            else {
                Write-Verbose -Message 'Calling Invoke-NcSsh (cluster ring show (priv advanced))'
                Invoke-NcSsh -Command 'set -privilege advanced -confirmations off; set -showseparator ~; set -showallfields true; cluster ring show' -Controller $Controller -ErrorVariable +NetAppDocsError -Verbose:$false | Convert-SystemCliFieldDelimitedText
            }
        )

        if ($systemData.System.OntapiVersion -ge [version]::Parse('1.30')) {
            $systemData.Cluster | Add-Member -MemberType NoteProperty -Name Contact -Value $(
                try {
                    Write-Verbose -Message 'Calling Get-NcClusterContact'
                    Get-NcClusterContact -Controller $Controller -ErrorAction Stop
                }
                catch {
                    if ($_.Exception.Message -like "*entry doesn't exist*") {
                        $Global:Error.RemoveAt(0)
                    }
                    else {
                        $NetAppDocsError += $_
                    }
                }
            )
        }

        #endregion Cluster Information

        #region Cluster HA Information

        Write-Verbose -Message '- Gathering cluster HA information -'
        $systemData.Cluster | Add-Member -MemberType NoteProperty -Name Ha -Value ([pscustomobject]@{ })

        #If we use the new NcClusterHaInfo cmdlet in the future, we will need to change this 'Info' to 'Status' and update types files accordingly
        Write-Verbose -Message 'Calling Get-NcClusterHa'
        $systemData.Cluster.Ha | Add-Member -MemberType NoteProperty -Name Info -Value ($systemData.Node | Get-NcClusterHa -Controller $Controller -ErrorVariable +NetAppDocsError)

        #ZAPI: HA Configured property - 'cluster-ha-get' introduced in 8.2 (use system-cli to get this property for older releases) - This ZAPI may be removed in the future.
        $systemData.Cluster.Ha | Add-Member -MemberType NoteProperty -Name Configured -Value $(
            if ($systemData.System.OntapiVersion -ge [version]::Parse('1.20')) {
                Write-Verbose -Message 'Calling Invoke-NcSystemApi (cluster-ha-get)'
                ((Invoke-NcSystemApi -Request '<cluster-ha-get />' -Controller $Controller -ErrorVariable +NetAppDocsError).results.attributes.'cluster-ha-info' | Convert-GenericXmlNode).HaConfigured
            }
            else {
                if ($useSystemCli) {
                    Write-Verbose -Message 'Calling Invoke-NcSystemApi (cluster ha show (priv advanced))'
                    ((Invoke-NcSystemApi -Request '<system-cli><priv>advanced</priv><args><arg>off;</arg><arg>cluster ha show</arg></args></system-cli>' -Controller $Controller -ErrorVariable +NetAppDocsError).results.'cli-output' | Convert-SystemCliTextInstance).HighAvailabilityConfigured
                }
                else {
                    Write-Verbose -Message 'Calling Invoke-NcSsh (cluster ha show (priv advanced))'
                    (Invoke-NcSsh -Command 'set -privilege advanced -confirmations off; cluster ha show' -Controller $Controller -ErrorVariable +NetAppDocsError -Verbose:$false | Convert-SystemCliTextInstance).HighAvailabilityConfigured
                }
            }
        )

        #endregion Cluster HA Information

        #region Cluster LogForward Information

        if ($systemData.System.OntapiVersion -ge [version]::Parse('1.31')) {
            Write-Verbose -Message '- Gathering Cluster LogForward information -'
            $systemData.Cluster | Add-Member -MemberType NoteProperty -Name LogForward -Value ([pscustomobject]@{ })

            Write-Verbose -Message 'Calling Get-NcClusterLogForward'
            $systemData.Cluster.LogForward | Add-Member -MemberType NoteProperty -Name Info -Value (Get-NcClusterLogForward -Controller $Controller -ErrorVariable +NetAppDocsError)
        }

        #endregion Cluster LogForward Information

        #region Timezone Information

        Write-Verbose -Message 'Calling Get-NcTimezone'
        $systemData | Add-Member -MemberType NoteProperty -Name Timezone -Value (Get-NcTimezone -Controller $Controller -ErrorVariable +NetAppDocsError)

        #endregion Timezone Information

        #region License Information

        Write-Verbose -Message '- Gathering license information -'
        $systemData | Add-Member -MemberType NoteProperty -Name License -Value ([pscustomobject]@{ })

        #License v2.0 zapis introduced in 8.2. Prior to that, the unsupported interim-license-list-get zapi must be used to get license information
        $systemData.License | Add-Member -MemberType NoteProperty -Name Info -Value $(
            if ($systemData.System.OntapiVersion -ge [version]::Parse('1.20')) {
                Write-Verbose -Message 'Calling Get-NcLicense'
                Get-NcLicense -Controller $Controller -ErrorVariable +NetAppDocsError
            }
            else {
                Write-Verbose -Message 'Calling Invoke-NcSystemApi (interim-license-list-get)'
                ((Invoke-NcSystemApi -Request '<interim-license-list-get />' -Controller $Controller -ErrorVariable +NetAppDocsError).results.licenses.'interim-license-cluster-info' | Convert-GenericXmlNode)
            }
        )

        #endregion License Information

        #region NTP Server Information

        if ($systemData.System.OntapiVersion -ge [version]::Parse('1.30')) {
            Write-Verbose -Message '- Gathering Ntp server information -'
            $systemData | Add-Member -MemberType NoteProperty -Name Ntp -Value ([pscustomobject]@{ })

            Write-Verbose -Message 'Calling Get-NcNtpServer'
            $systemData.Ntp | Add-Member -MemberType NoteProperty -Name Server -Value (Get-NcNtpServer -Controller $Controller -ErrorVariable +NetAppDocsError)
        }

        #endregion NTP Server Information

        #region System Services Data

        Write-Verbose -Message '- Gathering system services data -'
        $systemData | Add-Member -MemberType NoteProperty -Name SystemServices -Value ([pscustomobject]@{ })

        $systemData.SystemServices | Add-Member -MemberType NoteProperty -Name Web -Value $(
            if ($systemData.System.OntapiVersion -lt [version]::Parse('1.31')) {
                $systemServicesWebShow = $(
                    if ($useSystemCli) {
                        Write-Verbose -Message 'Calling Invoke-NcSystemApi (system services web show)'
                        (Invoke-NcSystemApi -Request '<system-cli><args><arg>system services web show</arg></args></system-cli>' -Controller $Controller -ErrorVariable +NetAppDocsError).results.'cli-output' | Convert-SystemCliTextInstance
                    }
                    else {
                        Write-Verbose -Message 'Calling Invoke-NcSsh (system services web show)'
                        Invoke-NcSsh -Command 'system services web show' -Controller $Controller -ErrorVariable +NetAppDocsError -Verbose:$false | Convert-SystemCliTextInstance
                    }
                )

                $systemServicesWebShow | Select-Object -ExcludeProperty HttpProtocolPort, HttpsProtocolPort, ExternalWebServices, SslFips1402Enabled -Property *, @{
                    Name       = 'HttpPort'
                    Expression = { $_.HttpProtocolPort }
                }, @{
                    Name       = 'HttpsPort'
                    Expression = { $_.HttpsProtocolPort }
                }, @{
                    Name       = 'External'
                    Expression = { $_.ExternalWebServices }
                }, @{
                    Name       = 'SslFipsEnabled'
                    Expression = { $_.SslFips1402Enabled }
                }
            }
            else {
                Write-Verbose -Message 'Calling Get-NcSystemWebServices'
                Get-NcSystemWebServices -Controller $Controller -ErrorVariable +NetAppDocsError
            }
        )

        $systemData.SystemServices | Add-Member -MemberType NoteProperty -Name WebNode -Value $(
            if ($systemData.System.OntapiVersion -ge [version]::Parse('1.100')) {
                Write-Verbose -Message 'Calling Get-NcSystemServicesWebNode'
                Get-NcSystemServicesWebNode -Controller $Controller -ErrorVariable +NetAppDocsError
            }
            else {
                if ($useSystemCli) {
                    Write-Verbose -Message 'Calling Invoke-NcSystemApi (system services web node show)'
                    (Invoke-NcSystemApi -Request '<system-cli><args><arg>set -showseparator ~;</arg><arg>set -showallfields true;</arg><arg>system services web node show</arg></args></system-cli>' -Controller $Controller -ErrorVariable +NetAppDocsError).results.'cli-output' | Convert-SystemCliFieldDelimitedText
                }
                else {
                    Write-Verbose -Message 'Calling Invoke-NcSsh (system services web node show)'
                    Invoke-NcSsh -Command 'set -showseparator ~; set -showallfields true; system services web node show' -Controller $Controller -ErrorVariable +NetAppDocsError -Verbose:$false | Convert-SystemCliFieldDelimitedText
                }
            }
        )

        #endregion System Services Data

        #region Adapter Card Inventory

        Write-Verbose -Message '- Identifying adapter cards -'

        Write-Verbose -Message 'Calling Get-NtapExpansionSlotInventory'
        $systemData | Add-Member -MemberType NoteProperty -Name AdapterCard -Value (Get-NtapExpansionSlotInventory -SysconfigAc $sysconfigAc -SysconfigVorA $sysconfigV | Sort-Object -Property NodeName, SlotNumber)

        #endregion Adapter Card Inventory

        #region NDMP

        Write-Verbose -Message '- Gathering NDMP configuration -'
        $systemData | Add-Member -MemberType NoteProperty -Name Ndmp -Value ([pscustomobject]@{ })

        $systemData.Ndmp | Add-Member -MemberType NoteProperty -Name SystemServices -Value $(
            if ($useSystemCli) {
                Write-Verbose -Message 'Calling Invoke-NcSystemApi (system services ndmp show)'
                (Invoke-NcSystemApi -Request '<system-cli><args><arg>set -showseparator ~;</arg><arg>set -showallfields true;</arg><arg>system services ndmp show</arg></args></system-cli>' -Controller $Controller -ErrorVariable +NetAppDocsError).results.'cli-output' | Convert-SystemCliFieldDelimitedText
            }
            else {
                Write-Verbose -Message 'Calling Invoke-NcSsh (system services ndmp show)'
                Invoke-NcSsh -Command 'set -showseparator ~; set -showallfields true; system services ndmp show' -Controller $Controller -ErrorVariable +NetAppDocsError -Verbose:$false | Convert-SystemCliFieldDelimitedText
            }
        )

        if ($systemData.System.OntapiVersion -ge [version]::Parse('1.20')) {
            $systemData.Ndmp | Add-Member -MemberType NoteProperty -Name NodeScopeMode -Value (
                [regex]::Matches(
                    $(
                        if ($useSystemCli) {
                            Write-Verbose -Message 'Calling Invoke-NcSystemApi (system services ndmp node-scope-mode status)'
                            (Invoke-NcSystemApi -Request '<system-cli><args><arg>system services ndmp node-scope-mode status</arg></args></system-cli>' -Controller $Controller -ErrorVariable +NetAppDocsError).results.'cli-output'
                        }
                        else {
                            Write-Verbose -Message 'Calling Invoke-NcSsh (system services ndmp node-scope-mode status)'
                            Invoke-NcSsh -Command 'system services ndmp node-scope-mode status' -Controller $Controller -ErrorVariable +NetAppDocsError -Verbose:$false
                        }
                    ),
                    'enabled|disabled'
                )[0].Value
            )

            if ($systemData.Ndmp.NodeScopeMode -eq 'disabled') {
                $systemData.Ndmp | Add-Member -MemberType NoteProperty -Name VserverServices -Value $(
                    if ($useSystemCli) {
                        Write-Verbose -Message 'Calling Invoke-NcSystemApi (vserver services ndmp show)'
                        (Invoke-NcSystemApi -Request '<system-cli><args><arg>set -showseparator ~;</arg><arg>set -showallfields true;</arg><arg>vserver services ndmp show</arg></args></system-cli>' -Controller $Controller -ErrorVariable +NetAppDocsError).results.'cli-output' | Convert-SystemCliFieldDelimitedText
                    }
                    else {
                        Write-Verbose -Message 'Calling Invoke-NcSsh (vserver services ndmp show)'
                        Invoke-NcSsh -Command 'set -showseparator ~; set -showallfields true; vserver services ndmp show' -Controller $Controller -ErrorVariable +NetAppDocsError -Verbose:$false | Convert-SystemCliFieldDelimitedText
                    }
                )
            }
        }

        #endregion NDMP

        #region Remote Management Configuration

        Write-Verbose -Message '- Gathering remote management configuration -'
        $systemData | Add-Member -MemberType NoteProperty -Name ServiceProcessor -Value ([pscustomobject]@{ })

        if ($systemData.System.OntapiVersion -ge [version]::Parse('1.30')) {
            Write-Verbose -Message 'Calling Get-NcServiceProcessor'
            $systemData.ServiceProcessor | Add-Member -MemberType NoteProperty -Name Info -Value (Get-NcServiceProcessor -Controller $Controller -ErrorVariable +NetAppDocsError)

            $systemData.ServiceProcessor | Add-Member -MemberType NoteProperty -Name Image -Value $(
                foreach ($node in ($systemData.NodeInfo | Where-Object { $_ })) {
                    foreach ($firmwareImage in 'primary', 'backup') {
                        Write-Verbose -Message "Calling Get-NcServiceProcessorImage (Node: $( $node.SystemName )) (FirmwareImage: $firmwareImage)"
                        try {
                            Get-NcServiceProcessorImage -Node $node.SystemName -FirmwareImage $firmwareImage -Controller $Controller -ErrorAction Stop | Where-Object { $_.FirmwareImage -eq $firmwareImage }
                        }
                        catch {
                            if ($_.Exception.Message -like "*entry doesn't exist*") {
                                $Global:Error.RemoveAt(0)
                            }
                            else {
                                $NetAppDocsError += $_
                            }
                        }
                    }
                }
            )

            Write-Verbose -Message 'Calling Get-NcServiceProcessorNetwork'
            $systemData.ServiceProcessor | Add-Member -MemberType NoteProperty -Name Network -Value (Get-NcServiceProcessorNetwork -Controller $Controller -ErrorVariable +NetAppDocsError)

            try {
                Write-Verbose -Message 'Calling Get-NcServiceProcessorSsh'
                $systemData.ServiceProcessor | Add-Member -MemberType NoteProperty -Name Ssh -Value (Get-NcServiceProcessorSsh -Controller $Controller -ErrorAction Stop)
            }
            catch {
                if ($_.Exception.Message -like "*entry doesn't exist*") {
                    $Global:Error.RemoveAt(0)
                }
                else {
                    $NetAppDocsError += $_
                }
            }
        }
        elseif ($systemData.System.OntapiVersion -ge [version]::Parse('1.21')) {
            $systemData.ServiceProcessor | Add-Member -MemberType NoteProperty -Name Info -Value $(
                foreach ($node in ($systemData.NodeInfo | Where-Object { $_ })) {
                    Write-Verbose -Message "Calling Get-NcServiceProcessor (Node: $( $node.SystemName ))"
                    Get-NcServiceProcessor -Node $node.SystemName -Controller $Controller -ErrorVariable +NetAppDocsError
                }
            )

            $systemData.ServiceProcessor | Add-Member -MemberType NoteProperty -Name Image -Value $(
                foreach ($node in ($systemData.NodeInfo | Where-Object { $_ })) {
                    foreach ($firmwareImage in 'primary', 'backup') {
                        Write-Verbose -Message "Calling Get-NcServiceProcessorImage (Node: $( $node.SystemName )) (FirmwareImage: $firmwareImage)"
                        Get-NcServiceProcessorImage -Node $node.SystemName -FirmwareImage $firmwareImage -Controller $Controller -ErrorVariable +NetAppDocsError | Where-Object { $_.FirmwareImage -eq $firmwareImage }
                    }
                }
            )

            $systemData.ServiceProcessor | Add-Member -MemberType NoteProperty -Name Network -Value $(
                foreach ($node in ($systemData.NodeInfo | Where-Object { $_ })) {
                    'Ipv4', 'Ipv6' | ForEach-Object {
                        Write-Verbose -Message "Calling Get-NcServiceProcessorNetwork (Node: $( $node.SystemName )) (AddressType: $_)"
                        Get-NcServiceProcessorNetwork -Node $node.SystemName -AddressType $_ -Controller $Controller -ErrorVariable +NetAppDocsError
                    }
                }
            )
        }
        else {
            if (-not [string]::IsNullOrWhiteSpace($sysconfigV)) {
                Write-Verbose -Message 'Calling Get-NtapRemoteManagement'
                $systemData.ServiceProcessor | Add-Member -MemberType NoteProperty -Name Info -Value (Get-NtapRemoteManagement -Sysconfig $sysconfigV)
            }
            else {
                $sp = $(
                    switch -Wildcard ($sysconfigV) {
                        '*Service Processor*' {
                            if ($useSystemCli) {
                                Write-Verbose -Message 'Calling Invoke-NcSystemApi (system node run -node * -command sp status)'
                                (Invoke-NcSystemApi -Request '<system-cli><args><arg>system node run -node * -command sp status</arg></args></system-cli>' -Controller $Controller -ErrorVariable +NetAppDocsError).results.'cli-output'
                            }
                            else {
                                Write-Verbose -Message 'Calling Invoke-NcSsh (system node run -node * -command sp status)'
                                Invoke-NcSsh -Command 'system node run -node * -command sp status' -Controller $Controller -ErrorVariable +NetAppDocsError -Verbose:$false
                            }

                            break
                        }
                        '*Remote LAN Module*' {
                            if ($useSystemCli) {
                                Write-Verbose -Message 'Calling Invoke-NcSystemApi (system node run -node * -command rlm status)'
                                (Invoke-NcSystemApi -Request '<system-cli><args><arg>system node run -node * -command rlm status</arg></args></system-cli>' -Controller $Controller -ErrorVariable +NetAppDocsError).results.'cli-output'
                            }
                            else {
                                Write-Verbose -Message 'Calling Invoke-NcSsh (system node run -node * -command rlm status)'
                                Invoke-NcSsh -Command 'system node run -node * -command rlm status' -Controller $Controller -ErrorVariable +NetAppDocsError -Verbose:$false
                            }

                            break
                        }
                        '*Baseboard Management Controller*' {
                            if ($useSystemCli) {
                                Write-Verbose -Message 'Calling Invoke-NcSystemApi (system node run -node * -command bmc status)'
                                (Invoke-NcSystemApi -Request '<system-cli><args><arg>system node run -node * -command bmc status</arg></args></system-cli>' -Controller $Controller -ErrorVariable +NetAppDocsError).results.'cli-output'
                            }
                            else {
                                Write-Verbose -Message 'Calling Invoke-NcSsh (system node run -node * -command bmc status)'
                                Invoke-NcSsh -Command 'system node run -node * -command bmc status' -Controller $Controller -ErrorVariable +NetAppDocsError -Verbose:$false
                            }

                            break
                        }
                        default { }
                    }
                )

                if (-not [string]::IsNullOrWhiteSpace($sp)) {
                    Write-Verbose -Message 'Calling Get-NtapRemoteManagement'
                    $systemData.ServiceProcessor | Add-Member -MemberType NoteProperty -Name Info -Value (Get-NtapRemoteManagement -Sysconfig $sp)
                }
                else {
                    $NetAppDocsError += New-Object -TypeName System.Management.Automation.ErrorRecord -ArgumentList ('Unable to capture SP/RLM/BMC information.', 'NO_SP_INFORMATION', [System.Management.Automation.ErrorCategory]::InvalidData, $systemData.Info.ClusterName)
                }
            }
        }

        #endregion Remote Management Configuration
        #region Feature Usage Data

        Write-Verbose -Message '- Gathering feature usage data -'
        $systemData | Add-Member -MemberType NoteProperty -Name Feature -Value ([pscustomobject]@{ })

        if ($systemData.System.OntapiVersion -ge [version]::Parse('1.30')) {
            Write-Verbose -Message 'Calling Get-NcFeatureUsageSummary'
            $systemData.Feature | Add-Member -MemberType NoteProperty -Name UsageSummary -Value (Get-NcFeatureUsageSummary -Controller $Controller -ErrorVariable +NetAppDocsError)
        }

        #endregion Feature Usage Data

        #region SNMP Configuration

        Write-Verbose -Message '- Gathering SNMP information -'

        Write-Verbose -Message 'Calling Get-NcSnmp'
        $systemData | Add-Member -MemberType NoteProperty -Name Snmp -Value (Get-NcSnmp -Controller $Controller -ErrorVariable +NetAppDocsError)

        if ($systemData.System.OntapiVersion -ge [version]::Parse('1.120')) {
            $systemData.Snmp | Add-Member -MemberType NoteProperty -Name SnmpV3Enable -Value $(
                if ($useSystemCli) {
                    Write-Verbose -Message 'Calling Invoke-NcSystemApi (system snmp show -fields snmpv3-enable-value (priv advanced))'
                    ((Invoke-NcSystemApi -Request '<system-cli><priv>advanced</priv><args><arg>set -showseparator ~;</arg><arg>system snmp show -fields snmpv3-enable-value</arg></args></system-cli>' -Controller $Controller -ErrorVariable +NetAppDocsError).results.'cli-output' | Convert-SystemCliFieldDelimitedText).Snmpv3EnableValue
                }
                else {
                    Write-Verbose -Message 'Calling Invoke-NcSsh (system snmp show -fields snmpv3-enable-value (priv advanced))'
                    (Invoke-NcSsh -Command 'set -privilege advanced -confirmations off; set -showseparator ~; system snmp show -fields snmpv3-enable-value' -Controller $Controller -ErrorVariable +NetAppDocsError -Verbose:$false | Convert-SystemCliFieldDelimitedText).Snmpv3EnableValue
                }
            )
        }

        #endregion SNMP Configuration

        #region Syslog (EMS) Data

        Write-Verbose -Message '- Gathering EMS information -'
        $systemData | Add-Member -MemberType NoteProperty -Name Ems -Value ([pscustomobject]@{ })

        if ($systemData.System.OntapiVersion -lt [version]::Parse('1.100')) {
            Write-Verbose -Message 'Calling Get-NcEmsDestination'
            $systemData.Ems | Add-Member -MemberType NoteProperty -Name Destination -Value (Get-NcEmsDestination -Controller $Controller -ErrorVariable +NetAppDocsError)
        }

        if ($systemData.System.OntapiVersion -ge [version]::Parse('1.100')) {
            Write-Verbose -Message 'Calling Get-NcEmsEventFilter'
            $systemData.Ems | Add-Member -MemberType NoteProperty -Name EventFilter -Value (Get-NcEmsEventFilter -Controller $Controller -ErrorVariable +NetAppDocsError)

            $systemData.Ems | Add-Member -MemberType NoteProperty -Name EventNotification -Value ([pscustomobject]@{ })

            Write-Verbose -Message 'Calling Get-NcEmsEventNotification'
            $systemData.Ems.EventNotification | Add-Member -MemberType NoteProperty -Name Info -Value (Get-NcEmsEventNotification -Controller $Controller -ErrorVariable +NetAppDocsError)

            Write-Verbose -Message 'Calling Get-NcEmsEventNotificationDestination'
            $systemData.Ems.EventNotification | Add-Member -MemberType NoteProperty -Name Destination -Value (Get-NcEmsEventNotificationDestination -Controller $Controller -ErrorVariable +NetAppDocsError)
        }

        #endregion Syslog (EMS) Data

        #region Option Information

        Write-Verbose -Message '- Gathering options -'

        Write-Verbose -Message 'Calling Get-NcOption'
        $systemData | Add-Member -MemberType NoteProperty -Name Option -Value @( (Get-NcOption -Controller $Controller -ErrorVariable +NetAppDocsError) )

        $additionalOptions = @(
            'sis.max_active_ops'
            'sis.idedup_allow_non_aff_hya'
            'flexscale.flash_pool_caching'
            # The following properties must be explicitly called in 8.3.2 and below
            'raid.timeout'
            'raid.mirror_read_plex_pref'
        )

        $optionsRegex = '(?m)^Node[:]\s+(?<Node>.+?)(?:\s+Last login time[:]\s\S+\s\S+)?\s+(?<Name>.+?)\s+(?<Value>.+?)(?:\s+\(.*)?\s*$'

        foreach ($additionalOption in $additionalOptions) {
            if (-not ($systemData.Option | Where-Object { $_.Name -eq $additionalOption })) {
                $optionsCliOutput = $(
                    if ($useSystemCli) {
                        Write-Verbose -Message "Calling Invoke-NcSystemApi (system node run -node * -command options $additionalOption)"
                        (Invoke-NcSystemApi -Request "<system-cli><args><arg>system node run -node * -command options $additionalOption</arg></args></system-cli>" -Controller $Controller -ErrorVariable +NetAppDocsError).results.'cli-output'
                    }
                    else {
                        Write-Verbose -Message "Calling Invoke-NcSsh (system node run -node * -command options $additionalOption)"
                        Invoke-NcSsh -Command "system node run -node * -command options $additionalOption" -Controller $Controller -ErrorVariable +NetAppDocsError -Verbose:$false
                    }
                )

                [regex]::Matches($optionsCliOutput, $optionsRegex) | ForEach-Object {
                    $option = New-Object -TypeName DataONTAP.C.Types.Options.OptionInfo
                    $option.ClusterConstraint = 'none'
                    $option.PrivLevel = 'admin'

                    $option.Vserver, $option.Name, $option.Value = $_.Groups[1, 2, 3].Value

                    $systemData.Option += $option
                }
            }
        }

        #endregion Option Information

        #region Aggregate Status

        Write-Verbose -Message '- Gathering detailed aggregate information -'
        $systemData | Add-Member -MemberType NoteProperty -Name Aggr -Value ([pscustomobject]@{ })

        Write-Verbose -Message 'Calling Get-NcAggr'
        $systemData.Aggr | Add-Member -MemberType NoteProperty -Name Info -Value (Get-NcAggr -Controller $Controller -ErrorVariable +NetAppDocsError)

        $systemData.Aggr | Add-Member -MemberType NoteProperty -Name Option -Value @( )

        foreach ($aggr in ($systemData.Aggr.Info | Where-Object { $_ -and ($_.State -ne 'unknown' -and $_.State -ne 'remote_cluster') })) {
            Write-Verbose -Message "Calling Get-NcAggrOption (Aggr: $( $aggr.Name ))"
            $systemData.Aggr.Option += (Get-NcAggrOption -Name $aggr.Name -Controller $Controller -Hashtable -ErrorVariable +NetAppDocsError).Value | Add-Member -MemberType NoteProperty -Name AggregateName -Value $aggr.Name -PassThru
        }

        $aggrSnapSched = $(
            if ($useSystemCli) {
                Write-Verbose -Message 'Calling Invoke-NcSystemApi (system node run -node * -command snap sched -A)'
                (Invoke-NcSystemApi -Request '<system-cli><args><arg>system node run -node * -command snap sched -A</arg></args></system-cli>' -Controller $Controller -ErrorVariable +NetAppDocsError).results.'cli-output'
            }
            else {
                Write-Verbose -Message 'Calling Invoke-NcSsh (system node run -node * -command snap sched -A)'
                Invoke-NcSsh -Command 'system node run -node * -command snap sched -A' -Controller $Controller -ErrorVariable +NetAppDocsError -Verbose:$false
            }
        )

        $aggrSnapList = $(
            if ($useSystemCli) {
                Write-Verbose -Message 'Calling Invoke-NcSystemApi (system node run -node * -command snap list -A)'
                (Invoke-NcSystemApi -Request '<system-cli><args><arg>system node run -node * -command snap list -A</arg></args></system-cli>' -Controller $Controller -ErrorVariable +NetAppDocsError).results.'cli-output'
            }
            else {
                Write-Verbose -Message 'Calling Invoke-NcSsh (system node run -node * -command snap list -A)'
                Invoke-NcSsh -Command 'system node run -node * -command snap list -A' -Controller $Controller -ErrorVariable +NetAppDocsError -Verbose:$false
            }
        )

        Write-Verbose -Message 'Calling Get-AggrSnapInformation'
        $systemData.Aggr | Add-Member -MemberType NoteProperty -Name Snapshot -Value (Get-AggrSnapInformation -SnapSched $aggrSnapSched -SnapList $aggrSnapList)

        if ($systemData.System.OntapiVersion -ge [version]::Parse('1.30')) {
            Write-Verbose -Message 'Calling Get-NcAggrSpace'
            $systemData.Aggr | Add-Member -MemberType NoteProperty -Name Space -Value (Get-NcAggrSpace -Controller $Controller -ErrorVariable +NetAppDocsError)
            if ($systemData.System.OntapiVersion -ge [version]::Parse('1.120')) {
                $systemData.Aggr | Add-Member -MemberType NoteProperty -Name ObjectStore -Value ([pscustomobject]@{ })

                Write-Verbose -Message 'Calling Get-NcAggrObjectStore'
                $systemData.Aggr.ObjectStore | Add-Member -MemberType NoteProperty -Name Info -Value (Get-NcAggrObjectStore -Controller $Controller -ErrorVariable +NetAppDocsError)

                Write-Verbose -Message 'Calling Get-NcAggrObjectStoreConfig'
                $systemData.Aggr.ObjectStore | Add-Member -MemberType NoteProperty -Name Config -Value (Get-NcAggrObjectStoreConfig -Controller $Controller -ErrorVariable +NetAppDocsError)

                if ($systemData.System.OntapiVersion -ge [version]::Parse('1.130')) {
                    $systemData.Aggr | Add-Member -MemberType NoteProperty -Name EfficiencyConfig -Value $(
                        if ($useSystemCli) {
                            Write-Verbose -Message 'Calling Invoke-NcSystemApi (storage aggregate efficiency show)'
                            (Invoke-NcSystemApi -Request '<system-cli><args><arg>set -showseparator ~;</arg><arg>set -showallfields true;</arg><arg>storage aggregate efficiency show</arg></args></system-cli>' -Controller $Controller -ErrorVariable +NetAppDocsError).results.'cli-output' | Convert-SystemCliFieldDelimitedText
                        }
                        else {
                            Write-Verbose -Message 'Calling Invoke-NcSsh (storage aggregate efficiency show)'
                            Invoke-NcSsh -Command 'set -showseparator ~; set -showallfields true; storage aggregate efficiency show' -Controller $Controller -ErrorVariable +NetAppDocsError -Verbose:$false | Convert-SystemCliFieldDelimitedText
                        }
                    )
                }
            }
        }

        #endregion Aggregate Status

        #region Disk Information

        Write-Verbose -Message '- Gathering physical disk information -'

        Write-Verbose -Message 'Calling Get-NcDisk'
        $systemData | Add-Member -MemberType NoteProperty -Name Disk -Value (Get-NcDisk -Controller $Controller -ErrorVariable +NetAppDocsError)

        if ($systemData.System.OntapiVersion -ge [version]::Parse('1.30')) {
            $systemData | Add-Member -MemberType NoteProperty -Name PartitionDisk -Value $(
                if ($useSystemCli) {
                    Write-Verbose -Message 'Calling Invoke-NcSystemApi (storage disk partition show (priv diag))'
                    (Invoke-NcSystemApi -Request '<system-cli><args><arg>set -privilege diagnostic -confirmations off;</arg><arg>set -showseparator ~;</arg><arg>set -showallfields true;</arg><arg>storage disk partition show</arg></args></system-cli>' -Controller $Controller -ErrorVariable +NetAppDocsError).results.'cli-output' | Where-Object { $_ -notmatch '^Error[:].+' } | Convert-SystemCliFieldDelimitedText
                }
                else {
                    Write-Verbose -Message 'Calling Invoke-NcSsh (storage disk partition show (priv diag))'
                    Invoke-NcSsh -Command 'set -privilege diagnostic -confirmations off; set -showseparator ~; set -showallfields true; storage disk partition show' -Controller $Controller -ErrorVariable +NetAppDocsError -Verbose:$false | Where-Object { $_ -notmatch '^Error[:].+' } | Convert-SystemCliFieldDelimitedText
                }
            )
        }

        #endregion Disk Information

        #region Disk Option Information

        $systemData | Add-Member -MemberType NoteProperty -Name DiskOption -Value $(
            if ($useSystemCli) {
                Write-Verbose -Message 'Calling Invoke-NcSystemApi (storage disk option show)'
                (Invoke-NcSystemApi -Request '<system-cli><args><arg>set -showseparator ~;</arg><arg>set -showallfields true;</arg><arg>storage disk option show</arg></args></system-cli>' -Controller $Controller -ErrorVariable +NetAppDocsError).results.'cli-output' | Convert-SystemCliFieldDelimitedText
            }
            else {
                Write-Verbose -Message 'Calling Invoke-NcSsh (storage disk option show)'
                Invoke-NcSsh -Command 'set -showseparator ~; set -showallfields true; storage disk option show' -Controller $Controller -ErrorVariable +NetAppDocsError -Verbose:$false | Convert-SystemCliFieldDelimitedText
            }
        )

        #endregion Disk Option Information

        #region Disk Encrypt Information

        if ($systemData.System.OntapiVersion -ge [version]::Parse('1.31')) {
            Write-Verbose -Message '- Gathering disk encrypt information -'
            $systemData | Add-Member -MemberType NoteProperty -Name DiskEncrypt -Value ([pscustomobject]@{ })

            Write-Verbose -Message 'Calling Get-NcDiskEncrypt'
            $systemData.DiskEncrypt | Add-Member -MemberType NoteProperty -Name Info -Value (Get-NcDiskEncrypt -Controller $Controller -ErrorVariable +NetAppDocsError)
        }

        #endregion Disk Encrypt Information

        #region iSCSI Configuration

        Write-Verbose -Message '- Gathering iSCSI configuration data -'
        $systemData | Add-Member -MemberType NoteProperty -Name Iscsi -Value ([pscustomobject]@{ })

        Write-Verbose -Message 'Calling Get-NcIscsiService'
        $systemData.Iscsi | Add-Member -MemberType NoteProperty -Name Service -Value (Get-NcIscsiService -Controller $Controller -ErrorVariable +NetAppDocsError)

        Write-Verbose -Message 'Calling Get-NcIscsiInitiatorAuth'
        $systemData.Iscsi | Add-Member -MemberType NoteProperty -Name InitiatorAuth -Value (Get-NcIscsiInitiatorAuth -Controller $Controller -ErrorVariable +NetAppDocsError)

        #endregion iSCSI configuration

        #region Network Details

        Write-Verbose -Message '- Gathering network details -'
        $systemData | Add-Member -MemberType NoteProperty -Name Net -Value ([pscustomobject]@{ })

        Write-Verbose -Message 'Calling Get-NcNetInterface'
        $systemData.Net | Add-Member -MemberType NoteProperty -Name Interface -Value (Get-NcNetInterface -Controller $Controller -ErrorVariable +NetAppDocsError)

        $netInterfaceCli = $(
            if ($useSystemCli) {
                Write-Verbose -Message 'Calling Invoke-NcSystemApi (network interface show (priv advanced))'
                (Invoke-NcSystemApi -Request '<system-cli><priv>advanced</priv><args><arg>set -showseparator ~;</arg><arg>set -showallfields true;</arg><arg>network interface show</arg></args></system-cli>' -Controller $Controller -ErrorVariable +NetAppDocsError).results.'cli-output' | Convert-SystemCliFieldDelimitedText
            }
            else {
                Write-Verbose -Message 'Calling Invoke-NcSsh (network interface show (priv advanced))'
                Invoke-NcSsh -Command 'set -privilege advanced -confirmations off; set -showseparator ~; set -showallfields true; network interface show' -Controller $Controller -ErrorVariable +NetAppDocsError -Verbose:$false | Convert-SystemCliFieldDelimitedText
            }
        )

        foreach ($lif in ($systemData.Net.Interface | Where-Object { $_ })) {
            $netInterface = $netInterfaceCli | Where-Object { $_.Vserver -ceq $lif.Vserver -and $_.Lif -ceq $lif.InterfaceName }

            $lif | Add-Member -NotePropertyMembers @{
                NumericId     = $netinterface.NumericId
                ServicePolicy = $netinterface.ServicePolicy
            } -Force
        }

        $systemData.Net | Add-Member -MemberType NoteProperty -Name FailoverTarget -Value $(
            if ($useSystemCli) {
                Write-Verbose -Message 'Calling Invoke-NcSystemApi (network interface show -failover)'
                (Invoke-NcSystemApi -Request '<system-cli><args><arg>network interface show -failover</arg></args></system-cli>' -Controller $Controller -ErrorVariable +NetAppDocsError).results.'cli-output' | Convert-FailoverTarget
            }
            else {
                Write-Verbose -Message 'Calling Invoke-NcSsh (network interface show -failover)'
                Invoke-NcSsh -Command 'network interface show -failover' -Controller $Controller -ErrorVariable +NetAppDocsError -Verbose:$false | Convert-FailoverTarget
            }
        )

        if ($systemData.System.OntapiVersion -ge [version]::Parse('1.20')) {
            Write-Verbose -Message 'Calling Get-NcNetOption'
            $systemData.Net | Add-Member -MemberType NoteProperty -Name Option -Value (Get-NcNetOption -Controller $Controller -ErrorVariable +NetAppDocsError)

            if ($systemData.System.OntapiVersion -ge [version]::Parse('1.30')) {
                $systemData.Net | Add-Member -MemberType NoteProperty -Name Firewall -Value ([pscustomobject]@{ })

                Write-Verbose -Message 'Calling Get-NcNetFirewall'
                $systemData.Net.Firewall | Add-Member -MemberType NoteProperty -Name Config -Value (Get-NcNetFirewall -Controller $Controller -ErrorVariable +NetAppDocsError)

                if ($systemData.System.OntapiVersion -lt [version]::Parse('1.201')) {
                    Write-Verbose -Message 'Calling Get-NcNetFirewallPolicy'
                    $systemData.Net.Firewall | Add-Member -MemberType NoteProperty -Name Policy -Value (Get-NcNetFirewallPolicy -Controller $Controller -ErrorVariable +NetAppDocsError)
                }

                if ($systemData.System.OntapiVersion -ge [version]::Parse('1.150')) {
                    $systemData.Net | Add-Member -MemberType NoteProperty -Name ServicePolicy -Value ([pscustomobject]@{ })

                    $systemData.Net.ServicePolicy | Add-Member -MemberType NoteProperty -Name Info -Value $(
                        if ($useSystemCli) {
                            Write-Verbose -Message 'Calling Invoke-NcSystemApi (network interface service-policy show (priv diag))'
                            (Invoke-NcSystemApi -Request '<system-cli><args><arg>set -privilege diagnostic -confirmations off;</arg><arg>set -showseparator ~;</arg><arg>set -showallfields true;</arg><arg>network interface service-policy show</arg></args></system-cli>' -Controller $Controller -ErrorVariable +NetAppDocsError).results.'cli-output' | Convert-SystemCliFieldDelimitedText
                        }
                        else {
                            Write-Verbose -Message 'Calling Invoke-NcSsh (network interface service-policy show(priv diag))'
                            Invoke-NcSsh -Command 'set -privilege diagnostic -confirmations off; set -showseparator ~; set -showallfields true; network interface service-policy show' -Controller $Controller -ErrorVariable +NetAppDocsError -Verbose:$false | Convert-SystemCliFieldDelimitedText
                        }
                    )

                    #Write-Verbose -Message 'Calling Invoke-NcSystemApi (net-interface-service-policy-get-iter)'
                    #$systemData.Net.ServicePolicy | Add-Member -MemberType NoteProperty -Name Info -Value ((Invoke-NcSystemApi -Request '<net-interface-service-policy-get-iter />' -Controller $Controller -ErrorVariable +NetAppDocsError).results.'attributes-list'.'net-interface-service-policy-info' | Convert-GenericXmlNode)

                    #Write-Verbose -Message 'Calling Invoke-NcSystemApi (net-interface-service-policy-entry-get-iter)'
                    #$systemData.Net.ServicePolicy | Add-Member -MemberType NoteProperty -Name Entry -Value ((Invoke-NcSystemApi -Request '<net-interface-service-policy-entry-get-iter />' -Controller $Controller -ErrorVariable +NetAppDocsError).results.'attributes-list'.'net-interface-service-policy-entry-info' | Convert-GenericXmlNode)
                }
            }
        }

        #endregion Network Details

        #region Shelf Details

        Write-Verbose -Message '- Gathering shelf data -'
        $systemData | Add-Member -MemberType NoteProperty -Name Shelf -Value (
            [pscustomobject]@{
                Info        = @( )
                Environment = @( )
                Bay         = @( )
            }
        )

        #Note: This will throw an error for an unhealthy node. Currently, this is desired. (Use $systemData.NodeInfo and $node.SystemName if we do not want this error).
        foreach ($node in ($systemData.Node | Where-Object { $_ -and $_.NodeModel -notmatch '^(?:CB|CD|DO|FD)vM\d{2,}|SIMBOX' })) {
            Write-Verbose -Message "Calling Get-NcShelf (Node: $( $node.Node ))"
            $systemData.Shelf.Info += Get-NcShelf -NodeName $node.Node -Controller $Controller -ErrorVariable +NetAppDocsError

            Write-Verbose -Message "Calling Get-NcShelfEnvironment (Node: $( $node.Node ))"
            $systemData.Shelf.Environment += Get-NcShelfEnvironment -NodeName $node.Node -Controller $Controller -ErrorVariable +NetAppDocsError

            Write-Verbose -Message "Calling Get-NcShelfBay (Node: $( $node.Node ))"
            $systemData.Shelf.Bay += Get-NcShelfBay -NodeName $node.Node -Controller $Controller -ErrorVariable +NetAppDocsError
        }

        #endregion Shelf Details

        #region Vserver Information

        Write-Verbose -Message '- Gathering Vserver information -'
        $systemData | Add-Member -MemberType NoteProperty -Name Vserver -Value ([pscustomobject]@{ })

        Write-Verbose -Message 'Calling Get-NcVserver'
        $systemData.Vserver | Add-Member -MemberType NoteProperty -Name Info -Value (Get-NcVserver -Controller $Controller -ErrorVariable +NetAppDocsError)

        if ($systemData.System.OntapiVersion -ge [version]::Parse('1.31')) {
            Write-Verbose -Message '- Gathering login banner/MOTD information -'

            Write-Verbose -Message 'Calling Get-NcVserverLoginBanner'
            $systemData.Vserver | Add-Member -MemberType NoteProperty -Name LoginBanner -Value (Get-NcVserverLoginBanner -Controller $Controller -ErrorVariable +NetAppDocsError)

            Write-Verbose -Message 'Calling Get-NcVserverMotd'
            $systemData.Vserver | Add-Member -MemberType NoteProperty -Name Motd -Value (Get-NcVserverMotd -Controller $Controller -ErrorVariable +NetAppDocsError)
        }

        #endregion Vserver Information

        #region Vserver Peer Information

        if ($systemData.System.OntapiVersion -ge [version]::Parse('1.20')) {
            Write-Verbose -Message '- Gathering Vserver Peer information -'
            $systemData.Vserver | Add-Member -MemberType NoteProperty -Name Peer -Value ([pscustomobject]@{ })

            Write-Verbose -Message 'Calling Get-NcVserverPeer'
            $systemData.Vserver.Peer | Add-Member -MemberType NoteProperty -Name Info -Value (Get-NcVserverPeer -Controller $Controller -ErrorVariable +NetAppDocsError)
        }

        #endregion Vserver Peer Information

        #region Volume Configuration

        Write-Verbose -Message '- Gathering detailed volume information -'
        $systemData | Add-Member -MemberType NoteProperty -Name Vol -Value ([pscustomobject]@{ })

        #ZAPI BUG: Not all attributes are returned unless we inititalize the template (flexgroup aggr-list specifically)
        Write-Verbose -Message 'Calling Get-NcVol'
        $volTemplate = Set-AllNcTemplateProperties -Template (Get-NcVol -Template -Fill -Controller $Controller -ErrorVariable +NetAppDocsError)
        $systemData.Vol | Add-Member -MemberType NoteProperty -Name Info -Value (Get-NcVol -Attributes $volTemplate -Controller $Controller -ErrorVariable +NetAppDocsError)

        Write-Verbose -Message 'Calling Get-NcVolClone'
        $systemData.Vol | Add-Member -MemberType NoteProperty -Name CloneVol -Value (Get-NcVolClone -Controller $Controller -ErrorVariable +NetAppDocsError)

        if ($systemData.System.OntapiVersion -ge [version]::Parse('1.20')) {
            Write-Verbose -Message 'Calling Get-NcVolSpace'
            $systemData.Vol | Add-Member -MemberType NoteProperty -Name Space -Value (Get-NcVolSpace -Controller $Controller -ErrorVariable +NetAppDocsError)

            Write-Verbose -Message 'Calling Get-NcVolFootprint'
            $systemData.Vol | Add-Member -MemberType NoteProperty -Name Footprint -Value (Get-NcVolFootprint -Controller $Controller -ErrorVariable +NetAppDocsError)
        }

        # vol show -fields commands
        if ($systemData.System.OntapiVersion -ge [version]::Parse('1.140')) {
            Write-Verbose -Message '- Gathering misc vol fields -'
            $volShowFields = @( 'vserver', 'vserver-uuid', 'instance-uuid', 'tiering-minimum-cooling-days' )

            if ($systemData.System.OntapiVersion -ge [version]::Parse('1.160')) {
                $volShowFields += @('encryption-state', 'encryption-type' )

                if ($systemData.System.OntapiVersion -ge [version]::Parse('1.201')) {
                    $volShowFields += 'anti-ransomware-state'

                    if ($systemData.System.OntapiVersion -ge [version]::Parse('1.221')) {
                        $volShowFields += @( 'snapshot-locking-enabled', 'expiry-time', 'compliance-clock-time' )
                    }
                }

                $systemData.Vol | Add-Member -MemberType NoteProperty -Name CliFields -Value $(
                    if ($useSystemCli) {
                        Write-Verbose -Message "Calling Invoke-NcSystemApi (volume show -fields $( $volShowFields -join ',' ) (priv diag))"
                        (Invoke-NcSystemApi -Request "<system-cli><args><arg>set -privilege diagnostic -confirmations off;</arg><arg>set -showseparator ~;</arg><arg>volume show -fields $( $volShowFields -join ',' )</arg></args></system-cli>" -Controller $Controller -ErrorVariable +NetAppDocsError).results.'cli-output' | Convert-SystemCliFieldDelimitedText
                    }
                    else {
                        Write-Verbose -Message "Calling Invoke-NcSsh (vol show -fields $( $volShowFields -join ',' ) (priv diag))"
                        Invoke-NcSsh -Command "set -privilege diagnostic -confirmations off; set -showseparator ~; volume show -fields $( $volShowFields -join ',' )" -Controller $Controller -ErrorVariable +NetAppDocsError -Verbose:$false | Convert-SystemCliFieldDelimitedText
                    }
                )
            }
        }

        #endregion Volume Configuration

        #region SIS Configuration

        Write-Verbose -Message '- Gathering SIS configuration -'
        $systemData | Add-Member -MemberType NoteProperty -Name Sis -Value ([pscustomobject]@{ })

        Write-Verbose -Message 'Calling Get-NcSis'
        $systemData.Sis | Add-Member -MemberType NoteProperty -Name Info -Value (Get-NcSis -Controller $Controller -ErrorVariable +NetAppDocsError)

        Write-Verbose -Message 'Calling Get-NcSisPolicy'
        $systemData.Sis | Add-Member -MemberType NoteProperty -Name Policy -Value (Get-NcSisPolicy -Controller $Controller -ErrorVariable +NetAppDocsError)

        #endregion SIS Configuration

        #region Snapshot Information

        Write-Verbose -Message '- Gathering detailed Snapshot information -'
        $systemData | Add-Member -MemberType NoteProperty -Name Snapshot -Value ([pscustomobject]@{ })

        Write-Verbose -Message 'Calling Get-NcSnapshot'
        $snapshotTemplate = Set-AllNcTemplateProperties -Template (Get-NcSnapshot -Template -Controller $Controller -ErrorVariable +NetAppDocsError)
        $systemData.Snapshot | Add-Member -MemberType NoteProperty -Name Info -Value (Get-NcSnapshot -Attributes $snapshotTemplate -Controller $Controller -ErrorVariable +NetAppDocsError)

        $systemData.Snapshot | Add-Member -MemberType NoteProperty -Name Policy -Value @(
            if ($useSystemCli) {
                Write-Verbose -Message 'Calling Invoke-NcSystemApi (volume snapshot policy show (priv diag))'
                (Invoke-NcSystemApi -Request '<system-cli><args><arg>set -privilege diagnostic -confirmations off;</arg><arg>set -showseparator ~;</arg><arg>set -showallfields true;</arg><arg>volume snapshot policy show</arg></args></system-cli>' -Controller $Controller -ErrorVariable +NetAppDocsError).results.'cli-output' |
                Convert-SystemCliFieldDelimitedText | Select-Object -ExcludeProperty Counts, Prefixes, Schedules, SnapmirrorLabels, RetentionPeriods -Property *, @{
                    Name       = 'SnapshotPolicySchedules'
                    Expression = {
                        @(
                            for ($i = 0; $i -lt $_.TotalSchedules; $i++) {
                                [pscustomobject]@{
                                    Count           = $_.Counts -split '\s*,\s*' | Select-Object -Index $i
                                    Prefix          = $_.Prefixes -split '\s*,\s*' | Select-Object -Index $i
                                    Schedule        = $_.Schedules -split '\s*,\s*' | Select-Object -Index $i
                                    SnapmirrorLabel = $_.SnapmirrorLabels -split '\s*,\s*' | Select-Object -Index $i
                                    RetentionPeriod = ($_.RetentionPeriods -split '\s*,\s*' | Select-Object -Index $i) -replace '^"|"$'
                                }
                            }
                        )
                    }
                }
            }
            else {
                Write-Verbose -Message 'Calling Invoke-NcSsh (volume snapshot policy show (priv diag))'
                Invoke-NcSsh -Command 'set -privilege diagnostic -confirmations off; set -showseparator ~; set -showallfields true; volume snapshot policy show' -Controller $Controller -ErrorVariable +NetAppDocsError -Verbose:$false |
                Convert-SystemCliFieldDelimitedText | Select-Object -ExcludeProperty Vserver, Counts, Prefixes, Schedules, SnapmirrorLabels, RetentionPeriods -Property *, @{
                    Name       = 'VserverName'
                    Expression = { $_.Vserver }
                }, @{
                    Name       = 'SnapshotPolicySchedules'
                    Expression = {
                        @(
                            for ($i = 0; $i -lt $_.TotalSchedules; $i++) {
                                [pscustomobject]@{
                                    Count           = $_.Counts -split '\s*,\s*' | Select-Object -Index $i
                                    Prefix          = $_.Prefixes -split '\s*,\s*' | Select-Object -Index $i
                                    Schedule        = $_.Schedules -split '\s*,\s*' | Select-Object -Index $i
                                    SnapmirrorLabel = $_.SnapmirrorLabels -split '\s*,\s*' | Select-Object -Index $i
                                    RetentionPeriod = ($_.RetentionPeriods -split '\s*,\s*' | Select-Object -Index $i) -replace '^"|"$'
                                }
                            }
                        )
                    }
                }
            }
        )

        #endregion Snapshot Information

        #region User Data

        Write-Verbose -Message '- Gathering user data -'

        Write-Verbose -Message 'Calling Get-NcUser'
        $systemData | Add-Member -MemberType NoteProperty -Name User -Value (Get-NcUser -Controller $Controller -ErrorVariable +NetAppDocsError)

        Write-Verbose -Message 'Calling Get-NcUser (admin vserver diag user)'
        $systemData.User += Get-NcUser -Name diag -Vserver ($systemData.Vserver.Info | Where-Object { $_.VserverType -eq 'admin' }).VserverName -Controller $Controller -ErrorVariable +NetAppDocsError

        #endregion User Data

        #region Role Data

        Write-Verbose -Message '- Gathering role data -'
        $systemData | Add-Member -MemberType NoteProperty -Name Role -Value ([pscustomobject]@{ })

        Write-Verbose -Message 'Calling Get-NcRole'
        $systemData.Role | Add-Member -MemberType NoteProperty -Name Info -Value (Get-NcRole -Controller $Controller -ErrorVariable +NetAppDocsError)

        #CLI BURT: diag needed for this CLI call only because admin/advanced does not add the appropriate newline between records
        $systemData.Role | Add-Member -MemberType NoteProperty -Name Config -Value $(
            $roles = $(
                if ($useSystemCli) {
                    Write-Verbose -Message 'Calling Invoke-NcSystemApi (security login role config show (priv diag))'
                    (Invoke-NcSystemApi -Request '<system-cli><args><arg>set -privilege diagnostic -confirmations off;</arg><arg>set -showseparator ~;</arg><arg>set -showallfields true;</arg><arg>security login role config show</arg></args></system-cli>' -Controller $Controller -ErrorVariable +NetAppDocsError).results.'cli-output' | Convert-SystemCliFieldDelimitedText
                }
                else {
                    Write-Verbose -Message 'Calling Invoke-NcSsh (security login role config show (priv diag))'
                    Invoke-NcSsh -Command 'set -privilege diagnostic -confirmations off; set -showseparator ~; set -showallfields true; security login role config show' -Controller $Controller -ErrorVariable +NetAppDocsError -Verbose:$false | Convert-SystemCliFieldDelimitedText
                }
            )

            $roles | Select-Object -Property Vserver, @{
                Name       = 'RoleName'
                Expression = { $_.Role }
            }, @{
                Name       = 'MinUsernameSize'
                Expression = { $_.UsernameMinlength }
            }, @{
                Name       = 'RequireUsernameAlphaNumeric'
                Expression = { $_.UsernameAlphaNum -eq 'enabled' }
            }, @{
                Name       = 'MinPasswordSize'
                Expression = { $_.PasswdMinlength }
            }, @{
                Name       = 'RequirePasswordAlphaNumeric'
                Expression = { $_.PasswdAlphanum -eq 'enabled' }
            }, @{
                Name       = 'MinPasswdSpecialchar'
                Expression = { $_.PasswdMinSpecialChars }
            }, @{
                Name       = 'PasswordExpirationDuration'
                Expression = { $_.PasswdExpiryTime }
            }, @{
                Name       = 'RequireInitialPasswordUpdate'
                Expression = { $_.RequireInitialPasswdUpdate -eq 'enabled' }
            }, @{
                Name       = 'LastPasswordsDisallowedCount'
                Expression = { $_.DisallowedReuse }
            }, @{
                Name       = 'ChangePasswordDurationInDays'
                Expression = { $_.ChangeDelay }
            }, MaxFailedLoginAttempts, LockoutDuration, DelayAfterFailedLogin, PasswdMinLowercaseChars, PasswdMinUppercaseChars, PasswdMinDigits, PasswdExpiryWarnTime, AccountExpiryTime, AccountInactiveLimit
        )

        #endregion Role Data

        #region SNMP User Data

        if ($systemData.System.OntapiVersion -ge [version]::Parse('1.100')) {
            Write-Verbose -Message '- Gathering SNMP user data -'

            $systemData | Add-Member -MemberType NoteProperty -Name SnmpUser -Value $(
                if ($useSystemCli) {
                    Write-Verbose -Message 'Calling Invoke-NcSystemApi (security snmpusers)'
                    (Invoke-NcSystemApi -Request '<system-cli><args><arg>set -showseparator ~;</arg><arg>set -showallfields true;</arg><arg>security snmpusers</arg></args></system-cli>' -Controller $Controller -ErrorVariable +NetAppDocsError).results.'cli-output' | Convert-SystemCliFieldDelimitedText
                }
                else {
                    Write-Verbose -Message 'Calling Invoke-NcSsh (security snmpusers)'
                    Invoke-NcSsh -Command 'set -showseparator ~; set -showallfields true; security snmpusers' -Controller $Controller -ErrorVariable +NetAppDocsError -Verbose:$false | Convert-SystemCliFieldDelimitedText
                }
            )
        }

        #endregion SNMP User Data

        #region Vscan Configuration

        if ($systemData.System.OntapiVersion -ge [version]::Parse('1.21')) {
            Write-Verbose -Message '- Gathering Vscan information -'
            $systemData | Add-Member -MemberType NoteProperty -Name Vscan -Value ([pscustomobject]@{ })

            Write-Verbose -Message 'Calling Get-NcVscanOnAccessPolicy'
            $systemData.Vscan | Add-Member -MemberType NoteProperty -Name OnAccessPolicy -Value (Get-NcVscanOnAccessPolicy -Controller $Controller -ErrorVariable +NetAppDocsError)

            Write-Verbose -Message 'Calling Get-NcVscanStatus'
            $systemData.Vscan | Add-Member -MemberType NoteProperty -Name Status -Value (Get-NcVscanStatus -Controller $Controller -ErrorVariable +NetAppDocsError)
        }

        #endregion Vscan Configuration

        #region Audit Configuration

        Write-Verbose -Message '- Gathering Audit configuration -'
        $systemData | Add-Member -MemberType NoteProperty -Name Audit -Value ([pscustomobject]@{ })

        Write-Verbose -Message 'Calling Get-NcAudit'
        $systemData.Audit | Add-Member -MemberType NoteProperty -Name Info -Value (Get-NcAudit -Controller $Controller -ErrorVariable +NetAppDocsError)

        #endregion Audit Configuration

        #region Autosupport Configuration

        Write-Verbose -Message '- Gathering autosupport configuration -'
        $systemData | Add-Member -MemberType NoteProperty -Name Autosupport -Value ([pscustomobject]@{ })

        Write-Verbose -Message 'Calling Get-NcAutoSupportConfig'
        $systemData.Autosupport | Add-Member -MemberType NoteProperty -Name Config -Value (Get-NcAutoSupportConfig -Controller $Controller -ErrorVariable +NetAppDocsError)

        Write-Verbose -Message 'Calling Get-NcAutoSupportHistory'
        $systemData.Autosupport | Add-Member -MemberType NoteProperty -Name History -Value (Get-NcAutoSupportHistory -Controller $Controller -ErrorVariable +NetAppDocsError)

        #endregion Autosupport Configuration

        #region Export Policy Configuration

        Write-Verbose -Message '- Gathering export policy information -'
        $systemData | Add-Member -MemberType NoteProperty -Name Export -Value ([pscustomobject]@{ })

        Write-Verbose -Message 'Calling Get-NcExportPolicy'
        $systemData.Export | Add-Member -MemberType NoteProperty -Name Policy -Value (Get-NcExportPolicy -Controller $Controller -ErrorVariable +NetAppDocsError)

        Write-Verbose -Message 'Calling Get-NcExportRule'
        $systemData.Export | Add-Member -MemberType NoteProperty -Name Rule -Value (Get-NcExportRule -Controller $Controller -ErrorVariable +NetAppDocsError)

        #endregion Export Policy Configuration

        #region Fpolicy Configuration

        if ($systemData.System.OntapiVersion -ge [version]::Parse('1.20')) {
            Write-Verbose -Message '- Gathering Fpolicy configuration -'
            $systemData | Add-Member -MemberType NoteProperty -Name Fpolicy -Value ([pscustomobject]@{ })

            Write-Verbose -Message 'Calling Get-NcFpolicyStatus'
            $systemData.Fpolicy | Add-Member -MemberType NoteProperty -Name Status -Value (Get-NcFpolicyStatus -Controller $Controller -ErrorVariable +NetAppDocsError)

            Write-Verbose -Message 'Calling Get-NcFpolicyServerStatus'
            $systemData.Fpolicy | Add-Member -MemberType NoteProperty -Name ServerStatus -Value (Get-NcFpolicyServerStatus -Controller $Controller -ErrorVariable +NetAppDocsError)

            Write-Verbose -Message 'Calling Get-NcFpolicyExternalEngine'
            $systemData.Fpolicy | Add-Member -MemberType NoteProperty -Name ExternalEngine -Value (Get-NcFpolicyExternalEngine -Controller $Controller -ErrorVariable +NetAppDocsError)

            Write-Verbose -Message 'Calling Get-NcFpolicyPolicy'
            $systemData.Fpolicy | Add-Member -MemberType NoteProperty -Name Policy -Value (Get-NcFpolicyPolicy -Controller $Controller -ErrorVariable +NetAppDocsError)

            Write-Verbose -Message 'Calling Get-NcFpolicyEvent'
            $systemData.Fpolicy | Add-Member -MemberType NoteProperty -Name Event -Value (Get-NcFpolicyEvent -Controller $Controller -ErrorVariable +NetAppDocsError)

            Write-Verbose -Message 'Calling Get-NcFpolicyScope'
            $systemData.Fpolicy | Add-Member -MemberType NoteProperty -Name Scope -Value (Get-NcFpolicyScope -Controller $Controller -ErrorVariable +NetAppDocsError)
        }

        #endregion Fpolicy Configuration

        #region FileService Audit Configuration

        Write-Verbose -Message '- Gathering File Service Audit configuration -'
        $systemData | Add-Member -MemberType NoteProperty -Name FileServiceAudit -Value ([pscustomobject]@{ })

        if ($systemData.System.OntapiVersion -ge [version]::Parse('1.20')) {
            Write-Verbose -Message 'Calling Get-NcFileServiceAudit'
            $systemData.FileServiceAudit | Add-Member -MemberType NoteProperty -Name Info -Value (Get-NcFileserviceAudit -Controller $Controller -ErrorVariable +NetAppDocsError)
        }

        #endregion FileService Audit Configuration

        #region Cluster Peer Configuration

        Write-Verbose -Message '- Gathering Cluster Peer configuration -'
        $systemData.Cluster | Add-Member -MemberType NoteProperty -Name Peer -Value ([pscustomobject]@{ })

        Write-Verbose -Message 'Calling Get-NcClusterPeer'
        $systemData.Cluster.Peer | Add-Member -MemberType NoteProperty -Name Info -Value (Get-NcClusterPeer -Controller $Controller -ErrorVariable +NetAppDocsError)

        #endregion Cluster Peer Configuration

        #region Snapmirror Configuration

        Write-Verbose -Message '- Gathering Snapmirror configuration -'
        $systemData | Add-Member -MemberType NoteProperty -Name Snapmirror -Value ([pscustomobject]@{ })

        #BURTS 653291, 658442, 674884, 715998 (No way to get around this, there may be missing entries in the output)
        Write-Verbose -Message 'Calling Get-NcSnapmirror'
        $systemData.Snapmirror | Add-Member -MemberType NoteProperty -Name Info -Value $(
            if ($systemData.System.OntapiVersion -ge [version]::Parse('1.100')) {
                Get-NcSnapmirror -Expand -Controller $Controller -ErrorVariable +NetAppDocsError
            }
            else {
                Get-NcSnapmirror -Controller $Controller -ErrorVariable +NetAppDocsError
            }
        )

        if ($systemData.System.OntapiVersion -ge [version]::Parse('1.20')) {
            Write-Verbose -Message 'Calling Get-NcSnapmirrorDestination'
            $systemData.Snapmirror | Add-Member -MemberType NoteProperty -Name Destination -Value (Get-NcSnapmirrorDestination -Controller $Controller -ErrorVariable +NetAppDocsError)

            #Write-Verbose -Message 'Calling Get-NcSnapmirrorPolicy'
            #$systemData.Snapmirror | Add-Member -MemberType NoteProperty -Name Policy -Value (Get-NcSnapmirrorPolicy -Controller $Controller -ErrorVariable +NetAppDocsError)

            $systemData.Snapmirror | Add-Member -MemberType NoteProperty -Name Policy -Value @(
                if ($useSystemCli) {
                    Write-Verbose -Message 'Calling Invoke-NcSystemApi (snapmirror policy show (priv diag))'
                    (Invoke-NcSystemApi -Request '<system-cli><args><arg>set -privilege diagnostic -confirmations off;</arg><arg>set -showseparator ~;</arg><arg>set -showallfields true;</arg><arg>snapmirror policy show</arg></args></system-cli>' -Controller $Controller -ErrorVariable +NetAppDocsError).results.'cli-output' | Convert-SystemCliFieldDelimitedText
                }
                else {
                    Write-Verbose -Message 'Calling Invoke-NcSsh (snapmirror policy show (priv diag))'
                    Invoke-NcSsh -Command 'set -privilege diagnostic -confirmations off; set -showseparator ~; set -showallfields true; snapmirror policy show' -Controller $Controller -ErrorVariable +NetAppDocsError -Verbose:$false | Convert-SystemCliFieldDelimitedText
                }
            )
        }

        #endregion Snapmirror Configuration

        #region ConfigBackup Information

        Write-Verbose -Message '- Gathering config backup information -'
        $systemData | Add-Member -MemberType NoteProperty -Name ConfigBackup -Value ([pscustomobject]@{ })

        Write-Verbose -Message 'Calling Get-NcConfigBackup'
        $systemData.ConfigBackup | Add-Member -MemberType NoteProperty -Name Info -Value (Get-NcConfigBackup -Controller $Controller -ErrorVariable +NetAppDocsError)

        Write-Verbose -Message 'Calling Get-NcConfigBackupCount'
        $systemData.ConfigBackup | Add-Member -MemberType NoteProperty -Name Count -Value (Get-NcConfigBackupCount -Controller $Controller -ErrorVariable +NetAppDocsError)

        Write-Verbose -Message 'Calling Get-NcConfigBackupUrl'
        $systemData.ConfigBackup | Add-Member -MemberType NoteProperty -Name Url -Value (Get-NcConfigBackupUrl -Controller $Controller -ErrorVariable +NetAppDocsError)

        #endregion ConfigBackup Information

        #region Name Services Information (NIS/Kerberos/LDAP)

        Write-Verbose -Message '- Gathering name services information -'
        $systemData | Add-Member -MemberType NoteProperty -Name NameService -Value (
            [pscustomobject]@{
                Kerberos = [pscustomobject]@{ }
                Ldap     = [pscustomobject]@{ }
                Nis      = [pscustomobject]@{ }
            }
        )

        Write-Verbose -Message 'Calling Get-NcNis'
        $systemData.NameService.Nis | Add-Member -MemberType NoteProperty -Name Info -Value (Get-NcNis -Controller $Controller -ErrorVariable +NetAppDocsError)

        #Write-Verbose -Message 'Calling Get-NcKerberosConfig'
        #$systemData.NameService.Kerberos | Add-Member -MemberType NoteProperty -Name Config -Value (Get-NcKerberosConfig -Controller $Controller -ErrorVariable +NetAppDocsError)

        Write-Verbose -Message 'Calling Get-NcKerberosRealm'
        $systemData.NameService.Kerberos | Add-Member -MemberType NoteProperty -Name Realm -Value (Get-NcKerberosRealm -Controller $Controller -ErrorVariable +NetAppDocsError)

        #TOOLKIT: Get-NcLdapClient is not returning the TryChannelBinding property
        #Write-Verbose -Message 'Calling Get-NcLdapClient'
        #$systemData.NameService.Ldap | Add-Member -MemberType NoteProperty -Name Client -Value (Get-NcLdapClient -Controller $Controller -ErrorVariable +NetAppDocsError)

        Write-Verbose -Message 'Calling Invoke-NcSystemApi (ldap-client-get-iter)'
        $systemData.NameService.Ldap | Add-Member -MemberType NoteProperty -Name Client -Value ((Invoke-NcSystemApi -Request '<ldap-client-get-iter />' -Controller $Controller -ErrorVariable +NetAppDocsError).results.'attributes-list'.'ldap-client' | Convert-GenericXmlNode)

        #Write-Verbose -Message 'Calling Get-NcLdapClientSchema'
        #$systemData.NameService.Ldap | Add-Member -MemberType NoteProperty -Name ClientSchema -Value (Get-NcLdapClientSchema -Controller $Controller -ErrorVariable +NetAppDocsError)

        #BURT 636873
        #$systemData.NameService.Ldap | Add-Member -MemberType NoteProperty -Name Config -Value $(
        #    if ($systemData.System.OntapiVersion -ge [version]::Parse('1.20')) {
        #        Write-Verbose -Message 'Calling Get-NcLdapConfig'
        #        Get-NcLdapConfig -Controller $Controller -ErrorVariable +NetAppDocsError
        #    }
        #    else {
        #        if ($useSystemCli) {
        #            Write-Verbose -Message 'Calling Invoke-NcSystemApi (vserver services ldap show)'
        #            (Invoke-NcSystemApi -Request '<system-cli><args><arg>set -showseparator ~;</arg><arg>set -showallfields true;</arg><arg>vserver services ldap show</arg></args></system-cli>' -Controller $Controller -ErrorVariable +NetAppDocsError).results.'cli-output' | Convert-SystemCliFieldDelimitedText
        #        }
        #        else {
        #            Write-Verbose -Message 'Calling Invoke-NcSsh (vserver services ldap show)'
        #            Invoke-NcSsh -Command 'set -showseparator ~; set -showallfields true; vserver services ldap show' -Controller $Controller -ErrorVariable +NetAppDocsError -Verbose:$false | Convert-SystemCliFieldDelimitedText
        #        }
        #    }
        #)

        if ($systemData.System.OntapiVersion -ge [version]::Parse('1.30')) {
            Write-Verbose -Message 'Calling Get-NcNameServiceNsSwitch'
            $systemData.NameService | Add-Member -MemberType NoteProperty -Name NsSwitch -Value (Get-NcNameServiceNsSwitch -Controller $Controller -ErrorVariable +NetAppDocsError)
        }

        #endregion Name Service Information (NIS/Kerberos/LDAP)

        #region Job Information

        Write-Verbose -Message '- Gathering job information -'
        $systemData | Add-Member -MemberType NoteProperty -Name Job -Value ([pscustomobject]@{ })

        $jobSchedules = @(
            if ($useSystemCli) {
                Write-Verbose -Message 'Calling Invoke-NcSystemApi (job schedule show)'
                (Invoke-NcSystemApi -Request '<system-cli><args><arg>set -showseparator ~;</arg><arg>set -showallfields true;</arg><arg>job schedule show</arg></args></system-cli>' -Controller $Controller -ErrorVariable +NetAppDocsError).results.'cli-output' | Convert-SystemCliFieldDelimitedText
            }
            else {
                Write-Verbose -Message 'Calling Invoke-NcSsh (job schedule show)'
                Invoke-NcSsh -Command 'set -showseparator ~; set -showallfields true; job schedule show' -Controller $Controller -ErrorVariable +NetAppDocsError -Verbose:$false | Convert-SystemCliFieldDelimitedText
            }
        )

        $systemData.Job | Add-Member -MemberType NoteProperty -Name Schedule -Value (
            $jobSchedules | Select-Object -Property *, @{
                Name       = 'JobScheduleInfo'
                Expression = { ConvertFrom-JobScheduleDescription -JobScheduleDescription $_.Description }
            }
        )

        #endregion Job Information

        #region CIFS Configuration

        Write-Verbose -Message '- Gathering CIFS configuration information -'
        $systemData | Add-Member -MemberType NoteProperty -Name Cifs -Value ([pscustomobject]@{ })

        Write-Verbose -Message 'Calling Get-NcCifsServer'
        $systemData.Cifs | Add-Member -MemberType NoteProperty -Name Server -Value (Get-NcCifsServer -Controller $Controller -ErrorVariable +NetAppDocsError)

        #(MCC BURT: 969339)
        Write-Verbose -Message 'Calling Get-NcCifsSession'
        $systemData.Cifs | Add-Member -MemberType NoteProperty -Name Session -Value (Get-NcCifsSession -Controller $Controller -ErrorVariable +NetAppDocsError)

        Write-Verbose -Message 'Calling Get-NcCifsShare'
        $systemData.Cifs | Add-Member -MemberType NoteProperty -Name Share -Value (Get-NcCifsShare -Controller $Controller -ErrorVariable +NetAppDocsError)
        if ($systemData.System.OntapiVersion -lt [version]::Parse('1.20')) {
            foreach ($share in ($systemData.Cifs.Share | Where-Object { $_ -and [string]::IsNullOrWhiteSpace($_.Vserver) })) {
                $share | Add-Member -MemberType NoteProperty -Name Vserver -Value ($systemData.Cifs.Server | Where-Object { $_.CifsServer -ceq $share.CifsServer }).Vserver -ErrorAction Ignore -Force
            }
        }

        Write-Verbose -Message 'Calling Get-NcCifsShareAcl'
        $systemData.Cifs | Add-Member -MemberType NoteProperty -Name ShareAcl -Value (Get-NcCifsShareAcl -Controller $Controller -ErrorVariable +NetAppDocsError)

        Write-Verbose -Message 'Calling Get-NcCifsPreferredDomainController'
        $systemData.Cifs | Add-Member -MemberType NoteProperty -Name PreferredDomainController -Value (Get-NcCifsPreferredDomainController -Controller $Controller -ErrorVariable +NetAppDocsError)

        # Add Vserver specific properties
        ## As of 1.20+, these are cluster APIs but the TOOLKIT still enforces VserverContext
        $systemData.Cifs | Add-Member -NotePropertyMembers @{
            Option   = @( )
            Security = @( )
        }

        foreach ($cifsServer in ($systemData.Cifs.Server | Where-Object { $_ })) {
            Write-Verbose -Message "Calling Get-NcCifsOption (Vserver: $( $cifsServer.Vserver ))"
            Get-NcCifsOption -VserverContext $cifsServer.Vserver -Controller $Controller -ErrorVariable +NetAppDocsError | Where-Object { $_ } | ForEach-Object {
                $systemData.Cifs.Option += $_
                $systemData.Cifs.Option | Select-Object -Last 1 | Add-Member -MemberType NoteProperty -Name Vserver -Value $cifsServer.Vserver -ErrorAction Ignore -Force
            }

            # TOOLKIT: Missing 9.8+ property
            #Write-Verbose -Message "Calling Get-NcCifsSecurity (Vserver: $( $cifsServer.Vserver ))"
            #Get-NcCifsSecurity -VserverContext $cifsServer.Vserver -Controller $Controller -ErrorVariable +NetAppDocsError | Where-Object { $_ } | ForEach-Object {
            #    $systemData.Cifs.Security += $_
            #    $systemData.Cifs.Security | Select-Object -Last 1 | Add-Member -MemberType NoteProperty -Name Vserver -Value $cifsServer.Vserver -ErrorAction Ignore -Force
            #}

            if ($useSystemCli) {
                Write-Verbose -Message 'Calling Invoke-NcSystemApi (vserver cifs security show)'
                $systemData.Cifs.Security += (Invoke-NcSystemApi -Request "<system-cli><args><arg>set -showseparator ~;</arg><arg>set -showallfields true;</arg><arg>vserver cifs security show -vserver $( $cifsServer.Vserver )</arg></args></system-cli>" -Controller $Controller -ErrorVariable +NetAppDocsError).results.'cli-output' | Convert-SystemCliFieldDelimitedText
            }
            else {
                Write-Verbose -Message 'Calling Invoke-NcSsh (vserver cifs security show)'
                $systemData.Cifs.Security += Invoke-NcSsh -Command "set -showseparator ~; set -showallfields true; vserver cifs security show -vserver $( $cifsServer.Vserver )" -Controller $Controller -ErrorVariable +NetAppDocsError -Verbose:$false | Convert-SystemCliFieldDelimitedText
            }
        }

        if ($systemData.System.OntapiVersion -ge [version]::Parse('1.20')) {
            Write-Verbose -Message '- Gathering CIFS local users and groups -'
            $systemData.Cifs | Add-Member -MemberType NoteProperty -Name UserAndGroup -Value ([pscustomobject]@{ })

            Write-Verbose -Message 'Calling Get-NcCifsLocalUser'
            $systemData.Cifs.UserAndGroup | Add-Member -MemberType NoteProperty -Name LocalUser -Value (Get-NcCifsLocalUser -Controller $Controller -ErrorVariable +NetAppDocsError)

            Write-Verbose -Message 'Calling Get-NcCifsLocalGroup'
            $systemData.Cifs.UserAndGroup | Add-Member -MemberType NoteProperty -Name LocalGroup -Value (Get-NcCifsLocalGroup -Controller $Controller -ErrorVariable +NetAppDocsError)

            Write-Verbose -Message 'Calling Get-NcCifsLocalGroupMember'
            $systemData.Cifs.UserAndGroup | Add-Member -MemberType NoteProperty -Name LocalGroupMember -Value (Get-NcCifsLocalGroupMember -Controller $Controller -ErrorVariable +NetAppDocsError)

            if ($systemData.System.OntapiVersion -ge [version]::Parse('1.30')) {
                Write-Verbose -Message 'Calling Get-NcCifsHomeDirectoryConfig'
                $systemData.Cifs | Add-Member -MemberType NoteProperty -Name HomeDirectoryConfig -Value (Get-NcCifsHomeDirectoryConfig -Controller $Controller -ErrorVariable +NetAppDocsError)

                if ($systemData.System.OntapiVersion -ge [version]::Parse('1.130')) {
                    Write-Verbose -Message 'Calling Invoke-NcSystemApi (cifs-domain-server-discovery-mode-get-iter)'
                    $systemData.Cifs | Add-Member -MemberType NoteProperty -Name DomainServerDiscoveryMode -Value ((Invoke-NcSystemApi -Request '<cifs-domain-server-discovery-mode-get-iter />' -Controller $Controller -ErrorVariable +NetAppDocsError).results.'attributes-list'.'cifs-domain-server-discovery-mode-info' | Convert-GenericXmlNode)
                }
            }
        }

        #endregion CIFS Information

        #region NFS

        Write-Verbose -Message '- Gathering NFS configuration information -'
        $systemData | Add-Member -MemberType NoteProperty -Name Nfs -Value ([pscustomobject]@{ })

        #BURT 512073
        if ($systemData.System.OntapiVersion -ge [version]::Parse('1.17')) {
            Write-Verbose -Message 'Calling Get-NcNfsService'
            $systemData.Nfs | Add-Member -MemberType NoteProperty -Name Service -Value (Get-NcNfsService -Controller $Controller -ErrorVariable +NetAppDocsError)
        }
        else {
            $systemData.Nfs | Add-Member -MemberType NoteProperty -Name Service -Value @( )

            foreach ($vserver in ($systemData.Vserver.Info | Where-Object { $_.AllowedProtocols -contains 'nfs' -and $_.VserverType -in @( 'data', 'cluster' ) })) {
                if (-not $systemData.Nfs.Service) {
                    Write-Verbose -Message "Calling Get-NcNfsService (Vserver: $( $vserver.VserverName ))"
                    try {
                        Get-NcNfsService -VserverContext $vserver.VserverName -Controller $Controller -ErrorAction Stop | Where-Object { $_ } | ForEach-Object {
                            $systemData.Nfs.Service += $_
                            $systemData.Nfs.Service | Select-Object -Last 1 | Add-Member -MemberType NoteProperty -Name Vserver -Value $vserver.VserverName
                        }
                    }
                    catch {
                        if ($_.Exception.Message -like "*entry doesn't exist*") {
                            $Global:Error.RemoveAt(0)
                        }
                        else {
                            $NetAppDocsError += $_
                        }

                        continue
                    }
                }
            }
        }

        #endregion NFS

        #region Security Information

        if ($systemData.System.OntapiVersion -ge [version]::Parse('1.17')) {
            Write-Verbose -Message '- Gathering security related information -'
            $systemData | Add-Member -MemberType NoteProperty -Name Security -Value ([pscustomobject]@{ })

            $systemData.Security | Add-Member -MemberType NoteProperty -Name DomainTunnelVserver -Value $(
                if ($useSystemCli) {
                    Write-Verbose -Message 'Calling Invoke-NcSystemApi (security login domain-tunnel show)'
                    ((Invoke-NcSystemApi -Request '<system-cli><args><arg>security login domain-tunnel show</arg></args></system-cli>' -Controller $Controller -ErrorVariable +NetAppDocsError).results.'cli-output' | Convert-SystemCliTextInstance).TunnelVserver
                }
                else {
                    Write-Verbose -Message 'Calling Invoke-NcSsh (security login domain-tunnel show)'
                    (Invoke-NcSsh -Command 'security login domain-tunnel show' -Controller $Controller -ErrorVariable +NetAppDocsError -Verbose:$false | Convert-SystemCliTextInstance).TunnelVserver
                }
            )

            if ($systemData.System.OntapiVersion -ge [version]::Parse('1.20')) {
                #TOOLKIT: Get-NcSecurityCertificate is not returning the SelfSigned property
                #Write-Verbose -Message 'Calling Get-NcSecurityCertificate'
                #$systemData.Security | Add-Member -MemberType NoteProperty -Name Certificate -Value (Get-NcSecurityCertificate -Controller $Controller -ErrorVariable +NetAppDocsError)

                Write-Verbose -Message 'Calling Invoke-NcSystemApi (security-certificate-get-iter)'
                $systemData.Security | Add-Member -MemberType NoteProperty -Name Certificate -Value ((Invoke-NcSystemApi -Request '<security-certificate-get-iter />' -Controller $Controller -ErrorVariable +NetAppDocsError).results.'attributes-list'.'certificate-info' | Convert-GenericXmlNode)

                Write-Verbose -Message 'Calling Get-NcSecuritySsl'
                $systemData.Security | Add-Member -MemberType NoteProperty -Name Ssl -Value (Get-NcSecuritySsl -Controller $Controller -ErrorVariable +NetAppDocsError)

                if ($systemData.System.OntapiVersion -ge [version]::Parse('1.21')) {
                    Write-Verbose -Message 'Calling Get-NcSecuritySsh'
                    $systemData.Security | Add-Member -MemberType NoteProperty -Name Ssh -Value (Get-NcSecuritySsh -Controller $Controller -ErrorVariable +NetAppDocsError)
                }

                if ($systemData.System.OntapiVersion -ge [version]::Parse('1.100')) {
                    #TOOLKIT: Get-NcSecurityConfig is not returning the SupportedProtocols property and is also not updated to include SupportedCipherSuites
                    #Write-Verbose -Message 'Calling Get-NcSecurityConfig (Interface: ssl)'
                    #$systemData.Security | Add-Member -MemberType NoteProperty -Name Config -Value (Get-NcSecurityConfig -Interface ssl -Controller $Controller -ErrorVariable +NetAppDocsError)

                    Write-Verbose -Message 'Calling Invoke-NcSystemApi (security-config-get)'
                    $systemData.Security | Add-Member -MemberType NoteProperty -Name Config -Value (
                        (Invoke-NcSystemApi -Request '<security-config-get><interface>ssl</interface></security-config-get>' -Controller $Controller -ErrorVariable +NetAppDocsError).results.attributes.'security-config-info' | Convert-GenericXmlNode | Select-Object -ExcludeProperty SupportedCipherSuites -Property *, @{
                            Name       = 'SupportedCipherSuites'
                            Expression = { $_.SupportedCipherSuites }
                        }
                    )

                    $systemData.Security | Add-Member -MemberType NoteProperty -Name ConfigStatus -Value $(
                        if ($useSystemCli) {
                            Write-Verbose -Message 'Calling Invoke-NcSystemApi (security config status show (priv diag))'
                            (Invoke-NcSystemApi -Request '<system-cli><args><arg>set -privilege diagnostic -confirmations off;</arg><arg>set -showseparator ~;</arg><arg>set -showallfields true;</arg><arg>security config status show</arg></args></system-cli>' -Controller $Controller -ErrorVariable +NetAppDocsError).results.'cli-output' | Convert-SystemCliFieldDelimitedText
                        }
                        else {
                            Write-Verbose -Message 'Calling Invoke-NcSsh (security config status show (priv diag))'
                            Invoke-NcSsh -Command 'set -privilege diagnostic -confirmations off; set -showseparator ~; set -showallfields true; security config status show' -Controller $Controller -ErrorVariable +NetAppDocsError -Verbose:$false | Convert-SystemCliFieldDelimitedText
                        }
                    )

                    $systemData.Security | Add-Member -MemberType NoteProperty -Name Protocol -Value @( )

                    Write-Verbose -Message 'Calling Get-NcSecurityProtocol (Application: rsh)'
                    $systemData.Security.Protocol += Get-NcSecurityProtocol -Application rsh -Controller $Controller -ErrorVariable +NetAppDocsError

                    Write-Verbose -Message 'Calling Get-NcSecurityProtocol (Application: telnet)'
                    $systemData.Security.Protocol += Get-NcSecurityProtocol -Application telnet -Controller $Controller -ErrorVariable +NetAppDocsError

                    if ($systemData.System.OntapiVersion -ge [version]::Parse('1.110')) {
                        $systemData.Security | Add-Member -MemberType NoteProperty -Name Tpm -Value $(
                            if ($useSystemCli) {
                                Write-Verbose -Message 'Calling Invoke-NcSystemApi (security tpm show)'
                                (Invoke-NcSystemApi -Request '<system-cli><args><arg>set -showseparator ~;</arg><arg>set -showallfields true;</arg><arg>security tpm show</arg></args></system-cli>' -Controller $Controller -ErrorVariable +NetAppDocsError).results.'cli-output' | Convert-SystemCliFieldDelimitedText
                            }
                            else {
                                Write-Verbose -Message 'Calling Invoke-NcSsh (security tpm show)'
                                Invoke-NcSsh -Command 'set -showseparator ~; set -showallfields true; security tpm show' -Controller $Controller -ErrorVariable +NetAppDocsError -Verbose:$false | Convert-SystemCliFieldDelimitedText
                            }
                        )

                        $systemData.Security | Add-Member -MemberType NoteProperty -Name PublicKey -Value $(
                            if ($useSystemCli) {
                                Write-Verbose -Message 'Calling Invoke-NcSystemApi (security login publickey show)'
                                (Invoke-NcSystemApi -Request '<system-cli><args><arg>set -showseparator ~;</arg><arg>set -showallfields true;</arg><arg>security login publickey show</arg></args></system-cli>' -Controller $Controller -ErrorVariable +NetAppDocsError).results.'cli-output' | Convert-SystemCliFieldDelimitedText
                            }
                            else {
                                Write-Verbose -Message 'Calling Invoke-NcSsh (security login publickey show)'
                                Invoke-NcSsh -Command 'set -showseparator ~; set -showallfields true; security login publickey show' -Controller $Controller -ErrorVariable +NetAppDocsError -Verbose:$false | Convert-SystemCliFieldDelimitedText
                            }
                        )

                        if ($systemData.System.OntapiVersion -ge [version]::Parse('1.120')) {
                            Write-Verbose -Message 'Calling Get-NcSecurityProtocolSsh'
                            $systemData.Security | Add-Member -MemberType NoteProperty -Name ProtocolSsh -Value (Get-NcSecurityProtocolSsh -Controller $Controller -ErrorVariable +NetAppDocsError)

                            if ($systemData.System.OntapiVersion -ge [version]::Parse('1.130')) {
                                $systemData.Security | Add-Member -MemberType NoteProperty -Name SamlSp -Value $(
                                    try {
                                        Write-Verbose -Message 'Calling Get-NcSecuritySamlSp'
                                        Get-NcSecuritySamlSp -Controller $Controller -ErrorAction Stop
                                    }
                                    catch {
                                        if ($_.Exception.Message -like "*entry doesn't exist*") {
                                            $Global:Error.RemoveAt(0)
                                        }
                                        else {
                                            $NetAppDocsError += $_
                                        }
                                    }
                                )

                                if ($systemData.System.OntapiVersion -ge [version]::Parse('1.180')) {
                                    $systemData.Security | Add-Member -MemberType NoteProperty -Name IPSec -Value ([pscustomobject]@{ })

                                    $systemData.Security.IPSec | Add-Member -MemberType NoteProperty -Name Config -Value $(
                                        if ($useSystemCli) {
                                            Write-Verbose -Message 'Calling Invoke-NcSystemApi (security ipsec config show)'
                                            (Invoke-NcSystemApi -Request '<system-cli><args><arg>set -showseparator ~;</arg><arg>set -showallfields true;</arg><arg>security ipsec config show</arg></args></system-cli>' -Controller $Controller -ErrorVariable +NetAppDocsError).results.'cli-output' | Convert-SystemCliFieldDelimitedText
                                        }
                                        else {
                                            Write-Verbose -Message 'Calling Invoke-NcSsh (security ipsec config show)'
                                            Invoke-NcSsh -Command 'set -showseparator ~; set -showallfields true; security ipsec config show' -Controller $Controller -ErrorVariable +NetAppDocsError -Verbose:$false | Convert-SystemCliFieldDelimitedText
                                        }
                                    )

                                    $systemData.Security.IPSec | Add-Member -MemberType NoteProperty -Name Policy -Value $(
                                        if ($useSystemCli) {
                                            Write-Verbose -Message 'Calling Invoke-NcSystemApi (security ipsec policy show)'
                                            (Invoke-NcSystemApi -Request '<system-cli><args><arg>set -showseparator ~;</arg><arg>set -showallfields true;</arg><arg>security ipsec policy show</arg></args></system-cli>' -Controller $Controller -ErrorVariable +NetAppDocsError).results.'cli-output' | Convert-SystemCliFieldDelimitedText
                                        }
                                        else {
                                            Write-Verbose -Message 'Calling Invoke-NcSsh (security ipsec policy show)'
                                            Invoke-NcSsh -Command 'set -showseparator ~; set -showallfields true; security ipsec policy show' -Controller $Controller -ErrorVariable +NetAppDocsError -Verbose:$false | Convert-SystemCliFieldDelimitedText
                                        }
                                    )

                                    <#$systemData.Security.IPSec | Add-Member -MemberType NoteProperty -Name IkeSA -Value $(
                                        if ($useSystemCli) {
                                            Write-Verbose -Message 'Calling Invoke-NcSystemApi (security ipsec show-ikesasa show)'
                                            (Invoke-NcSystemApi -Request '<system-cli><args><arg>set -showseparator ~;</arg><arg>set -showallfields true;</arg><arg>security ipsec show-ikesasa show</arg></args></system-cli>' -Controller $Controller -ErrorVariable +NetAppDocsError).results.'cli-output' | Convert-SystemCliFieldDelimitedText
                                        }
                                        else {
                                            Write-Verbose -Message 'Calling Invoke-NcSsh (security ipsec show-ikesasa show)'
                                            Invoke-NcSsh -Command 'set -showseparator ~; set -showallfields true; security ipsec show-ikesasa show' -Controller $Controller -ErrorVariable +NetAppDocsError -Verbose:$false | Convert-SystemCliFieldDelimitedText
                                        }
                                    )

                                    $systemData.Security.IPSec | Add-Member -MemberType NoteProperty -Name SA -Value $(
                                        if ($useSystemCli) {
                                            Write-Verbose -Message 'Calling Invoke-NcSystemApi (security ipsec show-ipsecsa show)'
                                            (Invoke-NcSystemApi -Request '<system-cli><args><arg>set -showseparator ~;</arg><arg>set -showallfields true;</arg><arg>security ipsec show-ipsecsa show</arg></args></system-cli>' -Controller $Controller -ErrorVariable +NetAppDocsError).results.'cli-output' | Convert-SystemCliFieldDelimitedText
                                        }
                                        else {
                                            Write-Verbose -Message 'Calling Invoke-NcSsh (security ipsec show-ipsecsa show)'
                                            Invoke-NcSsh -Command 'set -showseparator ~; set -showallfields true; security ipsec show-ipsecsa show' -Controller $Controller -ErrorVariable +NetAppDocsError -Verbose:$false | Convert-SystemCliFieldDelimitedText
                                        }
                                    )#>

                                    if ($systemData.System.OntapiVersion -ge [version]::Parse('1.201')) {
                                        $systemData.Security | Add-Member -MemberType NoteProperty -Name AntiRansomwareVolume -Value $(
                                            if ($useSystemCli) {
                                                Write-Verbose -Message 'Calling Invoke-NcSystemApi (security anti-ransomware volume show)'
                                                (Invoke-NcSystemApi -Request '<system-cli><args><arg>set -showseparator ~;</arg><arg>set -showallfields true;</arg><arg>security anti-ransomware volume show</arg></args></system-cli>' -Controller $Controller -ErrorVariable +NetAppDocsError).results.'cli-output' | Convert-SystemCliFieldDelimitedText
                                            }
                                            else {
                                                Write-Verbose -Message 'Calling Invoke-NcSsh (security anti-ransomware volume show)'
                                                Invoke-NcSsh -Command 'set -showseparator ~; set -showallfields true; security anti-ransomware volume show' -Controller $Controller -ErrorVariable +NetAppDocsError -Verbose:$false | Convert-SystemCliFieldDelimitedText
                                            }
                                        )

                                        if ($systemData.System.OntapiVersion -ge [version]::Parse('1.211')) {
                                            $systemData.Security | Add-Member -MemberType NoteProperty -Name MultiAdminVerify -Value ([pscustomobject]@{ })

                                            $systemData.Security.MultiAdminVerify | Add-Member -MemberType NoteProperty -Name GlobalSetting -Value $(
                                                if ($useSystemCli) {
                                                    Write-Verbose -Message 'Calling Invoke-NcSystemApi (security multi-admin-verify show)'
                                                    (Invoke-NcSystemApi -Request '<system-cli><args><arg>set -showseparator ~;</arg><arg>set -showallfields true;</arg><arg>security multi-admin-verify show</arg></args></system-cli>' -Controller $Controller -ErrorVariable +NetAppDocsError).results.'cli-output' | Convert-SystemCliFieldDelimitedText
                                                }
                                                else {
                                                    Write-Verbose -Message 'Calling Invoke-NcSsh (security multi-admin-verify show)'
                                                    Invoke-NcSsh -Command 'set -showseparator ~; set -showallfields true; security multi-admin-verify show' -Controller $Controller -ErrorVariable +NetAppDocsError -Verbose:$false | Convert-SystemCliFieldDelimitedText
                                                }
                                            )

                                            $systemData.Security.MultiAdminVerify | Add-Member -MemberType NoteProperty -Name Rule -Value $(
                                                if ($useSystemCli) {
                                                    Write-Verbose -Message 'Calling Invoke-NcSystemApi (security multi-admin-verify rule show)'
                                                    (Invoke-NcSystemApi -Request '<system-cli><args><arg>set -showseparator ~;</arg><arg>set -showallfields true;</arg><arg>security multi-admin-verify rule show</arg></args></system-cli>' -Controller $Controller -ErrorVariable +NetAppDocsError).results.'cli-output' | Convert-SystemCliFieldDelimitedText
                                                }
                                                else {
                                                    Write-Verbose -Message 'Calling Invoke-NcSsh (security multi-admin-verify rule show)'
                                                    Invoke-NcSsh -Command 'set -showseparator ~; set -showallfields true; security multi-admin-verify rule show' -Controller $Controller -ErrorVariable +NetAppDocsError -Verbose:$false | Convert-SystemCliFieldDelimitedText
                                                }
                                            )
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }

        #endregion Security Information

        #region Security Key Manager Information

        if ($systemData.System.OntapiVersion -ge [version]::Parse('1.160')) {
            Write-Verbose -Message '- Gathering security key manager information -'
            $systemData.Security | Add-Member -MemberType NoteProperty -Name KeyManager -Value ([pscustomobject]@{ })

            Write-Verbose -Message 'Calling Get-NcSecurityKeyManagerKeyStore'
            $systemData.Security.KeyManager | Add-Member -MemberType NoteProperty -Name KeyStore -Value (Get-NcSecurityKeyManagerKeyStore -Controller $Controller -ErrorVariable +NetAppDocsError)
        }

        #endregion Security Key Manager Information

        #region SnapLock Information

        if ($systemData.System.OntapiVersion -ge [version]::Parse('1.100')) {
            Write-Verbose -Message '- Gathering SnapLock information -'
            $systemData | Add-Member -MemberType NoteProperty -Name Snaplock -Value ([pscustomobject]@{ })

            $systemData.Snaplock | Add-Member -MemberType NoteProperty -Name Volume -Value $(
                if ($useSystemCli) {
                    Write-Verbose -Message 'Calling Invoke-NcSystemApi (volume snaplock show)'
                    (Invoke-NcSystemApi -Request '<system-cli><args><arg>set -showseparator ~;</arg><arg>set -showallfields true;</arg><arg>volume snaplock show</arg></args></system-cli>' -Controller $Controller -ErrorVariable +NetAppDocsError).results.'cli-output' | Convert-SystemCliFieldDelimitedText
                }
                else {
                    Write-Verbose -Message 'Calling Invoke-NcSsh (volume snaplock show)'
                    Invoke-NcSsh -Command 'set -showseparator ~; set -showallfields true; volume snaplock show' -Controller $Controller -ErrorVariable +NetAppDocsError -Verbose:$false | Convert-SystemCliFieldDelimitedText
                }
            )
        }

        #endregion SnapLock Information

        #region Object Store Information

        if ($systemData.System.OntapiVersion -ge [version]::Parse('1.180')) {
            Write-Verbose -Message '- Gathering ObjectStore information -'
            $systemData | Add-Member -MemberType NoteProperty -Name ObjectStore -Value (
                [pscustomobject]@{
                    Server = $(
                        if ($useSystemCli) {
                            Write-Verbose -Message 'Calling Invoke-NcSystemApi (vserver object-store-server show)'
                            (Invoke-NcSystemApi -Request '<system-cli><args><arg>set -showseparator ~;</arg><arg>vserver object-store-server show</arg></args></system-cli>' -Controller $Controller -ErrorVariable +NetAppDocsError).results.'cli-output' | Convert-SystemCliFieldDelimitedText
                        }
                        else {
                            Write-Verbose -Message 'Calling Invoke-NcSsh (vserver object-store-server show)'
                            Invoke-NcSsh -Command 'set -showseparator ~; vserver object-store-server show' -Controller $Controller -ErrorVariable +NetAppDocsError -Verbose:$false | Convert-SystemCliFieldDelimitedText
                        }
                    )
                }
            )
        }

        #endregion Object Store Information
        #region Cluster Switch Information

        $systemData | Add-Member -MemberType NoteProperty -Name ClusterSwitch -Value ([pscustomobject]@{ })

        $systemData.ClusterSwitch | Add-Member -MemberType NoteProperty -Name Info -Value $(
            if ($systemData.System.OntapiVersion -ge [version]::Parse('1.32')) {
                if ($useSystemCli) {
                    Write-Verbose -Message 'Calling Invoke-NcSystemApi (system cluster-switch show-all (priv advanced))'
                    (Invoke-NcSystemApi -Request '<system-cli><priv>advanced</priv><args><arg>set -showseparator ~;</arg><arg>set -showallfields true;</arg><arg>system cluster-switch show-all</arg></args></system-cli>' -Controller $Controller -ErrorVariable +NetAppDocsError).results.'cli-output' | Convert-SystemCliFieldDelimitedText | Where-Object { $_.SwVersion -notlike 'NetApp Release*' }
                }
                else {
                    Write-Verbose -Message 'Calling Invoke-NcSsh (system cluster-switch show-all (priv advanced))'
                    Invoke-NcSsh -Command 'set -privilege advanced -confirmations off; set -showseparator ~; set -showallfields true; system cluster-switch show-all' -Controller $Controller -ErrorVariable +NetAppDocsError -Verbose:$false | Convert-SystemCliFieldDelimitedText
                }
            }
            elseif ($systemData.System.OntapiVersion -ge [version]::Parse('1.20')) {
                if ($useSystemCli) {
                    Write-Verbose -Message 'Calling Invoke-NcSystemApi (system health cluster-switch show)'
                    (Invoke-NcSystemApi -Request '<system-cli><args><arg>set -showseparator ~;</arg><arg>set -showallfields true;</arg><arg>system health cluster-switch show</arg></args></system-cli>' -Controller $Controller -ErrorVariable +NetAppDocsError).results.'cli-output' | Convert-SystemCliFieldDelimitedText | Where-Object { $_.SwVersion -notlike 'NetApp Release*' }
                }
                else {
                    Write-Verbose -Message 'Calling Invoke-NcSsh (system health cluster-switch show)'
                    Invoke-NcSsh -Command 'set -showseparator ~; set -showallfields true; system health cluster-switch show' -Controller $Controller -ErrorVariable +NetAppDocsError -Verbose:$false | Convert-SystemCliFieldDelimitedText | Where-Object { $_.SwVersion -notlike 'NetApp Release*' }
                }
            }
        )

        #endregion Cluster Switch Information

        #region Disk Qual Information

        Write-Verbose -Message '- Gathering DQP information -'
        $dqpRegex = '(?s)^(?:Node[:]\s*(?<Node>.*?))\s+.*?(?:Datecode[:]\s*(?<PackageDate>.*?))\s+.*?(?:FileName[=]\s*(?<FileName>.*?))\s+.*?(?:FileVersion[=]\s*(?<FileVersion>.*?))\s+.*?(?:DriveRecordCount[=]\s*(?<DriveRecordCount>.*?))\s+.*?(?:AliasRecordCount[=]\s*(?<AliasRecordCount>.*?))\s+.*?(?:DeviceRecordCount[=]\s*(?<DeviceRecordCount>.*?))\s+.*?(?:SystemRecordCount[=]\s*(?<SystemRecordCount>.*?))\s+.*$'

        $systemData | Add-Member -MemberType NoteProperty -Name DiskQualPackage -Value $(
            $diskQual = $(
                if ($useSystemCli) {
                    Write-Verbose -Message 'Calling Invoke-NcSystemApi (system node run -node * -command rdfile /etc/qual_devices_v3)'
                    (Invoke-NcSystemApi -Request '<system-cli><args><arg>system node run -node * -command rdfile /etc/qual_devices_v3</arg></args></system-cli>' -Controller $Controller -ErrorVariable +NetAppDocsError).results.'cli-output'
                }
                else {
                    Write-Verbose -Message 'Calling Invoke-NcSsh (system node run -node * -command rdfile /etc/qual_devices_v3)'
                    Invoke-NcSsh -Command 'system node run -node * -command rdfile /etc/qual_devices_v3' -Controller $Controller -ErrorVariable +NetAppDocsError -Verbose:$false
                }
            )

            if ($diskQual) {
                $diskQual -split '(?=Node[:]\s)' | Select-Object -Skip 1 | Import-WithRegex -RegexString $dqpRegex
            }
        )

        #endregion Disk Qual Information

        Write-Verbose -Message '- Getting current timestamp -'
        if ($systemData.System.OntapiVersion -lt [version]::Parse('1.30')) {
            $systemData | Add-Member -MemberType NoteProperty -Name ReferenceDT -Value (ConvertFrom-UnixTimestamp -Seconds (Get-NcClusterHa -Node ($systemData.Net.Interface | Where-Object { $_.Address -eq $Controller.Address.IpAddressToString }).CurrentNode -Controller $Controller -ErrorVariable +NetAppDocsError).CurrentTime)
        }
        else {
            $systemData | Add-Member -MemberType NoteProperty -Name ReferenceDT -Value (ConvertFrom-UnixTimestamp -Seconds (Get-NcTime -Controller $Controller -ErrorVariable +NetAppDocsError).UtcTime)
        }

        Write-Verbose -Message '- Identifying SystemName -'
        $systemData | Add-Member -MemberType NoteProperty -Name SystemName -Value $systemData.Cluster.Info.ClusterName

        Write-Verbose -Message '- Identifying any errors during processing -'
        $systemData | Add-Member -MemberType NoteProperty -Name DiscoveryError -Value @(
            foreach ($err in ($NetAppDocsError | Where-Object { $_ })) {
                $err.CategoryInfo | Select-Object -Property *, @{
                    Name       = 'Message'
                    Expression = { $err.Exception.Message }
                }
            }
        )

        if (($systemData.DiscoveryError | Measure-Object).Count -gt 0) {
            $msg = New-Object -TypeName System.Text.StringBuilder

            [void]$msg.AppendLine("Errors were encountered while collecting data from system: $( $Controller.Name )")
            [void]$msg.AppendLine('-------------------------------------------------------------------------------------')
            [void]$msg.AppendLine('Please review the output for accuracy and, if necessary, correct any error conditions')
            [void]$msg.AppendLine('then run these scripts again.')
            [void]$msg.AppendLine('-------------------------------------------------------------------------------------')
            [void]$msg.AppendLine()

            Write-Warning -Message $msg.ToString()
        }

        $systemData
    }
    catch {
        $Global:Error.RemoveAt(0)

        $msg = New-Object -TypeName System.Text.StringBuilder

        [void]$msg.AppendLine('An unexpected error occurred. Processing halted.')
        [void]$msg.AppendLine('--------------------------------------------------------------------------------------------------')
        [void]$msg.AppendLine('As this error is not expected, please consider contacting the support staff via e-mail or by')
        [void]$msg.AppendLine('starting a thread on the community site.')
        [void]$msg.AppendLine('            E-mail: ng-NetAppDocs-support@netapp.com')
        [void]$msg.AppendLine('    Community site: https://community.netapp.com/t5/NetAppDocs/bd-p/netappdocs')
        [void]$msg.AppendLine('--------------------------------------------------------------------------------------------------')
        [void]$msg.AppendLine($( $_.Exception.GetBaseException().Message ))
        [void]$msg.AppendLine("Location: $( $_.ScriptStackTrace )")
        [void]$msg.AppendLine()

        $PSCmdlet.ThrowTerminatingError((New-Object -TypeName System.Management.Automation.ErrorRecord -ArgumentList ($msg.ToString(), $_.FullyQualifiedErrorId, $_.CategoryInfo.Category, $_.CategoryInfo.TargetName)))
    }
}

# SIG # Begin signature block
# MIIy0QYJKoZIhvcNAQcCoIIywjCCMr4CAQExDzANBglghkgBZQMEAgEFADB5Bgor
# BgEEAYI3AgEEoGswaTA0BgorBgEEAYI3AgEeMCYCAwEAAAQQH8w7YFlLCE63JNLG
# KX7zUQIBAAIBAAIBAAIBAAIBADAxMA0GCWCGSAFlAwQCAQUABCAuoxWI96+Oc0RD
# +oFsZNWMLsA2GViAGqFonhd7ixJw5aCCFxcwggQ+MIIDJqADAgECAgRKU4woMA0G
# CSqGSIb3DQEBCwUAMIG+MQswCQYDVQQGEwJVUzEWMBQGA1UEChMNRW50cnVzdCwg
# SW5jLjEoMCYGA1UECxMfU2VlIHd3dy5lbnRydXN0Lm5ldC9sZWdhbC10ZXJtczE5
# MDcGA1UECxMwKGMpIDIwMDkgRW50cnVzdCwgSW5jLiAtIGZvciBhdXRob3JpemVk
# IHVzZSBvbmx5MTIwMAYDVQQDEylFbnRydXN0IFJvb3QgQ2VydGlmaWNhdGlvbiBB
# dXRob3JpdHkgLSBHMjAeFw0wOTA3MDcxNzI1NTRaFw0zMDEyMDcxNzU1NTRaMIG+
# MQswCQYDVQQGEwJVUzEWMBQGA1UEChMNRW50cnVzdCwgSW5jLjEoMCYGA1UECxMf
# U2VlIHd3dy5lbnRydXN0Lm5ldC9sZWdhbC10ZXJtczE5MDcGA1UECxMwKGMpIDIw
# MDkgRW50cnVzdCwgSW5jLiAtIGZvciBhdXRob3JpemVkIHVzZSBvbmx5MTIwMAYD
# VQQDEylFbnRydXN0IFJvb3QgQ2VydGlmaWNhdGlvbiBBdXRob3JpdHkgLSBHMjCC
# ASIwDQYJKoZIhvcNAQEBBQADggEPADCCAQoCggEBALqEtnLbngxr4pnpMAGnduoy
# uJVBGsnaYU5Ycs/+9oJ5v3NhBgqlJ9izX9NFThxy1k4y8nKKD/eDGdBqgIAARR6w
# x+eavxJXJxyjaC8Kh71qaw5eZfMcd9XUhY1wIbSzMueLotWGOQKxuNJHzuTJScQ7
# p977VH1XvvDobsJ5sjoLVeJQmBYyE1wveFbBwpSz8lrkJ5qfJNfG7NCbJYLjzMLE
# RcWMl3oGayoRn6kKbkg7b9vUERlC948Hv/VTX5w+9Bcs5mmsTjJMYnfqt+jluzS8
# GYuunFHnt361U7EzIuVtz3A8Gvrim2e2g/SNpa9iTE3gWKxkNBID+LaNlGMkpHEC
# AwEAAaNCMEAwDgYDVR0PAQH/BAQDAgEGMA8GA1UdEwEB/wQFMAMBAf8wHQYDVR0O
# BBYEFGpyJnrQHu995ztpUdRsjZ+QEmarMA0GCSqGSIb3DQEBCwUAA4IBAQB5nx2W
# xrZ5PyKNh9OHAwRgamuaLlmJcxGsQ9H1E/+NOSvA8r1PcIypL+oXxAtUntQblpgz
# PKitYqIAdqtZaW4GHX7EuUSNmK8S1GHbChlGR/Pr92PBQAVApdK39LWaNr+piHaI
# BFUEK5yHfxo3PH4tpRrY1Ileyr2sPWzYba/V83YPzTuIOCKdbJOaxD2/ghtlP6YP
# Xar85bIVyrWtxrw90ITo6gZysE05Mni/PhGcC6SdmiHz8JsLMHjbwdyHQ/68Y5rK
# xcIcyceN/zsSWAjmtj3seixO+4OWzgw8aYdUc6RzwpP/URCsFVQB2PwFsYmhf3SD
# mknX3E57ikhvi0X2MIIF3zCCBMegAwIBAgIQTkDkN1Tt5owAAAAAUdOUfzANBgkq
# hkiG9w0BAQsFADCBvjELMAkGA1UEBhMCVVMxFjAUBgNVBAoTDUVudHJ1c3QsIElu
# Yy4xKDAmBgNVBAsTH1NlZSB3d3cuZW50cnVzdC5uZXQvbGVnYWwtdGVybXMxOTA3
# BgNVBAsTMChjKSAyMDA5IEVudHJ1c3QsIEluYy4gLSBmb3IgYXV0aG9yaXplZCB1
# c2Ugb25seTEyMDAGA1UEAxMpRW50cnVzdCBSb290IENlcnRpZmljYXRpb24gQXV0
# aG9yaXR5IC0gRzIwHhcNMjEwNTA3MTU0MzQ1WhcNMzAxMTA3MTYxMzQ1WjBpMQsw
# CQYDVQQGEwJVUzEWMBQGA1UECgwNRW50cnVzdCwgSW5jLjFCMEAGA1UEAww5RW50
# cnVzdCBDb2RlIFNpZ25pbmcgUm9vdCBDZXJ0aWZpY2F0aW9uIEF1dGhvcml0eSAt
# IENTQlIxMIICIjANBgkqhkiG9w0BAQEFAAOCAg8AMIICCgKCAgEAp4GP9xRFtmJD
# 8tiu0yVeSE9Rv8V9n1AcNdHWfmEqlBltJ0akphpd91RRaoAixqhmdU1Ug8leaBur
# 9ltksK2tIL1U70ZrbQLnRa519o6KuTIui7h3HFJNeYhfpToYyVAslyctv9oAfWN/
# 7zLsRodj25qfw1ohNnv5m9XKoG5yLPzh8Z5wTQhWFW+Qq/tIurnXwYJ4hWUuf7XJ
# wOIUtzcRQQbiiuCo9uV+tngFAcNg7U8HQS4KE0njkJt/3b36rL9kUdFcm7T1XOdc
# /zubpaAa130JssK3/24cvMh95ukr/HKzFOlKVRKEnEQldR32KvBPpSA9aCXrYZd8
# D+W2PfOuw8ERvBuOzOBHMF5CAIZx41isBsplH3uUpktXZwx+Xq14Z1tV417rx9js
# TG6Gy/Pc+J+HqnJYEg99pvj4Qjk7PCzkMk1JjODhAMI4oJz6hD5B3G5WrsYaW/Rn
# aAUBzRu/roe8nVP2Lui2a+SZ3sVPh1io0mUeyB/Vcm7uWRxXOwlyndfKt5DGzXtF
# kpFCA0x9P8ryqrjCDobzEJ9GLqRmhmhaaBhwKTgRgGBrikOjc2zjs2s3/+adZwGS
# ht8vSNH7UGDVXP4h0wFCY/7vcLQXwI+o7tPBS18S6v39Lg6HRGDjqfTCGKPj/c4M
# hCIN86d42pPz2zjPuS8zxv8HPF6+RdMCAwEAAaOCASswggEnMA4GA1UdDwEB/wQE
# AwIBhjASBgNVHRMBAf8ECDAGAQH/AgEBMB0GA1UdJQQWMBQGCCsGAQUFBwMDBggr
# BgEFBQcDCDA7BgNVHSAENDAyMDAGBFUdIAAwKDAmBggrBgEFBQcCARYaaHR0cDov
# L3d3dy5lbnRydXN0Lm5ldC9ycGEwMwYIKwYBBQUHAQEEJzAlMCMGCCsGAQUFBzAB
# hhdodHRwOi8vb2NzcC5lbnRydXN0Lm5ldDAwBgNVHR8EKTAnMCWgI6Ahhh9odHRw
# Oi8vY3JsLmVudHJ1c3QubmV0L2cyY2EuY3JsMB0GA1UdDgQWBBSCutY9l86fz3Ho
# kjev/bO1aTVXzzAfBgNVHSMEGDAWgBRqciZ60B7vfec7aVHUbI2fkBJmqzANBgkq
# hkiG9w0BAQsFAAOCAQEAH15BBLaDcCRTLFVzHWU6wOy0ewSYXlk4EwmkWZRCXlC/
# T2xuJSCQk1hADfUZtGLuJF7CAVgVAh0QCW+o1PuSfjc4Pi8UfY8dQzZks2YTXxTM
# pXH3WyFLxpe+3JX8cH0RHNMh3dAkOSnF/goapc97ee46b97cv+kR3RaDCNMsjX9N
# qBR5LwVhUjjrYPMUaH3LsoqtwJRc5CYOLIrdRsPO5FZRxVbjhbhNm0VyiwfxivtJ
# uF/R8paBXWlSJPEII9LWIw/ri9d+i8GTa/rxYntY6VCbl24XiA3hxkOY14FhtoWd
# R+yxnq4/IDtDndiiHODUfAjCr3YG+GJmerb3+sivNTCCBmcwggRPoAMCAQICEEbM
# 41QNEgkq80rzt39+/oswDQYJKoZIhvcNAQELBQAwYzELMAkGA1UEBhMCVVMxFjAU
# BgNVBAoTDUVudHJ1c3QsIEluYy4xPDA6BgNVBAMTM0VudHJ1c3QgRXh0ZW5kZWQg
# VmFsaWRhdGlvbiBDb2RlIFNpZ25pbmcgQ0EgLSBFVkNTMjAeFw0yNDEwMjMxNTIy
# MTRaFw0yNTExMDkxNTIyMTNaMIHCMQswCQYDVQQGEwJVUzETMBEGA1UECBMKQ2Fs
# aWZvcm5pYTERMA8GA1UEBxMIU2FuIEpvc2UxEzARBgsrBgEEAYI3PAIBAxMCVVMx
# GTAXBgsrBgEEAYI3PAIBAhMIRGVsYXdhcmUxFDASBgNVBAoTC05ldEFwcCwgSW5j
# MR0wGwYDVQQPExRQcml2YXRlIE9yZ2FuaXphdGlvbjEQMA4GA1UEBRMHMzQ1Mjc0
# MzEUMBIGA1UEAxMLTmV0QXBwLCBJbmMwggGiMA0GCSqGSIb3DQEBAQUAA4IBjwAw
# ggGKAoIBgQCIHyusYBqN6d8MjDVGb9yKy/jxOJi5Ry9vtMsEpSe0eX1NwlzjdPpo
# R+kSTTj0HXwjn48v4UtO3lOWVVdqDuAuECTvG+n31c1RmaB4qS0qg9VEDsUP0/gr
# R4GlmEnGRIeDNWnRIV4oHGTqA1NjNU3esZYLQSV9j5DSzh0W+vyq10stDZiLBNai
# KoBzi5fs+C7QWW+xd+fSFV3Z+zIfE/jHFa4ZKP0n8Bva/5XKZSg6XVBN09oY/EF2
# 67wlz2018jiSwKZi++kCqTIpp9j7dKP5x+0/bK4GlGeivT/NObhjDN8e0Iu5rC2L
# tCalpr5CBMzjKmwfV8mVd9L/qPKk8QZxBdBrUpA1qHDCHbd9BRcd2zT8doIbM0vT
# dHnT9pRPZYws7BfT33KMoV7nShbrNViWhkP3i3UkQv7W9RptiZwfEDpt4Lxn0Uw8
# Zf94fbwMQaV3A28ItUbueLN23+k41N0jqdFTbpCaqQpUT/NxlRHECtn0Lp6kjZTy
# wKF7FqH+jpcCAwEAAaOCATUwggExMAwGA1UdEwEB/wQCMAAwHQYDVR0OBBYEFGzw
# 5EcKP/wp/cuZ0BIs8j6slZQcMB8GA1UdIwQYMBaAFM6JT4JRqhWihGLKMSNh0mH7
# +P54MA4GA1UdDwEB/wQEAwIHgDATBgNVHSUEDDAKBggrBgEFBQcDAzBnBggrBgEF
# BQcBAQRbMFkwIwYIKwYBBQUHMAGGF2h0dHA6Ly9vY3NwLmVudHJ1c3QubmV0MDIG
# CCsGAQUFBzAChiZodHRwOi8vYWlhLmVudHJ1c3QubmV0L2V2Y3MyLWNoYWluLnA3
# YzAxBgNVHR8EKjAoMCagJKAihiBodHRwOi8vY3JsLmVudHJ1c3QubmV0L2V2Y3My
# LmNybDAgBgNVHSAEGTAXMAcGBWeBDAEDMAwGCmCGSAGG+mwKAQIwDQYJKoZIhvcN
# AQELBQADggIBADAwfASY7H1rvIVgDB4LCw7V6xYShLM1XBV0YV8mgn50EVUgkXRr
# qwdn9JgVJvPU6jQ30ab9bb2505N/e1hWQN+Skw21Kbixw3mJ2lZNjY9CvwPUAc4B
# 9xXFEl/w7w1GuATgHO77YCpe0QTbYpbuSTXA0YBs1LlzmuoMxCD+E82srnED8Crg
# m4IHe71kpqqbsIYDnxujU+w/+/CAijPCuxXD2Ff1D7Ms97n9OFvLZHU2e+7xTP66
# RFXF81uCNcLYZSAMiXpjFCiXz74yNHtqZYecnNKvA3hUnvE0BH2IbJFpFiwQO9H2
# rSxdf6qTJANWvCTm7AkDMPhCneufSIFeGHyINH5sKNc+MgsNawZwy2Bfld2YE+ii
# +RyoUgQvl+OZu58dcPYvbgSkc+nE7Xlq/nzE4qNB83ZUuo4MQuGgZ1ZdROSgO6BD
# lCDa1Mr+viR84DtT6dwMYDf0PYWe0LoDYBVLRknZHpSNCMULURCTENBJdJo1mrOn
# hmmm5ZJ3Hg/CmyrujK67YvNFUsQnpLKYEut7ntmJM5ptEPm7Ql5zzaOp3o68a/Q6
# 0+s6p4NwZQYo4ekHk+DM4oaPiv8K+dEPxvppQCSoSPG0S59A8oFBXzoaidOTC06j
# xfGFkPFtOEIO269lK0NCIgr2lJvlUdlUPSnM2eWkEkslz42ShJgON0SaMIIGgzCC
# BGugAwIBAgIQNa+3e500H2r8j4RGqzE1KzANBgkqhkiG9w0BAQ0FADBpMQswCQYD
# VQQGEwJVUzEWMBQGA1UECgwNRW50cnVzdCwgSW5jLjFCMEAGA1UEAww5RW50cnVz
# dCBDb2RlIFNpZ25pbmcgUm9vdCBDZXJ0aWZpY2F0aW9uIEF1dGhvcml0eSAtIENT
# QlIxMB4XDTIxMDUwNzE5MTk1MloXDTQwMTIyOTIzNTkwMFowYzELMAkGA1UEBhMC
# VVMxFjAUBgNVBAoTDUVudHJ1c3QsIEluYy4xPDA6BgNVBAMTM0VudHJ1c3QgRXh0
# ZW5kZWQgVmFsaWRhdGlvbiBDb2RlIFNpZ25pbmcgQ0EgLSBFVkNTMjCCAiIwDQYJ
# KoZIhvcNAQEBBQADggIPADCCAgoCggIBAL69pznJpX3sXWXx9Cuph9DnrRrFGjsY
# zuGhUY1y+s5YH1y4JEIPRtUxl9BKTeObMMm6l6ic/kU2zyeA53u4bsEkt9+ndNyF
# 8qMkWEXMlJQ7AuvEjXxG9VxmguOkwdMfrG4MUyMO1Dr62kLxg1RfNTJW8rV4m1cA
# SB6pYWEnDnMDQ7bWcJL71IWaMMaz5ppeS+8dKthmqxZG/wvYD6aJSgJRV0E8QThO
# l8dRMm1njmahXk2fNSKv1Wq3f0BfaDXMafrxBfDqhabqMoXLwcHKg2lFSQbcCWy6
# SWUZjPm3NyeMZJ414+Xs5wegnahyvG+FOiymFk49nM8I5oL1RH0owL2JrWwv3C94
# eRHXHHBL3Z0ITF4u+o29p91j9n/wUjGEbjrY2VyFRJ5jBmnQhlh4iZuHu1gcpChs
# xv5pCpwerBFgal7JaWUu7UMtafF4tzstNfKqT+If4wFvkEaq1agNBFegtKzjbb2d
# GyiAJ0bH2qpnlfHRh3vHyCXphAyPiTbSvjPhhcAz1aA8GYuvOPLlk4C/xsOre5PE
# PZ257kV2wNRobzBePLQ2+ddFQuASBoDbpSH85wV6KI20jmB798i1SkesFGaXoFpp
# cjFXa1OEzWG6cwcVcDt7AfynP4wtPYeM+wjX5S8Xg36Cq08J8inhflV3ZZQFHVnU
# Ct2TfuMUXeK7AgMBAAGjggErMIIBJzASBgNVHRMBAf8ECDAGAQH/AgEAMB0GA1Ud
# DgQWBBTOiU+CUaoVooRiyjEjYdJh+/j+eDAfBgNVHSMEGDAWgBSCutY9l86fz3Ho
# kjev/bO1aTVXzzAzBggrBgEFBQcBAQQnMCUwIwYIKwYBBQUHMAGGF2h0dHA6Ly9v
# Y3NwLmVudHJ1c3QubmV0MDEGA1UdHwQqMCgwJqAkoCKGIGh0dHA6Ly9jcmwuZW50
# cnVzdC5uZXQvY3NicjEuY3JsMA4GA1UdDwEB/wQEAwIBhjATBgNVHSUEDDAKBggr
# BgEFBQcDAzBEBgNVHSAEPTA7MDAGBFUdIAAwKDAmBggrBgEFBQcCARYaaHR0cDov
# L3d3dy5lbnRydXN0Lm5ldC9ycGEwBwYFZ4EMAQMwDQYJKoZIhvcNAQENBQADggIB
# AD4AVLgq849mr2EWxFiTZPRBi2RVjRs1M6GbkdirRsqrX7y+fnDk0tcHqJYH14bR
# VwoI0NB4Tfgq37IE85rh13zwwQB6wUCh34qMt8u0HQFh8piapt24gwXKqSwW3Jwt
# Dv6nl+RQqZeVwUsqjFHjxALga3w1TVO8S5QTi1MYFl6mCqe4NMFssess5DF9DCzG
# fOGkVugtdtWyE3XqgwCuAHfGb6k97mMUgVAW/FtPEhkOWw+N6kvOBkyJS64gzI5H
# pnXWZe4vMOhdNI8fgk1cQqbyFExQIJwJonQkXDnYiTKFPK+M5Wqe5gQ6pRP/qh3N
# R0suAgW0ao/rhU+B7wrbfZ8pj6XCP1I4UkGVO7w+W1QwQiMJY95QjYk1RfqruA+P
# oq17ehGT8Y8ohHtoeUdq6GQpTR/0HS9tHsiUhjzTWpl6a3yrNfcrOUtPuT8Wku8p
# jI2rrAEazHFEOctAPiASzghw40f+3IDXCADRC2rqIbV5ZhfpaqpW3c0VeLEDwBSt
# PkcYde0KU0syk83/gLGQ1hPl5EF4Iu1BguUO37DOlSFF5osB0xn39CtVrNlWc2MQ
# 4LigbctUlpigmSFRBqqmDDorY8t52kO50hLM3o9VeukJ8+Ka0yXBezaS2uDlUmfN
# 4+ZUCqWd1HOj0y9dBmSFA3d/YNjCvHTJlZFot7d+YRl1MYIbEDCCGwwCAQEwdzBj
# MQswCQYDVQQGEwJVUzEWMBQGA1UEChMNRW50cnVzdCwgSW5jLjE8MDoGA1UEAxMz
# RW50cnVzdCBFeHRlbmRlZCBWYWxpZGF0aW9uIENvZGUgU2lnbmluZyBDQSAtIEVW
# Q1MyAhBGzONUDRIJKvNK87d/fv6LMA0GCWCGSAFlAwQCAQUAoIGEMBgGCisGAQQB
# gjcCAQwxCjAIoAKAAKECgAAwGQYJKoZIhvcNAQkDMQwGCisGAQQBgjcCAQQwHAYK
# KwYBBAGCNwIBCzEOMAwGCisGAQQBgjcCARUwLwYJKoZIhvcNAQkEMSIEIH/SA2Ou
# HjuxMVEoBHbpQscXmg1FRISWbr6F+TboqDRVMA0GCSqGSIb3DQEBAQUABIIBgAIp
# y04gAe1zKpTbg035kqhDr+hwrBh8yhpRvnmNVCJV5wJ3LFnhguJURgefRcQ8+Z1P
# z1b55PY3N+ijDkFO8JImGabtB9M086KQZT+e03XOrr76b7VYhmoFyGgX+zA7H/gW
# A+sSG7IBa2FY5TFW3yItClkfqlE1ozuQLDpFBpU6MirPDD7EU862ErzQXIjVRdfh
# U4R3pCNyvuS+93TmMH6Dz4104tSjJAgqntfYeEsZZMzBRafyuy2q9deoEEQNrmBN
# KL+MZBBxVIg71JntPh1yPTZ/ReVDHhFLYxiOaaxj9k3VM4WfRb6RDfCNBwZK2bEe
# 5Rb/YmxboJOkqQQggZQqjRQmne0S8A2XjlB2WajODcEeDivy26OwTh5KPCF/jPKX
# s5l8F6WKlxodFDjgHRyMO1pUDjxgHumIGRGOtdyrb7hStNID8eqc1Z1IFco942rS
# kKBPG3ksZmqWUGDDfhS6N6Yo0zKnDoMSK/QLaWEi8IyLsIy5UwdYz3V8PFuee6GC
# GGMwghhfBgorBgEEAYI3AwMBMYIYTzCCGEsGCSqGSIb3DQEHAqCCGDwwghg4AgED
# MQ0wCwYJYIZIAWUDBAIDMIHzBgsqhkiG9w0BCRABBKCB4wSB4DCB3QIBAQYKYIZI
# AYb6bAoDBTAxMA0GCWCGSAFlAwQCAQUABCDHolKih4bwi6qOUPpU/noab1OJ8DUg
# I3KvaJGEMrBa2wIIGPYzwEd5xvcYDzIwMjUwMTA5MDMwNjI5WjADAgEBoHmkdzB1
# MQswCQYDVQQGEwJDQTEQMA4GA1UECBMHT250YXJpbzEPMA0GA1UEBxMGT3R0YXdh
# MRYwFAYDVQQKEw1FbnRydXN0LCBJbmMuMSswKQYDVQQDEyJFbnRydXN0IFRpbWVz
# dGFtcCBBdXRob3JpdHkgLSBUU0EyoIITDjCCBd8wggTHoAMCAQICEE5A5DdU7eaM
# AAAAAFHTlH8wDQYJKoZIhvcNAQELBQAwgb4xCzAJBgNVBAYTAlVTMRYwFAYDVQQK
# Ew1FbnRydXN0LCBJbmMuMSgwJgYDVQQLEx9TZWUgd3d3LmVudHJ1c3QubmV0L2xl
# Z2FsLXRlcm1zMTkwNwYDVQQLEzAoYykgMjAwOSBFbnRydXN0LCBJbmMuIC0gZm9y
# IGF1dGhvcml6ZWQgdXNlIG9ubHkxMjAwBgNVBAMTKUVudHJ1c3QgUm9vdCBDZXJ0
# aWZpY2F0aW9uIEF1dGhvcml0eSAtIEcyMB4XDTIxMDUwNzE1NDM0NVoXDTMwMTEw
# NzE2MTM0NVowaTELMAkGA1UEBhMCVVMxFjAUBgNVBAoMDUVudHJ1c3QsIEluYy4x
# QjBABgNVBAMMOUVudHJ1c3QgQ29kZSBTaWduaW5nIFJvb3QgQ2VydGlmaWNhdGlv
# biBBdXRob3JpdHkgLSBDU0JSMTCCAiIwDQYJKoZIhvcNAQEBBQADggIPADCCAgoC
# ggIBAKeBj/cURbZiQ/LYrtMlXkhPUb/FfZ9QHDXR1n5hKpQZbSdGpKYaXfdUUWqA
# IsaoZnVNVIPJXmgbq/ZbZLCtrSC9VO9Ga20C50WudfaOirkyLou4dxxSTXmIX6U6
# GMlQLJcnLb/aAH1jf+8y7EaHY9uan8NaITZ7+ZvVyqBuciz84fGecE0IVhVvkKv7
# SLq518GCeIVlLn+1ycDiFLc3EUEG4orgqPblfrZ4BQHDYO1PB0EuChNJ45Cbf929
# +qy/ZFHRXJu09VznXP87m6WgGtd9CbLCt/9uHLzIfebpK/xysxTpSlUShJxEJXUd
# 9irwT6UgPWgl62GXfA/ltj3zrsPBEbwbjszgRzBeQgCGceNYrAbKZR97lKZLV2cM
# fl6teGdbVeNe68fY7Exuhsvz3Pifh6pyWBIPfab4+EI5Ozws5DJNSYzg4QDCOKCc
# +oQ+QdxuVq7GGlv0Z2gFAc0bv66HvJ1T9i7otmvkmd7FT4dYqNJlHsgf1XJu7lkc
# VzsJcp3XyreQxs17RZKRQgNMfT/K8qq4wg6G8xCfRi6kZoZoWmgYcCk4EYBga4pD
# o3Ns47NrN//mnWcBkobfL0jR+1Bg1Vz+IdMBQmP+73C0F8CPqO7TwUtfEur9/S4O
# h0Rg46n0whij4/3ODIQiDfOneNqT89s4z7kvM8b/BzxevkXTAgMBAAGjggErMIIB
# JzAOBgNVHQ8BAf8EBAMCAYYwEgYDVR0TAQH/BAgwBgEB/wIBATAdBgNVHSUEFjAU
# BggrBgEFBQcDAwYIKwYBBQUHAwgwOwYDVR0gBDQwMjAwBgRVHSAAMCgwJgYIKwYB
# BQUHAgEWGmh0dHA6Ly93d3cuZW50cnVzdC5uZXQvcnBhMDMGCCsGAQUFBwEBBCcw
# JTAjBggrBgEFBQcwAYYXaHR0cDovL29jc3AuZW50cnVzdC5uZXQwMAYDVR0fBCkw
# JzAloCOgIYYfaHR0cDovL2NybC5lbnRydXN0Lm5ldC9nMmNhLmNybDAdBgNVHQ4E
# FgQUgrrWPZfOn89x6JI3r/2ztWk1V88wHwYDVR0jBBgwFoAUanImetAe733nO2lR
# 1GyNn5ASZqswDQYJKoZIhvcNAQELBQADggEBAB9eQQS2g3AkUyxVcx1lOsDstHsE
# mF5ZOBMJpFmUQl5Qv09sbiUgkJNYQA31GbRi7iRewgFYFQIdEAlvqNT7kn43OD4v
# FH2PHUM2ZLNmE18UzKVx91shS8aXvtyV/HB9ERzTId3QJDkpxf4KGqXPe3nuOm/e
# 3L/pEd0WgwjTLI1/TagUeS8FYVI462DzFGh9y7KKrcCUXOQmDiyK3UbDzuRWUcVW
# 44W4TZtFcosH8Yr7Sbhf0fKWgV1pUiTxCCPS1iMP64vXfovBk2v68WJ7WOlQm5du
# F4gN4cZDmNeBYbaFnUfssZ6uPyA7Q53Yohzg1HwIwq92BvhiZnq29/rIrzUwggZv
# MIIEV6ADAgECAhAlvCvzKcoQfx6puoiF1J07MA0GCSqGSIb3DQEBDQUAMGkxCzAJ
# BgNVBAYTAlVTMRYwFAYDVQQKDA1FbnRydXN0LCBJbmMuMUIwQAYDVQQDDDlFbnRy
# dXN0IENvZGUgU2lnbmluZyBSb290IENlcnRpZmljYXRpb24gQXV0aG9yaXR5IC0g
# Q1NCUjEwHhcNMjEwNTA3MTkyMjE0WhcNNDAxMjI5MjM1OTAwWjBOMQswCQYDVQQG
# EwJVUzEWMBQGA1UEChMNRW50cnVzdCwgSW5jLjEnMCUGA1UEAxMeRW50cnVzdCBU
# aW1lIFN0YW1waW5nIENBIC0gVFMyMIICIjANBgkqhkiG9w0BAQEFAAOCAg8AMIIC
# CgKCAgEAtQMqBraETzTMnjG+6o7Ge24gJA8D172BNjPmb9I+u8SeqqDMZl1CYDRR
# 4i1hg7o8nmi4S8NmjqXeK7qbLUGl3tDLVwUtiNnARohtPDUNGZCsuAwZ8EzF20Vb
# 3DvCvI1EAOA4//9dyM8Imj3Usut7lcPEpQDgYgFkWV+M43uLqflFt8awogo3Q7hG
# gauGfn8v3vBf680gzKUtMeOoLpmb4Hjd9NtdgmN9MDmIfiQG5nxghHjIXCnllcv1
# e0/PWNyQz6eiexRUxOA3mDDBm7aunCn2ayVD5xJqPhmLvsDngddKMAh+fI3fPPNI
# kyHegYNqnQA+12C60bNiCJbptkkAOPRcdg3ad2o9oJXeHqHMWXpen8/pdl1A8rUV
# tWBoLPNaySOD2PUaa2kopLbUrmIG3iZFvd7pVneq3GSwQc3ME/FdcXSylAhLTzJu
# fnC4d+nVMjfCet5CEuhxHOLYYr+qa0GGBlvmab4mCfSXwv5M9IByhG7b3sUHqY4V
# om3hG+AJD/WgB6C7xHdzCTwHk40BtRJnLU0ZtbjFyRCrcM9C/6UnHdYS1uQYRGD7
# OV2Eu2a1UxKdAxSCZSm07Ely/bND5cMCKSS6H/i4vl2jty6p/M9i2fqXxBN441W5
# WnqE6/+7z2mIwLb5i2CFQRBQxwUQ0l52nT417PZPivkSez+vCzcCAwEAAaOCASww
# ggEoMBIGA1UdEwEB/wQIMAYBAf8CAQAwHQYDVR0OBBYEFCYP8MRICBvN3ZH1VFS2
# s7P8mfEIMB8GA1UdIwQYMBaAFIK61j2Xzp/PceiSN6/9s7VpNVfPMDMGCCsGAQUF
# BwEBBCcwJTAjBggrBgEFBQcwAYYXaHR0cDovL29jc3AuZW50cnVzdC5uZXQwMQYD
# VR0fBCowKDAmoCSgIoYgaHR0cDovL2NybC5lbnRydXN0Lm5ldC9jc2JyMS5jcmww
# DgYDVR0PAQH/BAQDAgGGMBMGA1UdJQQMMAoGCCsGAQUFBwMIMEUGA1UdIAQ+MDww
# MAYEVR0gADAoMCYGCCsGAQUFBwIBFhpodHRwOi8vd3d3LmVudHJ1c3QubmV0L3Jw
# YTAIBgZngQwBBAIwDQYJKoZIhvcNAQENBQADggIBAHY9RmiFXwnAzsn4x7jXfqdc
# OwtGc84iU4s49hG6d8TChyGPJ9kMlevoS+Vc15AVb6gqLIpq4CIRNHn6vZZ5yHYW
# zbFE6nFRD5tVyRh81YJ5nPOmt5OpBThyAgaXoaiyeX5eCOJubHmaqIeUzn+PVTZA
# D58/UdVmvnurhuZuh7pOY12x2I4fy2fDDXDaeuuTshSE0j59kz8WDDagXHkAnwtu
# 5YQq2IjBbQS3N/OSmnDgOwP9zbsngs4v/I3kssq++CWFxu10yD01XzgJPXlP6xvg
# QV6ldIE/URXA+EmIj7q7BdStRYIzecEljV9UPZHkuJ8XmABdrSjlT18b+/hUtaXE
# JPzwdjv4DV7MR7DKqrtuUd9TtUka8WCHt/n9aQZ1X3PWEZDygFiO5cMPA5iMBFOz
# KYffP/ew6UxXgSREeWj+GzYH1mU97G/WwrWqYXxmBuzS5WmA+HGSLlziEUYYRUvW
# +gVk77veBspDJb3fjckBdGGI2/DHpj3jDHG8f0vnyOBu4NqA+QCewGsl1aO6XCv+
# xGAgFEoHIzHlGsuRRLKae0PxW5i1IUu4MTsiVO55YfealBtmJyrG3+FlXe9xOeZQ
# Jf2HNLmuXXwnO+dlk/gohLFvhpMsk5O2cK+h7sikYamEO/sKt/i4bSbuR071XgX6
# nA8RfdfLka5DIPcVWWO8MIIGtDCCBJygAwIBAgIQW3AmzJb2eBq7hfYR9W27bjAN
# BgkqhkiG9w0BAQ0FADBOMQswCQYDVQQGEwJVUzEWMBQGA1UEChMNRW50cnVzdCwg
# SW5jLjEnMCUGA1UEAxMeRW50cnVzdCBUaW1lIFN0YW1waW5nIENBIC0gVFMyMB4X
# DTI0MDExOTE2NDc0N1oXDTM1MDQxODAwMDAwMFowdTELMAkGA1UEBhMCQ0ExEDAO
# BgNVBAgTB09udGFyaW8xDzANBgNVBAcTBk90dGF3YTEWMBQGA1UEChMNRW50cnVz
# dCwgSW5jLjErMCkGA1UEAxMiRW50cnVzdCBUaW1lc3RhbXAgQXV0aG9yaXR5IC0g
# VFNBMjCCAiIwDQYJKoZIhvcNAQEBBQADggIPADCCAgoCggIBAKqGBDhej2i/g/Va
# /VSbxX06dZ3Epy1PbenQBR9R4zO7L186sZZUiukIfDStiYNVpJ8CzCML2Bt6QKTM
# n0LqE398uOC3SgIF4oNJBG8Ghs1zQamD1Jhn+vRSGXrjSlkoh1YXXUjxhpELuCJk
# 20H5XVS1NaRuONwIDRpWGEDDfpg8QjOEDcWlYVHOIG1PAMtC8Kh1G9tG/incG57X
# +BgFUXYwm3S9lR8ZU2UVRyxWY2vpukisCji2wXL9IAqjdBprB3/R3awXzrUcx3qJ
# DOxtHDCNBfCvPZfMNbS1BQNmup52/DnayJrWEhLkQcKFIkDwy7qI6dKHwqvGNDvl
# qw0ABg4GtYt70Qsi2jyoyVZ78DUElupZRoyP1KC1qYOwzoT7S0TlQagW86/a0aAn
# cN8H7zfzrpHO1CF/hpjcXu6PwNpH22nKELmRlYAVY/ErGsKWoR/C9zzU9n0kByWo
# VJRvClXb2yRDbi47ontJu7XWGI/ZBLO+6se1ZJpLzlJhHisFHmM3Rp2xydVeh6Tq
# 1zSyIUIXuDooOl6l7T79Nxd/YyDye9ZfyvMUP+HO/4p3lGZticHg7H8EcDhkJ/Wn
# F31NygyBG8Xl8TbFeq59KaRHNTj6DfRTAykF/G+hi8nN8biIQNPS254lMa3X5lrY
# BLSEuwRGYWb/XTLe/NM+fE4Y737fAgMBAAGjggFlMIIBYTAMBgNVHRMBAf8EAjAA
# MB0GA1UdDgQWBBT1dhr4EJiSoRt3qQAs8ZViAgt5JzAfBgNVHSMEGDAWgBQmD/DE
# SAgbzd2R9VRUtrOz/JnxCDBoBggrBgEFBQcBAQRcMFowIwYIKwYBBQUHMAGGF2h0
# dHA6Ly9vY3NwLmVudHJ1c3QubmV0MDMGCCsGAQUFBzAChidodHRwOi8vYWlhLmVu
# dHJ1c3QubmV0L3RzMi1jaGFpbjI1Ni5wN2MwMQYDVR0fBCowKDAmoCSgIoYgaHR0
# cDovL2NybC5lbnRydXN0Lm5ldC90czJjYS5jcmwwDgYDVR0PAQH/BAQDAgeAMBYG
# A1UdJQEB/wQMMAoGCCsGAQUFBwMIMEwGA1UdIARFMEMwNwYKYIZIAYb6bAoBBzAp
# MCcGCCsGAQUFBwIBFhtodHRwczovL3d3dy5lbnRydXN0Lm5ldC9ycGEwCAYGZ4EM
# AQQCMA0GCSqGSIb3DQEBDQUAA4ICAQCpq32/GgCFW9ztcSZelIKvXIJL+hw5xVO1
# wwbdA2B++WGaQSDCrkEXLqVCCJEJXpcwmIFF1rK1hD2L6YcH2gE/4AhYZxEglB7A
# Fo5jd72bbAWNPbXLmjGpSlxiMqITzpCcDceQT2fD4Kh/h4RL1/LBIQ8Q5xe5+70J
# zL6KpgH+TYJPaSCadWvkDfdQJlP8M/f+G66nasCo6x6vItM9YBWfuuTVKaS1V5+x
# ZfaxTGHV0sjat6yqGB8gGIqm+PUH+XIr9mPYJZwW/vuU34Z0oiC08gjfFu85gHY6
# krHQloNq2nxTiVkbDVzxuynyIU9hLCSJ+HYJf9lokI5XDGTVdgBxWfPUhMg8oYh/
# qG/zvW5/sKiO5pNs6RTFdRwMK87yB/u1rtRBMb/BzGmYGK5bCI+rXGS5hGq9XDih
# 309Ct0tbUtPe9zvLn2/CWUIiTgsYtyLIL/xqCYXtUClResSUS7ZxZYpKhppLeEtW
# iqHeL4uTWzLMobnvqbYM2AYLFzC/DW+Pk6zLEfXxxwCe0BnXnF+OVHxrqUM7Y9zB
# RKu9jDpD8HI/JJXNhJFdIQANeGsNXCMkn0yvNMsUWt6x5EVQ/a9vpwWCp3PPz+mS
# 6nMqseHdogPmILwdZxpWwi72rF/KToGw5lodc8/SFPNQeCgFThVuhST4M6iIr3Rt
# 9FLOjKiEnjGCBBowggQWAgEBMGIwTjELMAkGA1UEBhMCVVMxFjAUBgNVBAoTDUVu
# dHJ1c3QsIEluYy4xJzAlBgNVBAMTHkVudHJ1c3QgVGltZSBTdGFtcGluZyBDQSAt
# IFRTMgIQW3AmzJb2eBq7hfYR9W27bjALBglghkgBZQMEAgOgggGLMBoGCSqGSIb3
# DQEJAzENBgsqhkiG9w0BCRABBDAcBgkqhkiG9w0BCQUxDxcNMjUwMTA5MDMwNjI5
# WjArBgkqhkiG9w0BCTQxHjAcMAsGCWCGSAFlAwQCA6ENBgkqhkiG9w0BAQ0FADBP
# BgkqhkiG9w0BCQQxQgRA2H4+omCH8GaN099o/Yaa9D1HZaQ4+Toc0nBQyLc5mHnS
# qYUdxI5sCqTs844/yjQ58tLp4TCKAyczYM5DUAsKUjCB0AYLKoZIhvcNAQkQAi8x
# gcAwgb0wgbowgbcwCwYJYIZIAWUDBAIDBEA5EUIuFwI+qpkkmXQODsjo0nLTVfxc
# 9mz5EVavl1U05ICv07x8TFtX79H/vNt1FGXg1AVahU6bETnZ9+xV1f4kMGYwUqRQ
# ME4xCzAJBgNVBAYTAlVTMRYwFAYDVQQKEw1FbnRydXN0LCBJbmMuMScwJQYDVQQD
# Ex5FbnRydXN0IFRpbWUgU3RhbXBpbmcgQ0EgLSBUUzICEFtwJsyW9ngau4X2EfVt
# u24wDQYJKoZIhvcNAQENBQAEggIALQA3db+k8JVp+1yWsjA+/Vx87abf9D7V33W6
# sv2bVTXaYyzsSjT3C0A6pLPDqyzTKD58C9DEVZMiTgWRuah767/YczqZwtWYqac0
# Uz6WBG+YsA7M0efWClmLr+ai+Nq+ztdoONJ8RPMc15qRIHT/Ybd+A8ajnuue4sAi
# C0yVJ1mafN4IoIJyRbH382VwxyJYy0YW10FmQVhHMLEVqovgWEOJmRGhUSEgke2R
# rIS4W4+NMWvTRvXXkt0WS94jAYr+eBhPh0IUD3zv6bakRykbfzDgDeNLOvsH0FqE
# AqPUXcYb6eSgrWf5cIKN8KNvI16i71Ie+edykUdR+fGQ7dojwZWZ7zvIYHoAPxW5
# I98luF2/fTOPX/Ofg4Z8LNVcHuJ7uM4k98zeMQV3nY+uUzhNBVmIalgddXESghfA
# 04RC7ri/usAVc3vZsytinRGDAnINLTmk2C4iK5LbpryOMAWCpmdo0bnzFKQ/K5T0
# cgntBhHHYod0d163JD2YiKBCE7CcKRXbLeyiXZA3Uv9knpTmI9lTr3rRiLvhJ9ZG
# XXAFKeofBlovhsreuOxqyczpUfhvVomJPAQiJnlCOsPLCuE7VWcAYq7XDhUR9oPg
# WWHzrJ8/97zX6x1uKLJG195X6ZqMXwtO6GmxExeEwruZrcuZl/IeltW17St3/z7X
# pk8+qsg=
# SIG # End signature block
