function Get-NtapClusterRrasData {
    <#
        .SYNOPSIS
            Collects ONTAP storage system data using ASUPs for the Ransomware Recovery Assurance Service report.

        .DESCRIPTION
            Uses the NetApp ONTAP PowerShell Toolkit to gather data required for the RRAS deliverable.

        .EXAMPLE
            PS> Get-NtapClusterRrasData -Name cluster1 | Export-Clixml -Path 'D:\Output\cluster1.xml'

            Collects the configuration information off the cluster 'cluster1' and saves the results to a .xml file for further
            processing.

        .EXAMPLE
            PS> Get-NtapClusterRrasData -Name cluster1 -Credential (Get-Credential) | Export-Clixml -Path 'D:\Output\cluster1.xml'

            Collects the configuration information off the cluster 'cluster1' using alternate credentials. The Add-NcCredential
            cmdlet may be used to cache credentials for the cluster to be accessed.

        .EXAMPLE
            PS> Get-NtapClusterRrasData -Name cluster1 | Format-NtapClusterRrasData | Out-NtapDocument -ExcelFile 'D:\Output\cluster1_RRAS.xlsx'

            Collects the configuration information from the cluster 'cluster1', formats the data and creates the specified
            Word and Excel documents.

        .LINK
            https://community.netapp.com/t5/NetAppDocs/bd-p/netappdocs

        .LINK
            Format-NtapClusterRrasData

        .INPUTS
            NetApp.Ontapi.AbstractController

        .INPUTS
            System.String

        .OUTPUTS
            System.Management.Automation.PSCustomObject
    #>

    [CmdletBinding(DefaultParameterSetName = 'Name')]
    [OutputType([System.Management.Automation.PSCustomObject])]
    param (
        # The system name or IP address of the cluster admin SVM to gather the data from.
        [Parameter(Mandatory, Position = 0, ParameterSetName = 'Name', ValueFromPipeline, ValueFromPipelineByPropertyName)]
        [ValidateNotNullOrEmpty()]
        [Alias('ClusterName', 'SystemName')]
        [string[]]$Name,

        # A NetApp ONTAP PowerShell Toolkit controller object (NcController).
        [Parameter(Mandatory, Position = 0, ParameterSetName = 'ControllerObject', ValueFromPipeline)]
        [ValidateNotNullOrEmpty()]
        [NetApp.Ontapi.AbstractController[]]$AbstractController,

        # Specifies explicit credentials to connect to the storage system. (Alias: Cred)
        [Parameter(ParameterSetName = 'Name')]
        [ValidateNotNullOrEmpty()]
        [Alias('Cred')]
        [pscredential]
        [System.Management.Automation.Credential()]
        $Credential = [pscredential]::Empty,

        # The port on which to connect to the storage system. If not set, it defaults to 80 for HTTP and 443 for HTTPS.
        [Parameter(ParameterSetName = 'Name')]
        [ValidateRange([uint16]::MinValue, [uint16]::MaxValue)]
        [uint16]$Port,

        # Connection timeout in milliseconds. Defaults to 60000 milliseconds.
        [Parameter()]
        [ValidateRange(0, [int]::MaxValue)]
        [int]$Timeout,

        # Forces the use of HTTPS instead of HTTP for the connection to the storage system.
        [Parameter(ParameterSetName = 'Name')]
        [Alias('ForceHTTPS')]
        [switch]$ForceSecure,

        # During the initial connection to the storage system, this switch disables a check that is performed to ensure that the LIF
        # being used for the connection is a cluster mgmt or a node mgmt port. This should only be used for a very specific scenario
        # where port forwarding is configured on the host connecting to the storage system.
        [Parameter()]
        [switch]$DisableLifCheck,

        # Disables the ping test prior to connecting to the storage system (useful in the event ICMP is being blocked). (Deprecated)
        [Parameter()]
        [ValidateScript({
                Write-Warning -Message '-DisablePingCheck has been deprecated (no longer performing an ICMP ping to test connectivity).'

                $true
            })]
        [switch]$DisablePingCheck,

        # Disables the manual generation of an ASUP for the storage system. (Deprecated)
        [Parameter()]
        [ValidateScript({
                Write-Warning -Message 'The -DisableAsupInvoke switch has been deprecated. The function no longer triggers ASUPs by default (use the -InvokeAsup switch to explicitly trigger ASUPs)'

                $true
            })]
        [switch]$DisableAsupInvoke,

        # Invokes a user generated ASUP for the storage system.
        [Parameter()]
        [switch]$InvokeAsup
    )

    process {
        Write-Verbose -Message "Initializing function: $( $PSCmdlet.MyInvocation.MyCommand.Name ) ($( $PSCmdlet.MyInvocation.MyCommand.ModuleName ) v$( (Get-NtapDocsVersion -AsVersion).ToString() ))"

        switch -Exact ($PSCmdlet.ParameterSetName) {
            'Name' {
                foreach ($cluster in ($Name | Where-Object { -not [string]::IsNullOrWhiteSpace($_) })) {
                    try {
                        Write-Verbose -Message "Validating connectivity to system: $cluster"

                        $ip = [System.Net.Dns]::GetHostAddresses($cluster)
                        $tcpClient = New-Object -TypeName System.Net.Sockets.TCPClient

                        if ($PSCmdlet.MyInvocation.BoundParameters.ContainsKey('Port')) {
                            $tcpConnect = $tcpClient.BeginConnect($ip, $Port, $null, $null)

                            if (-not $tcpConnect.AsyncWaitHandle.WaitOne(7500, $false)) {
                                $tcpClient.Close()

                                throw 'Connection failed (ResourceUnavailable).'
                            }
                        }
                        elseif ($ForceSecure.IsPresent) {
                            $tcpConnect = $tcpClient.BeginConnect($ip, 443, $null, $null)

                            if (-not $tcpConnect.AsyncWaitHandle.WaitOne(7500, $false)) {
                                $tcpClient.Close()

                                throw 'Connection failed (ResourceUnavailable).'
                            }
                        }
                        else {
                            $tcpConnect = $tcpClient.BeginConnect($ip, 443, $null, $null)

                            if (-not $tcpConnect.AsyncWaitHandle.WaitOne(7500, $false)) {
                                $tcpClient.Close()

                                $tcpClient = New-Object -TypeName System.Net.Sockets.TCPClient
                                $tcpConnect = $tcpClient.BeginConnect($ip, 80, $null, $null)

                                if (-not $tcpConnect.AsyncWaitHandle.WaitOne(7500, $false)) {
                                    $tcpClient.Close()

                                    throw 'Connection failed (ResourceUnavailable).'
                                }
                            }
                        }

                        [void]$tcpClient.EndConnect($tcpConnect)
                        $tcpClient.Close()

                        $params = @{
                            Name        = $cluster
                            ForceSecure = $ForceSecure.IsPresent
                        }

                        if ($Credential -ne [pscredential]::Empty) {
                            $params.Add('Credential', $Credential)
                        }

                        if ($PSCmdlet.MyInvocation.BoundParameters.ContainsKey('Port')) {
                            $params.Add('Port', $Port)
                        }

                        if ($PSCmdlet.MyInvocation.BoundParameters.ContainsKey('Timeout')) {
                            $params.Add('Timeout', $Timeout)
                        }

                        $controller = Connect-NtapStorageSystem @params -NcController -ErrorAction Stop

                        if ($controller.GetType().FullName -ne 'NetApp.Ontapi.Filer.C.NcController') {
                            throw 'Incorrect controller type specified for the Get-NtapClusterRrasData function.'
                        }

                        if (-not (Test-NtapStorageSystemConnection -Controller $controller -DisableLifCheck:$DisableLifCheck.IsPresent -ErrorAction Stop)) {
                            throw 'Unspecified error.'
                        }

                        Write-Verbose -Message "Valid connection established to system: $cluster"
                    }
                    catch {
                        $Global:Error.RemoveAt(0)

                        Write-Error -ErrorRecord (New-Object -TypeName System.Management.Automation.ErrorRecord -ArgumentList ("Unable to gather data from system. Reason: $( $_.Exception.GetBaseException().Message )", $_.FullyQualifiedErrorId, $_.CategoryInfo.Category, $cluster))

                        continue
                    }

                    Write-Verbose -Message "Gathering data from system: $( $controller.Name )"

                    try {
                        $functionStartTime = [datetime]::Now

                        $ntapData = Get-ClusterRrasData -Controller $controller

                        $ntapData

                        if ($ntapData -and $InvokeAsup.IsPresent) {
                            foreach ($node in ($ntapData.Node | Where-Object { $_ })) {
                                try {
                                    Write-Verbose -Message "Invoking ASUP on node: $( $node.Node )"
                                    [void](Invoke-NcAutoSupport -Node $node.Node -Message "NetAppDocs_$( $ntapData.ReferenceDT.ToString('u') )" -Type all -Controller $controller)
                                }
                                catch {
                                    $Global:Error.RemoveAt(0)

                                    Show-ErrorMessage -Message "Unable to invoke ASUP on node: $( $node.Node )"
                                }
                            }
                        }

                        $statsParams = @{
                            FunctionPSCallStack = Get-PSCallStack
                            TargetDetails       = [pscustomobject]@{
                                SystemUid     = $ntapData.Cluster.Info.ClusterUuid
                                SystemType    = $ntapData.pstypenames[0]
                                SystemVersion = ([regex]::Split(([regex]::Split($ntapData.System.VersionInfo.Version, 'Release\s(?:\w+[__])?')[1]), '\s+'))[0].Replace(':', '')
                            }
                            ElapsedTicks        = ([datetime]::Now - $functionStartTime).Ticks
                            ExcludeParameter    = 'Name'
                        }

                        Write-Statistics @statsParams
                    }
                    catch {
                        Write-Error -ErrorRecord $_
                    }
                }
            }
            'ControllerObject' {
                foreach ($controller in ($AbstractController | Where-Object { $_ })) {
                    Write-Verbose -Message "Validating connectivity to system: $( $controller.Name )"

                    try {
                        if ($controller.GetType().FullName -ne 'NetApp.Ontapi.Filer.C.NcController') {
                            throw 'Incorrect controller type specified for the Get-NtapClusterRrasData function.'
                        }

                        if ($controller.Protocol -like 'HTTP*' -and $PSCmdlet.MyInvocation.BoundParameters.ContainsKey('Timeout')) {
                            $controller.TimeoutMsec = $Timeout
                        }

                        if (-not (Test-NtapStorageSystemConnection -Controller $controller -DisableLifCheck:$DisableLifCheck.IsPresent -ErrorAction Stop)) {
                            throw 'Unspecified error.'
                        }

                        Write-Verbose -Message "Valid connection established to system: $( $controller.Name )"
                    }
                    catch {
                        $Global:Error.RemoveAt(0)

                        Write-Error -ErrorRecord (New-Object -TypeName System.Management.Automation.ErrorRecord -ArgumentList ("Unable to gather data from system. Reason: $( $_.Exception.GetBaseException().Message )", $_.FullyQualifiedErrorId, $_.CategoryInfo.Category, $controller.Name))

                        continue
                    }

                    Write-Verbose -Message "Gathering data from system: $( $controller.Name )"

                    try {
                        $functionStartTime = [datetime]::Now

                        $ntapData = Get-ClusterRrasData -Controller $controller

                        $ntapData

                        if ($ntapData -and $InvokeAsup.IsPresent) {
                            foreach ($node in ($ntapData.Node | Where-Object { $_ })) {
                                try {
                                    Write-Verbose -Message "Invoking ASUP on node: $( $node.Node )"
                                    [void](Invoke-NcAutoSupport -Node $node.Node -Message "NetAppDocs_$( $ntapData.ReferenceDT.ToString('u') )" -Type all -Controller $controller)
                                }
                                catch {
                                    $Global:Error.RemoveAt(0)

                                    Show-ErrorMessage -Message "Unable to invoke ASUP on node: $( $node.Node )"
                                }
                            }
                        }

                        $statsParams = @{
                            FunctionPSCallStack  = Get-PSCallStack
                            TargetDetails        = [pscustomobject]@{
                                SystemUid     = $ntapData.Cluster.Info.ClusterUuid
                                SystemType    = $ntapData.pstypenames[0]
                                SystemVersion = ([regex]::Split(([regex]::Split($ntapData.System.VersionInfo.Version, 'Release\s(?:\w+[__])?')[1]), '\s+'))[0].Replace(':', '')
                            }
                            ElapsedTicks         = ([datetime]::Now - $functionStartTime).Ticks
                            ParameterToSetAsBool = 'AbstractController'
                        }

                        Write-Statistics @statsParams
                    }
                    catch {
                        Write-Error -ErrorRecord $_
                    }
                }
            }
        }
    }
}

function Get-ClusterRrasData {
    [CmdletBinding()]
    param (
        [Parameter(Mandatory)]
        [ValidateNotNullOrEmpty()]
        [NetApp.Ontapi.Filer.C.NcController]$Controller
    )

    $systemData = [pscustomobject]@{ }
    $systemData.pstypenames.Insert(0, 'NetAppDocs.NcController.Data')

    Write-Verbose -Message '- Capturing module version -'
    $systemData | Add-Member -MemberType NoteProperty -Name NetAppDocsVersion (Get-NtapDocsVersion -AsVersion)

    Write-Verbose -Message '- Capturing PowerShell version -'
    $systemData | Add-Member -MemberType NoteProperty -Name PowerShellVersion -Value $PSVersionTable.PSVersion

    $NetAppDocsError = @( )

    try {
        Write-Verbose -Message 'Calling Get-NcSystemApi'
        $useSystemCli = $null -ne (Get-NcSystemApi -Name 'system-cli*' -Controller $Controller -ErrorVariable +NetAppDocsError)

        #region System Information

        Write-Verbose -Message '- Gathering basic system information -'
        $systemData | Add-Member -MemberType NoteProperty -Name System -Value ([pscustomobject]@{ })

        Write-Verbose -Message 'Calling Get-NcSystemVersionInfo'
        $systemData.System | Add-Member -MemberType NoteProperty -Name VersionInfo -Value (Get-NcSystemVersionInfo -Controller $Controller -ErrorVariable +NetAppDocsError)

        Write-Verbose -Message 'Calling Get-NcSystemOntapiVersion'
        $systemData.System | Add-Member -MemberType NoteProperty -Name OntapiVersion -Value ([version]::Parse((Get-NcSystemOntapiVersion -Controller $Controller -ErrorVariable +NetAppDocsError).ToString()))

        #endregion System Information

        #region Node Information

        Write-Verbose -Message '- Gathering node information-'

        Write-Verbose -Message 'Calling Get-NcNode'
        $systemData | Add-Member -MemberType NoteProperty -Name Node -Value (Get-NcNode -Controller $Controller -ErrorVariable +NetAppDocsError | Where-Object { -not [string]::IsNullOrWhiteSpace($_.Node) })

        #BURT 520762
        if ($systemData.System.OntapiVersion -lt [version]::Parse('1.17')) {
            $clusterShowCliOutput = $(
                if ($useSystemCli) {
                    Write-Verbose -Message 'Calling Invoke-NcSystemApi (cluster show (priv advanced))'
                    (Invoke-NcSystemApi -Request '<system-cli><priv>advanced</priv><args><arg>set -showseparator ~;</arg><arg>set -showallfields true;</arg><arg>cluster show</arg></args></system-cli>' -Controller $Controller -ErrorVariable +NetAppDocsError).results.'cli-output' | Convert-SystemCliFieldDelimitedText
                }
                else {
                    Write-Verbose -Message 'Calling Invoke-NcSsh (cluster show (priv advanced))'
                    Invoke-NcSsh -Command 'set -privilege advanced -confirmations off; set -showseparator ~; set -showallfields true; cluster show' -Controller $Controller -ErrorVariable +NetAppDocsError -Verbose:$false | Convert-SystemCliFieldDelimitedText
                }
            )

            foreach ($node in ($systemData.Node | Where-Object { $_ })) {
                $node | Add-Member -NotePropertyMembers @{
                    IsEpsilonNode         = ($clusterShowCliOutput | Where-Object { $_.NodeUuid -ceq $node.NodeUuid }).Epsilon
                    IsNodeHealthy         = ($clusterShowCliOutput | Where-Object { $_.NodeUuid -ceq $node.NodeUuid }).Health
                    IsNodeClusterEligible = ($clusterShowCliOutput | Where-Object { $_.NodeUuid -ceq $node.NodeUuid }).Eligibility
                }
            }
        }

        Write-Verbose -Message 'Calling Get-NcNodeInfo'
        $systemData | Add-Member -MemberType NoteProperty -Name NodeInfo -Value (Get-NcNodeInfo -Controller $Controller -ErrorVariable +NetAppDocsError)

        Write-Verbose -Message 'Identifying additional properties from parsed sysconfig'
        foreach ($node in ($systemData.NodeInfo | Where-Object { $_ })) {
            $node | Add-Member -MemberType NoteProperty -Name IoxmPresent -Value ([bool]($sysconfigProperties | Where-Object { $_.NodeName -ceq $node.SystemName }).IoxmPresent)

            $node | Add-Member -NotePropertyMembers @{
                PlatformConfig = Get-PlatformConfig -SystemModel $node.SystemModel -IsClustered (! [string]::IsNullOrWhiteSpace($node.PartnerSystemId)) -IoxmPresent $node.IoxmPresent
                LoaderVersion  = ($sysconfigProperties | Where-Object { $_.NodeName -ceq $node.SystemName }).LoaderVersion
            }
        }

        #endregion Node Information

        #region Cluster Information

        Write-Verbose -Message '- Gathering cluster information -'
        $systemData | Add-Member -MemberType NoteProperty -Name Cluster -Value ([pscustomobject]@{ })

        Write-Verbose -Message 'Calling Get-NcCluster'
        $systemData.Cluster | Add-Member -MemberType NoteProperty -Name Info -Value (Get-NcCluster -Controller $Controller -ErrorVariable +NetAppDocsError)

        $systemData.Cluster | Add-Member -MemberType NoteProperty -Name RingReplication -Value $(
            if ($useSystemCli) {
                Write-Verbose -Message 'Calling Invoke-NcSystemApi (cluster ring show (priv advanced))'
                (Invoke-NcSystemApi -Request '<system-cli><priv>advanced</priv><args><arg>set -showseparator ~;</arg><arg>set -showallfields true;</arg><arg>cluster ring show</arg></args></system-cli>' -Controller $Controller -ErrorVariable +NetAppDocsError).results.'cli-output' | Convert-SystemCliFieldDelimitedText
            }
            else {
                Write-Verbose -Message 'Calling Invoke-NcSsh (cluster ring show (priv advanced))'
                Invoke-NcSsh -Command 'set -privilege advanced -confirmations off; set -showseparator ~; set -showallfields true; cluster ring show' -Controller $Controller -ErrorVariable +NetAppDocsError -Verbose:$false | Convert-SystemCliFieldDelimitedText
            }
        )

        #endregion Cluster Information

        #region Vserver Information

        Write-Verbose -Message '- Gathering Vserver information -'
        $systemData | Add-Member -MemberType NoteProperty -Name Vserver -Value ([pscustomobject]@{ })

        Write-Verbose -Message 'Calling Get-NcVserver'
        $systemData.Vserver | Add-Member -MemberType NoteProperty -Name Info -Value (Get-NcVserver -Controller $Controller -ErrorVariable +NetAppDocsError)

        #endregion Vserver Information

        Write-Verbose -Message '- Gathering detailed volume information -'
        $systemData | Add-Member -MemberType NoteProperty -Name Vol -Value ([pscustomobject]@{ })

        #ZAPI BUG: Not all attributes are returned unless we inititalize the template (flexgroup aggr-list specifically)
        Write-Verbose -Message 'Calling Get-NcVol'
        $volTemplate = Set-AllNcTemplateProperties -Template (Get-NcVol -Template -Fill -Controller $Controller -ErrorVariable +NetAppDocsError)
        $systemData.Vol | Add-Member -MemberType NoteProperty -Name Info -Value (Get-NcVol -Attributes $volTemplate -Controller $Controller -ErrorVariable +NetAppDocsError)

        # vol show -fields commands
        if ($systemData.System.OntapiVersion -ge [version]::Parse('1.140')) {
            Write-Verbose -Message '- Gathering misc vol fields -'
            $volShowFields = @( 'vserver', 'vserver-uuid', 'instance-uuid', 'tiering-minimum-cooling-days' )

            if ($systemData.System.OntapiVersion -ge [version]::Parse('1.160')) {
                $volShowFields += @('encryption-state', 'encryption-type' )

                if ($systemData.System.OntapiVersion -ge [version]::Parse('1.201')) {
                    $volShowFields += 'anti-ransomware-state'

                    if ($systemData.System.OntapiVersion -ge [version]::Parse('1.221')) {
                        $volShowFields += @( 'snapshot-locking-enabled', 'expiry-time', 'compliance-clock-time' )
                    }
                }

                $systemData.Vol | Add-Member -MemberType NoteProperty -Name CliFields -Value $(
                    if ($useSystemCli) {
                        Write-Verbose -Message "Calling Invoke-NcSystemApi (volume show -fields $( $volShowFields -join ',' ) (priv diag))"
                        (Invoke-NcSystemApi -Request "<system-cli><args><arg>set -privilege diagnostic -confirmations off;</arg><arg>set -showseparator ~;</arg><arg>volume show -fields $( $volShowFields -join ',' )</arg></args></system-cli>" -Controller $Controller -ErrorVariable +NetAppDocsError).results.'cli-output' | Convert-SystemCliFieldDelimitedText
                    }
                    else {
                        Write-Verbose -Message "Calling Invoke-NcSsh (vol show -fields $( $volShowFields -join ',' ) (priv diag))"
                        Invoke-NcSsh -Command "set -privilege diagnostic -confirmations off; set -showseparator ~; volume show -fields $( $volShowFields -join ',' )" -Controller $Controller -ErrorVariable +NetAppDocsError -Verbose:$false | Convert-SystemCliFieldDelimitedText
                    }
                )
            }
        }

        #endregion Volume Configuration

        #region Snapshot Information

        ##Unused Cmdlet: Get-NcSnapshotPolicy - Using CLI for this output
        ##Unused Cmdlet: Get-NcSnapshotAutoDelete - Info exists in Get-NcVol
        ##Unused Cmdlet: Get-NcSnapshotDelta - Not needed for build docs
        ##Unused Cmdlet: Get-NcSnapshotReclaimable - Not needed for build docs
        ##Unused Cmdlet: Get-NcSnapshotReserve - Info exists in Get-NcVol
        ##Unused Cmdlet: Get-NcSnapshotRestoreStatus - Not needed for build docs

        Write-Verbose -Message '- Gathering detailed Snapshot information -'
        $systemData | Add-Member -MemberType NoteProperty -Name Snapshot -Value ([pscustomobject]@{ })

        Write-Verbose -Message 'Calling Get-NcSnapshot'
        $snapshotTemplate = Set-AllNcTemplateProperties -Template (Get-NcSnapshot -Template -Controller $Controller -ErrorVariable +NetAppDocsError)
        $systemData.Snapshot | Add-Member -MemberType NoteProperty -Name Info -Value (Get-NcSnapshot -Attributes $snapshotTemplate -Controller $Controller -ErrorVariable +NetAppDocsError)

        $systemData.Snapshot | Add-Member -MemberType NoteProperty -Name Policy -Value @(
            if ($useSystemCli) {
                Write-Verbose -Message 'Calling Invoke-NcSystemApi (volume snapshot policy show (priv diag))'
                (Invoke-NcSystemApi -Request '<system-cli><args><arg>set -privilege diagnostic -confirmations off;</arg><arg>set -showseparator ~;</arg><arg>set -showallfields true;</arg><arg>volume snapshot policy show</arg></args></system-cli>' -Controller $Controller -ErrorVariable +NetAppDocsError).results.'cli-output' |
                Convert-SystemCliFieldDelimitedText | Select-Object -ExcludeProperty Counts, Prefixes, Schedules, SnapmirrorLabels, RetentionPeriods -Property *, @{
                    Name       = 'SnapshotPolicySchedules'
                    Expression = {
                        @(
                            for ($i = 0; $i -lt $_.TotalSchedules; $i++) {
                                [pscustomobject]@{
                                    Count           = $_.Counts -split '\s*,\s*' | Select-Object -Index $i
                                    Prefix          = $_.Prefixes -split '\s*,\s*' | Select-Object -Index $i
                                    Schedule        = $_.Schedules -split '\s*,\s*' | Select-Object -Index $i
                                    SnapmirrorLabel = $_.SnapmirrorLabels -split '\s*,\s*' | Select-Object -Index $i
                                    RetentionPeriod = ($_.RetentionPeriods -split '\s*,\s*' | Select-Object -Index $i) -replace '^"|"$'
                                }
                            }
                        )
                    }
                }
            }
            else {
                Write-Verbose -Message 'Calling Invoke-NcSsh (volume snapshot policy show (priv diag))'
                Invoke-NcSsh -Command 'set -privilege diagnostic -confirmations off; set -showseparator ~; set -showallfields true; volume snapshot policy show' -Controller $Controller -ErrorVariable +NetAppDocsError -Verbose:$false |
                Convert-SystemCliFieldDelimitedText | Select-Object -ExcludeProperty Vserver, Counts, Prefixes, Schedules, SnapmirrorLabels, RetentionPeriods -Property *, @{
                    Name       = 'VserverName'
                    Expression = { $_.Vserver }
                }, @{
                    Name       = 'SnapshotPolicySchedules'
                    Expression = {
                        @(
                            for ($i = 0; $i -lt $_.TotalSchedules; $i++) {
                                [pscustomobject]@{
                                    Count           = $_.Counts -split '\s*,\s*' | Select-Object -Index $i
                                    Prefix          = $_.Prefixes -split '\s*,\s*' | Select-Object -Index $i
                                    Schedule        = $_.Schedules -split '\s*,\s*' | Select-Object -Index $i
                                    SnapmirrorLabel = $_.SnapmirrorLabels -split '\s*,\s*' | Select-Object -Index $i
                                    RetentionPeriod = ($_.RetentionPeriods -split '\s*,\s*' | Select-Object -Index $i) -replace '^"|"$'
                                }
                            }
                        )
                    }
                }
            }
        )

        #endregion Snapshot Information

        #region Snapmirror Configuration

        ##Unused Cmdlet: Get-NcSnapmirrorSnapshotOwner - Not currently needed for build docs
        ##Unused Cmdlet: Get-NcSnapmirrorHistory - (1.30+) Not currently needed for build docs
        ##Unused Cmdlet: Get-NcSnapmirrorDpPersonality - (1.120+) Currently not needed for build docs (Snapmirror.DpPersonality)
        ##Unused Cmdlet: Get-NcSnapmirrorUnprotected - (1.130+) Not currently needed for build docs
        ##Unused Cmdlet: Get-NcSnapmirrorCapabilities - (1.130+) Not currently needed for build docs

        Write-Verbose -Message '- Gathering Snapmirror configuration -'
        $systemData | Add-Member -MemberType NoteProperty -Name Snapmirror -Value ([pscustomobject]@{ })

        #BURTS 653291, 658442, 674884, 715998 (No way to get around this, there may be missing entries in the output)
        Write-Verbose -Message 'Calling Get-NcSnapmirror'
        $systemData.Snapmirror | Add-Member -MemberType NoteProperty -Name Info -Value $(
            if ($systemData.System.OntapiVersion -ge [version]::Parse('1.100')) {
                Get-NcSnapmirror -Expand -Controller $Controller -ErrorVariable +NetAppDocsError
            }
            else {
                Get-NcSnapmirror -Controller $Controller -ErrorVariable +NetAppDocsError
            }
        )

        if ($systemData.System.OntapiVersion -ge [version]::Parse('1.20')) {
            Write-Verbose -Message 'Calling Get-NcSnapmirrorDestination'
            $systemData.Snapmirror | Add-Member -MemberType NoteProperty -Name Destination -Value (Get-NcSnapmirrorDestination -Controller $Controller -ErrorVariable +NetAppDocsError)

            #Write-Verbose -Message 'Calling Get-NcSnapmirrorPolicy'
            #$systemData.Snapmirror | Add-Member -MemberType NoteProperty -Name Policy -Value (Get-NcSnapmirrorPolicy -Controller $Controller -ErrorVariable +NetAppDocsError)

            $systemData.Snapmirror | Add-Member -MemberType NoteProperty -Name Policy -Value @(
                if ($useSystemCli) {
                    Write-Verbose -Message 'Calling Invoke-NcSystemApi (snapmirror policy show (priv diag))'
                    (Invoke-NcSystemApi -Request '<system-cli><args><arg>set -privilege diagnostic -confirmations off;</arg><arg>set -showseparator ~;</arg><arg>set -showallfields true;</arg><arg>snapmirror policy show</arg></args></system-cli>' -Controller $Controller -ErrorVariable +NetAppDocsError).results.'cli-output' | Convert-SystemCliFieldDelimitedText
                }
                else {
                    Write-Verbose -Message 'Calling Invoke-NcSsh (snapmirror policy show (priv diag))'
                    Invoke-NcSsh -Command 'set -privilege diagnostic -confirmations off; set -showseparator ~; set -showallfields true; snapmirror policy show' -Controller $Controller -ErrorVariable +NetAppDocsError -Verbose:$false | Convert-SystemCliFieldDelimitedText
                }
            )
        }

        #endregion Snapmirror Configuration

        #region SnapLock Information

        if ($systemData.System.OntapiVersion -ge [version]::Parse('1.100')) {
            Write-Verbose -Message '- Gathering SnapLock information -'
            $systemData | Add-Member -MemberType NoteProperty -Name Snaplock -Value ([pscustomobject]@{ })

            $systemData.Snaplock | Add-Member -MemberType NoteProperty -Name Volume -Value $(
                if ($useSystemCli) {
                    Write-Verbose -Message 'Calling Invoke-NcSystemApi (volume snaplock show)'
                    (Invoke-NcSystemApi -Request '<system-cli><args><arg>set -showseparator ~;</arg><arg>set -showallfields true;</arg><arg>volume snaplock show</arg></args></system-cli>' -Controller $Controller -ErrorVariable +NetAppDocsError).results.'cli-output' | Convert-SystemCliFieldDelimitedText
                }
                else {
                    Write-Verbose -Message 'Calling Invoke-NcSsh (volume snaplock show)'
                    Invoke-NcSsh -Command 'set -showseparator ~; set -showallfields true; volume snaplock show' -Controller $Controller -ErrorVariable +NetAppDocsError -Verbose:$false | Convert-SystemCliFieldDelimitedText
                }
            )
        }

        #endregion SnapLock Information

        Write-Verbose -Message '- Getting current timestamp -'
        if ($systemData.System.OntapiVersion -lt [version]::Parse('1.30')) {
            $systemData | Add-Member -MemberType NoteProperty -Name ReferenceDT -Value (ConvertFrom-UnixTimestamp -Seconds (Get-NcClusterHa -Node ($systemData.Net.Interface | Where-Object { $_.Address -eq $Controller.Address.IpAddressToString }).CurrentNode -Controller $Controller -ErrorVariable +NetAppDocsError).CurrentTime)
        }
        else {
            $systemData | Add-Member -MemberType NoteProperty -Name ReferenceDT -Value (ConvertFrom-UnixTimestamp -Seconds (Get-NcTime -Controller $Controller -ErrorVariable +NetAppDocsError).UtcTime)
        }

        Write-Verbose -Message '- Identifying SystemName -'
        $systemData | Add-Member -MemberType NoteProperty -Name SystemName -Value $systemData.Cluster.Info.ClusterName

        Write-Verbose -Message '- Identifying any errors during processing -'
        $systemData | Add-Member -MemberType NoteProperty -Name DiscoveryError -Value @(
            foreach ($err in ($NetAppDocsError | Where-Object { $_ })) {
                $err.CategoryInfo | Select-Object -Property *, @{
                    Name       = 'Message'
                    Expression = { $err.Exception.Message }
                }
            }
        )

        if (($systemData.DiscoveryError | Measure-Object).Count -gt 0) {
            $msg = New-Object -TypeName System.Text.StringBuilder

            [void]$msg.AppendLine("Errors were encountered while collecting data from system: $( $Controller.Name )")
            [void]$msg.AppendLine('-------------------------------------------------------------------------------------')
            [void]$msg.AppendLine('Please review the output for accuracy and, if necessary, correct any error conditions')
            [void]$msg.AppendLine('then run these scripts again.')
            [void]$msg.AppendLine('-------------------------------------------------------------------------------------')
            [void]$msg.AppendLine()

            Write-Warning -Message $msg.ToString()
        }

        $systemData
    }
    catch {
        $Global:Error.RemoveAt(0)

        $msg = New-Object -TypeName System.Text.StringBuilder

        [void]$msg.AppendLine('An unexpected error occurred. Processing halted.')
        [void]$msg.AppendLine('--------------------------------------------------------------------------------------------------')
        [void]$msg.AppendLine('As this error is not expected, please consider contacting the support staff via e-mail or by')
        [void]$msg.AppendLine('starting a thread on the community site.')
        [void]$msg.AppendLine('            E-mail: ng-NetAppDocs-support@netapp.com')
        [void]$msg.AppendLine('    Community site: https://community.netapp.com/t5/NetAppDocs/bd-p/netappdocs')
        [void]$msg.AppendLine('--------------------------------------------------------------------------------------------------')
        [void]$msg.AppendLine($( $_.Exception.GetBaseException().Message ))
        [void]$msg.AppendLine("Location: $( $_.ScriptStackTrace )")
        [void]$msg.AppendLine()

        $PSCmdlet.ThrowTerminatingError((New-Object -TypeName System.Management.Automation.ErrorRecord -ArgumentList ($msg.ToString(), $_.FullyQualifiedErrorId, $_.CategoryInfo.Category, $_.CategoryInfo.TargetName)))
    }
}

# SIG # Begin signature block
# MIIy0gYJKoZIhvcNAQcCoIIywzCCMr8CAQExDzANBglghkgBZQMEAgEFADB5Bgor
# BgEEAYI3AgEEoGswaTA0BgorBgEEAYI3AgEeMCYCAwEAAAQQH8w7YFlLCE63JNLG
# KX7zUQIBAAIBAAIBAAIBAAIBADAxMA0GCWCGSAFlAwQCAQUABCBDq3+uc/prekgA
# ot/+FEQEgSMSt9LGplj6LrlM5hXU26CCFxcwggQ+MIIDJqADAgECAgRKU4woMA0G
# CSqGSIb3DQEBCwUAMIG+MQswCQYDVQQGEwJVUzEWMBQGA1UEChMNRW50cnVzdCwg
# SW5jLjEoMCYGA1UECxMfU2VlIHd3dy5lbnRydXN0Lm5ldC9sZWdhbC10ZXJtczE5
# MDcGA1UECxMwKGMpIDIwMDkgRW50cnVzdCwgSW5jLiAtIGZvciBhdXRob3JpemVk
# IHVzZSBvbmx5MTIwMAYDVQQDEylFbnRydXN0IFJvb3QgQ2VydGlmaWNhdGlvbiBB
# dXRob3JpdHkgLSBHMjAeFw0wOTA3MDcxNzI1NTRaFw0zMDEyMDcxNzU1NTRaMIG+
# MQswCQYDVQQGEwJVUzEWMBQGA1UEChMNRW50cnVzdCwgSW5jLjEoMCYGA1UECxMf
# U2VlIHd3dy5lbnRydXN0Lm5ldC9sZWdhbC10ZXJtczE5MDcGA1UECxMwKGMpIDIw
# MDkgRW50cnVzdCwgSW5jLiAtIGZvciBhdXRob3JpemVkIHVzZSBvbmx5MTIwMAYD
# VQQDEylFbnRydXN0IFJvb3QgQ2VydGlmaWNhdGlvbiBBdXRob3JpdHkgLSBHMjCC
# ASIwDQYJKoZIhvcNAQEBBQADggEPADCCAQoCggEBALqEtnLbngxr4pnpMAGnduoy
# uJVBGsnaYU5Ycs/+9oJ5v3NhBgqlJ9izX9NFThxy1k4y8nKKD/eDGdBqgIAARR6w
# x+eavxJXJxyjaC8Kh71qaw5eZfMcd9XUhY1wIbSzMueLotWGOQKxuNJHzuTJScQ7
# p977VH1XvvDobsJ5sjoLVeJQmBYyE1wveFbBwpSz8lrkJ5qfJNfG7NCbJYLjzMLE
# RcWMl3oGayoRn6kKbkg7b9vUERlC948Hv/VTX5w+9Bcs5mmsTjJMYnfqt+jluzS8
# GYuunFHnt361U7EzIuVtz3A8Gvrim2e2g/SNpa9iTE3gWKxkNBID+LaNlGMkpHEC
# AwEAAaNCMEAwDgYDVR0PAQH/BAQDAgEGMA8GA1UdEwEB/wQFMAMBAf8wHQYDVR0O
# BBYEFGpyJnrQHu995ztpUdRsjZ+QEmarMA0GCSqGSIb3DQEBCwUAA4IBAQB5nx2W
# xrZ5PyKNh9OHAwRgamuaLlmJcxGsQ9H1E/+NOSvA8r1PcIypL+oXxAtUntQblpgz
# PKitYqIAdqtZaW4GHX7EuUSNmK8S1GHbChlGR/Pr92PBQAVApdK39LWaNr+piHaI
# BFUEK5yHfxo3PH4tpRrY1Ileyr2sPWzYba/V83YPzTuIOCKdbJOaxD2/ghtlP6YP
# Xar85bIVyrWtxrw90ITo6gZysE05Mni/PhGcC6SdmiHz8JsLMHjbwdyHQ/68Y5rK
# xcIcyceN/zsSWAjmtj3seixO+4OWzgw8aYdUc6RzwpP/URCsFVQB2PwFsYmhf3SD
# mknX3E57ikhvi0X2MIIF3zCCBMegAwIBAgIQTkDkN1Tt5owAAAAAUdOUfzANBgkq
# hkiG9w0BAQsFADCBvjELMAkGA1UEBhMCVVMxFjAUBgNVBAoTDUVudHJ1c3QsIElu
# Yy4xKDAmBgNVBAsTH1NlZSB3d3cuZW50cnVzdC5uZXQvbGVnYWwtdGVybXMxOTA3
# BgNVBAsTMChjKSAyMDA5IEVudHJ1c3QsIEluYy4gLSBmb3IgYXV0aG9yaXplZCB1
# c2Ugb25seTEyMDAGA1UEAxMpRW50cnVzdCBSb290IENlcnRpZmljYXRpb24gQXV0
# aG9yaXR5IC0gRzIwHhcNMjEwNTA3MTU0MzQ1WhcNMzAxMTA3MTYxMzQ1WjBpMQsw
# CQYDVQQGEwJVUzEWMBQGA1UECgwNRW50cnVzdCwgSW5jLjFCMEAGA1UEAww5RW50
# cnVzdCBDb2RlIFNpZ25pbmcgUm9vdCBDZXJ0aWZpY2F0aW9uIEF1dGhvcml0eSAt
# IENTQlIxMIICIjANBgkqhkiG9w0BAQEFAAOCAg8AMIICCgKCAgEAp4GP9xRFtmJD
# 8tiu0yVeSE9Rv8V9n1AcNdHWfmEqlBltJ0akphpd91RRaoAixqhmdU1Ug8leaBur
# 9ltksK2tIL1U70ZrbQLnRa519o6KuTIui7h3HFJNeYhfpToYyVAslyctv9oAfWN/
# 7zLsRodj25qfw1ohNnv5m9XKoG5yLPzh8Z5wTQhWFW+Qq/tIurnXwYJ4hWUuf7XJ
# wOIUtzcRQQbiiuCo9uV+tngFAcNg7U8HQS4KE0njkJt/3b36rL9kUdFcm7T1XOdc
# /zubpaAa130JssK3/24cvMh95ukr/HKzFOlKVRKEnEQldR32KvBPpSA9aCXrYZd8
# D+W2PfOuw8ERvBuOzOBHMF5CAIZx41isBsplH3uUpktXZwx+Xq14Z1tV417rx9js
# TG6Gy/Pc+J+HqnJYEg99pvj4Qjk7PCzkMk1JjODhAMI4oJz6hD5B3G5WrsYaW/Rn
# aAUBzRu/roe8nVP2Lui2a+SZ3sVPh1io0mUeyB/Vcm7uWRxXOwlyndfKt5DGzXtF
# kpFCA0x9P8ryqrjCDobzEJ9GLqRmhmhaaBhwKTgRgGBrikOjc2zjs2s3/+adZwGS
# ht8vSNH7UGDVXP4h0wFCY/7vcLQXwI+o7tPBS18S6v39Lg6HRGDjqfTCGKPj/c4M
# hCIN86d42pPz2zjPuS8zxv8HPF6+RdMCAwEAAaOCASswggEnMA4GA1UdDwEB/wQE
# AwIBhjASBgNVHRMBAf8ECDAGAQH/AgEBMB0GA1UdJQQWMBQGCCsGAQUFBwMDBggr
# BgEFBQcDCDA7BgNVHSAENDAyMDAGBFUdIAAwKDAmBggrBgEFBQcCARYaaHR0cDov
# L3d3dy5lbnRydXN0Lm5ldC9ycGEwMwYIKwYBBQUHAQEEJzAlMCMGCCsGAQUFBzAB
# hhdodHRwOi8vb2NzcC5lbnRydXN0Lm5ldDAwBgNVHR8EKTAnMCWgI6Ahhh9odHRw
# Oi8vY3JsLmVudHJ1c3QubmV0L2cyY2EuY3JsMB0GA1UdDgQWBBSCutY9l86fz3Ho
# kjev/bO1aTVXzzAfBgNVHSMEGDAWgBRqciZ60B7vfec7aVHUbI2fkBJmqzANBgkq
# hkiG9w0BAQsFAAOCAQEAH15BBLaDcCRTLFVzHWU6wOy0ewSYXlk4EwmkWZRCXlC/
# T2xuJSCQk1hADfUZtGLuJF7CAVgVAh0QCW+o1PuSfjc4Pi8UfY8dQzZks2YTXxTM
# pXH3WyFLxpe+3JX8cH0RHNMh3dAkOSnF/goapc97ee46b97cv+kR3RaDCNMsjX9N
# qBR5LwVhUjjrYPMUaH3LsoqtwJRc5CYOLIrdRsPO5FZRxVbjhbhNm0VyiwfxivtJ
# uF/R8paBXWlSJPEII9LWIw/ri9d+i8GTa/rxYntY6VCbl24XiA3hxkOY14FhtoWd
# R+yxnq4/IDtDndiiHODUfAjCr3YG+GJmerb3+sivNTCCBmcwggRPoAMCAQICEEbM
# 41QNEgkq80rzt39+/oswDQYJKoZIhvcNAQELBQAwYzELMAkGA1UEBhMCVVMxFjAU
# BgNVBAoTDUVudHJ1c3QsIEluYy4xPDA6BgNVBAMTM0VudHJ1c3QgRXh0ZW5kZWQg
# VmFsaWRhdGlvbiBDb2RlIFNpZ25pbmcgQ0EgLSBFVkNTMjAeFw0yNDEwMjMxNTIy
# MTRaFw0yNTExMDkxNTIyMTNaMIHCMQswCQYDVQQGEwJVUzETMBEGA1UECBMKQ2Fs
# aWZvcm5pYTERMA8GA1UEBxMIU2FuIEpvc2UxEzARBgsrBgEEAYI3PAIBAxMCVVMx
# GTAXBgsrBgEEAYI3PAIBAhMIRGVsYXdhcmUxFDASBgNVBAoTC05ldEFwcCwgSW5j
# MR0wGwYDVQQPExRQcml2YXRlIE9yZ2FuaXphdGlvbjEQMA4GA1UEBRMHMzQ1Mjc0
# MzEUMBIGA1UEAxMLTmV0QXBwLCBJbmMwggGiMA0GCSqGSIb3DQEBAQUAA4IBjwAw
# ggGKAoIBgQCIHyusYBqN6d8MjDVGb9yKy/jxOJi5Ry9vtMsEpSe0eX1NwlzjdPpo
# R+kSTTj0HXwjn48v4UtO3lOWVVdqDuAuECTvG+n31c1RmaB4qS0qg9VEDsUP0/gr
# R4GlmEnGRIeDNWnRIV4oHGTqA1NjNU3esZYLQSV9j5DSzh0W+vyq10stDZiLBNai
# KoBzi5fs+C7QWW+xd+fSFV3Z+zIfE/jHFa4ZKP0n8Bva/5XKZSg6XVBN09oY/EF2
# 67wlz2018jiSwKZi++kCqTIpp9j7dKP5x+0/bK4GlGeivT/NObhjDN8e0Iu5rC2L
# tCalpr5CBMzjKmwfV8mVd9L/qPKk8QZxBdBrUpA1qHDCHbd9BRcd2zT8doIbM0vT
# dHnT9pRPZYws7BfT33KMoV7nShbrNViWhkP3i3UkQv7W9RptiZwfEDpt4Lxn0Uw8
# Zf94fbwMQaV3A28ItUbueLN23+k41N0jqdFTbpCaqQpUT/NxlRHECtn0Lp6kjZTy
# wKF7FqH+jpcCAwEAAaOCATUwggExMAwGA1UdEwEB/wQCMAAwHQYDVR0OBBYEFGzw
# 5EcKP/wp/cuZ0BIs8j6slZQcMB8GA1UdIwQYMBaAFM6JT4JRqhWihGLKMSNh0mH7
# +P54MA4GA1UdDwEB/wQEAwIHgDATBgNVHSUEDDAKBggrBgEFBQcDAzBnBggrBgEF
# BQcBAQRbMFkwIwYIKwYBBQUHMAGGF2h0dHA6Ly9vY3NwLmVudHJ1c3QubmV0MDIG
# CCsGAQUFBzAChiZodHRwOi8vYWlhLmVudHJ1c3QubmV0L2V2Y3MyLWNoYWluLnA3
# YzAxBgNVHR8EKjAoMCagJKAihiBodHRwOi8vY3JsLmVudHJ1c3QubmV0L2V2Y3My
# LmNybDAgBgNVHSAEGTAXMAcGBWeBDAEDMAwGCmCGSAGG+mwKAQIwDQYJKoZIhvcN
# AQELBQADggIBADAwfASY7H1rvIVgDB4LCw7V6xYShLM1XBV0YV8mgn50EVUgkXRr
# qwdn9JgVJvPU6jQ30ab9bb2505N/e1hWQN+Skw21Kbixw3mJ2lZNjY9CvwPUAc4B
# 9xXFEl/w7w1GuATgHO77YCpe0QTbYpbuSTXA0YBs1LlzmuoMxCD+E82srnED8Crg
# m4IHe71kpqqbsIYDnxujU+w/+/CAijPCuxXD2Ff1D7Ms97n9OFvLZHU2e+7xTP66
# RFXF81uCNcLYZSAMiXpjFCiXz74yNHtqZYecnNKvA3hUnvE0BH2IbJFpFiwQO9H2
# rSxdf6qTJANWvCTm7AkDMPhCneufSIFeGHyINH5sKNc+MgsNawZwy2Bfld2YE+ii
# +RyoUgQvl+OZu58dcPYvbgSkc+nE7Xlq/nzE4qNB83ZUuo4MQuGgZ1ZdROSgO6BD
# lCDa1Mr+viR84DtT6dwMYDf0PYWe0LoDYBVLRknZHpSNCMULURCTENBJdJo1mrOn
# hmmm5ZJ3Hg/CmyrujK67YvNFUsQnpLKYEut7ntmJM5ptEPm7Ql5zzaOp3o68a/Q6
# 0+s6p4NwZQYo4ekHk+DM4oaPiv8K+dEPxvppQCSoSPG0S59A8oFBXzoaidOTC06j
# xfGFkPFtOEIO269lK0NCIgr2lJvlUdlUPSnM2eWkEkslz42ShJgON0SaMIIGgzCC
# BGugAwIBAgIQNa+3e500H2r8j4RGqzE1KzANBgkqhkiG9w0BAQ0FADBpMQswCQYD
# VQQGEwJVUzEWMBQGA1UECgwNRW50cnVzdCwgSW5jLjFCMEAGA1UEAww5RW50cnVz
# dCBDb2RlIFNpZ25pbmcgUm9vdCBDZXJ0aWZpY2F0aW9uIEF1dGhvcml0eSAtIENT
# QlIxMB4XDTIxMDUwNzE5MTk1MloXDTQwMTIyOTIzNTkwMFowYzELMAkGA1UEBhMC
# VVMxFjAUBgNVBAoTDUVudHJ1c3QsIEluYy4xPDA6BgNVBAMTM0VudHJ1c3QgRXh0
# ZW5kZWQgVmFsaWRhdGlvbiBDb2RlIFNpZ25pbmcgQ0EgLSBFVkNTMjCCAiIwDQYJ
# KoZIhvcNAQEBBQADggIPADCCAgoCggIBAL69pznJpX3sXWXx9Cuph9DnrRrFGjsY
# zuGhUY1y+s5YH1y4JEIPRtUxl9BKTeObMMm6l6ic/kU2zyeA53u4bsEkt9+ndNyF
# 8qMkWEXMlJQ7AuvEjXxG9VxmguOkwdMfrG4MUyMO1Dr62kLxg1RfNTJW8rV4m1cA
# SB6pYWEnDnMDQ7bWcJL71IWaMMaz5ppeS+8dKthmqxZG/wvYD6aJSgJRV0E8QThO
# l8dRMm1njmahXk2fNSKv1Wq3f0BfaDXMafrxBfDqhabqMoXLwcHKg2lFSQbcCWy6
# SWUZjPm3NyeMZJ414+Xs5wegnahyvG+FOiymFk49nM8I5oL1RH0owL2JrWwv3C94
# eRHXHHBL3Z0ITF4u+o29p91j9n/wUjGEbjrY2VyFRJ5jBmnQhlh4iZuHu1gcpChs
# xv5pCpwerBFgal7JaWUu7UMtafF4tzstNfKqT+If4wFvkEaq1agNBFegtKzjbb2d
# GyiAJ0bH2qpnlfHRh3vHyCXphAyPiTbSvjPhhcAz1aA8GYuvOPLlk4C/xsOre5PE
# PZ257kV2wNRobzBePLQ2+ddFQuASBoDbpSH85wV6KI20jmB798i1SkesFGaXoFpp
# cjFXa1OEzWG6cwcVcDt7AfynP4wtPYeM+wjX5S8Xg36Cq08J8inhflV3ZZQFHVnU
# Ct2TfuMUXeK7AgMBAAGjggErMIIBJzASBgNVHRMBAf8ECDAGAQH/AgEAMB0GA1Ud
# DgQWBBTOiU+CUaoVooRiyjEjYdJh+/j+eDAfBgNVHSMEGDAWgBSCutY9l86fz3Ho
# kjev/bO1aTVXzzAzBggrBgEFBQcBAQQnMCUwIwYIKwYBBQUHMAGGF2h0dHA6Ly9v
# Y3NwLmVudHJ1c3QubmV0MDEGA1UdHwQqMCgwJqAkoCKGIGh0dHA6Ly9jcmwuZW50
# cnVzdC5uZXQvY3NicjEuY3JsMA4GA1UdDwEB/wQEAwIBhjATBgNVHSUEDDAKBggr
# BgEFBQcDAzBEBgNVHSAEPTA7MDAGBFUdIAAwKDAmBggrBgEFBQcCARYaaHR0cDov
# L3d3dy5lbnRydXN0Lm5ldC9ycGEwBwYFZ4EMAQMwDQYJKoZIhvcNAQENBQADggIB
# AD4AVLgq849mr2EWxFiTZPRBi2RVjRs1M6GbkdirRsqrX7y+fnDk0tcHqJYH14bR
# VwoI0NB4Tfgq37IE85rh13zwwQB6wUCh34qMt8u0HQFh8piapt24gwXKqSwW3Jwt
# Dv6nl+RQqZeVwUsqjFHjxALga3w1TVO8S5QTi1MYFl6mCqe4NMFssess5DF9DCzG
# fOGkVugtdtWyE3XqgwCuAHfGb6k97mMUgVAW/FtPEhkOWw+N6kvOBkyJS64gzI5H
# pnXWZe4vMOhdNI8fgk1cQqbyFExQIJwJonQkXDnYiTKFPK+M5Wqe5gQ6pRP/qh3N
# R0suAgW0ao/rhU+B7wrbfZ8pj6XCP1I4UkGVO7w+W1QwQiMJY95QjYk1RfqruA+P
# oq17ehGT8Y8ohHtoeUdq6GQpTR/0HS9tHsiUhjzTWpl6a3yrNfcrOUtPuT8Wku8p
# jI2rrAEazHFEOctAPiASzghw40f+3IDXCADRC2rqIbV5ZhfpaqpW3c0VeLEDwBSt
# PkcYde0KU0syk83/gLGQ1hPl5EF4Iu1BguUO37DOlSFF5osB0xn39CtVrNlWc2MQ
# 4LigbctUlpigmSFRBqqmDDorY8t52kO50hLM3o9VeukJ8+Ka0yXBezaS2uDlUmfN
# 4+ZUCqWd1HOj0y9dBmSFA3d/YNjCvHTJlZFot7d+YRl1MYIbETCCGw0CAQEwdzBj
# MQswCQYDVQQGEwJVUzEWMBQGA1UEChMNRW50cnVzdCwgSW5jLjE8MDoGA1UEAxMz
# RW50cnVzdCBFeHRlbmRlZCBWYWxpZGF0aW9uIENvZGUgU2lnbmluZyBDQSAtIEVW
# Q1MyAhBGzONUDRIJKvNK87d/fv6LMA0GCWCGSAFlAwQCAQUAoIGEMBgGCisGAQQB
# gjcCAQwxCjAIoAKAAKECgAAwGQYJKoZIhvcNAQkDMQwGCisGAQQBgjcCAQQwHAYK
# KwYBBAGCNwIBCzEOMAwGCisGAQQBgjcCARUwLwYJKoZIhvcNAQkEMSIEIESBeTKw
# kMYx5rRFJx7mtsMZVJoZJK/ntflp7zVHjssTMA0GCSqGSIb3DQEBAQUABIIBgEA9
# iCFSYJkUFRiOm6wcT82JRMtN73SeuBpRpGEakx7E/4x2eOJ8RNSGvhfhAbJhml8m
# GNVcGjM8FePhhPy2q8yL4qiYncNiMEZ/JSwe5KHUq/owhVxImZQ/fSmvnV61tBoU
# Ik/3PNH6f5Klahnx7pZqgEqqiCmII/ztQTBGGRFFheFdDnj6F5ZZ2x7fVGxDMnsc
# Tr54lA0RysVnDBitKWMFBzadjCrkKaisdSVjLAJvPJBvVZcbjEvn9+QWAKaQZ8iu
# b1TkfpWWozT0b0Hs6a+3HTU79FcnTuxdEP4UvNdi7gSoR8/UruGssh1E8vpvgW4k
# laLurGqK/nDmffOgbf0UsLhoGdIC1/RzwEQJYeGUskcTbUF4lnHQ84uxumHUGfaR
# +wY08tO5H01rYG1HqKS8ExC7/UdGLFdUfoKEt291/z2uDYBeZrtt6pyss+JKkXVg
# 7WccaL2t7deUuAgRcRQANP1vnv9wszlQYQVnehOOp/odYpZS5z/kXRbRF4M5oKGC
# GGQwghhgBgorBgEEAYI3AwMBMYIYUDCCGEwGCSqGSIb3DQEHAqCCGD0wghg5AgED
# MQ0wCwYJYIZIAWUDBAIDMIH0BgsqhkiG9w0BCRABBKCB5ASB4TCB3gIBAQYKYIZI
# AYb6bAoDBTAxMA0GCWCGSAFlAwQCAQUABCAZeMmfnylQ/wpYAH6zyumYHa6Ml6af
# ChF3WqvJaChUwgIJALXhhpbgqP1tGA8yMDI1MDEwOTAzMDYyOVowAwIBAaB5pHcw
# dTELMAkGA1UEBhMCQ0ExEDAOBgNVBAgTB09udGFyaW8xDzANBgNVBAcTBk90dGF3
# YTEWMBQGA1UEChMNRW50cnVzdCwgSW5jLjErMCkGA1UEAxMiRW50cnVzdCBUaW1l
# c3RhbXAgQXV0aG9yaXR5IC0gVFNBMqCCEw4wggXfMIIEx6ADAgECAhBOQOQ3VO3m
# jAAAAABR05R/MA0GCSqGSIb3DQEBCwUAMIG+MQswCQYDVQQGEwJVUzEWMBQGA1UE
# ChMNRW50cnVzdCwgSW5jLjEoMCYGA1UECxMfU2VlIHd3dy5lbnRydXN0Lm5ldC9s
# ZWdhbC10ZXJtczE5MDcGA1UECxMwKGMpIDIwMDkgRW50cnVzdCwgSW5jLiAtIGZv
# ciBhdXRob3JpemVkIHVzZSBvbmx5MTIwMAYDVQQDEylFbnRydXN0IFJvb3QgQ2Vy
# dGlmaWNhdGlvbiBBdXRob3JpdHkgLSBHMjAeFw0yMTA1MDcxNTQzNDVaFw0zMDEx
# MDcxNjEzNDVaMGkxCzAJBgNVBAYTAlVTMRYwFAYDVQQKDA1FbnRydXN0LCBJbmMu
# MUIwQAYDVQQDDDlFbnRydXN0IENvZGUgU2lnbmluZyBSb290IENlcnRpZmljYXRp
# b24gQXV0aG9yaXR5IC0gQ1NCUjEwggIiMA0GCSqGSIb3DQEBAQUAA4ICDwAwggIK
# AoICAQCngY/3FEW2YkPy2K7TJV5IT1G/xX2fUBw10dZ+YSqUGW0nRqSmGl33VFFq
# gCLGqGZ1TVSDyV5oG6v2W2Swra0gvVTvRmttAudFrnX2joq5Mi6LuHccUk15iF+l
# OhjJUCyXJy2/2gB9Y3/vMuxGh2Pbmp/DWiE2e/mb1cqgbnIs/OHxnnBNCFYVb5Cr
# +0i6udfBgniFZS5/tcnA4hS3NxFBBuKK4Kj25X62eAUBw2DtTwdBLgoTSeOQm3/d
# vfqsv2RR0VybtPVc51z/O5uloBrXfQmywrf/bhy8yH3m6Sv8crMU6UpVEoScRCV1
# HfYq8E+lID1oJethl3wP5bY9867DwRG8G47M4EcwXkIAhnHjWKwGymUfe5SmS1dn
# DH5erXhnW1XjXuvH2OxMbobL89z4n4eqclgSD32m+PhCOTs8LOQyTUmM4OEAwjig
# nPqEPkHcblauxhpb9GdoBQHNG7+uh7ydU/Yu6LZr5JnexU+HWKjSZR7IH9Vybu5Z
# HFc7CXKd18q3kMbNe0WSkUIDTH0/yvKquMIOhvMQn0YupGaGaFpoGHApOBGAYGuK
# Q6NzbOOzazf/5p1nAZKG3y9I0ftQYNVc/iHTAUJj/u9wtBfAj6ju08FLXxLq/f0u
# DodEYOOp9MIYo+P9zgyEIg3zp3jak/PbOM+5LzPG/wc8Xr5F0wIDAQABo4IBKzCC
# AScwDgYDVR0PAQH/BAQDAgGGMBIGA1UdEwEB/wQIMAYBAf8CAQEwHQYDVR0lBBYw
# FAYIKwYBBQUHAwMGCCsGAQUFBwMIMDsGA1UdIAQ0MDIwMAYEVR0gADAoMCYGCCsG
# AQUFBwIBFhpodHRwOi8vd3d3LmVudHJ1c3QubmV0L3JwYTAzBggrBgEFBQcBAQQn
# MCUwIwYIKwYBBQUHMAGGF2h0dHA6Ly9vY3NwLmVudHJ1c3QubmV0MDAGA1UdHwQp
# MCcwJaAjoCGGH2h0dHA6Ly9jcmwuZW50cnVzdC5uZXQvZzJjYS5jcmwwHQYDVR0O
# BBYEFIK61j2Xzp/PceiSN6/9s7VpNVfPMB8GA1UdIwQYMBaAFGpyJnrQHu995ztp
# UdRsjZ+QEmarMA0GCSqGSIb3DQEBCwUAA4IBAQAfXkEEtoNwJFMsVXMdZTrA7LR7
# BJheWTgTCaRZlEJeUL9PbG4lIJCTWEAN9Rm0Yu4kXsIBWBUCHRAJb6jU+5J+Nzg+
# LxR9jx1DNmSzZhNfFMylcfdbIUvGl77clfxwfREc0yHd0CQ5KcX+Chqlz3t57jpv
# 3ty/6RHdFoMI0yyNf02oFHkvBWFSOOtg8xRofcuyiq3AlFzkJg4sit1Gw87kVlHF
# VuOFuE2bRXKLB/GK+0m4X9HyloFdaVIk8Qgj0tYjD+uL136LwZNr+vFie1jpUJuX
# bheIDeHGQ5jXgWG2hZ1H7LGerj8gO0Od2KIc4NR8CMKvdgb4YmZ6tvf6yK81MIIG
# bzCCBFegAwIBAgIQJbwr8ynKEH8eqbqIhdSdOzANBgkqhkiG9w0BAQ0FADBpMQsw
# CQYDVQQGEwJVUzEWMBQGA1UECgwNRW50cnVzdCwgSW5jLjFCMEAGA1UEAww5RW50
# cnVzdCBDb2RlIFNpZ25pbmcgUm9vdCBDZXJ0aWZpY2F0aW9uIEF1dGhvcml0eSAt
# IENTQlIxMB4XDTIxMDUwNzE5MjIxNFoXDTQwMTIyOTIzNTkwMFowTjELMAkGA1UE
# BhMCVVMxFjAUBgNVBAoTDUVudHJ1c3QsIEluYy4xJzAlBgNVBAMTHkVudHJ1c3Qg
# VGltZSBTdGFtcGluZyBDQSAtIFRTMjCCAiIwDQYJKoZIhvcNAQEBBQADggIPADCC
# AgoCggIBALUDKga2hE80zJ4xvuqOxntuICQPA9e9gTYz5m/SPrvEnqqgzGZdQmA0
# UeItYYO6PJ5ouEvDZo6l3iu6my1Bpd7Qy1cFLYjZwEaIbTw1DRmQrLgMGfBMxdtF
# W9w7wryNRADgOP//XcjPCJo91LLre5XDxKUA4GIBZFlfjON7i6n5RbfGsKIKN0O4
# RoGrhn5/L97wX+vNIMylLTHjqC6Zm+B43fTbXYJjfTA5iH4kBuZ8YIR4yFwp5ZXL
# 9XtPz1jckM+nonsUVMTgN5gwwZu2rpwp9mslQ+cSaj4Zi77A54HXSjAIfnyN3zzz
# SJMh3oGDap0APtdgutGzYgiW6bZJADj0XHYN2ndqPaCV3h6hzFl6Xp/P6XZdQPK1
# FbVgaCzzWskjg9j1GmtpKKS21K5iBt4mRb3e6VZ3qtxksEHNzBPxXXF0spQIS08y
# bn5wuHfp1TI3wnreQhLocRzi2GK/qmtBhgZb5mm+Jgn0l8L+TPSAcoRu297FB6mO
# FaJt4RvgCQ/1oAegu8R3cwk8B5ONAbUSZy1NGbW4xckQq3DPQv+lJx3WEtbkGERg
# +zldhLtmtVMSnQMUgmUptOxJcv2zQ+XDAikkuh/4uL5do7cuqfzPYtn6l8QTeONV
# uVp6hOv/u89piMC2+YtghUEQUMcFENJedp0+Nez2T4r5Ens/rws3AgMBAAGjggEs
# MIIBKDASBgNVHRMBAf8ECDAGAQH/AgEAMB0GA1UdDgQWBBQmD/DESAgbzd2R9VRU
# trOz/JnxCDAfBgNVHSMEGDAWgBSCutY9l86fz3Hokjev/bO1aTVXzzAzBggrBgEF
# BQcBAQQnMCUwIwYIKwYBBQUHMAGGF2h0dHA6Ly9vY3NwLmVudHJ1c3QubmV0MDEG
# A1UdHwQqMCgwJqAkoCKGIGh0dHA6Ly9jcmwuZW50cnVzdC5uZXQvY3NicjEuY3Js
# MA4GA1UdDwEB/wQEAwIBhjATBgNVHSUEDDAKBggrBgEFBQcDCDBFBgNVHSAEPjA8
# MDAGBFUdIAAwKDAmBggrBgEFBQcCARYaaHR0cDovL3d3dy5lbnRydXN0Lm5ldC9y
# cGEwCAYGZ4EMAQQCMA0GCSqGSIb3DQEBDQUAA4ICAQB2PUZohV8JwM7J+Me4136n
# XDsLRnPOIlOLOPYRunfEwochjyfZDJXr6EvlXNeQFW+oKiyKauAiETR5+r2Wech2
# Fs2xROpxUQ+bVckYfNWCeZzzpreTqQU4cgIGl6Gosnl+Xgjibmx5mqiHlM5/j1U2
# QA+fP1HVZr57q4bmboe6TmNdsdiOH8tnww1w2nrrk7IUhNI+fZM/Fgw2oFx5AJ8L
# buWEKtiIwW0Etzfzkppw4DsD/c27J4LOL/yN5LLKvvglhcbtdMg9NV84CT15T+sb
# 4EFepXSBP1EVwPhJiI+6uwXUrUWCM3nBJY1fVD2R5LifF5gAXa0o5U9fG/v4VLWl
# xCT88HY7+A1ezEewyqq7blHfU7VJGvFgh7f5/WkGdV9z1hGQ8oBYjuXDDwOYjART
# symH3z/3sOlMV4EkRHlo/hs2B9ZlPexv1sK1qmF8Zgbs0uVpgPhxki5c4hFGGEVL
# 1voFZO+73gbKQyW9343JAXRhiNvwx6Y94wxxvH9L58jgbuDagPkAnsBrJdWjulwr
# /sRgIBRKByMx5RrLkUSymntD8VuYtSFLuDE7IlTueWH3mpQbZicqxt/hZV3vcTnm
# UCX9hzS5rl18JzvnZZP4KISxb4aTLJOTtnCvoe7IpGGphDv7Crf4uG0m7kdO9V4F
# +pwPEX3Xy5GuQyD3FVljvDCCBrQwggScoAMCAQICEFtwJsyW9ngau4X2EfVtu24w
# DQYJKoZIhvcNAQENBQAwTjELMAkGA1UEBhMCVVMxFjAUBgNVBAoTDUVudHJ1c3Qs
# IEluYy4xJzAlBgNVBAMTHkVudHJ1c3QgVGltZSBTdGFtcGluZyBDQSAtIFRTMjAe
# Fw0yNDAxMTkxNjQ3NDdaFw0zNTA0MTgwMDAwMDBaMHUxCzAJBgNVBAYTAkNBMRAw
# DgYDVQQIEwdPbnRhcmlvMQ8wDQYDVQQHEwZPdHRhd2ExFjAUBgNVBAoTDUVudHJ1
# c3QsIEluYy4xKzApBgNVBAMTIkVudHJ1c3QgVGltZXN0YW1wIEF1dGhvcml0eSAt
# IFRTQTIwggIiMA0GCSqGSIb3DQEBAQUAA4ICDwAwggIKAoICAQCqhgQ4Xo9ov4P1
# Wv1Um8V9OnWdxKctT23p0AUfUeMzuy9fOrGWVIrpCHw0rYmDVaSfAswjC9gbekCk
# zJ9C6hN/fLjgt0oCBeKDSQRvBobNc0Gpg9SYZ/r0Uhl640pZKIdWF11I8YaRC7gi
# ZNtB+V1UtTWkbjjcCA0aVhhAw36YPEIzhA3FpWFRziBtTwDLQvCodRvbRv4p3Bue
# 1/gYBVF2MJt0vZUfGVNlFUcsVmNr6bpIrAo4tsFy/SAKo3Qaawd/0d2sF861HMd6
# iQzsbRwwjQXwrz2XzDW0tQUDZrqedvw52sia1hIS5EHChSJA8Mu6iOnSh8KrxjQ7
# 5asNAAYOBrWLe9ELIto8qMlWe/A1BJbqWUaMj9SgtamDsM6E+0tE5UGoFvOv2tGg
# J3DfB+83866RztQhf4aY3F7uj8DaR9tpyhC5kZWAFWPxKxrClqEfwvc81PZ9JAcl
# qFSUbwpV29skQ24uO6J7Sbu11hiP2QSzvurHtWSaS85SYR4rBR5jN0adscnVXoek
# 6tc0siFCF7g6KDpepe0+/TcXf2Mg8nvWX8rzFD/hzv+Kd5RmbYnB4Ox/BHA4ZCf1
# pxd9TcoMgRvF5fE2xXqufSmkRzU4+g30UwMpBfxvoYvJzfG4iEDT0tueJTGt1+Za
# 2AS0hLsERmFm/10y3vzTPnxOGO9+3wIDAQABo4IBZTCCAWEwDAYDVR0TAQH/BAIw
# ADAdBgNVHQ4EFgQU9XYa+BCYkqEbd6kALPGVYgILeScwHwYDVR0jBBgwFoAUJg/w
# xEgIG83dkfVUVLazs/yZ8QgwaAYIKwYBBQUHAQEEXDBaMCMGCCsGAQUFBzABhhdo
# dHRwOi8vb2NzcC5lbnRydXN0Lm5ldDAzBggrBgEFBQcwAoYnaHR0cDovL2FpYS5l
# bnRydXN0Lm5ldC90czItY2hhaW4yNTYucDdjMDEGA1UdHwQqMCgwJqAkoCKGIGh0
# dHA6Ly9jcmwuZW50cnVzdC5uZXQvdHMyY2EuY3JsMA4GA1UdDwEB/wQEAwIHgDAW
# BgNVHSUBAf8EDDAKBggrBgEFBQcDCDBMBgNVHSAERTBDMDcGCmCGSAGG+mwKAQcw
# KTAnBggrBgEFBQcCARYbaHR0cHM6Ly93d3cuZW50cnVzdC5uZXQvcnBhMAgGBmeB
# DAEEAjANBgkqhkiG9w0BAQ0FAAOCAgEAqat9vxoAhVvc7XEmXpSCr1yCS/ocOcVT
# tcMG3QNgfvlhmkEgwq5BFy6lQgiRCV6XMJiBRdaytYQ9i+mHB9oBP+AIWGcRIJQe
# wBaOY3e9m2wFjT21y5oxqUpcYjKiE86QnA3HkE9nw+Cof4eES9fywSEPEOcXufu9
# Ccy+iqYB/k2CT2kgmnVr5A33UCZT/DP3/huup2rAqOseryLTPWAVn7rk1SmktVef
# sWX2sUxh1dLI2resqhgfIBiKpvj1B/lyK/Zj2CWcFv77lN+GdKIgtPII3xbvOYB2
# OpKx0JaDatp8U4lZGw1c8bsp8iFPYSwkifh2CX/ZaJCOVwxk1XYAcVnz1ITIPKGI
# f6hv871uf7CojuaTbOkUxXUcDCvO8gf7ta7UQTG/wcxpmBiuWwiPq1xkuYRqvVw4
# od9PQrdLW1LT3vc7y59vwllCIk4LGLciyC/8agmF7VApUXrElEu2cWWKSoaaS3hL
# Voqh3i+Lk1syzKG576m2DNgGCxcwvw1vj5OsyxH18ccAntAZ15xfjlR8a6lDO2Pc
# wUSrvYw6Q/ByPySVzYSRXSEADXhrDVwjJJ9MrzTLFFreseRFUP2vb6cFgqdzz8/p
# kupzKrHh3aID5iC8HWcaVsIu9qxfyk6BsOZaHXPP0hTzUHgoBU4VboUk+DOoiK90
# bfRSzoyohJ4xggQaMIIEFgIBATBiME4xCzAJBgNVBAYTAlVTMRYwFAYDVQQKEw1F
# bnRydXN0LCBJbmMuMScwJQYDVQQDEx5FbnRydXN0IFRpbWUgU3RhbXBpbmcgQ0Eg
# LSBUUzICEFtwJsyW9ngau4X2EfVtu24wCwYJYIZIAWUDBAIDoIIBizAaBgkqhkiG
# 9w0BCQMxDQYLKoZIhvcNAQkQAQQwHAYJKoZIhvcNAQkFMQ8XDTI1MDEwOTAzMDYy
# OVowKwYJKoZIhvcNAQk0MR4wHDALBglghkgBZQMEAgOhDQYJKoZIhvcNAQENBQAw
# TwYJKoZIhvcNAQkEMUIEQKTIIfxMG2qyKn1jzlidvGJmNqtrbJH4zuVjhFXx9DbG
# /SYalamQkbn/fRykfnPTiFc9V6xETC/rO4gqcWWjGm8wgdAGCyqGSIb3DQEJEAIv
# MYHAMIG9MIG6MIG3MAsGCWCGSAFlAwQCAwRAORFCLhcCPqqZJJl0Dg7I6NJy01X8
# XPZs+RFWr5dVNOSAr9O8fExbV+/R/7zbdRRl4NQFWoVOmxE52ffsVdX+JDBmMFKk
# UDBOMQswCQYDVQQGEwJVUzEWMBQGA1UEChMNRW50cnVzdCwgSW5jLjEnMCUGA1UE
# AxMeRW50cnVzdCBUaW1lIFN0YW1waW5nIENBIC0gVFMyAhBbcCbMlvZ4GruF9hH1
# bbtuMA0GCSqGSIb3DQEBDQUABIICAJ9QSPXyJwM7lDr+HkIv74kc9HuUn29shgxm
# E6OlUe4+05egCOBBo7sgu/qepC2aLzAiTTbl+01sxulRwWrZdP9xPBb6H53l8IbU
# UxcK75YHHm3HvO4vAPIGnBqlz9kghzDciIKQ4BgAbiG27JV7QVLUAJdkaLY47XJw
# ui1W1WRsnHM9V6JLZ9xxGDo8mk7kK2+ktCfiOOICZVpDr32jdL35j5peaIM+sxdN
# IdSsDtL72jl8OnzHQ+sYTU/AmuLljmwCpbf4cAMMGT1RmoRNUT+dTaSu0ttboEBR
# J424ZrGwbLFA2QunWdPkCEAxObGWqabZ5BhtgMGTb1ItcN0PkiCvPReiuEalF1YS
# faoDSlNKGSwhB3yo2t2v6BkZlMN/OHToKJqfNMgPLnzuGMzzoWozlbTD7KfdupbF
# 5JJt80YB20VIZaMlmsYsptlGWMpnzQ/Z+ZnGKmtBgUb3LtpkIksh6VXgswZDTvtp
# GT/YVHhhF1IF9OlenV4KUtvc2sdQgtzXmyEeVxN/JoariEnrrcfCHytqXa0/QGfn
# TCkkTAQjqtxZCgwP16yG1wdgT+4aqfDn+Bx91ndNvhYvZfUQW7gcqeJ0GHqwhtiL
# CUE1gDHFrMVrdyyrbm8fy+iASOw1CZ6h3gv8E9Y+3E7U566ZefkuNwj5SGtjNUJD
# mDWtzMa+
# SIG # End signature block
