function Get-NtapFilerData {
    <#
        .SYNOPSIS
            Collects data from Data ONTAP 7-Mode storage systems.

        .DESCRIPTION
            Uses the NetApp ONTAP PowerShell Toolkit to gather as much configuration information as possible. This
            data can be used to automate build docs and health checks. It can also be used for historical
            configuration information.

        .EXAMPLE
            PS> Get-NtapFilerData -Name controller1, controller2 | Export-Clixml -Path 'D:\Output\HaPair.xml'

            Collects the controller information from the system names specified and saves the results to a .xml file for
            further processing.

        .EXAMPLE
            PS> Get-NtapFilerData -Name controller1, controller2 -Credential (Get-Credential) | Export-Clixml -Path 'D:\Output\HaPair.xml'

            Collects the controller information from the system names specified using alternate credentials and saves the
            results to a .xml file for further processing. The Add-NaCredential cmdlet may be used to cache
            credentials for the system to be accessed.

        .EXAMPLE
            PS> Get-NtapFilerData -Name controller1, controller2 | Format-NtapFilerData | Out-NtapDocument -WordFile 'D:\Output\HaPair.docx' -ExcelFile 'D:\Output\HaPair.xlsx'

            Collects the controller information from the system names specified, formats the data and creates the specified
            Word and Excel documents.

        .LINK
            https://community.netapp.com/t5/NetAppDocs/bd-p/netappdocs

        .LINK
            Format-NtapFilerData

        .INPUTS
            NetApp.Ontapi.AbstractController

        .INPUTS
            System.String

        .OUTPUTS
            System.Management.Automation.PSCustomObject

        .NOTES
            AUTHOR : Jason Cole
    #>

    [CmdletBinding(DefaultParameterSetName = 'Name')]
    [OutputType([System.Management.Automation.PSCustomObject])]
    param (
        # The system name or IP address of the storage system to gather the data from.
        [Parameter(Mandatory, Position = 0, ParameterSetName = 'Name', ValueFromPipeline, ValueFromPipelineByPropertyName)]
        [ValidateNotNullOrEmpty()]
        [Alias('FilerName', 'SystemName')]
        [string[]]$Name,

        # A NetApp ONTAP PowerShell Toolkit controller object (NaController).
        [Parameter(Mandatory, Position = 0, ParameterSetName = 'ControllerObject', ValueFromPipeline)]
        [ValidateNotNullOrEmpty()]
        [NetApp.Ontapi.AbstractController[]]$AbstractController,

        # Specifies explicit credentials to connect to the storage system. (Alias: Cred)
        [Parameter(ParameterSetName = 'Name')]
        [ValidateNotNullOrEmpty()]
        [Alias('Cred')]
        [pscredential]
        [System.Management.Automation.Credential()]
        $Credential = [pscredential]::Empty,

        # The port on which to connect to the storage system. If not set, it defaults to 80 for HTTP and 443 for HTTPS.
        [Parameter(ParameterSetName = 'Name')]
        [ValidateRange([uint16]::MinValue, [uint16]::MaxValue)]
        [uint16]$Port,

        # Connection timeout in milliseconds. Only applies to HTTP/HTTPS connections. Defaults to 60000 milliseconds.
        [Parameter()]
        [ValidateRange(0, [int]::MaxValue)]
        [int]$Timeout,

        # Forces the use of RPC for the connection to the storage system.
        [Parameter(ParameterSetName = 'Name')]
        [switch]$ForceRPC,

        # Forces the use of HTTPS instead of HTTP for the connection to the storage system.
        [Parameter(ParameterSetName = 'Name')]
        [Alias('ForceHTTPS')]
        [switch]$ForceSecure,

        # Disables the ping test prior to connecting to the storage system (useful in the event ICMP is being blocked). (Deprecated)
        [Parameter()]
        [ValidateScript({
                Write-Warning -Message '-DisablePingCheck has been deprecated (no longer performing an ICMP ping to test connectivity).'

                $true
            })]
        [switch]$DisablePingCheck,

        # Disables the manual generation of an ASUP for the storage system. (Deprecated)
        [Parameter()]
        [ValidateScript({
                Write-Warning -Message 'The -DisableAsupInvoke switch has been deprecated. The function no longer triggers ASUPs by default (use the -InvokeAsup switch to explicitly trigger ASUPs)'

                $true
            })]
        [switch]$DisableAsupInvoke,

        # Invokes a user generated ASUP for the storage system.
        [Parameter()]
        [switch]$InvokeAsup
    )

    process {
        Write-Verbose -Message "Initializing function: $( $PSCmdlet.MyInvocation.MyCommand.Name ) ($( $PSCmdlet.MyInvocation.MyCommand.ModuleName ) v$( (Get-NtapDocsVersion -AsVersion).ToString() ))"

        switch -Exact ($PSCmdlet.ParameterSetName) {
            'Name' {
                foreach ($filer in ($Name | Where-Object { -not [string]::IsNullOrWhiteSpace($_) })) {
                    try {
                        Write-Verbose -Message "Validating connectivity to system: $filer"

                        $ip = [System.Net.Dns]::GetHostAddresses($filer)
                        $tcpClient = New-Object -TypeName System.Net.Sockets.TCPClient

                        if ($PSCmdlet.MyInvocation.BoundParameters.ContainsKey('Port')) {
                            $tcpConnect = $tcpClient.BeginConnect($ip, $Port, $null, $null)

                            if (-not $tcpConnect.AsyncWaitHandle.WaitOne(7500, $false)) {
                                $tcpClient.Close()

                                throw 'Connection failed (ResourceUnavailable).'
                            }
                        }
                        elseif ($ForceRPC.IsPresent) {
                            $tcpConnect = $tcpClient.BeginConnect($ip, 445, $null, $null)

                            if (-not $tcpConnect.AsyncWaitHandle.WaitOne(7500, $false)) {
                                $tcpClient.Close()

                                throw 'Connection failed (ResourceUnavailable).'
                            }
                        }
                        elseif ($ForceSecure.IsPresent) {
                            $tcpConnect = $tcpClient.BeginConnect($ip, 443, $null, $null)

                            if (-not $tcpConnect.AsyncWaitHandle.WaitOne(7500, $false)) {
                                $tcpClient.Close()

                                throw 'Connection failed (ResourceUnavailable).'
                            }
                        }
                        else {
                            $tcpConnect = $tcpClient.BeginConnect($ip, 443, $null, $null)

                            if (-not $tcpConnect.AsyncWaitHandle.WaitOne(7500, $false)) {
                                $tcpClient.Close()

                                $tcpClient = New-Object -TypeName System.Net.Sockets.TCPClient
                                $tcpConnect = $tcpClient.BeginConnect($ip, 80, $null, $null)

                                if (-not $tcpConnect.AsyncWaitHandle.WaitOne(7500, $false)) {
                                    $tcpClient.Close()

                                    throw 'Connection failed (ResourceUnavailable).'
                                }
                            }
                        }

                        [void]$tcpClient.EndConnect($tcpConnect)
                        $tcpClient.Close()

                        $params = @{
                            Name        = $filer
                            ForceRPC    = $ForceRPC.IsPresent
                            ForceSecure = $ForceSecure.IsPresent
                        }

                        if ($Credential -ne [pscredential]::Empty) {
                            $params.Add('Credential', $Credential)
                        }

                        if ($PSCmdlet.MyInvocation.BoundParameters.ContainsKey('Port')) {
                            $params.Add('Port', $Port)
                        }

                        if ($PSCmdlet.MyInvocation.BoundParameters.ContainsKey('Timeout')) {
                            $params.Add('Timeout', $Timeout)
                        }

                        $controller = Connect-NtapStorageSystem @params -NaController -ErrorAction Stop

                        if ($controller.GetType().FullName -ne 'NetApp.Ontapi.Filer.NaController') {
                            throw 'Incorrect controller type specified for the Get-NtapFilerData function.'
                        }

                        if (-not (Test-NtapStorageSystemConnection -Controller $controller -ErrorAction Stop)) {
                            throw 'Unspecified error.'
                        }

                        Write-Verbose -Message "Valid connection established to system: $filer"
                    }
                    catch {
                        $Global:Error.RemoveAt(0)

                        Write-Error -ErrorRecord (New-Object -TypeName System.Management.Automation.ErrorRecord -ArgumentList ("Unable to gather data from system. Reason: $( $_.Exception.GetBaseException().Message )", $_.FullyQualifiedErrorId, $_.CategoryInfo.Category, $filer))

                        continue
                    }

                    Write-Verbose -Message "Gathering data from system: $filer"

                    try {
                        $functionStartTime = [datetime]::Now

                        $ntapData = Get-FilerData -Controller $controller

                        $ntapData

                        if ($ntapData -and $InvokeAsup.IsPresent) {
                            Write-Verbose -Message '- Generating ASUPs -'

                            try {
                                Write-Verbose -Message "Invoking ASUP on system: $( $ntapData.SystemInfo.SystemName )"
                                [void](Invoke-NaSystemApi -Request "<system-cli><args><arg>options</arg><arg>autosupport.doit</arg><arg>`"NetAppDocs_$( $ntapData.ReferenceDT.ToString('u') )`"</arg></args></system-cli>" -Controller $controller)
                            }
                            catch {
                                $Global:Error.RemoveAt(0)

                                Show-ErrorMessage -Message "Unable to invoke ASUP on system: $( $ntapData.SystemInfo.SystemName )"
                            }
                        }

                        $statsParams = @{
                            FunctionPSCallStack = Get-PSCallStack
                            TargetDetails       = [pscustomobject]@{
                                SystemUid     = [string]::Join('_', $ntapData.SystemInfo.SystemId, $ntapData.SystemInfo.SystemSerialNumber)
                                SystemType    = $ntapData.pstypenames[0]
                                SystemVersion = $ntapData.SystemVersion
                            }
                            ElapsedTicks        = ([datetime]::Now - $functionStartTime).Ticks
                            ExcludeParameter    = 'Name'
                        }

                        Write-Statistics @statsParams
                    }
                    catch {
                        Write-Error -ErrorRecord $_
                    }
                }
            }
            'ControllerObject' {
                foreach ($controller in ($AbstractController | Where-Object { $_ })) {
                    Write-Verbose -Message "Validating connectivity to system: $( $controller.Name )"

                    try {
                        if ($controller.GetType().FullName -ne 'NetApp.Ontapi.Filer.NaController') {
                            throw 'Incorrect controller type specified for the Get-NtapFilerData function.'
                        }

                        if ($controller.Protocol -like 'HTTP*' -and $PSCmdlet.MyInvocation.BoundParameters.ContainsKey('Timeout')) {
                            $controller.TimeoutMsec = $Timeout
                        }

                        if (-not (Test-NtapStorageSystemConnection -Controller $controller -ErrorAction Stop)) {
                            throw 'Unspecified error.'
                        }

                        Write-Verbose -Message "Valid connection established to system: $( $controller.Name )"
                    }
                    catch {
                        $Global:Error.RemoveAt(0)

                        Write-Error -ErrorRecord (New-Object -TypeName System.Management.Automation.ErrorRecord -ArgumentList ("Unable to gather data from system. Reason: $( $_.Exception.GetBaseException().Message )", $_.FullyQualifiedErrorId, $_.CategoryInfo.Category, $controller.Name))

                        continue
                    }

                    Write-Verbose -Message "Gathering data from system: $( $controller.Name )"

                    try {
                        $functionStartTime = [datetime]::Now

                        $ntapData = Get-FilerData -Controller $controller

                        $ntapData

                        if ($ntapData -and $InvokeAsup.IsPresent) {
                            Write-Verbose -Message '- Generating ASUPs -'

                            try {
                                Write-Verbose -Message "Invoking ASUP on system: $( $ntapData.SystemInfo.SystemName )"
                                [void](Invoke-NaSystemApi -Request "<system-cli><args><arg>options</arg><arg>autosupport.doit</arg><arg>`"NetAppDocs_$( $ntapData.ReferenceDT.ToString('u') )`"</arg></args></system-cli>" -Controller $controller)
                            }
                            catch {
                                $Global:Error.RemoveAt(0)

                                Show-ErrorMessage -Message "Unable to invoke ASUP on system: $( $ntapData.SystemInfo.SystemName )"
                            }
                        }

                        $statsParams = @{
                            FunctionPSCallStack  = Get-PSCallStack
                            TargetDetails        = [pscustomobject]@{
                                SystemUid     = [string]::Join('_', $ntapData.SystemInfo.SystemId, $ntapData.SystemInfo.SystemSerialNumber)
                                SystemType    = $ntapData.pstypenames[0]
                                SystemVersion = $ntapData.SystemVersion
                            }
                            ElapsedTicks         = ([datetime]::Now - $functionStartTime).Ticks
                            ParameterToSetAsBool = 'AbstractController'
                        }

                        Write-Statistics @statsParams
                    }
                    catch {
                        Write-Error -ErrorRecord $_
                    }
                }
            }
        }
    }
}

function Get-FilerData {
    <#
        .NOTES
            BURTS
            : 439564 - storage-shelf-list-info doesn't do Jakes
            :   Workaround: Parsing sysconfig -v output manually
            :   Fixed: RR.0
            : 457555 - storage-shelf-list-info zapi fails most of the time with shelf-id specified
            :   Workaround: Calling per channel instead of per shelf (then parsing by shelf)
            :   Fixed: RR.0
            : 174666 - vfiler-get-allowed-protocols and vfiler-get-disallowed-protocols does not return all the protocols
            :   Workaround: Using Invoke-NaSystemApi or Invoke-NaSsh to call relevant CLI command
            :   Target Release: RR.0
            : 443207 - Missing vfiler support for cifs-share-acl-list-iter-start API
            :   Workaround: None, skipping for CIFS shares present in vfiler other than vfiler0
            :   Target Release: SN.0
            : 419019 - Panic in "system-cli" vfilerized zapi session with root username changed to NULL
            :   Workaround: Using 'vfiler run <vfiler> <command>' from vfiler0 context instead
            :   Fixed: IC.5, RR.0
            : 384034 - options-list-info, options-get and options-set does not support JAF installable options eg. ndmpd.enable
            :   Workaround: Using Invoke-NaSystemApi or Invoke-NaSsh to call 'options ndmpd.enable'
            :   Target Release: RR.0
            : 389554 - Filer panicked while cleaning up an uninitialized snapvault iterator
            :   Workaround: None - system version is checked before calling
            :   Fixed: RR.0, BR.2, IC.6
            : 613219 - "fcadmin config" does not show storage HBA card information
            :   Workaround: Ignore the error messages from Get-NaFcAdapter on Vespa controllers
            :   Fixed: Not fixed in customer release of ONTAP
            : 560283 - zapidoc: Improve description for storage-disk-get-iter max-records field
            :   Workaround: Use <next-tag> in a do loop
            :   Fixed: RR.1
            : 456727 - cifs-share-list-iter-start fails: "Unable to create iterator"
            :   Workaround: None - This only occurs in a Vfiler context with the /etc volume is full
            :   Fixed: None
    #>

    [CmdletBinding()]
    param (
        [Parameter(Mandatory)]
        [ValidateNotNullOrEmpty()]
        [NetApp.Ontapi.Filer.NaController]$Controller
    )

    $systemData = [pscustomobject]@{ }
    $systemData.pstypenames.Insert(0, 'NetAppDocs.NaController.Data')

    Write-Verbose -Message '- Capturing module version -'
    $systemData | Add-Member -MemberType NoteProperty -Name NetAppDocsVersion (Get-NtapDocsVersion -AsVersion)

    Write-Verbose -Message '- Capturing PowerShell version -'
    $systemData | Add-Member -MemberType NoteProperty -Name PowerShellVersion -Value $PSVersionTable.PSVersion

    $NetAppDocsError = @( )

    try {
        #region Physical Filer Data

        #region Raw SysConfig Data

        Write-Verbose -Message '- Collecting raw sysconfig data -'

        Write-Verbose -Message 'Calling Invoke-NaSystemApi (sysconfig)'
        $sysconfig = (Invoke-NaSystemApi -Request '<system-cli><args><arg>sysconfig</arg></args></system-cli>' -Controller $Controller -ErrorAction SilentlyContinue -ErrorVariable +NetAppDocsError).results.'cli-output'

        Write-Verbose -Message 'Calling Invoke-NaSystemApi (sysconfig -ac)'
        $sysconfigAc = (Invoke-NaSystemApi -Request '<system-cli><args><arg>sysconfig</arg><arg>-ac</arg></args></system-cli>' -Controller $Controller -ErrorVariable +NetAppDocsError).results.'cli-output'

        Write-Verbose -Message 'Calling Invoke-NaSystemApi (storage show)'
        $storageShow = (Invoke-NaSystemApi -Request '<system-cli><args><arg>storage</arg><arg>show</arg></args></system-cli>' -Controller $Controller -ErrorVariable +NetAppDocsError).results.'cli-output'

        Write-Verbose -Message 'Calling Invoke-NaSystemApi (sysconfig -v)'
        $sysconfigV = (Invoke-NaSystemApi -Request '<system-cli><args><arg>sysconfig</arg><arg>-v</arg></args></system-cli>' -Controller $Controller -ErrorAction SilentlyContinue -ErrorVariable +NetAppDocsError).results.'cli-output'

        Write-Verbose -Message 'Calling Invoke-NaSystemApi (storage show shelf)'
        $storageShowShelf = (Invoke-NaSystemApi -Request '<system-cli><args><arg>storage</arg><arg>show</arg><arg>shelf</arg></args></system-cli>' -Controller $Controller -ErrorVariable +NetAppDocsError).results.'cli-output'

        #endregion Raw SysConfig Data

        #region Basic System Information

        ##Unused Cmdlet: Get-NaClusterPartner - Info exists in Get-NaCluster
        ##Unused Cmdlet: Get-NaSystemApi - Not needed
        ##Unused Cmdlet: Get-NaSystemAvailableReplicationTransfers - Not needed for build docs
        ##Unused Cmdlet: Get-NaSystemVendorInfo - Not needed for build docs
        ##Unused Cmdlet: Get-NaStorageAdapterInfo - Not needed for build docs
        ##Unused Cmdlet: Get-NaClusterHwAssistStats - Not needed for build docs

        Write-Verbose -Message '- Gathering basic system information -'

        Write-Verbose -Message 'Calling Get-NaSystemInfo'
        $systemData | Add-Member -MemberType NoteProperty -Name SystemInfo -Value (Get-NaSystemInfo -Controller $Controller -ErrorVariable +NetAppDocsError)

        Write-Verbose -Message 'Identifying IOXM status and PlatformConfig'
        $systemData.SystemInfo | Add-Member -MemberType NoteProperty -Name IoxmPresent -Value ([bool](Select-String -InputObject $sysconfig -Pattern 'IOXM\s+Status:\s+present' -Quiet))
        $systemData.SystemInfo | Add-Member -MemberType NoteProperty -Name PlatformConfig -Value (Get-PlatformConfig -SystemModel $systemData.SystemInfo.SystemModel -IsClustered (! [string]::IsNullOrWhiteSpace($systemData.SystemInfo.PartnerSystemId)) -IoxmPresent $systemData.SystemInfo.IoxmPresent)

        Write-Verbose -Message 'Identifying SystemName'
        $systemData | Add-Member -MemberType NoteProperty -Name SystemName -Value $systemData.SystemInfo.SystemName

        Write-Verbose -Message 'Calling Get-NaSystemOntapiVersion'
        $systemData | Add-Member -MemberType NoteProperty -Name OntapiVersion -Value ([version]::Parse((Get-NaSystemOntapiVersion -Controller $Controller -ErrorVariable +NetAppDocsError).ToString()))

        Write-Verbose -Message 'Calling Get-NaLicense'
        $systemData | Add-Member -MemberType NoteProperty -Name License -Value (Get-NaLicense -Controller $Controller -ErrorVariable +NetAppDocsError)

        $systemData | Add-Member -MemberType NoteProperty -Name Cluster -Value $(
            if (
                ($systemData.OntapiVersion -ge [version]::Parse('1.20')) -or
                ($systemData.License | Where-Object { $_.Service -in @( 'cluster', 'cluster_remote', 'cf', 'cf_remote' ) -and $_.IsLicensed } | Measure-Object).Count -gt 0
            ) {
                Write-Verbose -Message 'Calling Get-NaCluster'
                (Get-NaCluster -Controller $Controller -ErrorVariable +NetAppDocsError)
            }
        )

        Write-Verbose -Message 'Calling Get-NaSystemVersion'
        $systemData | Add-Member -MemberType NoteProperty -Name SystemVersion -Value ([regex]::Split(([regex]::Split((Get-NaSystemVersion -Controller $Controller -ErrorVariable +NetAppDocsError), 'Release\s(?:\w+[__])?')[1]), '\s+'))[0].Replace(':', '')

        Write-Verbose -Message 'Calling Get-NaTimeZone'
        $systemData | Add-Member -MemberType NoteProperty -Name Timezone -Value (Get-NaTimezone -Controller $Controller -ErrorVariable +NetAppDocsError)

        Write-Verbose -Message 'Calling Get-NaVolRoot'
        $systemData | Add-Member -MemberType NoteProperty -Name VolRoot -Value (Get-NaVolRoot -Controller $Controller -ErrorVariable +NetAppDocsError)

        #endregion Basic System Information

        #region Node Health Validation

        if ($systemData.Cluster) {
            if ($systemData.Cluster.Partner -and $systemData.Cluster.State -ne 'CONNECTED') {
                $nodeHaStateMsg = [string]::Format(
                    'Node: {0} (HA State: {1})', $systemData.SystemName, $(
                        if ($systemData.Cluster.State) {
                            $systemData.Cluster.State
                        }
                        else {
                            'UNKNOWN'
                        }
                    )
                )

                $msg = New-Object -TypeName System.Text.StringBuilder

                [void]$msg.AppendLine("HA pair '$( $systemData.SystemName )/$( $systemData.Cluster.Partner )' is not healthy. Errors may be encountered and data may be incomplete.")
                [void]$msg.AppendLine('------------------------------------------------------------------------------')
                [void]$msg.AppendLine($nodeHaStateMsg)
                [void]$msg.AppendLine('Both nodes of the HA pair must be healthy to ensure accurate data is gathered.')
                [void]$msg.AppendLine('------------------------------------------------------------------------------')
                [void]$msg.AppendLine()

                Show-ErrorMessage -Message $msg.ToString()

                $NetAppDocsError += New-Object -TypeName System.Management.Automation.ErrorRecord -ArgumentList ("HA pair is not healthy. Reason: $nodeHaStateMsg", 'HAPAIR_NOT_HEALTHY', [System.Management.Automation.ErrorCategory]::DeviceError, ([string]::Format('{0}/{1}', $systemData.SystemName, $systemData.Cluster.Partner)))
            }
        }

        #endregion Node Health Validation

        #region Feature Status Information

        if ($systemData.OntapiVersion -ge [version]::Parse('1.20')) {
            Write-Verbose -Message 'Calling Get-NaFeatureStatus'
            $systemData | Add-Member -MemberType NoteProperty -Name FeatureStatus -Value (Get-NaFeatureStatus -Controller $Controller -ErrorVariable +NetAppDocsError)
        }

        #endregion Feature Status Information

        #region Detailed System Info

        Write-Verbose -Message '- Gathering more detailed system info -'

        Write-Verbose -Message 'Identifying adapter cards'
        $systemData | Add-Member -MemberType NoteProperty -Name AdapterCard -Value (Get-NtapExpansionSlotInventory -SysconfigAc $sysconfigAc -SysconfigVorA $sysconfigV | Sort-Object -Property SlotNumber)

        Write-Verbose -Message 'Calling Get-NaSnmp'
        $systemData | Add-Member -MemberType NoteProperty -Name Snmp -Value (Get-NaSnmp -Controller $Controller -ErrorVariable +NetAppDocsError)

        Write-Verbose -Message '- Identifying system storage configuration -'
        $systemData | Add-Member -MemberType NoteProperty -Name Storage -Value ([pscustomobject]@{ })

        $storageConfiguration = $sysconfig -split '\n' | Select-String -Pattern 'System Storage Configuration:' -SimpleMatch
        $systemData.Storage | Add-Member -MemberType NoteProperty -Name Configuration -Value $(
            if ($storageConfiguration) {
                ($storageConfiguration.ToString().Split(':'))[1].Trim()
            }
            else {
                'NA'
            }
        )

        Write-Verbose -Message '- Identifying system ACP connectivity information -'

        $systemAcpConnectivity = $sysconfig -split '\n' | Select-String -Pattern 'System ACP Connectivity:' -SimpleMatch
        $systemData.Storage | Add-Member -MemberType NoteProperty -Name SystemAcpConnectivity -Value $(
            if ($systemAcpConnectivity) {
                ($systemAcpConnectivity.ToString().Split(':'))[1].Trim()
            }
            else {
                'NA'
            }
        )

        Write-Verbose -Message '- Gathering system storage adapter configuration -'
        $systemData.Storage | Add-Member -MemberType NoteProperty -Name Adapter -Value ([pscustomobject]@{ })

        Write-Verbose -Message 'Calling Get-NaStorageAdapter'
        $adapterList = Get-NaStorageAdapter -Controller $Controller -ErrorVariable +NetAppDocsError

        $systemData.Storage.Adapter | Add-Member -MemberType NoteProperty -Name Info -Value @( )
        $adapterList | Where-Object { $_ } | ForEach-Object {
            Write-Verbose -Message "Calling Invoke-NaSystemApi (storage-adapter-get-adapter-info) $( $_.AdapterName )"
            $systemData.Storage.Adapter.Info += (Invoke-NaSystemApi -Request "<storage-adapter-get-adapter-info><adapter-name>$( $_.AdapterName )</adapter-name></storage-adapter-get-adapter-info>" -Controller $Controller -ErrorVariable +NetAppDocsError).results.'adapter-details'.'adapter-detail-info' | Convert-GenericXmlNode
        }

        Write-Verbose -Message 'Calling Get-NaReallocate'
        $systemData | Add-Member -MemberType NoteProperty -Name Reallocate -Value (Get-NaReallocate -Controller $Controller -ErrorVariable +NetAppDocsError)

        #endregion Detailed System Info

        #region Remote Management Configuration

        Write-Verbose -Message '- Gathering remote management configuration -'

        if (-not [string]::IsNullOrWhiteSpace($sysconfigV)) {
            $systemData | Add-Member -MemberType NoteProperty -Name RemoteManagement -Value (Get-NtapRemoteManagement -Sysconfig $sysconfigV)
        }
        else {
            $sp = $(
                switch -Wildcard ($sysconfig) {
                    '*Service Processor*' {
                        Write-Verbose -Message 'Calling Invoke-NaSystemApi (sp status)'
                        (Invoke-NaSystemApi -Request '<system-cli><args><arg>sp</arg><arg>status</arg></args></system-cli>' -Controller $Controller -ErrorVariable +NetAppDocsError).results.'cli-output'

                        break
                    }
                    '*Remote LAN Module*' {
                        Write-Verbose -Message 'Calling Invoke-NaSystemApi (rlm status)'
                        (Invoke-NaSystemApi -Request '<system-cli><args><arg>rlm</arg><arg>status</arg></args></system-cli>' -Controller $Controller -ErrorVariable +NetAppDocsError).results.'cli-output'

                        break
                    }
                    '*Baseboard Management Controller*' {
                        Write-Verbose -Message 'Calling Invoke-NaSystemApi (bmc status)'
                        (Invoke-NaSystemApi -Request '<system-cli><args><arg>bmc</arg><arg>status</arg></args></system-cli>' -Controller $Controller -ErrorVariable +NetAppDocsError).results.'cli-output'

                        break
                    }
                    default { $null }
                }
            )

            if (-not [string]::IsNullOrWhiteSpace($sp)) {
                Write-Verbose -Message 'Calling Get-NtapRemoteManagement'
                $systemData | Add-Member -MemberType NoteProperty -Name RemoteManagement -Value (Get-NtapRemoteManagement -Sysconfig $sp)
            }
            else {
                $NetAppDocsError += New-Object -TypeName System.Management.Automation.ErrorRecord -ArgumentList ('Unable to capture SP/RLM/BMC information.', 'NO_SP_INFORMATION', [System.Management.Automation.ErrorCategory]::InvalidData, $systemData.SystemName)
            }
        }

        #endregion Remote Management Configuration

        #region Aggregate Status

        ##Unused Cmdlet: Get-NaAggrFilerInfo - Not needed for build docs
        ##Unused Cmdlet: Get-NaAggrMediaScrub - Not needed for build docs
        ##Unused Cmdlet: Get-NaAggrScrub - Not needed for build docs
        ##Unused Cmdlet: Get-NaAggrVerify - Not needed for build docs

        Write-Verbose -Message '- Gathering detailed aggregate information -'

        Write-Verbose -Message 'Calling Get-NaAggr'
        $systemData | Add-Member -MemberType NoteProperty -Name Aggr -Value (Get-NaAggr -Controller $Controller -ErrorVariable +NetAppDocsError)

        foreach ($aggr in ($systemData.Aggr | Where-Object { $_ })) {
            Write-Verbose -Message "Calling Get-NaAggrOption (Aggr: $( $aggr.Name ))"
            $aggr | Add-Member -MemberType NoteProperty -Name Option -Value (Get-NaAggrOption -Name $aggr.Name -Controller $Controller -ErrorVariable +NetAppDocsError)

            if ($aggr.State -eq 'online') {
                Write-Verbose -Message "Calling Get-NaAggrSpace (Aggr: $( $aggr.Name ))"
                $aggr | Add-Member -MemberType NoteProperty -Name Space -Value (Get-NaAggrSpace -Name $aggr.Name -Controller $Controller -ErrorVariable +NetAppDocsError)

                Write-Verbose -Message "Calling Get-NaSnapshot (Aggr: $( $aggr.Name ))"
                $aggr | Add-Member -MemberType NoteProperty -Name Snapshot -Value (Get-NaSnapshot -TargetName $aggr.Name -TargetType Aggregate -Terse -Controller $Controller -ErrorVariable +NetAppDocsError)

                Write-Verbose -Message "Calling Get-NaSnapshotReserve (Aggr: $( $aggr.Name ))"
                $aggr | Add-Member -MemberType NoteProperty -Name SnapshotReserve -Value (Get-NaSnapshotReserve -TargetName $aggr.Name -TargetType Aggregate -Controller $Controller -ErrorVariable +NetAppDocsError -Verbose:$false)
            }
        }

        #endregion Aggregate Status

        #region Flash Device Details

        ##Unused Cmdlet: Get-NaFlashProfile - Not needed for build docs (Flash.Profile - add Name property)
        ##Unused Cmdlet: Get-NaFlashThreshold - Not needed for build docs (Flash.Threshold)

        Write-Verbose -Message '- Gathering flash device data -'
        $systemData | Add-Member -MemberType NoteProperty -Name Flash -Value ([pscustomobject]@{ })

        if ($systemData.OntapiVersion -ge [version]::Parse('1.13')) {
            Write-Verbose -Message 'Calling Get-NaFlashDevice'
            $systemData.Flash | Add-Member -MemberType NoteProperty -Name Device -Value (Get-NaFlashDevice -Controller $Controller -ErrorVariable +NetAppDocsError)
        }

        #endregion Flash Device Details

        #region Cluster HW Assist Details

        ##Unused Cmdlet: Get-NaClusterHwAssist - Not needed for build docs (if $systemData.Cluster and $systemData.Cluster.State -ne 'Error')

        #endregion Cluster HW Assist Details

        #region Priority Scheduling Data

        ##Unused Cmdlet: Get-NaPriority - Not needed for build docs (Priority.Info)
        ##Unused Cmdlet: Get-NaPriorityDefault - Not needed for build docs (Priority.Default)
        ##Unused Cmdlet: Get-NaPriorityVolume - Not needed for build docs (Priority.Volume)

        #endregion Priority Scheduling Data

        #region Disk Information

        ##Unused Cmdlet: Get-NaDiskOwnerFiler - Info exists in Get-NaDisk / Get-NaDiskOwner

        Write-Verbose -Message '- Gathering physical disk information -'

        $systemData | Add-Member -MemberType NoteProperty -Name Disk -Value $(
            if ($systemData.OntapiVersion -ge [version]::Parse('1.19')) {
                (Get-NaDisk -Controller $Controller -ErrorVariable +NetAppDocsError)
            }
            else {
                (Get-NaDisk -Old -Controller $Controller -ErrorVariable +NetAppDocsError)
            }
        )

        if ($storageShow -notlike '*SANOWN not enabled*') {
            Write-Verbose -Message 'Calling Get-NaDiskOwner'
            $systemData | Add-Member -MemberType NoteProperty -Name DiskOwner -Value (Get-NaDiskOwner -Controller $Controller -ErrorVariable +NetAppDocsError)
        }

        #endregion Disk Information

        #region FC/FCP Configuration

        ##Unused Cmdlet: Get-NaFcpAdapterStatistics - Not needed for build docs
        ##Unused Cmdlet: Get-NaFcpNodeName - Data exists in Get-NcFcpAdapter
        ##Unused Cmdlet: Get-NaFcpPortName - Data exists in Get-NcFcpAdapter
        ##Unused Cmdlet: Get-NaFcpPortNameAlias - ?? Should we add? Errors if wwpn alias not set
        ##Unused Cmdlet: Get-NaFcpClusterMode - Not needed for build docs (Fcp.FcpClusterMode - if ($systemData.Cluster))
        ##Unused Cmdlet: Get-NaFcpInitiator - Not needed for build docs (Fcp.FcpInitiator)
        ##Unused Cmdlet: Get-NaPortSet - Not needed for build docs (Fcp.PortSet - if (Fcp.FcpAdapter.Count -gt 1))
        ##Unused Cmdlet: Test-NaLunConfig - Not needed for build docs (Fcp.LunConfigCheck.Info)
        ##Unused Cmdlet: Test-NaLunConfig -CfMode - Not needed for build docs (Fcp.LunConfigCheck.CfMode)
        ##Unused Cmdlet: Test-NaLunConfig -SSI - Not needed for build docs (Fcp.LunConfigCheck.SSI - if Fcp.FcpClusterMode -eq 'single_image')

        Write-Verbose -Message '- Gathering FC adapter data -'

        Write-Verbose -Message 'Calling Get-NaFcAdapter'
        #BURT 613219
        if ($systemData.SystemInfo.SystemModel -match 'FAS2[0|2][2|4|5]0') {
            $systemData | Add-Member -MemberType NoteProperty -Name FcAdapter -Value (Get-NaFcAdapter -Controller $Controller -ErrorAction SilentlyContinue)
        }
        else {
            $systemData | Add-Member -MemberType NoteProperty -Name FcAdapter -Value (Get-NaFcAdapter -Controller $Controller -ErrorVariable +NetAppDocsError)
        }

        Write-Verbose -Message '- Gathering FCP configuration data -'
        $systemData | Add-Member -MemberType NoteProperty -Name Fcp -Value ([pscustomobject]@{ })

        Write-Verbose -Message 'Calling Get-NaOption (licensed_feature.fcp.enable)'
        $optionLicensedFeatureFcpEnable = Get-NaOption -OptionNames 'licensed_feature.fcp.enable' -Controller $Controller -ErrorAction SilentlyContinue

        if (
            ($systemData.OntapiVersion -ge [version]::Parse('1.20') -and (($systemData.License | Where-Object { $_.Service -eq 'fcp' }).IsLicensed -and ($optionLicensedFeatureFcpEnable.Value -eq 'on'))) -or
            ($systemData.OntapiVersion -lt [version]::Parse('1.20') -and (($systemData.License | Where-Object { $_.Service -eq 'fcp' }).IsLicensed))
        ) {
            Write-Verbose -Message 'Calling Get-NaFcpAdapter'
            $systemData.Fcp | Add-Member -MemberType NoteProperty -Name FcpAdapter -Value (Get-NaFcpAdapter -Controller $Controller -ErrorVariable +NetAppDocsError)
        }

        #endregion FC/FCP Configuration

        #region iSCSI Configuration

        try {
            $iscsiInterface = Get-NaIscsiInterface -Controller $Controller -ErrorAction Stop

            Write-Verbose -Message '- Gathering iSCSI configuration data -'
            $systemData | Add-Member -MemberType NoteProperty -Name Iscsi -Value ([pscustomobject]@{ })

            Write-Verbose -Message 'Calling Get-NaIscsiInterface'
            $systemData.Iscsi | Add-Member -MemberType NoteProperty -Name Interface -Value $iscsiInterface
        }
        catch {
            if ($_.Exception.ErrorName -eq 'EAPILICENSE') {
                $Global:Error.RemoveAt(0)
            }
            else {
                $NetAppDocsError += $_
            }
        }

        if ($systemData.Iscsi) {
            Write-Verbose -Message 'Calling Get-NaIscsiTargetPortalGroup'
            $systemData.Iscsi | Add-Member -MemberType NoteProperty -Name TargetPortalGroup -Value (Get-NaIscsiTargetPortalGroup -Controller $Controller -ErrorVariable +NetAppDocsError)
        }

        #endregion iSCSI configuration

        #region Network Details

        ##Unused Cmdlet: Get-NaNetDcb - Not yet needed for build docs
        ##Unused Cmdlet: Get-NaNetDcbPriority - Not yet needed for build docs
        ##Unused Cmdlet: Get-NaNetActiveConfig - Not needed for build docs (Net.ActiveConfig)
        ##Unused Cmdlet: Get-NaNetVif - Not needed for build docs (Net.Vif)
        ##Unused Cmdlet: Get-NaNetVlan - Not needed for build docs (Net.Vlan)

        Write-Verbose -Message '- Gathering network details -'
        $systemData | Add-Member -MemberType NoteProperty -Name Net -Value ([pscustomobject]@{ })

        if ($systemData.OntapiVersion -ge [version]::Parse('1.11')) {
            Write-Verbose -Message 'Calling Get-NaNetInterface'
            $systemData.Net | Add-Member -MemberType NoteProperty -Name Interface -Value (Get-NaNetInterface -Controller $Controller -ErrorVariable +NetAppDocsError)
        }

        if ($systemData.OntapiVersion -ge [version]::Parse('1.11')) {
            Write-Verbose -Message 'Calling Get-NaNetPersistentConfig'
            $systemData.Net | Add-Member -MemberType NoteProperty -Name PersistentConfig -Value (Get-NaNetPersistentConfig -Controller $Controller -ErrorVariable +NetAppDocsError)
        }

        if ($systemData.OntapiVersion -ge [version]::Parse('1.11')) {
            Write-Verbose -Message 'Calling Get-NaNetRoute'
            $systemData.Net | Add-Member -MemberType NoteProperty -Name Route -Value (Get-NaNetRoute -Controller $Controller -ErrorVariable +NetAppDocsError)
        }

        #endregion Network Details

        #region Shelf Details

        Write-Verbose -Message '- Gathering shelf data -'

        $systemData | Add-Member -MemberType NoteProperty -Name Shelf -Value ([pscustomobject]@{ })

        if (($systemData.Disk | Where-Object { $_.DiskType -ne 'LUN' } | Measure-Object).Count -gt 0) {
            if ($systemData.OntapiVersion -ge [version]::Parse('1.15')) {
                Write-Verbose -Message 'Calling Get-NaShelf'
                $systemData.Shelf | Add-Member -MemberType NoteProperty -Name Info -Value (Get-NaShelf -Controller $Controller -ErrorVariable +NetAppDocsError | Where-Object { -not [string]::IsNullOrWhiteSpace($_.ShelfId) })

                Write-Verbose -Message 'Calling Invoke-NaSystemApi (storage-shelf-bay-list-info)'
                $systemData.Shelf | Add-Member -MemberType NoteProperty -Name Bay -Value ((Invoke-NaSystemApi -Request '<storage-shelf-bay-list-info />' -Controller $Controller -ErrorVariable +NetAppDocsError).results.'shelf-bay-list'.'shelf-bay-list-info' | Convert-GenericXmlNode)
            }
            else {
                Write-Verbose -Message 'Calling Get-NaShelf'
                $shelfNonAta = Get-NaShelf -Controller $Controller -ErrorVariable +NetAppDocsError | Where-Object { -not [string]::IsNullOrWhiteSpace($_.ShelfId) }

                #BURT 439564
                $shelfAta = Get-NaShelfAta -Sysconfig $sysconfigV -StorageShowShelf $storageShowShelf

                $systemData.Shelf | Add-Member -MemberType NoteProperty -Name Info -Value (
                    ($shelfNonAta, $shelfAta) | ForEach-Object {
                        if ($_) {
                            $_
                        }
                    }
                )
            }

            Write-Verbose -Message 'Calling Get-NaShelfEnvironment'
            #BURT 457555
            $systemData.Shelf | Add-Member -MemberType NoteProperty -Name Environment -Value (
                $systemData.Shelf.Info | Where-Object { $_.ChannelName -and $_.ChannelName -ne 'UNUSED' } | Select-Object -Property ChannelName -Unique | ForEach-Object {
                    Get-NaShelfEnvironment -Channel $_.ChannelName -Controller $Controller -ErrorVariable +NetAppDocsError
                }
            )
        }

        #endregion Shelf Details

        #region Vfiler Configuration

        ##Unused Cmdlet: Get-NaVfilerDr - Not needed for build docs
        ##Unused Cmdlet: Get-NaVfilerMigrate - Not needed for build docs
        ##Unused Cmdlet: Get-NaVfilerStatus - Info exists in Get-NaVfiler

        Write-Verbose -Message '- Gathering MultiStore (vFiler) configurations -'

        try {
            Write-Verbose -Message 'Calling Get-NaVfiler'
            $systemData | Add-Member -MemberType NoteProperty -Name Vfiler -Value (Get-NaVfiler -Controller $Controller -ErrorAction Stop)
        }
        catch {
            if ($_.Exception.ErrorName -eq 'EAPILICENSE') {
                $Global:Error.RemoveAt(0)
            }
            else {
                $NetAppDocsError += $_
            }
        }

        if ($systemData.Vfiler) {
            foreach ($vfiler in ($systemData.Vfiler | Where-Object { $_ })) {
                #BURT 174666
                $vfiler | Add-Member -MemberType NoteProperty -Name Protocol -Value $(
                    if ($systemData.OntapiVersion -ge [version]::Parse('1.15')) {
                        (Get-NaVfilerProtocol -Name $vfiler.Name -Controller $Controller -ErrorVariable +NetAppDocsError)
                    }
                    else {
                        Write-Verbose -Message 'Calling Invoke-NaSystemApi (vfiler allow)'
                        $allowedProtocols = (Invoke-NaSystemApi -Request "<system-cli><args><arg>vfiler</arg><arg>allow</arg><arg>$( $vfiler.Name )</arg></args></system-cli>" -Controller $Controller -ErrorVariable +NetAppDocsError).results.'cli-output'

                        Write-Verbose -Message 'Calling Invoke-NaSystemApi (vfiler disallow)'
                        $disallowedProtocols = (Invoke-NaSystemApi -Request "<system-cli><args><arg>vfiler</arg><arg>disallow</arg><arg>$( $vfiler.Name )</arg></args></system-cli>" -Controller $Controller -ErrorVariable +NetAppDocsError).results.'cli-output'

                        [pscustomobject]@{
                            AllowedProtocols    = $allowedProtocols -split '\n' | ForEach-Object {
                                ($_ -split 'proto=')[1] | Where-Object { -not [string]::IsNullOrWhiteSpace($_) }
                            }
                            DisallowedProtocols = $disallowedProtocols -split '\n' | ForEach-Object {
                                ($_ -split 'proto=')[1] | Where-Object { -not [string]::IsNullOrWhiteSpace($_) }
                            }
                        }
                    }
                )
            }

            if ($systemData.OntapiVersion -ge [version]::Parse('1.14')) {
                Write-Verbose -Message 'Calling Get-NaNetIpspace'
                $systemData.Net | Add-Member -MemberType NoteProperty -Name Ipspace -Value (Get-NaNetIpspace -Controller $Controller -ErrorVariable +NetAppDocsError)
            }
        }
        else {
            Write-Verbose -Message '- Identifying default vfiler configuration -'

            Write-Verbose -Message 'Calling Read-NaFile (/etc/resolv.conf)'
            $resolvConf = Read-NaFile "/vol/$( ($systemData.VolRoot).Name )/etc/resolv.conf" -Controller $Controller -ErrorAction SilentlyContinue

            Write-Verbose -Message 'Calling Get-NaOption (dns.domainname, nis.domainname, nis.servers)'
            $systemData | Add-Member -MemberType NoteProperty -Name Vfiler -Value (
                [pscustomobject]@{
                    Name     = 'vfiler0'
                    Status   = 'running'
                    DnsInfo  = [pscustomobject]@{
                        DnsDomainname = (Get-NaOption -OptionNames 'dns.domainname' -Controller $Controller -ErrorVariable +NetAppDocsError).Value
                        DnsServers    = $(
                            if (-not [string]::IsNullOrWhiteSpace($resolvConf)) {
                                $resolvConf -split '\n' | Where-Object { $_ -match '^\s*nameserver' } | ForEach-Object {
                                    [pscustomobject]@{
                                        Server = ($_ -replace 'nameserver').Trim()
                                    }
                                }
                            }
                        )
                    }
                    NisInfo  = [pscustomobject]@{
                        NisDomainname = (Get-NaOption -OptionNames 'nis.domainname' -Controller $Controller -ErrorVariable +NetAppDocsError).Value
                        NisServers    = (Get-NaOption -OptionNames 'nis.servers' -Controller $Controller -ErrorVariable +NetAppDocsError).Value -split ',' | ForEach-Object {
                            [pscustomobject]@{
                                Server = $_.Trim()
                            }
                        }
                    }
                    Protocol = [pscustomobject]@{
                        AllowedProtocols    = 'rsh', 'ssh', 'nfs', 'cifs', 'iscsi', 'ftp', 'http'
                        DisallowedProtocols = @( )
                    }
                }
            )

            $systemData.Vfiler | Add-Member -MemberType NoteProperty -Name Ipspace -Value $null
        }

        #endregion Vfiler Configuration

        #region Volume Configuration

        Write-Verbose -Message '- Gathering detailed volume information -'

        ##Unused Cmdlet: Get-NaSnapshotDelta - Not needed for build docs
        ##Unused Cmdlet: Get-NaSnapshotReclaimable - Not needed for build docs
        ##Unused Cmdlet: Get-NaSnapshotRestoreStatus - Not needed for build docs
        ##Unused Cmdlet: Get-NaSnapshotVolumeSpace - Not needed for build docs
        ##Unused Cmdlet: Get-NaVolCharmap - Not needed for build docs
        ##Unused Cmdlet: Get-NaVolCloneSplit - Not needed for build docs
        ##Unused Cmdlet: Get-NaVolCloneSplitEstimate - Not needed for build docs
        ##Unused Cmdlet: Get-NaVolContainer - Info exists in Get-NaAggr
        ##Unused Cmdlet: Get-NaVolMove - Not needed for build docs
        ##Unused Cmdlet: Get-NaVolSis - ????
        ##Unused Cmdlet: Get-NaVolSize - Info exists in Get-NaVol
        ##Unused Cmdlet: Get-NaSnapshotAutodelete - Info exists in Get-NaVol
        ##Unused Cmdlet: Get-NaVolAutosize - Info exists in Get-NaVol

        Write-Verbose -Message 'Calling Get-NaVol'
        $systemData | Add-Member -MemberType NoteProperty -Name Vol -Value (Get-NaVol -Controller $Controller -ErrorVariable +NetAppDocsError)

        if ($systemData.OntapiVersion -ge [version]::Parse('1.20')) {
            Write-Verbose -Message 'Calling Get-NaVolFootprint'
            $systemData | Add-Member -MemberType NoteProperty -Name VolFootprint -Value (Get-NaVolFootprint -Controller $Controller -ErrorVariable +NetAppDocsError)
        }

        if ($systemData.OntapiVersion -ge [version]::Parse('1.20')) {
            Write-Verbose -Message 'Calling Get-NaVolSpace'
            $systemData | Add-Member -MemberType NoteProperty -Name VolSpace -Value (Get-NaVolSpace -Controller $Controller -ErrorVariable +NetAppDocsError)
        }

        foreach ($vol in ($systemData.Vol | Where-Object { $_ })) {
            Write-Verbose -Message "Calling Get-NaVolOption (Vol: $( $vol.Name ))"
            $vol | Add-Member -MemberType NoteProperty -Name Option -Value (Get-NaVolOption -Name $vol.Name -Controller $Controller -ErrorVariable +NetAppDocsError)

            if ($vol.State -eq 'online') {
                Write-Verbose -Message "Calling Get-NaSnapshot (Vol: $( $vol.Name ))"
                $vol | Add-Member -MemberType NoteProperty -Name Snapshot -Value (Get-NaSnapshot -TargetName $vol.Name -TargetType Volume -Controller $Controller -ErrorVariable +NetAppDocsError)

                Write-Verbose -Message "Calling Get-NaSnapshotSchedule (Vol: $( $vol.Name ))"
                $vol | Add-Member -MemberType NoteProperty -Name SnapshotSchedule -Value (Get-NaSnapshotSchedule -TargetName $vol.Name -Controller $Controller -ErrorVariable +NetAppDocsError)

                Write-Verbose -Message "Calling Get-NaSnapshotReserve (Vol: $( $vol.Name ))"
                $vol | Add-Member -MemberType NoteProperty -Name SnapshotReserve -Value (Get-NaSnapshotReserve -TargetName $vol.Name -TargetType Volume -Controller $Controller -ErrorVariable +NetAppDocsError)

                Write-Verbose -Message "Calling Get-NaVolLanguage (Vol: $( $vol.Name ))"
                $vol | Add-Member -MemberType NoteProperty -Name Language -Value (Get-NaVolLanguage -Name $vol.Name -Controller $Controller -ErrorVariable +NetAppDocsError)

                if ($vol.RaidStatus -notlike '*read-only*' -and $vol.Type -ne 'trad') {
                    #If the volume belongs to a vfiler, this call to Get-NaQuotaStatus will fail with an error message (will get called again during vfiler loop below)
                    if (-not $vol.OwningVfiler -or $vol.OwningVfiler -ceq 'vfiler0') {
                        Write-Verbose -Message "Calling Get-NaVolQuotaStatus (Vol: $( $vol.Name ))"
                        $vol | Add-Member -MemberType NoteProperty -Name QuotaStatus -Value (Get-NaVolQuotaStatus -Volume $vol.Name -Controller $Controller -ErrorVariable +NetAppDocsError)
                    }
                }
            }
        }

        #endregion Volume Configuration

        #region SIS Configuration

        Write-Verbose -Message '- Gathering SIS information -'

        try {
            Write-Verbose -Message 'Calling Get-NaSis'
            $systemData | Add-Member -MemberType NoteProperty -Name Sis -Value (Get-NaSis -Controller $Controller -ErrorAction Stop)
        }
        catch {
            if ($_.Exception.Message -eq 'No Status entry found.') {
                $Global:Error.RemoveAt(0)
            }
            else {
                $NetAppDocsError += $_
            }
        }

        #endregion SIS Configuration

        #region Qtree Configuration

        Write-Verbose -Message '- Gathering qtree data -'

        Write-Verbose -Message 'Calling Get-NaQtree'
        $systemData | Add-Member -MemberType NoteProperty -Name Qtree -Value (Get-NaQtree -Controller $Controller -ErrorVariable +NetAppDocsError)

        #endregion Qtree Configuration

        #region SnapLock Configuration

        ##Unused Cmdlet: Get-NaSnaplockComplianceClock - Currently no Snaplock support in NetAppDocs
        ##Unused Cmdlet: Get-NaSnaplockLog - Currently no Snaplock support in NetAppDocs
        ##Unused Cmdlet: Get-NaSnaplockLogVolume - Currently no Snaplock support in NetAppDocs
        ##Unused Cmdlet: Get-NaSnaplockOption - Currently no Snaplock support in NetAppDocs
        ##Unused Cmdlet: Get-NaSnaplockRetentionTime - Currently no Snaplock support in NetAppDocs

        #endregion SnapLock Configuration

        #region UCM Information

        if ($systemData.OntapiVersion -ge [version]::Parse('1.21')) {
            Write-Verbose -Message '- Gathering UCM information -'
            $systemData | Add-Member -MemberType NoteProperty -Name Ucm -Value ([pscustomobject]@{ })

            Write-Verbose -Message 'Calling Get-NaUcmAdapter'
            $systemData.Ucm | Add-Member -MemberType NoteProperty -Name Adapter -Value (Get-NaUcmAdapter -Controller $Controller -ErrorVariable +NetAppDocsError)
        }

        #endregion UCM Information

        #endregion Physical Filer Data

        #region Virtual Filer Data

        foreach ($vfiler in ($systemData.Vfiler | Where-Object { $_ -and $_.Status -ne 'DR backup' })) {
            $Controller.Vfiler = $vfiler.Name

            #region Basic vFiler System Info

            Write-Verbose -Message '- Identifying basic vFiler system info -'

            $vfiler | Add-Member -MemberType NoteProperty -Name RootPath -Value $(
                if ($vfiler.Name -ceq 'vfiler0') {
                    "/vol/$( $systemData.VolRoot.Name )"
                }
                else {
                    ($vfiler.VfStores | Where-Object { $_.IsEtc }).Path
                }
            )

            Write-Verbose -Message 'Calling Get-NaOption'
            $vfiler | Add-Member -MemberType NoteProperty -Name Option -Value (Get-NaOption -Controller $Controller -ErrorVariable +NetAppDocsError)

            #endregion Basic vFiler System Info

            #region Qtree Configuration
            #This is necessary in the vfiler context for the OwningVfiler property to be correct

            Write-Verbose -Message '- Gathering qtree data -'

            if ($vfiler.Name -cne 'vfiler0') {
                Write-Verbose -Message 'Calling Get-NaQtree'

                foreach ($vfQtree in (Get-NaQtree -Controller $Controller -ErrorVariable +NetAppDocsError | Where-Object { $_ })) {
                    $systemData.Qtree | Where-Object { $_.Volume -ceq $vfQtree.Volume -and $_.Qtree -ceq $vfQtree.Qtree } | Add-Member -MemberType NoteProperty -Name OwningVfiler -Value $vfiler.Name -Force
                }
            }

            #endregion Qtree Configuration

            #region Cifs

            ##Unused Cmdlet: Get-NaCifsBranchCacheHashStatistics - Not needed for build docs
            ##Unused Cmdlet: Get-NaCifsOrganizationalUnit - Not needed for build docs
            ##Unused Cmdlet: Get-NaCifsSession - Not needed for build docs
            ##Unused Cmdlet: Get-NaCifsSite - Not needed for build docs
            ##Unused Cmdlet: Get-NaCifsStatistics - Not needed for build docs
            ##Unused Cmdlet: Get-NaCifsNetBiosAlias - Not needed for build docs (vFiler.Cifs.NetBiosAlias)

            if (($systemData.License | Where-Object { $_.Service -eq 'cifs' }).IsLicensed -and (Test-NaCifs -Controller $Controller -ErrorAction Ignore) -eq 'started') {
                Write-Verbose -Message '- Gathering CIFS Configuration information -'
                $vfiler | Add-Member -MemberType NoteProperty -Name Cifs -Value ([pscustomobject]@{ })

                Write-Verbose -Message 'Calling Get-NaCifs'
                $vfiler.Cifs | Add-Member -MemberType NoteProperty -Name Config -Value (Get-NaCifs -Controller $Controller -ErrorVariable +NetAppDocsError)

                Write-Verbose -Message 'Calling Get-NaCifsHomeDirectory'
                $vfiler.Cifs | Add-Member -MemberType NoteProperty -Name HomeDirectory -Value (Get-NaCifsHomeDirectory -Controller $Controller -ErrorVariable +NetAppDocsError)

                #BURT 456727
                try {
                    Write-Verbose -Message 'Calling Get-NaCifsShare'
                    $vfiler.Cifs | Add-Member -MemberType NoteProperty -Name Share -Value (Get-NaCifsShare -Controller $Controller -ErrorAction Stop)
                }
                catch {
                    if ($_.Exception.Message -like '*Unable to create iterator*') {
                        $Global:Error.RemoveAt(0)
                    }
                    else {
                        $NetAppDocsError += $_
                    }
                }

                #BURT 443207
                if ($vfiler.Name -ceq 'vfiler0') {
                    if ($systemData.OntapiVersion -ge [version]::Parse('1.14')) {
                        #BURT 456727
                        try {
                            Write-Verbose -Message 'Calling Get-NaCifsShareAcl'
                            $vfiler.Cifs | Add-Member -MemberType NoteProperty -Name ShareAcl -Value (Get-NaCifsShareAcl -Controller $Controller -ErrorAction Stop)
                        }
                        catch {
                            if ($_.Exception.Message -like '*Unable to create iterator*') {
                                $Global:Error.RemoveAt(0)
                            }
                            else {
                                $NetAppDocsError += $_
                            }
                        }
                    }
                    else {
                        $vfiler.Cifs | Add-Member -MemberType NoteProperty -Name ShareAcl -Value $(
                            Write-Verbose -Message 'Calling Invoke-NaSystemApi (cifs shares)'
                            Convert-VfilerCifsShareAcl -CifsShareString (Invoke-NaSystemApi -Request '<system-cli><args><arg>cifs</arg><arg>shares</arg></args></system-cli>' -Controller $Controller -ErrorVariable +NetAppDocsError).results.'cli-output'
                        )
                    }
                }
                else {
                    if ($systemData.OntapiVersion -ge [version]::Parse('1.20')) {
                        #BURT 456727
                        try {
                            Write-Verbose -Message 'Calling Get-NaCifsShareAcl'
                            $vfiler.Cifs | Add-Member -MemberType NoteProperty -Name ShareAcl -Value (Get-NaCifsShareAcl -Controller $Controller -ErrorAction Stop)
                        }
                        catch {
                            if ($_.Exception.Message -like '*Unable to create iterator*') {
                                $Global:Error.RemoveAt(0)
                            }
                            else {
                                $NetAppDocsError += $_
                            }
                        }
                    }
                }

                Write-Verbose -Message 'Calling Get-NaOption (cifs.wins_servers)'
                $winsServers = (Get-NaOption -OptionNames 'cifs.wins_servers' -Controller $Controller -ErrorAction Ignore).Value
                $vfiler.Cifs | Add-Member -MemberType NoteProperty -Name WinsServers -Value $(
                    if ($winsServers) {
                        $winsServers -split '\s*,\s*' | Where-Object { -not [string]::IsNullOrWhiteSpace($_) } | ForEach-Object {
                            [pscustomobject]@{
                                Server = $_.Trim()
                            }
                        }
                    }
                )
            }

            #endregion Cifs

            #region FPolicy

            ##Unused Cmdlet: Get-NaFpolicyOption - Not needed for build docs (fpolicy.Option)
            ##Unused Cmdlet: Get-NaFpolicyServer - Not needed for build docs (fpolicy.Server)
            ##Unused Cmdlet: Get-NaFpolicySecondaryServer - Not needed for build docs (fpolicy.SecondaryServer)
            ##Unused Cmdlet: Get-NaFpolicyVolumeList - Not needed for build docs (fpolicy.VolumeList)

            Write-Verbose -Message 'Capturing Fpolicy configuration data'

            if (
                ($systemData.License | Where-Object { $_.Service -eq 'cifs' }).IsLicensed -and
                (Test-NaCifs -Controller $Controller -ErrorAction Ignore) -eq 'started' -and
                (Test-NaFpolicy -Controller $Controller -ErrorAction Ignore)
            ) {
                $vfiler | Add-Member -MemberType NoteProperty -Name Fpolicy -Value ([pscustomobject]@{ })

                Write-Verbose -Message 'Calling Get-NaFpolicy'
                $vfiler.Fpolicy | Add-Member -MemberType NoteProperty -Name Info -Value (Get-NaFpolicy -Controller $Controller -ErrorVariable +NetAppDocsError)

                foreach ($fpolicy in ($vfiler.Fpolicy.Info | Where-Object { $_ })) {
                    Write-Verbose -Message "Calling Get-NaFpolicyExtensionList (FPolicy: $( $fpolicy.Name ))"
                    $fpolicy | Add-Member -MemberType NoteProperty -Name ExtensionList -Value (Get-NaFpolicyExtensionList -Name $fpolicy.Name -Controller $Controller -ErrorVariable +NetAppDocsError | Select-Object -Property Include, Exclude)
                }
            }

            #endregion FPolicy

            #region User/Group Data

            ##Unused Cmdlet: Get-NaUnixGroup - Not needed for build docs?

            Write-Verbose -Message '- Gathering local group and user data -'

            Write-Verbose -Message 'Calling Get-NaGroup'
            $vfiler | Add-Member -MemberType NoteProperty -Name Group -Value (Get-NaGroup -GetCapabilities -Controller $Controller -ErrorVariable +NetAppDocsError)
            foreach ($group in ($vfiler.Group | Where-Object { $_ })) {
                Write-Verbose -Message "Calling Get-NaDomainUser (Group: $group)"
                $group | Add-Member -MemberType NoteProperty -Name DomainUser -Value (Get-NaDomainUser -Group $group.Name -Controller $Controller -ErrorVariable +NetAppDocsError -Verbose:$false)
            }

            Write-Verbose -Message 'Calling Get-NaUser'
            $vfiler | Add-Member -MemberType NoteProperty -Name User -Value (Get-NaUser -Controller $Controller -ErrorVariable +NetAppDocsError)

            Write-Verbose -Message 'Calling Get-NaRole'
            $vfiler | Add-Member -MemberType NoteProperty -Name Role -Value (Get-NaRole -Controller $Controller -ErrorVariable +NetAppDocsError)

            #endregion User/Group Data

            #region iSCSI

            ##Unused Cmdlet: Get-NaIscsiConnection - Not needed for build docs
            ##Unused Cmdlet: Get-NaIscsiInitiator - Not yet needed for build docs?
            ##Unused Cmdlet: Get-NaIscsiPortal - Not needed for build docs??
            ##Unused Cmdlet: Get-NaIscsiSession - Not needed for build docs
            ##Unused Cmdlet: Get-NaIscsiStatistics - Not needed for build docs
            ##Unused Cmdlet: Get-NaIscsiInitiatorAuth - Not needed for build docs (vFiler.Iscsi.InitiatorAuth)

            if ($systemData.Iscsi) {
                Write-Verbose -Message '- Gathering iSCSI configuration -'
                $vfiler | Add-Member -MemberType NoteProperty -Name Iscsi -Value ([pscustomobject]@{ })

                Write-Verbose -Message 'Calling Test-NaIscsi'
                if (Test-NaIscsi -Controller $Controller -ErrorAction Ignore) {
                    Write-Verbose -Message 'Calling Get-NaIscsiInitiatorDefaultAuth'
                    $vfiler.Iscsi | Add-Member -MemberType NoteProperty -Name DefaultAuth -Value (Get-NaIscsiInitiatorDefaultAuth -Controller $Controller -ErrorVariable +NetAppDocsError)

                    if ($systemData.OntapiVersion -ge [version]::Parse('1.3')) {
                        Write-Verbose -Message 'Calling Get-NaIsns'
                        $vfiler.Iscsi | Add-Member -MemberType NoteProperty -Name Isns -Value (Get-NaIsns -Controller $Controller -ErrorVariable +NetAppDocsError)
                    }

                    Write-Verbose -Message 'Calling Get-NaIscsiNodeName'
                    $vfiler.Iscsi | Add-Member -MemberType NoteProperty -Name NodeName -Value (Get-NaIscsiNodeName -Controller $Controller -ErrorVariable +NetAppDocsError)

                    Write-Verbose -Message 'Calling Get-NaIscsiTargetAlias'
                    $vfiler.Iscsi | Add-Member -MemberType NoteProperty -Name TargetAlias -Value (Get-NaIscsiTargetAlias -Controller $Controller -ErrorVariable +NetAppDocsError)
                }
            }

            #endregion iSCSI

            #region LUN/iGroup Information

            ##Unused Cmdlet: Get-NaLunAttribute - Info exists in Get-NaLun
            ##Unused Cmdlet: Get-NaLunByIgroup - Not needed
            ##Unused Cmdlet: Get-NaLunClone - Not needed for build docs
            ##Unused Cmdlet: Get-NaLunCloneSplit - Not needed for build docs
            ##Unused Cmdlet: Get-NaLunComment - Info exists in Get-NaLun
            ##Unused Cmdlet: Get-NaLunGeometry - Not needed for build docs
            ##Unused Cmdlet: Get-NaLunInquiry - Not needed for build docs
            ##Unused Cmdlet: Get-NaLunMapByInitiator - Not needed
            ##Unused Cmdlet: Get-NaLunMaxSize - Not needed for build docs
            ##Unused Cmdlet: Get-NaLunMinSize - Not needed for build docs
            ##Unused Cmdlet: Get-NaLunOccupiedSize - Info exists in Get-NaLun
            ##Unused Cmdlet: Get-NaLunPersistentReservation - Not needed for build docs
            ##Unused Cmdlet: Get-NaLunSelect - Not needed for build docs
            ##Unused Cmdlet: Get-NaLunSerialNumber - Info exists in Get-NaLun
            ##Unused Cmdlet: Get-NaLunSignature - Not needed for build docs
            ##Unused Cmdlet: Get-NaLunSnapUsage - Not needed for build docs
            ##Unused Cmdlet: Get-NaLunSpaceReserved - Info exists in Get-NaLun
            ##Unused Cmdlet: Get-NaLunStatistics - Not needed for build docs
            ##Unused Cmdlet: Get-NaLunTargetDeviceId - Not needed for build docs

            Write-Verbose -Message '- Gathering LUN/iGroup information -'

            Write-Verbose -Message 'Calling Get-NaLun'
            $vfiler | Add-Member -MemberType NoteProperty -Name Lun -Value (Get-NaLun -Controller $Controller -ErrorVariable +NetAppDocsError)

            foreach ($lun in ($vfiler.Lun | Where-Object { $_ })) {
                Write-Verbose -Message "Calling Get-NaLunMap (LUN: $( $lun.Path ))"
                $lun | Add-Member -MemberType NoteProperty -Name Map -Value (Get-NaLunMap -Path $lun.Path -Controller $Controller -ErrorVariable +NetAppDocsError)
            }

            Write-Verbose -Message '- Gathering iGroup information -'

            Write-Verbose -Message 'Calling Get-NaIgroup'
            $vfiler | Add-Member -MemberType NoteProperty -Name Igroup -Value (Get-NaIgroup -Controller $Controller -ErrorVariable +NetAppDocsError)

            foreach ($igroup in ($vfiler.Igroup | Where-Object { $_.Initiators })) {
                foreach ($initiator in ($igroup.Initiators | Where-Object { $_ })) {
                    Write-Verbose -Message "Calling Confirm-NaLunInitatiorLoggedIn (Initiator: $( $initiator.InitiatorName ))"
                    $initiator | Add-Member -MemberType NoteProperty -Name IsLoggedIn -Value (Confirm-NaLunInitiatorLoggedIn -Initiator $initiator.InitiatorName -Controller $Controller -ErrorVariable +NetAppDocsError)
                }
            }

            #endregion LUN/iGroup Information

            #region NFS

            ##Unused Cmdlet: Get-NaNfsExportStoragePath - Info exists in Get-NaNfsExport
            ##Unused Cmdlet: Get-NaNfsMonitor - Not needed for build docs
            ##Unused Cmdlet: Get-NaNfsSecurityFlavor - Not needed for build docs
            ##Unused Cmdlet: Get-NaNfsStatistics - Not needed for build docs
            ##Unused Cmdlet: Get-NaNfsTopClient - Not needed for build docs

            Write-Verbose -Message '- Gathering NFS configuration -'
            $vfiler | Add-Member -MemberType NoteProperty -Name Nfs -Value ([pscustomobject]@{ })

            try {
                Write-Verbose -Message 'Calling Get-NaNfsExport'
                $vfiler.Nfs | Add-Member -MemberType NoteProperty -Name Export -Value (Get-NaNfsExport -Controller $Controller -ErrorAction Stop)

                Write-Verbose -Message 'Calling Get-NaNfsExport -Persistent'
                $vfiler.Nfs | Add-Member -MemberType NoteProperty -Name PersistentExport -Value (Get-NaNfsExport -Persistent -Controller $Controller -ErrorAction Stop)
            }
            catch {
                if ($_.Exception.ErrorName -eq 'EAPILICENSE') {
                    $Global:Error.RemoveAt(0)
                }
                else {
                    $NetAppDocsError += $_
                }
            }

            try {
                Write-Verbose -Message 'Calling Read-NaFile (/etc/netgroup)'
                $vfiler.Nfs | Add-Member -MemberType NoteProperty -Name Netgroup -Value (Convert-NetgroupEntry -Netgroup (Read-NaFile -Path "$( $vfiler.RootPath )/etc/netgroup" -Controller $Controller -ErrorAction Stop))
            }
            catch {
                if ($_.Exception.ErrorName -eq 'EONTAPI_ENOENT') {
                    $Global:Error.RemoveAt(0)
                }
                else {
                    $NetAppDocsError += $_
                }
            }

            #endregion NFS

            #region NameMapping

            Write-Verbose -Message '- Gathering name mappings information -'

            try {
                Write-Verbose -Message 'Calling Read-NaFile (/etc/usermap.cfg)'
                $vfiler | Add-Member -MemberType NoteProperty -Name NameMapping -Value (Convert-UsermapCfgEntry -UserMapCfg (Read-NaFile -Path "$( $vfiler.RootPath )/etc/usermap.cfg" -Controller $Controller -ErrorAction Stop))
            }
            catch {
                if ($_.Exception.ErrorName -eq 'EONTAPI_ENOENT') {
                    $Global:Error.RemoveAt(0)
                }
                else {
                    $NetAppDocsError += $_
                }
            }

            try {
                Write-Verbose -Message 'Calling Read-NaFile (/etc/passwd)'
                $vfiler | Add-Member -MemberType NoteProperty -Name NameMappingUnixUser -Value (Convert-PasswdEntry -Passwd (Read-NaFile -Path "$( $vfiler.RootPath )/etc/passwd" -Controller $Controller -ErrorAction Stop))
            }
            catch {
                if ($_.Exception.ErrorName -eq 'EONTAPI_ENOENT') {
                    $Global:Error.RemoveAt(0)
                }
                else {
                    $NetAppDocsError += $_
                }
            }

            #endregion NameMapping

            #region Snapmirror

            Write-Verbose -Message '- Identifying SnapMirror configuration information -'

            if (
                (
                    $systemData.OntapiVersion -ge [version]::Parse('1.20') -and
                    $null -ne ($systemData.License | Where-Object { $_.Service -eq 'snapmirror' -and ($_.Type -ne 'Demo' -or (-not [string]::IsNullOrWhiteSpace($_.ExpirationTime) -and (ConvertFrom-UnixTimestamp -Seconds $_.ExpirationTime) -gt [datetime]::UtcNow)) })
                ) -or
                (
                    $systemData.OntapiVersion -lt [version]::Parse('1.20') -and
                    ($systemData.License | Where-Object { $_.Service -eq 'snapmirror' }).IsLicensed
                )
            ) {
                $vfiler | Add-Member -MemberType NoteProperty -Name Snapmirror -Value ([pscustomobject]@{ })

                Write-Verbose -Message 'Calling Get-NaSnapmirror'
                $vfiler.Snapmirror | Add-Member -MemberType NoteProperty -Name Relationship -Value (Get-NaSnapmirror -Controller $Controller -ErrorVariable +NetAppDocsError -WarningAction SilentlyContinue)

                if ($systemData.OntapiVersion -ge [version]::Parse('1.2')) {
                    Write-Verbose -Message 'Calling Get-NaSnapmirrorSchedule'
                    $vfiler.Snapmirror | Add-Member -MemberType NoteProperty -Name Schedule -Value (Get-NaSnapmirrorSchedule -Controller $Controller -ErrorVariable +NetAppDocsError)
                }

                Write-Verbose -Message 'Calling Get-NaSnapmirrorSyncSchedule'
                $vfiler.Snapmirror | Add-Member -MemberType NoteProperty -Name SyncSchedule -Value (Get-NaSnapmirrorSyncSchedule -Controller $Controller -ErrorVariable +NetAppDocsError)

                Write-Verbose -Message 'Calling Get-NaSnapmirrorDestination'
                $vfiler.Snapmirror | Add-Member -MemberType NoteProperty -Name Destination -Value (Get-NaSnapmirrorDestination -Controller $Controller -ErrorVariable +NetAppDocsError)

                if ($systemData.OntapiVersion -ge [version]::Parse('1.3')) {
                    Write-Verbose -Message 'Calling Get-NaSnapmirrorConnection'
                    $vfiler.Snapmirror | Add-Member -MemberType NoteProperty -Name Connection -Value (Get-NaSnapmirrorConnection -Controller $Controller -ErrorVariable +NetAppDocsError)
                }
            }

            #endregion Snapmirror

            #region Snapvault

            ##Unused Cmdlet: Get-NaSnapvaultPriSnapshotScheduleStatus - Not needed for build docs
            ##Unused Cmdlet: Get-NaSnapvaultSecSnapshotScheduleStatus - Not needed for build docs
            ##Unused Cmdlet: Get-NaSnapvaultSoftlock - Not needed for build docs
            ##Unused Cmdlet: Get-NaSnapvaultSoftlockedSnapshot - Not needed for build docs

            Write-Verbose -Message '- Gathering Snapvault configuration information -'
            $vfiler | Add-Member -MemberType NoteProperty -Name Snapvault -Value ([pscustomobject]@{ })

            if (
                (
                    $systemData.OntapiVersion -ge [version]::Parse('1.20') -and
                    $null -ne ($systemData.License | Where-Object { $_.Service -eq 'snapvault' -and ($_.Type -ne 'Demo' -or (-not [string]::IsNullOrWhiteSpace($_.ExpirationTime) -and (ConvertFrom-UnixTimestamp -Seconds $_.ExpirationTime) -gt [datetime]::UtcNow)) })
                ) -or
                (
                    $systemData.OntapiVersion -lt [version]::Parse('1.20') -and
                    ($systemData.License | Where-Object { $_.Service -eq 'sv_ontap_sec' }).IsLicensed
                )
            ) {
                Write-Verbose -Message 'Calling Get-NaSnapvaultSecStatus'
                $vfiler.Snapvault | Add-Member -MemberType NoteProperty -Name Status -Value (Get-NaSnapvaultSecStatus -Controller $Controller -ErrorVariable +NetAppDocsError)

                Write-Verbose -Message 'Calling Get-NaSnapvault'
                $vfiler.Snapvault | Add-Member -MemberType NoteProperty -Name Config -Value (Get-NaSnapvault -Controller $Controller -ErrorVariable +NetAppDocsError)

                Write-Verbose -Message 'Calling Get-NaSnapvaultSecSnapshotSchedule'
                $vfiler.Snapvault | Add-Member -MemberType NoteProperty -Name Schedule -Value (Get-NaSnapvaultSecSnapshotSchedule -Controller $Controller -ErrorVariable +NetAppDocsError)

                Write-Verbose -Message 'Calling Get-NaSnapvaultSecDestinations'
                $vfiler.Snapvault | Add-Member -MemberType NoteProperty -Name Destination -Value (Get-NaSnapvaultSecDestinations -Controller $Controller -ErrorVariable +NetAppDocsError)
            }
            elseif (
                $systemData.OntapiVersion -lt [version]::Parse('1.20') -and
                ($systemData.License | Where-Object { $_.Service -eq 'sv_ontap_pri' }).IsLicensed
            ) {
                #BURT 389554
                $systemVersion = [version]::Parse($systemData.SystemVersion -replace '[A-Z].*$')

                if (
                    $systemVersion -gt [version]::Parse('8.0.2') -or
                    ($systemVersion -ge [version]::Parse('7.3.6') -and $systemVersion -lt [version]::Parse('8.0'))
                ) {
                    Write-Verbose -Message 'Calling Get-NaSnapvaultPriStatus'
                    $vfiler.Snapvault | Add-Member -MemberType NoteProperty -Name Status -Value (Get-NaSnapvaultPriStatus -Controller $Controller -ErrorVariable +NetAppDocsError)

                    Write-Verbose -Message 'Calling Get-NaSnapvaultPriSnapshotSchedule'
                    $vfiler.Snapvault | Add-Member -MemberType NoteProperty -Name Schedule -Value (Get-NaSnapvaultPriSnapshotSchedule -Controller $Controller -ErrorVariable +NetAppDocsError)

                    Write-Verbose -Message 'Calling Get-NaSnapvaultPriDestinations'
                    $vfiler.Snapvault | Add-Member -MemberType NoteProperty -Name Destination -Value (Get-NaSnapvaultPriDestinations -Controller $Controller -ErrorVariable +NetAppDocsError)
                }
            }

            #endregion Snapvault

            #region Quota Information

            ##Unused Cmdlet: Get-NaQuota - Info exists in Get-NaQuotaEntry
            ##Unused Cmdlet: Get-NaQuotaReport - Info exists in Get-NaQuotaEntry

            Write-Verbose -Message '- Gathering quota information -'

            if ($vfiler.Status -eq 'running') {
                Write-Verbose -Message 'Calling Get-NaQuotaEntry'
                $vfiler | Add-Member -MemberType NoteProperty -Name QuotaEntry -Value (Get-NaQuotaEntry -Controller $Controller -ErrorVariable +NetAppDocsError)
            }

            foreach ($vol in ($systemData.Vol | Where-Object { "/vol/$( $_.Name )" -cin (($vfiler | Where-Object { $_.Name -cne 'vfiler0' }).VfStores.Path | Where-Object { -not [string]::IsNullOrWhiteSpace($_) }) -and -not [string]::IsNullOrWhiteSpace($_.QuotaStatus) -and $_.State -eq 'online' -and $_.RaidStatus -notlike '*read-only*' })) {
                Write-Verbose -Message "Calling Get-NaVolQuotaStatus (Vol: $( $vol.Name ))"
                $vol | Add-Member -MemberType NoteProperty -Name QuotaStatus -Value (Get-NaVolQuotaStatus -Volume $vol.Name -Controller $Controller -ErrorVariable +NetAppDocsError)
            }

            #endregion Quota Information

            #region Vfiler0 Context Required

            ###The following section requires the vfiler to be running and the vfiler context be set to vfiler0.
            ###This section must remain at the end of the foreach loop
            #BURT 419019
            if ($vfiler.Status -eq 'running') {
                $Controller.Vfiler = 'vfiler0'

                #region Additional vFiler Options

                Write-Verbose -Message '- Gathering additional hidden options -'

                # BURT 384034
                $additionalOptions = @(
                    'ndmpd.enable'
                )

                foreach ($additionalOption in ($additionalOptions | Where-Object { $_ })) {
                    if (-not ($vfiler.Option | Where-Object { $_.Name -eq $additionalOption })) {
                        $optRequest = New-Object -TypeName System.Xml.XmlDocument

                        if ($vfiler.Name -ceq 'vfiler0') {
                            $optRequest.LoadXml("<system-cli><args><arg>options</arg><arg>$additionalOption</arg></args></system-cli>")
                        }
                        else {
                            $optRequest.LoadXml("<system-cli><args><arg>vfiler</arg><arg>run</arg><arg>-q</arg><arg>$( $vfiler.Name )</arg><arg>options</arg><arg>$additionalOption</arg></args></system-cli>")
                        }

                        Write-Verbose -Message 'Calling Invoke-NaSystemApi (options)'
                        $optResults = (Invoke-NaSystemApi -RequestXML $optRequest -Controller $Controller -ErrorVariable +NetAppDocsError).results.'cli-output' -split '\s+'

                        $vfiler.Option += [pscustomobject]@{
                            ClusterConstraint = 'none'
                            Name              = $optResults[0].Trim()
                            Value             = $optResults[1].Trim()
                        }
                    }
                }

                #endregion Addition vFiler Options

                #region Cifs Share Acls

                if ($vfiler.Name -cne 'vfiler0' -and $vfiler.Cifs -and (-not $vfiler.Cifs.ShareAcl)) {
                    $vfiler.Cifs | Add-Member -MemberType NoteProperty -Name ShareAcl -Value $(
                        Write-Verbose -Message 'Calling Invoke-NaSystemApi (cifs shares)'
                        Convert-VfilerCifsShareAcl -CifsShareString (Invoke-NaSystemApi -Request "<system-cli><args><arg>vfiler</arg><arg>run</arg><arg>-q</arg><arg>$( $vfiler.Name )</arg><arg>cifs</arg><arg>shares</arg></args></system-cli>" -Controller $Controller -ErrorVariable +NetAppDocsError).results.'cli-output'
                    )
                }

                #endregion Cifs Share Acls

                #region Vscan Settings

                Write-Verbose -Message '- Gathering Vscan settings -'

                $vfiler | Add-Member -MemberType NoteProperty -Name Vscan -Value ([pscustomobject]@{ })

                $vscanRequest = New-Object -TypeName System.Xml.XmlDocument
                $vscanOptionsRequest = New-Object -TypeName System.Xml.XmlDocument

                if ($vfiler.Name -ceq 'vfiler0') {
                    $vscanRequest.LoadXml('<system-cli><args><arg>vscan</arg></args></system-cli>')
                    $vscanOptionsRequest.LoadXml('<system-cli><args><arg>vscan</arg><arg>options</arg></args></system-cli>')
                }
                else {
                    $vscanRequest.LoadXml("<system-cli><args><arg>vfiler</arg><arg>run</arg><arg>-q</arg><arg>$( $vfiler.Name )</arg><arg>vscan</arg></args></system-cli>")
                    $vscanOptionsRequest.LoadXml("<system-cli><args><arg>vfiler</arg><arg>run</arg><arg>-q</arg><arg>$( $vfiler.Name )</arg><arg>vscan</arg><arg>options</arg></args></system-cli>")
                }

                Write-Verbose -Message 'Calling Invoke-NaSystemApi (vscan)'
                $vscanResults = (Invoke-NaSystemApi -RequestXML $vscanRequest -Controller $Controller -ErrorVariable +NetAppDocsError).results.'cli-output'

                Write-Verbose -Message 'Calling Invoke-NaSystemApi (vscan options)'
                $vscanOptionsResults = (Invoke-NaSystemApi -RequestXML $vscanOptionsRequest -Controller $Controller -ErrorVariable +NetAppDocsError).results.'cli-output'

                Write-Verbose -Message 'Calling Get-NaVscan'
                $vfiler.Vscan = Get-NaVscan -Vscan $vscanResults -VscanOptions $vscanOptionsResults

                #endregion Vscan Settings
            }

            #endregion Vfiler0 Context Required
        }

        #endregion Virtual Filer Data

        $Controller.Vfiler = $null

        Write-Verbose -Message '- Getting current timestamp -'

        Write-Verbose -Message 'Calling Get-NaTime'
        $systemData | Add-Member -MemberType NoteProperty -Name ReferenceDT -Value (ConvertFrom-UnixTimestamp -Seconds (Get-NaTime -Controller $Controller -ErrorVariable +NetAppDocsError).UtcTime)

        Write-Verbose -Message '- Getting uptime -'

        Write-Verbose -Message 'Calling Get-NaPerfData -Name system -Counters uptime'
        $systemData.SystemInfo | Add-Member -MemberType NoteProperty -Name NodeUptime -Value ([long](New-TimeSpan -End ([datetime]::Now.AddSeconds(((Get-NaPerfData -Name system -Counters uptime -Controller $Controller -ErrorVariable +NetAppDocsError).Counters | Select-Object -First 1).Value))).TotalSeconds)

        Write-Verbose -Message '- Identifying any errors during processing -'
        $systemData | Add-Member -MemberType NoteProperty -Name DiscoveryError -Value @(
            foreach ($err in ($NetAppDocsError | Where-Object { $_ })) {
                $err.CategoryInfo | Select-Object -Property *, @{
                    Name       = 'Message'
                    Expression = { $err.Exception.Message }
                }
            }
        )

        if (($systemData.DiscoveryError | Measure-Object).Count -gt 0) {
            $msg = New-Object -TypeName System.Text.StringBuilder

            [void]$msg.AppendLine("Errors were encountered while collecting data from system: $( $Controller.Name )")
            [void]$msg.AppendLine('-------------------------------------------------------------------------------------')
            [void]$msg.AppendLine('Please review the output for accuracy and, if necessary, correct any error conditions')
            [void]$msg.AppendLine('then run these scripts again.')
            [void]$msg.AppendLine('-------------------------------------------------------------------------------------')
            [void]$msg.AppendLine()

            Write-Warning -Message $msg.ToString()
        }

        $systemData
    }
    catch {
        $Global:Error.RemoveAt(0)

        $msg = New-Object -TypeName System.Text.StringBuilder

        [void]$msg.AppendLine('An unexpected error occurred. Processing halted.')
        [void]$msg.AppendLine('--------------------------------------------------------------------------------------------------')
        [void]$msg.AppendLine('As this error is not expected, please consider contacting the support staff via e-mail or by')
        [void]$msg.AppendLine('starting a thread on the community site.')
        [void]$msg.AppendLine('            E-mail: ng-NetAppDocs-support@netapp.com')
        [void]$msg.AppendLine('    Community site: https://community.netapp.com/t5/NetAppDocs/bd-p/netappdocs')
        [void]$msg.AppendLine('--------------------------------------------------------------------------------------------------')
        [void]$msg.AppendLine($( $_.Exception.GetBaseException().Message ))
        [void]$msg.AppendLine("Location: $( $_.ScriptStackTrace )")
        [void]$msg.AppendLine()

        $PSCmdlet.ThrowTerminatingError((New-Object -TypeName System.Management.Automation.ErrorRecord -ArgumentList ($msg.ToString(), $_.FullyQualifiedErrorId, $_.CategoryInfo.Category, $_.CategoryInfo.TargetName)))
    }
    finally {
        $Controller.Vfiler = $null
    }
}

#region Misc Unused Cmdlets

##Unused Cmdlet: Get-NaClone - Not needed for build docs
##Unused Cmdlet: Get-NaCopyOffload - Not needed for build docs
##Unused Cmdlet: Get-NaFile - Not needed for build docs
##Unused Cmdlet: Get-NaFileFingerprint - Not needed for build docs
##Unused Cmdlet: Get-NaFileSymlink - Not needed for build docs
##Unused Cmdlet: Get-NaLockStatus - Not needed for build docs
##Unused Cmdlet: Get-NaNdmpCopy - Not needed for build docs
##Unused Cmdlet: Get-NaPerfCounter - Not needed for build docs
##Unused Cmdlet: Get-NaPerfInstance - Not needed for build docs
##Unused Cmdlet: Get-NaPerfObject - Not needed for build docs
##Unused Cmdlet: Get-NaRadiusServer - Not yet needed for build docs?
##Unused Cmdlet: Get-NaRadiusStatistics - Not needed for build docs
##Unused Cmdlet: Get-NaReallocateJobs - Not needed for build docs
##Unused Cmdlet: Get-NaRshSession - Not needed for build docs
##Unused Cmdlet: Get-NaRshStatistics - Not needed for build docs
##Unused Cmdlet: Get-NaSystemLog - Not needed for build docs (generates too much data)

#endregion Unused Cmdlets

#region Functions

function Get-NaVscan {
    <#
        .SYNOPSIS
            Collects the Vscan configuration data via system-cli API calls.

        .DESCRIPTION
            This function is only necessary as there are no Vscan APIs.

        .NOTES
            CAVEAT : This function parses raw text output, so at any point this may break due to changes in the text.
    #>

    param (
        # The raw output string from the 'vscan' CLI command.
        [Parameter(Mandatory)]
        [AllowEmptyString()]
        [string]$Vscan,

        # The raw output string from the 'vscan options' CLI command.
        [Parameter(Mandatory)]
        [AllowEmptyString()]
        [string]$VscanOptions
    )

    $ipv4Regex = [regex]'(?<First>2[0-4]\d|25[0-5]|[01]?\d\d?)\.(?<Second>2[0-4]\d|25[0-5]|[01]?\d\d?)\.(?<Third>2[0-4]\d|25[0-5]|[01]?\d\d?)\.(?<Fourth>2[0-4]\d|25[0-5]|[01]?\d\d?)'

    $vscanSections = $Vscan -split '\n\n' | ForEach-Object { $_.Trim() }

    $vscanObj = [pscustomobject]@{
        Scanners = @( )
        Options  = @( )
    }

    (($vscanSections[1] -split '--*\n')[1] -split '\n') | ForEach-Object {
        if ($_ -match "(?<IP>$ipv4Regex)\s+\\\\(?<Name>.*)\s+(?<PriOrSec>Pri|Sec)") {
            $vscanObj.Scanners += [pscustomobject]@{
                IP       = $Matches.IP
                Name     = $Matches.Name
                PriOrSec = $Matches.PriOrSec
            }

            if ($Matches) {
                $Matches.Clear()
            }
        }
    }

    $VscanOptions -replace 'vscan options ' -replace ':' -split '\n' | Where-Object { -not [string]::IsNullOrWhiteSpace($_) } | ForEach-Object {
        $opt = $_ -split '\s+'
        $vscanObj.Options += [pscustomobject]@{
            Name  = $opt[0].Trim()
            Value = $opt[1].Trim()
        }
    }

    [pscustomobject]@{
        Status        = [regex]::Match($vscanSections[0], 'enabled|disabled').Value
        InclusionList = ($vscanSections[2] -split ':' | ForEach-Object { if ($_ -notlike '*list is empty*') { $_.Trim() } })[1]
        ExclusionList = ($vscanSections[3] -split ':' | ForEach-Object { if ($_ -notlike '*list is empty*') { $_.Trim() } })[1]
        Scanner       = $vscanObj.Scanners
        Option        = $vscanObj.Options
    }
}

function Get-NaShelfAta {
    <#
        .SYNOPSIS
            Parses the raw sysconfig -v string to gather ATA shelf information.

        .DESCRIPTION
            This function is only necessary for older version of ONTAPi that did not return ATA shelf information. Only returns
            limited information as compared to other modules output from the API.

        .NOTES
            CAVEAT : This function parses raw text output, so at any point this may break due to changes in the text.
    #>

    param (
        # The raw output string from the 'sysconfig -v' output.
        [Parameter(Mandatory)]
        [AllowEmptyString()]
        [string]$Sysconfig,

        # The raw output string from the 'storage show shelf' output.
        [Parameter(Mandatory)]
        [AllowEmptyString()]
        [string]$StorageShowShelf
    )

    $storageShowShelfArray = @( )

    foreach ($line in ($StorageShowShelf -split '\n' | Select-String -Pattern ': ', '\[\s*\d*\]')) {
        switch -Regex ($line) {
            '(.+):\s+(.+)' {
                $Key, $Value = $matches[1..2]

                if ($Key -like '*Shelf name*') {
                    if ($shelfObj) {
                        $storageShowShelfArray += $shelfObj
                    }

                    $shelfObj = [pscustomobject]@{
                        ShelfName = $Value.ToLower()
                        BayCount  = 0
                        Bays      = 0
                    }
                }
                else {
                    $shelfObj | Add-Member -MemberType NoteProperty -Name ((Get-Culture).TextInfo.ToTitleCase($Key) -replace '\s', '' -replace 'Termswitch', 'TerminationSwitch' -replace 'ShelfUID', 'ShelfUid') -Value $Value.ToLower()
                }

                break
            }
            '\s*\[\s*\d*\]' {
                $shelfObj.BayCount++
                $shelfObj.Bays++

                break
            }
            default { }
        }
    }

    $storageShowShelfArray += $shelfObj

    $fwRegex = [regex]'(?:\s*Shelf\s(?<ShelfId>\d+)\:\s+(?<ShelfType>.*)\s+Firmware.*[A]\:\s+(?<FirmwareRevA>\d+)\s+AT-FCX\s[B]\:\s+(?<FirmwareRevB>\d+|--)?)'
    $hbaRegex = [regex]'(?:\s*slot.*Adapter\s(?<ChannelName>\d+\w+)\s.*?)'

    foreach ($line in ($Sysconfig -split '\n' | Select-String -Pattern 'FC Host Adapter', 'Shelf.*AT-FCX')) {
        switch -Regex ($line) {
            $hbaRegex {
                $hbaPort = $Matches.ChannelName.Trim()

                break
            }
            $fwRegex {
                $shelfMatch = $Matches
                $shelfName = "$hbaPort.shelf$( $shelfMatch.ShelfId.Trim() )"
                $storageShowShelfEntry = $storageShowShelfArray | Where-Object { $_.ShelfName -eq $shelfName }

                [pscustomobject]@{
                    Name              = $shelfName
                    Status            = $storageShowShelfEntry.ShelfState
                    Channel           = $hbaPort
                    ID                = $shelfMatch.ShelfId.Trim()
                    Bays              = $storageShowShelfEntry.BayCount
                    Type              = $shelfMatch.ShelfType.Trim()
                    ChannelName       = $hbaPort
                    FirmwareRevA      = $shelfMatch.FirmwareRevA.Trim()
                    FirmwareRevB      = $shelfMatch.FirmwareRevB.Trim()
                    Module            = $storageShowShelfEntry.Module
                    ModuleState       = $storageShowShelfEntry.ModuleState
                    BayCount          = $storageShowShelfEntry.BayCount
                    ShelfId           = $shelfMatch.ShelfId.Trim()
                    ShelfName         = $shelfName
                    ShelfState        = $storageShowShelfEntry.ShelfState
                    ShelfType         = $shelfMatch.ShelfType.Trim()
                    ShelfUid          = $storageShowShelfEntry.ShelfUid
                    TerminationSwitch = $storageShowShelfEntry.TerminationSwitch
                }

                if ($Matches) {
                    $Matches.Clear()
                }

                break
            }
            default { }
        }
    }
}

function Convert-UsermapCfgEntry {
    param (
        [Parameter(Mandatory)]
        [AllowEmptyString()]
        [string]$UserMapCfg
    )

    if (-not [string]::IsNullOrWhiteSpace($UserMapCfg)) {
        $i = 1

        $userEntryObj = [pscustomobject]@{
            Position           = 0
            WindowsIpQualifier = $null
            WindowsName        = $null
            Direction          = $null
            UnixIpQualifier    = $null
            UnixName           = $null
        }

        foreach ($line in ($UserMapCfg -split '\n' | Where-Object { -not [string]::IsNullOrWhiteSpace($_) -and $_ -notmatch '^(?:\s*|[#].*)$' })) {
            $entry = $line -split '(?s)\s+(?=(?:[^"]*"[^"]*")*[^"]*$)', 3 | Where-Object { -not [string]::IsNullOrWhiteSpace($_) }

            $outputObj = $userEntryObj.psobject.Copy()

            switch -Exact ($entry.Count) {
                '3' {
                    $outputObj.Direction = $(
                        switch -Exact ($entry[1]) {
                            '=>' { 'win_unix' }
                            '<=' { 'unix_win' }
                            '==' { 'bidirectional' }
                        }
                    )

                    $outputObj.Position = $i
                    $outputObj.WindowsName, $outputObj.WindowsIpQualifier = ($entry[0].Split(':', 2))[1, 0] | ForEach-Object { $_.Trim() }
                    $outputObj.UnixName, $outputObj.UnixIpQualifier = ($entry[2].Split(':', 2))[1, 0] | ForEach-Object { $_.Trim() }

                    $outputObj
                }
                '2' {
                    $outputObj.Direction = 'bidirectional'
                    $outputObj.Position = $i
                    $outputObj.WindowsName, $outputObj.WindowsIpQualifier = ($entry[0].Split(':', 2))[1, 0] | ForEach-Object { $_.Trim() }
                    $outputObj.UnixName, $outputObj.UnixIpQualifier = ($entry[1].Split(':', 2))[1, 0] | ForEach-Object { $_.Trim() }

                    $outputObj
                }
                default {
                    Write-Warning -Message "There is an invalid entry in the usermap.cfg file. This entry (#$i) has been skipped and the output data may be invalid."
                }
            }

            $i++
        }
    }
}

function Convert-VfilerCifsShareAcl {
    param (
        [Parameter(Mandatory)]
        [AllowEmptyString()]
        [string]$CifsShareString
    )

    foreach ($line in (($CifsShareString -split "`n") | Where-Object { -not [string]::IsNullOrWhiteSpace($_) -and $_ -notmatch '^(?:Name.+|[----].+|\s*)$' })) {
        if ($line -match '^\S') {
            if ($outputObj) {
                $outputObj
            }

            $outputObj = [pscustomobject]@{
                ShareName   = ($line -split '[/]')[0].Trim()
                UserAclInfo = @( )
            }
        }
        elseif ($line -match '\s+[/]\s+') {
            $user, $accessRight = $line -split '\s+[/]\s+', 2

            if ($user -match '{g}') {
                $outputObj.UserAclInfo += [pscustomobject]@{
                    UserName      = $null
                    AccessRights  = $AccessRight.Trim()
                    UnixGroupName = ($user -replace '{g}').Trim()
                }
            }
            else {
                $outputObj.UserAclInfo += [pscustomobject]@{
                    UserName      = $user.Trim()
                    AccessRights  = $AccessRight.Trim()
                    UnixGroupName = $null
                }
            }
        }
    }

    if (($outputObj.UserAclInfo | Measure-Object).Count -gt 0) {
        $outputObj
    }
}

function Convert-PasswdEntry {
    param (
        [Parameter(Mandatory)]
        [AllowEmptyString()]
        [string]$Passwd
    )

    #NAME:PASSWD:UID:GID:COMMENT:DIRECTORY:SHELL

    if (-not [string]::IsNullOrWhiteSpace($Passwd)) {
        foreach ($line in ($Passwd -split '\n' | Where-Object { -not [string]::IsNullOrWhiteSpace($_) -and $_ -notmatch '^(?:\s*[#]|\s*$)' })) {
            $entry = $line -split ':', 7

            [pscustomobject]@{
                UserName = $entry[0]
                UserId   = $entry[2]
                GroupId  = $entry[3]
            }
        }
    }
}

function Convert-NetgroupEntry {
    param (
        [Parameter(Mandatory)]
        [AllowEmptyString()]
        [string]$Netgroup
    )

    #groupname member-list (hostname, username, domainname)

    if (-not [string]::IsNullOrWhiteSpace($Netgroup)) {
        foreach ($line in ($Netgroup -split '\n' | Where-Object { -not [string]::IsNullOrWhiteSpace($_) -and $_ -notmatch '^(?:\s*[#]|\s*$)' })) {
            $entry = $line -split '\s+'

            foreach ($member in ($entry | Select-Object -Skip 1 | Where-Object { -not [string]::IsNullOrWhiteSpace($_) })) {
                if ($member -like '(*)') {
                    $hostName, $userName, $domainName = ($member -replace '[)]' -replace '[(]') -split '\s*,\s*', 3

                    [pscustomobject]@{
                        NetgroupName   = $entry[0]
                        MemberNetgroup = $null
                        Host           = $hostName
                        User           = $userName
                        Domain         = $domainName
                    }
                }
                else {
                    [pscustomobject]@{
                        NetgroupName   = $entry[0]
                        MemberNetgroup = $member
                        Host           = $null
                        User           = $null
                        Domain         = $null
                    }
                }
            }
        }
    }
}

#endregion Functions

# SIG # Begin signature block
# MIIy0QYJKoZIhvcNAQcCoIIywjCCMr4CAQExDzANBglghkgBZQMEAgEFADB5Bgor
# BgEEAYI3AgEEoGswaTA0BgorBgEEAYI3AgEeMCYCAwEAAAQQH8w7YFlLCE63JNLG
# KX7zUQIBAAIBAAIBAAIBAAIBADAxMA0GCWCGSAFlAwQCAQUABCDWK74uSK3gIgxb
# h8UgkTGgs7qXAINWCMAPvakDdBuR+6CCFxcwggQ+MIIDJqADAgECAgRKU4woMA0G
# CSqGSIb3DQEBCwUAMIG+MQswCQYDVQQGEwJVUzEWMBQGA1UEChMNRW50cnVzdCwg
# SW5jLjEoMCYGA1UECxMfU2VlIHd3dy5lbnRydXN0Lm5ldC9sZWdhbC10ZXJtczE5
# MDcGA1UECxMwKGMpIDIwMDkgRW50cnVzdCwgSW5jLiAtIGZvciBhdXRob3JpemVk
# IHVzZSBvbmx5MTIwMAYDVQQDEylFbnRydXN0IFJvb3QgQ2VydGlmaWNhdGlvbiBB
# dXRob3JpdHkgLSBHMjAeFw0wOTA3MDcxNzI1NTRaFw0zMDEyMDcxNzU1NTRaMIG+
# MQswCQYDVQQGEwJVUzEWMBQGA1UEChMNRW50cnVzdCwgSW5jLjEoMCYGA1UECxMf
# U2VlIHd3dy5lbnRydXN0Lm5ldC9sZWdhbC10ZXJtczE5MDcGA1UECxMwKGMpIDIw
# MDkgRW50cnVzdCwgSW5jLiAtIGZvciBhdXRob3JpemVkIHVzZSBvbmx5MTIwMAYD
# VQQDEylFbnRydXN0IFJvb3QgQ2VydGlmaWNhdGlvbiBBdXRob3JpdHkgLSBHMjCC
# ASIwDQYJKoZIhvcNAQEBBQADggEPADCCAQoCggEBALqEtnLbngxr4pnpMAGnduoy
# uJVBGsnaYU5Ycs/+9oJ5v3NhBgqlJ9izX9NFThxy1k4y8nKKD/eDGdBqgIAARR6w
# x+eavxJXJxyjaC8Kh71qaw5eZfMcd9XUhY1wIbSzMueLotWGOQKxuNJHzuTJScQ7
# p977VH1XvvDobsJ5sjoLVeJQmBYyE1wveFbBwpSz8lrkJ5qfJNfG7NCbJYLjzMLE
# RcWMl3oGayoRn6kKbkg7b9vUERlC948Hv/VTX5w+9Bcs5mmsTjJMYnfqt+jluzS8
# GYuunFHnt361U7EzIuVtz3A8Gvrim2e2g/SNpa9iTE3gWKxkNBID+LaNlGMkpHEC
# AwEAAaNCMEAwDgYDVR0PAQH/BAQDAgEGMA8GA1UdEwEB/wQFMAMBAf8wHQYDVR0O
# BBYEFGpyJnrQHu995ztpUdRsjZ+QEmarMA0GCSqGSIb3DQEBCwUAA4IBAQB5nx2W
# xrZ5PyKNh9OHAwRgamuaLlmJcxGsQ9H1E/+NOSvA8r1PcIypL+oXxAtUntQblpgz
# PKitYqIAdqtZaW4GHX7EuUSNmK8S1GHbChlGR/Pr92PBQAVApdK39LWaNr+piHaI
# BFUEK5yHfxo3PH4tpRrY1Ileyr2sPWzYba/V83YPzTuIOCKdbJOaxD2/ghtlP6YP
# Xar85bIVyrWtxrw90ITo6gZysE05Mni/PhGcC6SdmiHz8JsLMHjbwdyHQ/68Y5rK
# xcIcyceN/zsSWAjmtj3seixO+4OWzgw8aYdUc6RzwpP/URCsFVQB2PwFsYmhf3SD
# mknX3E57ikhvi0X2MIIF3zCCBMegAwIBAgIQTkDkN1Tt5owAAAAAUdOUfzANBgkq
# hkiG9w0BAQsFADCBvjELMAkGA1UEBhMCVVMxFjAUBgNVBAoTDUVudHJ1c3QsIElu
# Yy4xKDAmBgNVBAsTH1NlZSB3d3cuZW50cnVzdC5uZXQvbGVnYWwtdGVybXMxOTA3
# BgNVBAsTMChjKSAyMDA5IEVudHJ1c3QsIEluYy4gLSBmb3IgYXV0aG9yaXplZCB1
# c2Ugb25seTEyMDAGA1UEAxMpRW50cnVzdCBSb290IENlcnRpZmljYXRpb24gQXV0
# aG9yaXR5IC0gRzIwHhcNMjEwNTA3MTU0MzQ1WhcNMzAxMTA3MTYxMzQ1WjBpMQsw
# CQYDVQQGEwJVUzEWMBQGA1UECgwNRW50cnVzdCwgSW5jLjFCMEAGA1UEAww5RW50
# cnVzdCBDb2RlIFNpZ25pbmcgUm9vdCBDZXJ0aWZpY2F0aW9uIEF1dGhvcml0eSAt
# IENTQlIxMIICIjANBgkqhkiG9w0BAQEFAAOCAg8AMIICCgKCAgEAp4GP9xRFtmJD
# 8tiu0yVeSE9Rv8V9n1AcNdHWfmEqlBltJ0akphpd91RRaoAixqhmdU1Ug8leaBur
# 9ltksK2tIL1U70ZrbQLnRa519o6KuTIui7h3HFJNeYhfpToYyVAslyctv9oAfWN/
# 7zLsRodj25qfw1ohNnv5m9XKoG5yLPzh8Z5wTQhWFW+Qq/tIurnXwYJ4hWUuf7XJ
# wOIUtzcRQQbiiuCo9uV+tngFAcNg7U8HQS4KE0njkJt/3b36rL9kUdFcm7T1XOdc
# /zubpaAa130JssK3/24cvMh95ukr/HKzFOlKVRKEnEQldR32KvBPpSA9aCXrYZd8
# D+W2PfOuw8ERvBuOzOBHMF5CAIZx41isBsplH3uUpktXZwx+Xq14Z1tV417rx9js
# TG6Gy/Pc+J+HqnJYEg99pvj4Qjk7PCzkMk1JjODhAMI4oJz6hD5B3G5WrsYaW/Rn
# aAUBzRu/roe8nVP2Lui2a+SZ3sVPh1io0mUeyB/Vcm7uWRxXOwlyndfKt5DGzXtF
# kpFCA0x9P8ryqrjCDobzEJ9GLqRmhmhaaBhwKTgRgGBrikOjc2zjs2s3/+adZwGS
# ht8vSNH7UGDVXP4h0wFCY/7vcLQXwI+o7tPBS18S6v39Lg6HRGDjqfTCGKPj/c4M
# hCIN86d42pPz2zjPuS8zxv8HPF6+RdMCAwEAAaOCASswggEnMA4GA1UdDwEB/wQE
# AwIBhjASBgNVHRMBAf8ECDAGAQH/AgEBMB0GA1UdJQQWMBQGCCsGAQUFBwMDBggr
# BgEFBQcDCDA7BgNVHSAENDAyMDAGBFUdIAAwKDAmBggrBgEFBQcCARYaaHR0cDov
# L3d3dy5lbnRydXN0Lm5ldC9ycGEwMwYIKwYBBQUHAQEEJzAlMCMGCCsGAQUFBzAB
# hhdodHRwOi8vb2NzcC5lbnRydXN0Lm5ldDAwBgNVHR8EKTAnMCWgI6Ahhh9odHRw
# Oi8vY3JsLmVudHJ1c3QubmV0L2cyY2EuY3JsMB0GA1UdDgQWBBSCutY9l86fz3Ho
# kjev/bO1aTVXzzAfBgNVHSMEGDAWgBRqciZ60B7vfec7aVHUbI2fkBJmqzANBgkq
# hkiG9w0BAQsFAAOCAQEAH15BBLaDcCRTLFVzHWU6wOy0ewSYXlk4EwmkWZRCXlC/
# T2xuJSCQk1hADfUZtGLuJF7CAVgVAh0QCW+o1PuSfjc4Pi8UfY8dQzZks2YTXxTM
# pXH3WyFLxpe+3JX8cH0RHNMh3dAkOSnF/goapc97ee46b97cv+kR3RaDCNMsjX9N
# qBR5LwVhUjjrYPMUaH3LsoqtwJRc5CYOLIrdRsPO5FZRxVbjhbhNm0VyiwfxivtJ
# uF/R8paBXWlSJPEII9LWIw/ri9d+i8GTa/rxYntY6VCbl24XiA3hxkOY14FhtoWd
# R+yxnq4/IDtDndiiHODUfAjCr3YG+GJmerb3+sivNTCCBmcwggRPoAMCAQICEEbM
# 41QNEgkq80rzt39+/oswDQYJKoZIhvcNAQELBQAwYzELMAkGA1UEBhMCVVMxFjAU
# BgNVBAoTDUVudHJ1c3QsIEluYy4xPDA6BgNVBAMTM0VudHJ1c3QgRXh0ZW5kZWQg
# VmFsaWRhdGlvbiBDb2RlIFNpZ25pbmcgQ0EgLSBFVkNTMjAeFw0yNDEwMjMxNTIy
# MTRaFw0yNTExMDkxNTIyMTNaMIHCMQswCQYDVQQGEwJVUzETMBEGA1UECBMKQ2Fs
# aWZvcm5pYTERMA8GA1UEBxMIU2FuIEpvc2UxEzARBgsrBgEEAYI3PAIBAxMCVVMx
# GTAXBgsrBgEEAYI3PAIBAhMIRGVsYXdhcmUxFDASBgNVBAoTC05ldEFwcCwgSW5j
# MR0wGwYDVQQPExRQcml2YXRlIE9yZ2FuaXphdGlvbjEQMA4GA1UEBRMHMzQ1Mjc0
# MzEUMBIGA1UEAxMLTmV0QXBwLCBJbmMwggGiMA0GCSqGSIb3DQEBAQUAA4IBjwAw
# ggGKAoIBgQCIHyusYBqN6d8MjDVGb9yKy/jxOJi5Ry9vtMsEpSe0eX1NwlzjdPpo
# R+kSTTj0HXwjn48v4UtO3lOWVVdqDuAuECTvG+n31c1RmaB4qS0qg9VEDsUP0/gr
# R4GlmEnGRIeDNWnRIV4oHGTqA1NjNU3esZYLQSV9j5DSzh0W+vyq10stDZiLBNai
# KoBzi5fs+C7QWW+xd+fSFV3Z+zIfE/jHFa4ZKP0n8Bva/5XKZSg6XVBN09oY/EF2
# 67wlz2018jiSwKZi++kCqTIpp9j7dKP5x+0/bK4GlGeivT/NObhjDN8e0Iu5rC2L
# tCalpr5CBMzjKmwfV8mVd9L/qPKk8QZxBdBrUpA1qHDCHbd9BRcd2zT8doIbM0vT
# dHnT9pRPZYws7BfT33KMoV7nShbrNViWhkP3i3UkQv7W9RptiZwfEDpt4Lxn0Uw8
# Zf94fbwMQaV3A28ItUbueLN23+k41N0jqdFTbpCaqQpUT/NxlRHECtn0Lp6kjZTy
# wKF7FqH+jpcCAwEAAaOCATUwggExMAwGA1UdEwEB/wQCMAAwHQYDVR0OBBYEFGzw
# 5EcKP/wp/cuZ0BIs8j6slZQcMB8GA1UdIwQYMBaAFM6JT4JRqhWihGLKMSNh0mH7
# +P54MA4GA1UdDwEB/wQEAwIHgDATBgNVHSUEDDAKBggrBgEFBQcDAzBnBggrBgEF
# BQcBAQRbMFkwIwYIKwYBBQUHMAGGF2h0dHA6Ly9vY3NwLmVudHJ1c3QubmV0MDIG
# CCsGAQUFBzAChiZodHRwOi8vYWlhLmVudHJ1c3QubmV0L2V2Y3MyLWNoYWluLnA3
# YzAxBgNVHR8EKjAoMCagJKAihiBodHRwOi8vY3JsLmVudHJ1c3QubmV0L2V2Y3My
# LmNybDAgBgNVHSAEGTAXMAcGBWeBDAEDMAwGCmCGSAGG+mwKAQIwDQYJKoZIhvcN
# AQELBQADggIBADAwfASY7H1rvIVgDB4LCw7V6xYShLM1XBV0YV8mgn50EVUgkXRr
# qwdn9JgVJvPU6jQ30ab9bb2505N/e1hWQN+Skw21Kbixw3mJ2lZNjY9CvwPUAc4B
# 9xXFEl/w7w1GuATgHO77YCpe0QTbYpbuSTXA0YBs1LlzmuoMxCD+E82srnED8Crg
# m4IHe71kpqqbsIYDnxujU+w/+/CAijPCuxXD2Ff1D7Ms97n9OFvLZHU2e+7xTP66
# RFXF81uCNcLYZSAMiXpjFCiXz74yNHtqZYecnNKvA3hUnvE0BH2IbJFpFiwQO9H2
# rSxdf6qTJANWvCTm7AkDMPhCneufSIFeGHyINH5sKNc+MgsNawZwy2Bfld2YE+ii
# +RyoUgQvl+OZu58dcPYvbgSkc+nE7Xlq/nzE4qNB83ZUuo4MQuGgZ1ZdROSgO6BD
# lCDa1Mr+viR84DtT6dwMYDf0PYWe0LoDYBVLRknZHpSNCMULURCTENBJdJo1mrOn
# hmmm5ZJ3Hg/CmyrujK67YvNFUsQnpLKYEut7ntmJM5ptEPm7Ql5zzaOp3o68a/Q6
# 0+s6p4NwZQYo4ekHk+DM4oaPiv8K+dEPxvppQCSoSPG0S59A8oFBXzoaidOTC06j
# xfGFkPFtOEIO269lK0NCIgr2lJvlUdlUPSnM2eWkEkslz42ShJgON0SaMIIGgzCC
# BGugAwIBAgIQNa+3e500H2r8j4RGqzE1KzANBgkqhkiG9w0BAQ0FADBpMQswCQYD
# VQQGEwJVUzEWMBQGA1UECgwNRW50cnVzdCwgSW5jLjFCMEAGA1UEAww5RW50cnVz
# dCBDb2RlIFNpZ25pbmcgUm9vdCBDZXJ0aWZpY2F0aW9uIEF1dGhvcml0eSAtIENT
# QlIxMB4XDTIxMDUwNzE5MTk1MloXDTQwMTIyOTIzNTkwMFowYzELMAkGA1UEBhMC
# VVMxFjAUBgNVBAoTDUVudHJ1c3QsIEluYy4xPDA6BgNVBAMTM0VudHJ1c3QgRXh0
# ZW5kZWQgVmFsaWRhdGlvbiBDb2RlIFNpZ25pbmcgQ0EgLSBFVkNTMjCCAiIwDQYJ
# KoZIhvcNAQEBBQADggIPADCCAgoCggIBAL69pznJpX3sXWXx9Cuph9DnrRrFGjsY
# zuGhUY1y+s5YH1y4JEIPRtUxl9BKTeObMMm6l6ic/kU2zyeA53u4bsEkt9+ndNyF
# 8qMkWEXMlJQ7AuvEjXxG9VxmguOkwdMfrG4MUyMO1Dr62kLxg1RfNTJW8rV4m1cA
# SB6pYWEnDnMDQ7bWcJL71IWaMMaz5ppeS+8dKthmqxZG/wvYD6aJSgJRV0E8QThO
# l8dRMm1njmahXk2fNSKv1Wq3f0BfaDXMafrxBfDqhabqMoXLwcHKg2lFSQbcCWy6
# SWUZjPm3NyeMZJ414+Xs5wegnahyvG+FOiymFk49nM8I5oL1RH0owL2JrWwv3C94
# eRHXHHBL3Z0ITF4u+o29p91j9n/wUjGEbjrY2VyFRJ5jBmnQhlh4iZuHu1gcpChs
# xv5pCpwerBFgal7JaWUu7UMtafF4tzstNfKqT+If4wFvkEaq1agNBFegtKzjbb2d
# GyiAJ0bH2qpnlfHRh3vHyCXphAyPiTbSvjPhhcAz1aA8GYuvOPLlk4C/xsOre5PE
# PZ257kV2wNRobzBePLQ2+ddFQuASBoDbpSH85wV6KI20jmB798i1SkesFGaXoFpp
# cjFXa1OEzWG6cwcVcDt7AfynP4wtPYeM+wjX5S8Xg36Cq08J8inhflV3ZZQFHVnU
# Ct2TfuMUXeK7AgMBAAGjggErMIIBJzASBgNVHRMBAf8ECDAGAQH/AgEAMB0GA1Ud
# DgQWBBTOiU+CUaoVooRiyjEjYdJh+/j+eDAfBgNVHSMEGDAWgBSCutY9l86fz3Ho
# kjev/bO1aTVXzzAzBggrBgEFBQcBAQQnMCUwIwYIKwYBBQUHMAGGF2h0dHA6Ly9v
# Y3NwLmVudHJ1c3QubmV0MDEGA1UdHwQqMCgwJqAkoCKGIGh0dHA6Ly9jcmwuZW50
# cnVzdC5uZXQvY3NicjEuY3JsMA4GA1UdDwEB/wQEAwIBhjATBgNVHSUEDDAKBggr
# BgEFBQcDAzBEBgNVHSAEPTA7MDAGBFUdIAAwKDAmBggrBgEFBQcCARYaaHR0cDov
# L3d3dy5lbnRydXN0Lm5ldC9ycGEwBwYFZ4EMAQMwDQYJKoZIhvcNAQENBQADggIB
# AD4AVLgq849mr2EWxFiTZPRBi2RVjRs1M6GbkdirRsqrX7y+fnDk0tcHqJYH14bR
# VwoI0NB4Tfgq37IE85rh13zwwQB6wUCh34qMt8u0HQFh8piapt24gwXKqSwW3Jwt
# Dv6nl+RQqZeVwUsqjFHjxALga3w1TVO8S5QTi1MYFl6mCqe4NMFssess5DF9DCzG
# fOGkVugtdtWyE3XqgwCuAHfGb6k97mMUgVAW/FtPEhkOWw+N6kvOBkyJS64gzI5H
# pnXWZe4vMOhdNI8fgk1cQqbyFExQIJwJonQkXDnYiTKFPK+M5Wqe5gQ6pRP/qh3N
# R0suAgW0ao/rhU+B7wrbfZ8pj6XCP1I4UkGVO7w+W1QwQiMJY95QjYk1RfqruA+P
# oq17ehGT8Y8ohHtoeUdq6GQpTR/0HS9tHsiUhjzTWpl6a3yrNfcrOUtPuT8Wku8p
# jI2rrAEazHFEOctAPiASzghw40f+3IDXCADRC2rqIbV5ZhfpaqpW3c0VeLEDwBSt
# PkcYde0KU0syk83/gLGQ1hPl5EF4Iu1BguUO37DOlSFF5osB0xn39CtVrNlWc2MQ
# 4LigbctUlpigmSFRBqqmDDorY8t52kO50hLM3o9VeukJ8+Ka0yXBezaS2uDlUmfN
# 4+ZUCqWd1HOj0y9dBmSFA3d/YNjCvHTJlZFot7d+YRl1MYIbEDCCGwwCAQEwdzBj
# MQswCQYDVQQGEwJVUzEWMBQGA1UEChMNRW50cnVzdCwgSW5jLjE8MDoGA1UEAxMz
# RW50cnVzdCBFeHRlbmRlZCBWYWxpZGF0aW9uIENvZGUgU2lnbmluZyBDQSAtIEVW
# Q1MyAhBGzONUDRIJKvNK87d/fv6LMA0GCWCGSAFlAwQCAQUAoIGEMBgGCisGAQQB
# gjcCAQwxCjAIoAKAAKECgAAwGQYJKoZIhvcNAQkDMQwGCisGAQQBgjcCAQQwHAYK
# KwYBBAGCNwIBCzEOMAwGCisGAQQBgjcCARUwLwYJKoZIhvcNAQkEMSIEID2+hLZj
# DiQYOsudYniTEbCszBJSG92RNnNbHQvafdexMA0GCSqGSIb3DQEBAQUABIIBgF5I
# nocpmLOOBgeQChSrMi8/HkMkQnKSXYHInnSQP8a3eiNb0Kh5+2a3G2Cv/8D4YY0H
# fvOqvI+z5cqEW3x2XgNJcPhgKvsVxVTtyQMqpgZzVO2vsCoGl7+bTHMh61LVpO4G
# 0Vrh6yTBOKtaiSLFQP1G6D3UdwgAtBbSur6dhmXDUQdqyPg3PadmVrVLwIfNylFQ
# HnRYDP0ACSAWpVw6YAr74nPeizRh+x9qsVJxD/JM5wc3KNf5v98dzxCvy3xi1U1r
# U+p+NnHOIqToj+VNn7hIeGD9qtpYsPVbw/F4Lczd74Cvwfxf8fmPh1WbSX/NsKad
# 6Amyf7l6Ma+8qiGHuYHPqB8jMiBjE7bE79F0jtN9vDThvzE/cyVYYQJGSc7ZEl1C
# AwucOfAHVKQ1sxemDngxw9d0fmvOGck5Wg0s4yTrOqyZjmoz0SI3p12AiGOPr/x5
# TVQDWlFftxr+1BpiwwsKDADakZh390hjGIvR6/1+Nkoc6SgoHmT25NATzMro2KGC
# GGMwghhfBgorBgEEAYI3AwMBMYIYTzCCGEsGCSqGSIb3DQEHAqCCGDwwghg4AgED
# MQ0wCwYJYIZIAWUDBAIDMIHzBgsqhkiG9w0BCRABBKCB4wSB4DCB3QIBAQYKYIZI
# AYb6bAoDBTAxMA0GCWCGSAFlAwQCAQUABCC/ZgqEDGbVZ9tBrDklyE0FNzto6ab9
# oM71Y83CB9a+jgIIIeXwJe5Vf88YDzIwMjUwMTA5MDMwNjI5WjADAgEBoHmkdzB1
# MQswCQYDVQQGEwJDQTEQMA4GA1UECBMHT250YXJpbzEPMA0GA1UEBxMGT3R0YXdh
# MRYwFAYDVQQKEw1FbnRydXN0LCBJbmMuMSswKQYDVQQDEyJFbnRydXN0IFRpbWVz
# dGFtcCBBdXRob3JpdHkgLSBUU0EyoIITDjCCBd8wggTHoAMCAQICEE5A5DdU7eaM
# AAAAAFHTlH8wDQYJKoZIhvcNAQELBQAwgb4xCzAJBgNVBAYTAlVTMRYwFAYDVQQK
# Ew1FbnRydXN0LCBJbmMuMSgwJgYDVQQLEx9TZWUgd3d3LmVudHJ1c3QubmV0L2xl
# Z2FsLXRlcm1zMTkwNwYDVQQLEzAoYykgMjAwOSBFbnRydXN0LCBJbmMuIC0gZm9y
# IGF1dGhvcml6ZWQgdXNlIG9ubHkxMjAwBgNVBAMTKUVudHJ1c3QgUm9vdCBDZXJ0
# aWZpY2F0aW9uIEF1dGhvcml0eSAtIEcyMB4XDTIxMDUwNzE1NDM0NVoXDTMwMTEw
# NzE2MTM0NVowaTELMAkGA1UEBhMCVVMxFjAUBgNVBAoMDUVudHJ1c3QsIEluYy4x
# QjBABgNVBAMMOUVudHJ1c3QgQ29kZSBTaWduaW5nIFJvb3QgQ2VydGlmaWNhdGlv
# biBBdXRob3JpdHkgLSBDU0JSMTCCAiIwDQYJKoZIhvcNAQEBBQADggIPADCCAgoC
# ggIBAKeBj/cURbZiQ/LYrtMlXkhPUb/FfZ9QHDXR1n5hKpQZbSdGpKYaXfdUUWqA
# IsaoZnVNVIPJXmgbq/ZbZLCtrSC9VO9Ga20C50WudfaOirkyLou4dxxSTXmIX6U6
# GMlQLJcnLb/aAH1jf+8y7EaHY9uan8NaITZ7+ZvVyqBuciz84fGecE0IVhVvkKv7
# SLq518GCeIVlLn+1ycDiFLc3EUEG4orgqPblfrZ4BQHDYO1PB0EuChNJ45Cbf929
# +qy/ZFHRXJu09VznXP87m6WgGtd9CbLCt/9uHLzIfebpK/xysxTpSlUShJxEJXUd
# 9irwT6UgPWgl62GXfA/ltj3zrsPBEbwbjszgRzBeQgCGceNYrAbKZR97lKZLV2cM
# fl6teGdbVeNe68fY7Exuhsvz3Pifh6pyWBIPfab4+EI5Ozws5DJNSYzg4QDCOKCc
# +oQ+QdxuVq7GGlv0Z2gFAc0bv66HvJ1T9i7otmvkmd7FT4dYqNJlHsgf1XJu7lkc
# VzsJcp3XyreQxs17RZKRQgNMfT/K8qq4wg6G8xCfRi6kZoZoWmgYcCk4EYBga4pD
# o3Ns47NrN//mnWcBkobfL0jR+1Bg1Vz+IdMBQmP+73C0F8CPqO7TwUtfEur9/S4O
# h0Rg46n0whij4/3ODIQiDfOneNqT89s4z7kvM8b/BzxevkXTAgMBAAGjggErMIIB
# JzAOBgNVHQ8BAf8EBAMCAYYwEgYDVR0TAQH/BAgwBgEB/wIBATAdBgNVHSUEFjAU
# BggrBgEFBQcDAwYIKwYBBQUHAwgwOwYDVR0gBDQwMjAwBgRVHSAAMCgwJgYIKwYB
# BQUHAgEWGmh0dHA6Ly93d3cuZW50cnVzdC5uZXQvcnBhMDMGCCsGAQUFBwEBBCcw
# JTAjBggrBgEFBQcwAYYXaHR0cDovL29jc3AuZW50cnVzdC5uZXQwMAYDVR0fBCkw
# JzAloCOgIYYfaHR0cDovL2NybC5lbnRydXN0Lm5ldC9nMmNhLmNybDAdBgNVHQ4E
# FgQUgrrWPZfOn89x6JI3r/2ztWk1V88wHwYDVR0jBBgwFoAUanImetAe733nO2lR
# 1GyNn5ASZqswDQYJKoZIhvcNAQELBQADggEBAB9eQQS2g3AkUyxVcx1lOsDstHsE
# mF5ZOBMJpFmUQl5Qv09sbiUgkJNYQA31GbRi7iRewgFYFQIdEAlvqNT7kn43OD4v
# FH2PHUM2ZLNmE18UzKVx91shS8aXvtyV/HB9ERzTId3QJDkpxf4KGqXPe3nuOm/e
# 3L/pEd0WgwjTLI1/TagUeS8FYVI462DzFGh9y7KKrcCUXOQmDiyK3UbDzuRWUcVW
# 44W4TZtFcosH8Yr7Sbhf0fKWgV1pUiTxCCPS1iMP64vXfovBk2v68WJ7WOlQm5du
# F4gN4cZDmNeBYbaFnUfssZ6uPyA7Q53Yohzg1HwIwq92BvhiZnq29/rIrzUwggZv
# MIIEV6ADAgECAhAlvCvzKcoQfx6puoiF1J07MA0GCSqGSIb3DQEBDQUAMGkxCzAJ
# BgNVBAYTAlVTMRYwFAYDVQQKDA1FbnRydXN0LCBJbmMuMUIwQAYDVQQDDDlFbnRy
# dXN0IENvZGUgU2lnbmluZyBSb290IENlcnRpZmljYXRpb24gQXV0aG9yaXR5IC0g
# Q1NCUjEwHhcNMjEwNTA3MTkyMjE0WhcNNDAxMjI5MjM1OTAwWjBOMQswCQYDVQQG
# EwJVUzEWMBQGA1UEChMNRW50cnVzdCwgSW5jLjEnMCUGA1UEAxMeRW50cnVzdCBU
# aW1lIFN0YW1waW5nIENBIC0gVFMyMIICIjANBgkqhkiG9w0BAQEFAAOCAg8AMIIC
# CgKCAgEAtQMqBraETzTMnjG+6o7Ge24gJA8D172BNjPmb9I+u8SeqqDMZl1CYDRR
# 4i1hg7o8nmi4S8NmjqXeK7qbLUGl3tDLVwUtiNnARohtPDUNGZCsuAwZ8EzF20Vb
# 3DvCvI1EAOA4//9dyM8Imj3Usut7lcPEpQDgYgFkWV+M43uLqflFt8awogo3Q7hG
# gauGfn8v3vBf680gzKUtMeOoLpmb4Hjd9NtdgmN9MDmIfiQG5nxghHjIXCnllcv1
# e0/PWNyQz6eiexRUxOA3mDDBm7aunCn2ayVD5xJqPhmLvsDngddKMAh+fI3fPPNI
# kyHegYNqnQA+12C60bNiCJbptkkAOPRcdg3ad2o9oJXeHqHMWXpen8/pdl1A8rUV
# tWBoLPNaySOD2PUaa2kopLbUrmIG3iZFvd7pVneq3GSwQc3ME/FdcXSylAhLTzJu
# fnC4d+nVMjfCet5CEuhxHOLYYr+qa0GGBlvmab4mCfSXwv5M9IByhG7b3sUHqY4V
# om3hG+AJD/WgB6C7xHdzCTwHk40BtRJnLU0ZtbjFyRCrcM9C/6UnHdYS1uQYRGD7
# OV2Eu2a1UxKdAxSCZSm07Ely/bND5cMCKSS6H/i4vl2jty6p/M9i2fqXxBN441W5
# WnqE6/+7z2mIwLb5i2CFQRBQxwUQ0l52nT417PZPivkSez+vCzcCAwEAAaOCASww
# ggEoMBIGA1UdEwEB/wQIMAYBAf8CAQAwHQYDVR0OBBYEFCYP8MRICBvN3ZH1VFS2
# s7P8mfEIMB8GA1UdIwQYMBaAFIK61j2Xzp/PceiSN6/9s7VpNVfPMDMGCCsGAQUF
# BwEBBCcwJTAjBggrBgEFBQcwAYYXaHR0cDovL29jc3AuZW50cnVzdC5uZXQwMQYD
# VR0fBCowKDAmoCSgIoYgaHR0cDovL2NybC5lbnRydXN0Lm5ldC9jc2JyMS5jcmww
# DgYDVR0PAQH/BAQDAgGGMBMGA1UdJQQMMAoGCCsGAQUFBwMIMEUGA1UdIAQ+MDww
# MAYEVR0gADAoMCYGCCsGAQUFBwIBFhpodHRwOi8vd3d3LmVudHJ1c3QubmV0L3Jw
# YTAIBgZngQwBBAIwDQYJKoZIhvcNAQENBQADggIBAHY9RmiFXwnAzsn4x7jXfqdc
# OwtGc84iU4s49hG6d8TChyGPJ9kMlevoS+Vc15AVb6gqLIpq4CIRNHn6vZZ5yHYW
# zbFE6nFRD5tVyRh81YJ5nPOmt5OpBThyAgaXoaiyeX5eCOJubHmaqIeUzn+PVTZA
# D58/UdVmvnurhuZuh7pOY12x2I4fy2fDDXDaeuuTshSE0j59kz8WDDagXHkAnwtu
# 5YQq2IjBbQS3N/OSmnDgOwP9zbsngs4v/I3kssq++CWFxu10yD01XzgJPXlP6xvg
# QV6ldIE/URXA+EmIj7q7BdStRYIzecEljV9UPZHkuJ8XmABdrSjlT18b+/hUtaXE
# JPzwdjv4DV7MR7DKqrtuUd9TtUka8WCHt/n9aQZ1X3PWEZDygFiO5cMPA5iMBFOz
# KYffP/ew6UxXgSREeWj+GzYH1mU97G/WwrWqYXxmBuzS5WmA+HGSLlziEUYYRUvW
# +gVk77veBspDJb3fjckBdGGI2/DHpj3jDHG8f0vnyOBu4NqA+QCewGsl1aO6XCv+
# xGAgFEoHIzHlGsuRRLKae0PxW5i1IUu4MTsiVO55YfealBtmJyrG3+FlXe9xOeZQ
# Jf2HNLmuXXwnO+dlk/gohLFvhpMsk5O2cK+h7sikYamEO/sKt/i4bSbuR071XgX6
# nA8RfdfLka5DIPcVWWO8MIIGtDCCBJygAwIBAgIQW3AmzJb2eBq7hfYR9W27bjAN
# BgkqhkiG9w0BAQ0FADBOMQswCQYDVQQGEwJVUzEWMBQGA1UEChMNRW50cnVzdCwg
# SW5jLjEnMCUGA1UEAxMeRW50cnVzdCBUaW1lIFN0YW1waW5nIENBIC0gVFMyMB4X
# DTI0MDExOTE2NDc0N1oXDTM1MDQxODAwMDAwMFowdTELMAkGA1UEBhMCQ0ExEDAO
# BgNVBAgTB09udGFyaW8xDzANBgNVBAcTBk90dGF3YTEWMBQGA1UEChMNRW50cnVz
# dCwgSW5jLjErMCkGA1UEAxMiRW50cnVzdCBUaW1lc3RhbXAgQXV0aG9yaXR5IC0g
# VFNBMjCCAiIwDQYJKoZIhvcNAQEBBQADggIPADCCAgoCggIBAKqGBDhej2i/g/Va
# /VSbxX06dZ3Epy1PbenQBR9R4zO7L186sZZUiukIfDStiYNVpJ8CzCML2Bt6QKTM
# n0LqE398uOC3SgIF4oNJBG8Ghs1zQamD1Jhn+vRSGXrjSlkoh1YXXUjxhpELuCJk
# 20H5XVS1NaRuONwIDRpWGEDDfpg8QjOEDcWlYVHOIG1PAMtC8Kh1G9tG/incG57X
# +BgFUXYwm3S9lR8ZU2UVRyxWY2vpukisCji2wXL9IAqjdBprB3/R3awXzrUcx3qJ
# DOxtHDCNBfCvPZfMNbS1BQNmup52/DnayJrWEhLkQcKFIkDwy7qI6dKHwqvGNDvl
# qw0ABg4GtYt70Qsi2jyoyVZ78DUElupZRoyP1KC1qYOwzoT7S0TlQagW86/a0aAn
# cN8H7zfzrpHO1CF/hpjcXu6PwNpH22nKELmRlYAVY/ErGsKWoR/C9zzU9n0kByWo
# VJRvClXb2yRDbi47ontJu7XWGI/ZBLO+6se1ZJpLzlJhHisFHmM3Rp2xydVeh6Tq
# 1zSyIUIXuDooOl6l7T79Nxd/YyDye9ZfyvMUP+HO/4p3lGZticHg7H8EcDhkJ/Wn
# F31NygyBG8Xl8TbFeq59KaRHNTj6DfRTAykF/G+hi8nN8biIQNPS254lMa3X5lrY
# BLSEuwRGYWb/XTLe/NM+fE4Y737fAgMBAAGjggFlMIIBYTAMBgNVHRMBAf8EAjAA
# MB0GA1UdDgQWBBT1dhr4EJiSoRt3qQAs8ZViAgt5JzAfBgNVHSMEGDAWgBQmD/DE
# SAgbzd2R9VRUtrOz/JnxCDBoBggrBgEFBQcBAQRcMFowIwYIKwYBBQUHMAGGF2h0
# dHA6Ly9vY3NwLmVudHJ1c3QubmV0MDMGCCsGAQUFBzAChidodHRwOi8vYWlhLmVu
# dHJ1c3QubmV0L3RzMi1jaGFpbjI1Ni5wN2MwMQYDVR0fBCowKDAmoCSgIoYgaHR0
# cDovL2NybC5lbnRydXN0Lm5ldC90czJjYS5jcmwwDgYDVR0PAQH/BAQDAgeAMBYG
# A1UdJQEB/wQMMAoGCCsGAQUFBwMIMEwGA1UdIARFMEMwNwYKYIZIAYb6bAoBBzAp
# MCcGCCsGAQUFBwIBFhtodHRwczovL3d3dy5lbnRydXN0Lm5ldC9ycGEwCAYGZ4EM
# AQQCMA0GCSqGSIb3DQEBDQUAA4ICAQCpq32/GgCFW9ztcSZelIKvXIJL+hw5xVO1
# wwbdA2B++WGaQSDCrkEXLqVCCJEJXpcwmIFF1rK1hD2L6YcH2gE/4AhYZxEglB7A
# Fo5jd72bbAWNPbXLmjGpSlxiMqITzpCcDceQT2fD4Kh/h4RL1/LBIQ8Q5xe5+70J
# zL6KpgH+TYJPaSCadWvkDfdQJlP8M/f+G66nasCo6x6vItM9YBWfuuTVKaS1V5+x
# ZfaxTGHV0sjat6yqGB8gGIqm+PUH+XIr9mPYJZwW/vuU34Z0oiC08gjfFu85gHY6
# krHQloNq2nxTiVkbDVzxuynyIU9hLCSJ+HYJf9lokI5XDGTVdgBxWfPUhMg8oYh/
# qG/zvW5/sKiO5pNs6RTFdRwMK87yB/u1rtRBMb/BzGmYGK5bCI+rXGS5hGq9XDih
# 309Ct0tbUtPe9zvLn2/CWUIiTgsYtyLIL/xqCYXtUClResSUS7ZxZYpKhppLeEtW
# iqHeL4uTWzLMobnvqbYM2AYLFzC/DW+Pk6zLEfXxxwCe0BnXnF+OVHxrqUM7Y9zB
# RKu9jDpD8HI/JJXNhJFdIQANeGsNXCMkn0yvNMsUWt6x5EVQ/a9vpwWCp3PPz+mS
# 6nMqseHdogPmILwdZxpWwi72rF/KToGw5lodc8/SFPNQeCgFThVuhST4M6iIr3Rt
# 9FLOjKiEnjGCBBowggQWAgEBMGIwTjELMAkGA1UEBhMCVVMxFjAUBgNVBAoTDUVu
# dHJ1c3QsIEluYy4xJzAlBgNVBAMTHkVudHJ1c3QgVGltZSBTdGFtcGluZyBDQSAt
# IFRTMgIQW3AmzJb2eBq7hfYR9W27bjALBglghkgBZQMEAgOgggGLMBoGCSqGSIb3
# DQEJAzENBgsqhkiG9w0BCRABBDAcBgkqhkiG9w0BCQUxDxcNMjUwMTA5MDMwNjI5
# WjArBgkqhkiG9w0BCTQxHjAcMAsGCWCGSAFlAwQCA6ENBgkqhkiG9w0BAQ0FADBP
# BgkqhkiG9w0BCQQxQgRAHhErCNLMugtZqDUFYOuw+DRODzkogULtfdiWlqivbaWK
# wRjQ6N5Ya7yDuv5bnaNZoPCkzBaB0BBI1eE+sFtPjjCB0AYLKoZIhvcNAQkQAi8x
# gcAwgb0wgbowgbcwCwYJYIZIAWUDBAIDBEA5EUIuFwI+qpkkmXQODsjo0nLTVfxc
# 9mz5EVavl1U05ICv07x8TFtX79H/vNt1FGXg1AVahU6bETnZ9+xV1f4kMGYwUqRQ
# ME4xCzAJBgNVBAYTAlVTMRYwFAYDVQQKEw1FbnRydXN0LCBJbmMuMScwJQYDVQQD
# Ex5FbnRydXN0IFRpbWUgU3RhbXBpbmcgQ0EgLSBUUzICEFtwJsyW9ngau4X2EfVt
# u24wDQYJKoZIhvcNAQENBQAEggIAPObBvjoxAkhTBlr30xTmwQhG7eQCbkNXuZv8
# 9qPt11oQ+f0Zi+J+7YjVDmHWniXK0BQ8ol+N1nwaUh4ZsJmJbf/wFcvBBbMndH2I
# jHiLBFY7bmkzvsblgpOSFx0X+QImImNd6ti7vyeY5Acdwr0F5Xr4VIVfamVyuzVy
# ttvjOCAuaePcAwT/4TCiVXCQDiaW8XencKOOqskiJ2iOGAkSvAIo9/IdSbg7t8JA
# AHXVBE870dteXZ4PaGO6RZu3Pzejv2Ys0q9UmsDDgMGMuzE0jnH6h7bvM6AiSBvw
# en3hr0KsmeBiZOUgXlv/6BmFeeM4rj7unQxsnBGUHzRgKt0oN6VkNH0Jw/Lz+Pgp
# 7ATSsVF/VIE7Krso2AAgI0tMTfuJNDrLLZOe9+6ZMucqTV6y3aYwp+MCq++NPNHn
# x4u3anIu6uEh5S2r4QsyC3+USorqNRO1p+TEPy/sLFRKSlDvzmRFRlwqbTvrnm5T
# g6VrXVqA1CvHsL1h6MLzXfvOJ0W2RdCq9jt/bSYaIA4wC2+GmhnwVue4m/CRk8uy
# MOuPswI1cxLrjnKIynq2MoULAB2Q5atH3Mip9pMAtcCN+obXAZhZO5jnTDuPCUfu
# SvSyyNddshCcziC50V7QTt9PQ+BjalqVXQjiFSsW/z2EskqILL580likmmCeeSOv
# GwVk3/s=
# SIG # End signature block
